PRAGMA foreign_keys=OFF;
BEGIN TRANSACTION;
CREATE TABLE metadata (
	"key" VARCHAR NOT NULL,
	value VARCHAR NOT NULL,
	PRIMARY KEY ("key")
);
INSERT INTO metadata VALUES('db_version','1');
INSERT INTO metadata VALUES('mitre_version','2.0');
CREATE TABLE technique (
	id VARCHAR NOT NULL,
	name VARCHAR NOT NULL,
	description VARCHAR,
	created_time DATETIME,
	modified_time DATETIME,
	mitre_version VARCHAR,
	mitre_detection VARCHAR,
	network_requirements BOOLEAN,
	remote_support BOOLEAN,
	revoked_by VARCHAR,
	deprecated BOOLEAN,
	subtechnique_of VARCHAR,
	PRIMARY KEY (id),
	CHECK (network_requirements IN (0, 1)),
	CHECK (remote_support IN (0, 1)),
	CHECK (deprecated IN (0, 1))
);
INSERT INTO technique VALUES('attack-pattern--01df3350-ce05-4bdf-bdf8-0a919a66d4a8','.bash_profile and .bashrc',NULL,'2017-12-14 16:46:06.044000','2020-01-24 14:14:05.452000',NULL,NULL,0,0,'attack-pattern--b63a34e8-0a61-4c97-a23b-bf8a2ed812e2',0,NULL);
INSERT INTO technique VALUES('attack-pattern--b63a34e8-0a61-4c97-a23b-bf8a2ed812e2','.bash_profile and .bashrc',replace('Adversaries may establish persistence by executing malicious content triggered by a user’s shell. <code>~/.bash_profile</code> and <code>~/.bashrc</code> are shell scripts that contain shell commands. These files are executed in a user''s context when a new shell opens or when a user logs in so that their environment is set correctly.\n\n<code>~/.bash_profile</code> is executed for login shells and <code>~/.bashrc</code> is executed for interactive non-login shells. This means that when a user logs in (via username and password) to the console (either locally or remotely via something like SSH), the <code>~/.bash_profile</code> script is executed before the initial command prompt is returned to the user. After that, every time a new shell is opened, the <code>~/.bashrc</code> script is executed. This allows users more fine-grained control over when they want certain commands executed. These shell scripts are meant to be written to by the local user to configure their own environment.\n\nThe macOS Terminal.app is a little different in that it runs a login shell by default each time a new terminal window is opened, thus calling <code>~/.bash_profile</code> each time instead of <code>~/.bashrc</code>.\n\nAdversaries may abuse these shell scripts by inserting arbitrary shell commands that may be used to execute other binaries to gain persistence. Every time the user logs in or opens a new shell, the modified ~/.bash_profile and/or ~/.bashrc scripts will be executed.(Citation: amnesia malware)','\n',char(10)),'2020-01-24 14:13:45.936000','2020-03-24 16:28:04.990000','1.0','While users may customize their <code>~/.bashrc</code> and <code>~/.bash_profile</code> files , there are only certain types of commands that typically appear in these files. Monitor for abnormal commands such as execution of unknown programs, opening network sockets, or reaching out across the network when user profiles are loaded during the login process.',0,0,NULL,0,'attack-pattern--b6301b64-ef57-4cce-bb0b-77026f14a8db');
INSERT INTO technique VALUES('attack-pattern--d0b4fcdb-d67d-4ed2-99ce-788b12f8c0f4','/etc/passwd and /etc/shadow',replace('Adversaries may attempt to dump the contents of <code>/etc/passwd</code> and <code>/etc/shadow</code> to enable offline password cracking. Most modern Linux operating systems use a combination of <code>/etc/passwd</code> and <code>/etc/shadow</code> to store user account information including password hashes in <code>/etc/shadow</code>. By default, <code>/etc/shadow</code> is only readable by the root user.(Citation: Linux Password and Shadow File Formats)\n\nThe Linux utility, unshadow, can be used to combine the two files in a format suited for password cracking utilities such as John the Ripper:(Citation: nixCraft - John the Ripper) <code># /usr/bin/unshadow /etc/passwd /etc/shadow > /tmp/crack.password.db</code>\n','\n',char(10)),'2020-02-11 18:46:56.263000','2020-03-20 15:56:55.022000','1.0','The AuditD monitoring tool, which ships stock in many Linux distributions, can be used to watch for hostile processes attempting to access <code>/etc/passwd</code> and <code>/etc/shadow</code>, alerting on the pid, process name, and arguments of such programs.',0,0,NULL,0,'attack-pattern--0a3ead4e-6d47-4ccb-854c-a6a4f9d96b22');
INSERT INTO technique VALUES('attack-pattern--cabe189c-a0e3-4965-a473-dcff00f17213','ARP Cache Poisoning',replace('Adversaries may poison Address Resolution Protocol (ARP) caches to position themselves between the communication of two or more networked devices. This activity may be used to enable follow-on behaviors such as [Network Sniffing](https://attack.mitre.org/techniques/T1040) or [Transmitted Data Manipulation](https://attack.mitre.org/techniques/T1565/002).\n\nThe ARP protocol is used to resolve IPv4 addresses to link layer addresses, such as a media access control (MAC) address.(Citation: RFC826 ARP) Devices in a local network segment communicate with each other by using link layer addresses. If a networked device does not have the link layer address of a particular networked device, it may send out a broadcast ARP request to the local network to translate the IP address to a MAC address. The device with the associated IP address directly replies with its MAC address. The networked device that made the ARP request will then use as well as store that information in its ARP cache.\n\nAn adversary may passively wait for an ARP request to poison the ARP cache of the requesting device. The adversary may reply with their MAC address, thus deceiving the victim by making them believe that they are communicating with the intended networked device. For the adversary to poison the ARP cache, their reply must be faster than the one made by the legitimate IP address owner. Adversaries may also send a gratuitous ARP reply that maliciously announces the ownership of a particular IP address to all the devices in the local network segment.\n\nThe ARP protocol is stateless and does not require authentication. Therefore, devices may wrongly add or update the MAC address of the IP address in their ARP cache.(Citation: Sans ARP Spoofing Aug 2003)(Citation: Cylance Cleaver)\n\nAdversaries may use ARP cache poisoning as a means to man-in-the-middle (MiTM) network traffic. This activity may be used to collect and/or relay data such as credentials, especially those sent over an insecure, unencrypted protocol.(Citation: Sans ARP Spoofing Aug 2003)\n','\n',char(10)),'2020-10-15 12:05:58.755000','2020-10-16 15:22:11.604000','1.0',replace('Monitor network traffic for unusual ARP traffic, gratuitous ARP replies may be suspicious. \n\nConsider collecting changes to ARP caches across endpoints for signs of ARP poisoning. For example, if multiple IP addresses map to a single MAC address, this could be an indicator that the ARP cache has been poisoned.','\n',char(10)),0,0,NULL,0,'attack-pattern--035bb001-ab69-4a0b-9f6c-2de8b09e1b9d');
INSERT INTO technique VALUES('attack-pattern--3986e7fd-a8e9-4ecb-bfc6-55920855912b','AS-REP Roasting',replace('Adversaries may reveal credentials of accounts that have disabled Kerberos preauthentication by [Password Cracking](https://attack.mitre.org/techniques/T1110/002) Kerberos messages.(Citation: Harmj0y Roasting AS-REPs Jan 2017) \n\nPreauthentication offers protection against offline [Password Cracking](https://attack.mitre.org/techniques/T1110/002). When enabled, a user requesting access to a resource initiates communication with the Domain Controller (DC) by sending an Authentication Server Request (AS-REQ) message with a timestamp that is encrypted with the hash of their password. If and only if the DC is able to successfully decrypt the timestamp with the hash of the user’s password, it will then send an Authentication Server Response (AS-REP) message that contains the Ticket Granting Ticket (TGT) to the user. Part of the AS-REP message is signed with the user’s password.(Citation: Microsoft Kerberos Preauth 2014)\n\nFor each account found without preauthentication, an adversary may send an AS-REQ message without the encrypted timestamp and receive an AS-REP message with TGT data which may be encrypted with an insecure algorithm such as RC4. The recovered encrypted data may be vulnerable to offline [Password Cracking](https://attack.mitre.org/techniques/T1110/002) attacks similarly to [Kerberoasting](https://attack.mitre.org/techniques/T1558/003) and expose plaintext credentials. (Citation: Harmj0y Roasting AS-REPs Jan 2017)(Citation: Stealthbits Cracking AS-REP Roasting Jun 2019) \n\nAn account registered to a domain, with or without special privileges, can be abused to list all domain accounts that have preauthentication disabled by utilizing Windows tools like [PowerShell](https://attack.mitre.org/techniques/T1059/001) with an LDAP filter. Alternatively, the adversary may send an AS-REQ message for each user. If the DC responds without errors, the account does not require preauthentication and the AS-REP message will already contain the encrypted data. (Citation: Harmj0y Roasting AS-REPs Jan 2017)(Citation: Stealthbits Cracking AS-REP Roasting Jun 2019)\n\nCracked hashes may enable [Persistence](https://attack.mitre.org/tactics/TA0003), [Privilege Escalation](https://attack.mitre.org/tactics/TA0004), and [Lateral Movement](https://attack.mitre.org/tactics/TA0008) via access to [Valid Accounts](https://attack.mitre.org/techniques/T1078).(Citation: SANS Attacking Kerberos Nov 2014)','\n',char(10)),'2020-08-24 13:43:00.028000','2020-10-20 19:30:11.783000','1.0','Enable Audit Kerberos Service Ticket Operations to log Kerberos TGS service ticket requests. Particularly investigate irregular patterns of activity (ex: accounts making numerous requests, Event ID 4768 and 4769, within a small time frame, especially if they also request RC4 encryption [Type 0x17], pre-authentication not required [Type: 0x0]).(Citation: AdSecurity Cracking Kerberos Dec 2015)(Citation: Microsoft Detecting Kerberoasting Feb 2018)(Citation: Microsoft 4768 TGT 2017)',0,0,NULL,0,'attack-pattern--3fc01293-ef5e-41c6-86ce-61f10706b64a');
INSERT INTO technique VALUES('attack-pattern--67720091-eee3-4d2d-ae16-8264567f6f5b','Abuse Elevation Control Mechanism','Adversaries may circumvent mechanisms designed to control elevate privileges to gain higher-level permissions. Most modern systems contain native elevation control mechanisms that are intended to limit privileges that a user can perform on a machine. Authorization has to be granted to specific users in order to perform tasks that can be considered of higher risk. An adversary can perform several methods to take advantage of built-in control mechanisms in order to escalate privileges on a system.','2020-01-30 13:58:14.373000','2020-07-22 21:36:52.825000','1.0',replace('Monitor the file system for files that have the setuid or setgid bits set. Also look for any process API calls for behavior that may be indicative of [Process Injection](https://attack.mitre.org/techniques/T1055) and unusual loaded DLLs through [DLL Search Order Hijacking](https://attack.mitre.org/techniques/T1574/001), which indicate attempts to gain access to higher privileged processes. On Linux, auditd can alert every time a user''s actual ID and effective ID are different (this is what happens when you sudo).\n\nConsider monitoring for <code>/usr/libexec/security_authtrampoline</code> executions which may indicate that AuthorizationExecuteWithPrivileges is being executed. MacOS system logs may also indicate when AuthorizationExecuteWithPrivileges is being called. Monitoring OS API callbacks for the execution can also be a way to detect this behavior but requires specialized security tooling.\n\nOn Linux, auditd can alert every time a user''s actual ID and effective ID are different (this is what happens when you sudo). This technique is abusing normal functionality in macOS and Linux systems, but sudo has the ability to log all input and output based on the <code>LOG_INPUT</code> and <code>LOG_OUTPUT</code> directives in the <code>/etc/sudoers</code> file.\n\nThere are many ways to perform UAC bypasses when a user is in the local administrator group on a system, so it may be difficult to target detection on all variations. Efforts should likely be placed on mitigation and collecting enough information on process launches and actions that could be performed before and after a UAC bypass is performed. Some UAC bypass methods rely on modifying specific, user-accessible Registry settings. Analysts should monitor Registry settings for unauthorized changes.','\n',char(10)),0,0,NULL,0,NULL);
INSERT INTO technique VALUES('attack-pattern--dcaa092b-7de9-4a21-977f-7fcb77e89c48','Access Token Manipulation',replace('Adversaries may modify access tokens to operate under a different user or system security context to perform actions and bypass access controls. Windows uses access tokens to determine the ownership of a running process. A user can manipulate access tokens to make a running process appear as though it is the child of a different process or belongs to someone other than the user that started the process. When this occurs, the process also takes on the security context associated with the new token.\n\nAn adversary can use built-in Windows API functions to copy access tokens from existing processes; this is known as token stealing. These token can then be applied to an existing process (i.e. [Token Impersonation/Theft](https://attack.mitre.org/techniques/T1134/001)) or used to spawn a new process (i.e. [Create Process with Token](https://attack.mitre.org/techniques/T1134/002)). An adversary must already be in a privileged user context (i.e. administrator) to steal a token. However, adversaries commonly use token stealing to elevate their security context from the administrator level to the SYSTEM level. An adversary can then use a token to authenticate to a remote system as the account for that token if the account has appropriate permissions on the remote system.(Citation: Pentestlab Token Manipulation)\n\nAny standard user can use the <code>runas</code> command, and the Windows API functions, to create impersonation tokens; it does not require access to an administrator account. There are also other mechanisms, such as Active Directory fields, that can be used to modify access tokens.','\n',char(10)),'2017-12-14 16:46:06.044000','2020-04-16 19:37:02.355000','2.0',replace('If an adversary is using a standard command-line shell, analysts can detect token manipulation by auditing command-line activity. Specifically, analysts should look for use of the <code>runas</code> command. Detailed command-line logging is not enabled by default in Windows.(Citation: Microsoft Command-line Logging)\n\nIf an adversary is using a payload that calls the Windows token APIs directly, analysts can detect token manipulation only through careful analysis of user network activity, examination of running processes, and correlation with other endpoint and network behavior. \n\nThere are many Windows API calls a payload can take advantage of to manipulate access tokens (e.g., <code>LogonUser</code> (Citation: Microsoft LogonUser), <code>DuplicateTokenEx</code>(Citation: Microsoft DuplicateTokenEx), and <code>ImpersonateLoggedOnUser</code>(Citation: Microsoft ImpersonateLoggedOnUser)). Please see the referenced Windows API pages for more information.\n\nQuery systems for process and thread token information and look for inconsistencies such as user owns processes impersonating the local SYSTEM account.(Citation: BlackHat Atkinson Winchester Token Manipulation)\n\nLook for inconsistencies between the various fields that store PPID information, such as the EventHeader ProcessId from data collected via Event Tracing for Windows (ETW), Creator Process ID/Name from Windows event logs, and the ProcessID and ParentProcessID (which are also produced from ETW and other utilities such as Task Manager and Process Explorer). The ETW provided EventHeader ProcessId identifies the actual parent process.','\n',char(10)),0,0,NULL,0,NULL);
INSERT INTO technique VALUES('attack-pattern--9b99b83a-1aac-4e29-b975-b374950551a3','Accessibility Features',NULL,'2017-05-31 21:30:26.946000','2020-05-13 20:37:30.008000',NULL,NULL,0,0,'attack-pattern--70e52b04-2a0c-4cea-9d18-7149f1df9dc5',0,NULL);
INSERT INTO technique VALUES('attack-pattern--70e52b04-2a0c-4cea-9d18-7149f1df9dc5','Accessibility Features',replace('Adversaries may establish persistence and/or elevate privileges by executing malicious content triggered by accessibility features. Windows contains accessibility features that may be launched with a key combination before a user has logged in (ex: when the user is on the Windows logon screen). An adversary can modify the way these programs are launched to get a command prompt or backdoor without logging in to the system.\n\nTwo common accessibility programs are <code>C:\Windows\System32\sethc.exe</code>, launched when the shift key is pressed five times and <code>C:\Windows\System32\utilman.exe</code>, launched when the Windows + U key combination is pressed. The sethc.exe program is often referred to as "sticky keys", and has been used by adversaries for unauthenticated access through a remote desktop login screen. (Citation: FireEye Hikit Rootkit)\n\nDepending on the version of Windows, an adversary may take advantage of these features in different ways. Common methods used by adversaries include replacing accessibility feature binaries or pointers/references to these binaries in the Registry. In newer versions of Windows, the replaced binary needs to be digitally signed for x64 systems, the binary must reside in <code>%systemdir%\</code>, and it must be protected by Windows File or Resource Protection (WFP/WRP). (Citation: DEFCON2016 Sticky Keys) The [Image File Execution Options Injection](https://attack.mitre.org/techniques/T1546/012) debugger method was likely discovered as a potential workaround because it does not require the corresponding accessibility feature binary to be replaced.\n\nFor simple binary replacement on Windows XP and later as well as and Windows Server 2003/R2 and later, for example, the program (e.g., <code>C:\Windows\System32\utilman.exe</code>) may be replaced with "cmd.exe" (or another program that provides backdoor access). Subsequently, pressing the appropriate key combination at the login screen while sitting at the keyboard or when connected over [Remote Desktop Protocol](https://attack.mitre.org/techniques/T1021/001) will cause the replaced file to be executed with SYSTEM privileges. (Citation: Tilbury 2014)\n\nOther accessibility features exist that may also be leveraged in a similar fashion: (Citation: DEFCON2016 Sticky Keys)(Citation: Narrator Accessibility Abuse)\n\n* On-Screen Keyboard: <code>C:\Windows\System32\osk.exe</code>\n* Magnifier: <code>C:\Windows\System32\Magnify.exe</code>\n* Narrator: <code>C:\Windows\System32\Narrator.exe</code>\n* Display Switcher: <code>C:\Windows\System32\DisplaySwitch.exe</code>\n* App Switcher: <code>C:\Windows\System32\AtBroker.exe</code>','\n',char(10)),'2020-01-24 14:32:40.315000','2020-05-13 20:37:30.048000','1.0','Changes to accessibility utility binaries or binary paths that do not correlate with known software, patch cycles, etc., are suspicious. Command line invocation of tools capable of modifying the Registry for associated keys are also suspicious. Utility arguments and the binaries themselves should be monitored for changes. Monitor Registry keys within <code>HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows NT\CurrentVersion\Image File Execution Options</code>.',0,0,NULL,0,'attack-pattern--b6301b64-ef57-4cce-bb0b-77026f14a8db');
INSERT INTO technique VALUES('attack-pattern--b24e2a20-3b3d-4bf0-823b-1ed765398fb0','Account Access Removal',replace('Adversaries may interrupt availability of system and network resources by inhibiting access to accounts utilized by legitimate users. Accounts may be deleted, locked, or manipulated (ex: changed credentials) to remove access to accounts.\n\nAdversaries may also subsequently log off and/or reboot boxes to set malicious changes into place.(Citation: CarbonBlack LockerGoga 2019)(Citation: Unit42 LockerGoga 2019)','\n',char(10)),'2019-10-09 18:48:31.906000','2020-07-14 19:15:29.911000','1.0',replace('Use process monitoring to monitor the execution and command line parameters of binaries involved in deleting accounts or changing passwords, such as use of [Net](https://attack.mitre.org/software/S0039). Windows event logs may also designate activity associated with an adversary''s attempt to remove access to an account:\n\n* Event ID 4723 - An attempt was made to change an account''s password\n* Event ID 4724 - An attempt was made to reset an account''s password\n* Event ID 4726 - A user account was deleted\n* Event ID 4740 - A user account was locked out\n\nAlerting on [Net](https://attack.mitre.org/software/S0039) and these Event IDs may generate a high degree of false positives, so compare against baseline knowledge for how systems are typically used and correlate modification events with other indications of malicious activity where possible.','\n',char(10)),0,0,NULL,0,NULL);
INSERT INTO technique VALUES('attack-pattern--72b74d71-8169-42aa-92e0-e7b04b9f5a08','Account Discovery','Adversaries may attempt to get a listing of accounts on a system or within an environment. This information can help adversaries determine which accounts exist to aid in follow-on behavior.','2017-05-31 21:31:06.988000','2020-09-16 15:10:18.260000','2.2',replace('System and network discovery techniques normally occur throughout an operation as an adversary learns the environment. Data and events should not be viewed in isolation, but as part of a chain of behavior that could lead to other activities, such as Lateral Movement, based on the information obtained.\n\nMonitor processes and command-line arguments for actions that could be taken to gather system and network information. Remote access tools with built-in features may interact directly with the Windows API to gather information. Information may also be acquired through Windows system management tools such as [Windows Management Instrumentation](https://attack.mitre.org/techniques/T1047) and [PowerShell](https://attack.mitre.org/techniques/T1059/001).','\n',char(10)),0,0,NULL,0,NULL);
INSERT INTO technique VALUES('attack-pattern--a10641f4-87b4-45a3-a906-92a149cb2c27','Account Manipulation','Adversaries may manipulate accounts to maintain access to victim systems. Account manipulation may consist of any action that preserves adversary access to a compromised account, such as modifying credentials or permission groups. These actions could also include account activity designed to subvert security policies, such as performing iterative password updates to bypass password duration policies and preserve the life of compromised credentials. In order to create or manipulate accounts, the adversary must already have sufficient permissions on systems or the domain.','2017-05-31 21:31:12.196000','2020-10-05 16:43:29.473000','2.1',replace('Collect events that correlate with changes to account objects and/or permissions on systems and the domain, such as event IDs 4738, 4728 and 4670.(Citation: Microsoft User Modified Event)(Citation: Microsoft Security Event 4670)(Citation: Microsoft Security Event 4670) Monitor for modification of accounts in correlation with other suspicious activity. Changes may occur at unusual times or from unusual systems. Especially flag events where the subject and target accounts differ(Citation: InsiderThreat ChangeNTLM July 2017) or that include additional flags such as changing a password without knowledge of the old password.(Citation: GitHub Mimikatz Issue 92 June 2017)\n\nMonitor for use of credentials at unusual times or to unusual systems or services. This may also correlate with other suspicious activity.\n\nMonitor for unusual permissions changes that may indicate excessively broad permissions being granted to compromised accounts.','\n',char(10)),0,0,NULL,0,NULL);
INSERT INTO technique VALUES('attack-pattern--0458aab9-ad42-4eac-9e22-706a95bafee2','Acquire Infrastructure',replace('Before compromising a victim, adversaries may buy, lease, or rent infrastructure that can be used during targeting. A wide variety of infrastructure exists for hosting and orchestrating adversary operations. Infrastructure solutions include physical or cloud servers, domains, and third-party web services.(Citation: TrendmicroHideoutsLease) Additionally, botnets are available for rent or purchase.\n\nUse of these infrastructure solutions allows an adversary to stage, launch, and execute an operation. Solutions may help adversary operations blend in with traffic that is seen as normal, such as contact to third-party web services. Depending on the implementation, adversaries may use infrastructure that makes it difficult to physically tie back to them as well as utilize infrastructure that can be rapidly provisioned, modified, and shut down.','\n',char(10)),'2020-09-30 16:37:40.271000','2020-10-22 17:59:17.606000','1.0',replace('Consider use of services that may aid in tracking of newly acquired infrastructure, such as WHOIS databases for domain registration information. Much of this activity may take place outside the visibility of the target organization, making detection of this behavior difficult.\n\nDetection efforts may be focused on related stages of the adversary lifecycle, such as during Command and Control.','\n',char(10)),0,0,NULL,0,NULL);
INSERT INTO technique VALUES('attack-pattern--67073dde-d720-45ae-83da-b12d5e73ca3b','Active Scanning',replace('Before compromising a victim, adversaries may execute active reconnaissance scans to gather information that can be used during targeting. Active scans are those where the adversary probes victim infrastructure via network traffic, as opposed to other forms of reconnaissance that do not involve direct interaction.\n\nAdversaries may perform different forms of active scanning depending on what information they seek to gather. These scans can also be performed in various ways, including using native features of network protocols such as ICMP.(Citation: Botnet Scan)(Citation: OWASP Fingerprinting) Information from these scans may reveal opportunities for other forms of reconnaissance (ex: [Search Open Websites/Domains](https://attack.mitre.org/techniques/T1593) or [Search Open Technical Databases](https://attack.mitre.org/techniques/T1596)), establishing operational resources (ex: [Develop Capabilities](https://attack.mitre.org/techniques/T1587) or [Obtain Capabilities](https://attack.mitre.org/techniques/T1588)), and/or initial access (ex: [External Remote Services](https://attack.mitre.org/techniques/T1133) or [Exploit Public-Facing Application](https://attack.mitre.org/techniques/T1190)).','\n',char(10)),'2020-10-02 16:53:16.526000','2020-10-24 04:06:50.402000','1.0',replace('Monitor for suspicious network traffic that could be indicative of scanning, such as large quantities originating from a single source (especially if the source is known to be associated with an adversary/botnet). Analyzing web metadata may also reveal artifacts that can be attributed to potentially malicious activity, such as referer or user-agent string HTTP/S fields.\n\nMuch of this activity may have a very high occurrence and associated false positive rate, as well as potentially taking place outside the visibility of the target organization, making detection difficult for defenders.\n\nDetection efforts may be focused on related stages of the adversary lifecycle, such as during Initial Access.','\n',char(10)),0,0,NULL,0,NULL);
INSERT INTO technique VALUES('attack-pattern--2dbbdcd5-92cf-44c0-aea2-fe24783a6bc3','Add Office 365 Global Administrator Role',replace('An adversary may add the Global Administrator role to an adversary-controlled account to maintain persistent access to an Office 365 tenant.(Citation: Microsoft Support O365 Add Another Admin, October 2019)(Citation: Microsoft O365 Admin Roles) With sufficient permissions, a compromised account can gain almost unlimited access to data and settings (including the ability to reset the passwords of other admins) via the global admin role.(Citation: Microsoft O365 Admin Roles) \n\nThis account modification may immediately follow [Create Account](https://attack.mitre.org/techniques/T1136) or other malicious account activity.','\n',char(10)),'2020-01-19 16:59:45.362000','2020-03-24 12:40:02.331000','1.0','Collect usage logs from cloud administrator accounts to identify unusual activity in the assignment of roles to those accounts. Monitor for accounts assigned to admin roles that go over a certain threshold of known admins. ',0,0,NULL,0,'attack-pattern--a10641f4-87b4-45a3-a906-92a149cb2c27');
INSERT INTO technique VALUES('attack-pattern--34f1d81d-fe88-4f97-bd3b-a3164536255d','Add-ins',replace('Adversaries may abuse Microsoft Office add-ins to obtain persistence on a compromised system. Office add-ins can be used to add functionality to Office programs. (Citation: Microsoft Office Add-ins) There are different types of add-ins that can be used by the various Office products; including Word/Excel add-in Libraries (WLL/XLL), VBA add-ins, Office Component Object Model (COM) add-ins, automation add-ins, VBA Editor (VBE), Visual Studio Tools for Office (VSTO) add-ins, and Outlook add-ins. (Citation: MRWLabs Office Persistence Add-ins)(Citation: FireEye Mail CDS 2018)\n\nAdd-ins can be used to obtain persistence because they can be set to execute code when an Office application starts. ','\n',char(10)),'2019-11-07 19:52:52.801000','2020-03-26 17:34:02.877000','1.0',replace('Monitor and validate the Office trusted locations on the file system and audit the Registry entries relevant for enabling add-ins.(Citation: GlobalDotName Jun 2019)(Citation: MRWLabs Office Persistence Add-ins)\n\nCollect process execution information including process IDs (PID) and parent process IDs (PPID) and look for abnormal chains of activity resulting from Office processes. Non-standard process execution trees may also indicate suspicious or malicious behavior','\n',char(10)),0,0,NULL,0,'attack-pattern--2c4d4e92-0ccf-4a97-b54c-86d662988a53');
INSERT INTO technique VALUES('attack-pattern--4bf5845d-a814-4490-bc5c-ccdee6043025','AppCert DLLs',NULL,'2018-01-16 16:13:52.465000','2020-01-24 14:48:05.567000',NULL,NULL,0,0,'attack-pattern--7d57b371-10c2-45e5-b3cc-83a8fb380e4c',0,NULL);
INSERT INTO technique VALUES('attack-pattern--7d57b371-10c2-45e5-b3cc-83a8fb380e4c','AppCert DLLs',replace('Adversaries may establish persistence and/or elevate privileges by executing malicious content triggered by AppCert DLLs loaded into processes. Dynamic-link libraries (DLLs) that are specified in the <code>AppCertDLLs</code> Registry key under <code>HKEY_LOCAL_MACHINE\System\CurrentControlSet\Control\Session Manager\</code> are loaded into every process that calls the ubiquitously used application programming interface (API) functions <code>CreateProcess</code>, <code>CreateProcessAsUser</code>, <code>CreateProcessWithLoginW</code>, <code>CreateProcessWithTokenW</code>, or <code>WinExec</code>. (Citation: Endgame Process Injection July 2017)\n\nSimilar to [Process Injection](https://attack.mitre.org/techniques/T1055), this value can be abused to obtain elevated privileges by causing a malicious DLL to be loaded and run in the context of separate processes on the computer. Malicious AppCert DLLs may also provide persistence by continuously being triggered by API activity. ','\n',char(10)),'2020-01-24 14:47:41.795000','2020-03-24 20:22:45.298000','1.0',replace('Monitor DLL loads by processes, specifically looking for DLLs that are not recognized or not normally loaded into a process. Monitor the AppCertDLLs Registry value for modifications that do not correlate with known software, patch cycles, etc. Monitor and analyze application programming interface (API) calls that are indicative of Registry edits such as RegCreateKeyEx and RegSetValueEx. (Citation: Endgame Process Injection July 2017) \n\nTools such as Sysinternals Autoruns may overlook AppCert DLLs as an auto-starting location. (Citation: TechNet Autoruns) (Citation: Sysinternals AppCertDlls Oct 2007)\n\nLook for abnormal process behavior that may be due to a process loading a malicious DLL. Data and events should not be viewed in isolation, but as part of a chain of behavior that could lead to other activities, such as making network connections for Command and Control, learning details about the environment through Discovery, and conducting Lateral Movement.','\n',char(10)),0,0,NULL,0,'attack-pattern--b6301b64-ef57-4cce-bb0b-77026f14a8db');
INSERT INTO technique VALUES('attack-pattern--317fefa6-46c7-4062-adb6-2008cf6bcb41','AppInit DLLs',NULL,'2017-05-31 21:31:15.409000','2020-01-24 14:52:55.240000',NULL,NULL,0,0,'attack-pattern--cc89ecbd-3d33-4a41-bcca-001e702d18fd',0,NULL);
INSERT INTO technique VALUES('attack-pattern--cc89ecbd-3d33-4a41-bcca-001e702d18fd','AppInit DLLs',replace('Adversaries may establish persistence and/or elevate privileges by executing malicious content triggered by AppInit DLLs loaded into processes. Dynamic-link libraries (DLLs) that are specified in the <code>AppInit_DLLs</code> value in the Registry keys <code>HKEY_LOCAL_MACHINE\Software\Microsoft\Windows NT\CurrentVersion\Windows</code> or <code>HKEY_LOCAL_MACHINE\Software\Wow6432Node\Microsoft\Windows NT\CurrentVersion\Windows</code> are loaded by user32.dll into every process that loads user32.dll. In practice this is nearly every program, since user32.dll is a very common library. (Citation: Endgame Process Injection July 2017)\n\nSimilar to Process Injection, these values can be abused to obtain elevated privileges by causing a malicious DLL to be loaded and run in the context of separate processes on the computer. (Citation: AppInit Registry) Malicious AppInit DLLs may also provide persistence by continuously being triggered by API activity. \n\nThe AppInit DLL functionality is disabled in Windows 8 and later versions when secure boot is enabled. (Citation: AppInit Secure Boot)','\n',char(10)),'2020-01-24 14:52:25.589000','2020-03-24 20:34:09.996000','1.0',replace('Monitor DLL loads by processes that load user32.dll and look for DLLs that are not recognized or not normally loaded into a process. Monitor the AppInit_DLLs Registry values for modifications that do not correlate with known software, patch cycles, etc. Monitor and analyze application programming interface (API) calls that are indicative of Registry edits such as <code>RegCreateKeyEx</code> and <code>RegSetValueEx</code>. (Citation: Endgame Process Injection July 2017)\n\nTools such as Sysinternals Autoruns may also be used to detect system changes that could be attempts at persistence, including listing current AppInit DLLs. (Citation: TechNet Autoruns) \n\nLook for abnormal process behavior that may be due to a process loading a malicious DLL. Data and events should not be viewed in isolation, but as part of a chain of behavior that could lead to other activities, such as making network connections for Command and Control, learning details about the environment through Discovery, and conducting Lateral Movement.','\n',char(10)),0,0,NULL,0,'attack-pattern--b6301b64-ef57-4cce-bb0b-77026f14a8db');
INSERT INTO technique VALUES('attack-pattern--5ad95aaa-49c1-4784-821d-2e83f47b079b','AppleScript',NULL,'2017-12-14 16:46:06.044000','2020-04-14 13:26:00.846000',NULL,NULL,0,0,'attack-pattern--37b11151-1776-4f8f-b328-30939fbf2ceb',0,NULL);
INSERT INTO technique VALUES('attack-pattern--37b11151-1776-4f8f-b328-30939fbf2ceb','AppleScript',replace('Adversaries may abuse AppleScript for execution. AppleScript is a macOS scripting language designed to control applications and parts of the OS via inter-application messages called AppleEvents.(Citation: Apple AppleScript) These AppleEvent messages can be sent independently or easily scripted with AppleScript. These events can locate open windows, send keystrokes, and interact with almost any open application locally or remotely.\n\nScripts can be run from the command-line via <code>osascript /path/to/script</code> or <code>osascript -e "script here"</code>. Aside from the command line, scripts can be executed in numerous ways including Mail rules, Calendar.app alarms, and Automator workflows. AppleScripts can also be executed as plain text shell scripts by adding <code>#!/usr/bin/osascript</code> to the start of the script file.(Citation: SentinelOne AppleScript)\n\nAppleScripts do not need to call <code>osascript</code> to execute, however. They may be executed from within mach-O binaries by using the macOS [Native API](https://attack.mitre.org/techniques/T1106)s <code>NSAppleScript</code> or <code>OSAScript</code>, both of which execute code independent of the <code>/usr/bin/osascript</code> command line utility.\n\nAdversaries may abuse AppleScript to execute various behaviors, such as interacting with an open SSH connection, moving to remote machines, and even presenting users with fake dialog boxes. These events cannot start applications remotely (they can start them locally), but they can interact with applications if they''re already running remotely. On macOS 10.10 Yosemite and higher, AppleScript has the ability to execute [Native API](https://attack.mitre.org/techniques/T1106)s, which otherwise would require compilation and execution in a mach-O binary file format.(Citation: SentinelOne macOS Red Team). Since this is a scripting language, it can be used to launch more common techniques as well such as a reverse shell via [Python](https://attack.mitre.org/techniques/T1059/006).(Citation: Macro Malware Targets Macs)','\n',char(10)),'2020-03-09 14:07:54.329000','2020-08-03 21:40:51.878000','1.1',replace('Monitor for execution of AppleScript through <code>osascript</code> and usage of the <code>NSAppleScript</code> and <code>OSAScript</code> APIs that may be related to other suspicious behavior occurring on the system. Scripts are likely to perform actions with various effects on a system that may generate events, depending on the types of monitoring used. Monitor processes and command-line arguments for script execution and subsequent behavior. Actions may be related to network and system information [Discovery](https://attack.mitre.org/tactics/TA0007), [Collection](https://attack.mitre.org/tactics/TA0009), or other scriptable post-compromise behaviors and could be used as indicators of detection leading back to the source script.\n\nUnderstanding standard usage patterns is important to avoid a high number of false positives. If scripting is restricted for normal users, then any attempts to enable scripts running on a system would be considered suspicious. If scripts are not commonly used on a system, but enabled, scripts running out of cycle from patching or other administrator functions are suspicious. Scripts should be captured from the file system when possible to determine their actions and intent.','\n',char(10)),0,0,NULL,0,'attack-pattern--7385dfaf-6886-4229-9ecd-6fd678040830');
INSERT INTO technique VALUES('attack-pattern--27960489-4e7f-461d-a62a-f5c0cb521e4a','Application Access Token',NULL,'2019-08-30 12:55:58.607000','2020-03-23 15:59:40.457000',NULL,NULL,0,0,'attack-pattern--f005e783-57d4-4837-88ad-dbe7faee1c51',0,NULL);
INSERT INTO technique VALUES('attack-pattern--f005e783-57d4-4837-88ad-dbe7faee1c51','Application Access Token',replace('Adversaries may use stolen application access tokens to bypass the typical authentication process and access restricted accounts, information, or services on remote systems. These tokens are typically stolen from users and used in lieu of login credentials.\n\nApplication access tokens are used to make authorized API requests on behalf of a user and are commonly used as a way to access resources in cloud-based applications and software-as-a-service (SaaS).(Citation: Auth0 - Why You Should Always Use Access Tokens to Secure APIs Sept 2019) OAuth is one commonly implemented framework that issues tokens to users for access to systems. These frameworks are used collaboratively to verify the user and determine what actions the user is allowed to perform. Once identity is established, the token allows actions to be authorized, without passing the actual credentials of the user. Therefore, compromise of the token can grant the adversary access to resources of other sites through a malicious application.(Citation: okta)\n\nFor example, with a cloud-based email service once an OAuth access token is granted to a malicious application, it can potentially gain long-term access to features of the user account if a "refresh" token enabling background access is awarded.(Citation: Microsoft Identity Platform Access 2019) With an OAuth access token an adversary can use the user-granted REST API to perform functions such as email searching and contact enumeration.(Citation: Staaldraad Phishing with OAuth 2017)\n\nCompromised access tokens may be used as an initial step in compromising other services. For example, if a token grants access to a victim’s primary email, the adversary may be able to extend access to all other services which the target subscribes by triggering forgotten password routines. Direct API access through a token negates the effectiveness of a second authentication factor and may be immune to intuitive countermeasures like changing passwords. Access abuse over an API channel can be difficult to detect even from the service provider end, as the access can still align well with a legitimate workflow.','\n',char(10)),'2020-01-30 17:37:22.261000','2020-09-16 19:40:02.024000','1.1','Monitor access token activity for abnormal use and permissions granted to unusual or suspicious applications and APIs.',0,0,NULL,0,'attack-pattern--51a14c76-dd3b-440b-9c20-2bf91d25a814');
INSERT INTO technique VALUES('attack-pattern--327f3cc5-eea1-42d4-a6cd-ed34b7ce8f61','Application Deployment Software',NULL,'2017-05-31 21:30:27.755000','2020-03-23 15:40:50.965000',NULL,NULL,0,0,'attack-pattern--92a78814-b191-47ca-909c-1ccfe3777414',0,NULL);
INSERT INTO technique VALUES('attack-pattern--18cffc21-3260-437e-80e4-4ab8bf2ba5e9','Application Exhaustion Flood','Adversaries may target resource intensive features of web applications to cause a denial of service (DoS). Specific features in web applications may be highly resource intensive. Repeated requests to those features may be able to exhaust system resources and deny access to the application or the server itself. (Citation: Arbor AnnualDoSreport Jan 2018)','2020-02-20 15:35:00.025000','2020-03-29 02:01:10.832000','1.0',replace('Detection of Endpoint DoS can sometimes be achieved before the effect is sufficient to cause significant impact to the availability of the service, but such response time typically requires very aggressive monitoring and responsiveness. Typical network throughput monitoring tools such as netflow, SNMP, and custom scripts can be used to detect sudden increases in circuit utilization.(Citation: Cisco DoSdetectNetflow) Real-time, automated, and qualitative study of the network traffic can identify a sudden surge in one type of protocol can be used to detect an attack as it starts.\n\nIn addition to network level detections, endpoint logging and instrumentation can be useful for detection. Attacks targeting web applications may generate logs in the web server, application server, and/or database server that can be used to identify the type of attack, possibly before the impact is felt.','\n',char(10)),0,0,NULL,0,'attack-pattern--c675646d-e204-4aa8-978d-e3d6d65885c4');
INSERT INTO technique VALUES('attack-pattern--355be19c-ffc9-46d5-8d50-d6a036c675b6','Application Layer Protocol',replace('Adversaries may communicate using application layer protocols to avoid detection/network filtering by blending in with existing traffic. Commands to the remote system, and often the results of those commands, will be embedded within the protocol traffic between the client and server. \n\nAdversaries may utilize many different protocols, including those used for web browsing, transferring files, electronic mail, or DNS. For connections that occur internally within an enclave (such as those between a proxy or pivot node and other nodes), commonly used protocols are SMB, SSH, or RDP. ','\n',char(10)),'2017-05-31 21:30:56.776000','2020-10-21 16:35:45.986000','2.0','Analyze network data for uncommon data flows (e.g., a client sending significantly more data than it receives from a server). Processes utilizing the network that do not normally have network communication or have never been seen before are suspicious. Analyze packet contents to detect application layer protocols that do not follow the expected protocol standards regarding syntax, structure, or any other variable adversaries could leverage to conceal data.(Citation: University of Birmingham C2)',1,0,NULL,0,NULL);
INSERT INTO technique VALUES('attack-pattern--7c93aa74-4bc0-4a9e-90ea-f25f86301566','Application Shimming',NULL,'2017-12-14 16:46:06.044000','2020-01-24 15:00:47.185000',NULL,NULL,0,0,'attack-pattern--42fe883a-21ea-4cfb-b94a-78b6476dcc83',0,NULL);
INSERT INTO technique VALUES('attack-pattern--42fe883a-21ea-4cfb-b94a-78b6476dcc83','Application Shimming',replace('Adversaries may establish persistence and/or elevate privileges by executing malicious content triggered by application shims. The Microsoft Windows Application Compatibility Infrastructure/Framework (Application Shim) was created to allow for backward compatibility of software as the operating system codebase changes over time. For example, the application shimming feature allows developers to apply fixes to applications (without rewriting code) that were created for Windows XP so that it will work with Windows 10. (Citation: Endgame Process Injection July 2017)\n\nWithin the framework, shims are created to act as a buffer between the program (or more specifically, the Import Address Table) and the Windows OS. When a program is executed, the shim cache is referenced to determine if the program requires the use of the shim database (.sdb). If so, the shim database uses hooking to redirect the code as necessary in order to communicate with the OS. \n\nA list of all shims currently installed by the default Windows installer (sdbinst.exe) is kept in:\n\n* <code>%WINDIR%\AppPatch\sysmain.sdb</code> and\n* <code>hklm\software\microsoft\windows nt\currentversion\appcompatflags\installedsdb</code>\n\nCustom databases are stored in:\n\n* <code>%WINDIR%\AppPatch\custom & %WINDIR%\AppPatch\AppPatch64\Custom</code> and\n* <code>hklm\software\microsoft\windows nt\currentversion\appcompatflags\custom</code>\n\nTo keep shims secure, Windows designed them to run in user mode so they cannot modify the kernel and you must have administrator privileges to install a shim. However, certain shims can be used to [Bypass User Account Control](https://attack.mitre.org/techniques/T1548/002) (UAC and RedirectEXE), inject DLLs into processes (InjectDLL), disable Data Execution Prevention (DisableNX) and Structure Exception Handling (DisableSEH), and intercept memory addresses (GetProcAddress).\n\nUtilizing these shims may allow an adversary to perform several malicious acts such as elevate privileges, install backdoors, disable defenses like Windows Defender, etc. (Citation: FireEye Application Shimming) Shims can also be abused to establish persistence by continuously being invoked by affected programs.','\n',char(10)),'2020-01-24 14:56:24.231000','2020-05-04 19:05:30.140000','1.0',replace('There are several public tools available that will detect shims that are currently available (Citation: Black Hat 2015 App Shim):\n\n* Shim-Process-Scanner - checks memory of every running process for any shim flags\n* Shim-Detector-Lite - detects installation of custom shim databases\n* Shim-Guard - monitors registry for any shim installations\n* ShimScanner - forensic tool to find active shims in memory\n* ShimCacheMem - Volatility plug-in that pulls shim cache from memory (note: shims are only cached after reboot)\n\nMonitor process execution for sdbinst.exe and command-line arguments for potential indications of application shim abuse.','\n',char(10)),0,0,NULL,0,'attack-pattern--b6301b64-ef57-4cce-bb0b-77026f14a8db');
INSERT INTO technique VALUES('attack-pattern--4ae4f953-fe58-4cc8-a327-33257e30a830','Application Window Discovery','Adversaries may attempt to get a listing of open application windows. Window listings could convey information about how the system is used or give context to information collected by a keylogger.','2017-05-31 21:30:24.512000','2020-03-26 15:44:27.068000','1.1',replace('System and network discovery techniques normally occur throughout an operation as an adversary learns the environment. Data and events should not be viewed in isolation, but as part of a chain of behavior that could lead to other activities based on the information obtained.\n\nMonitor processes and command-line arguments for actions that could be taken to gather system and network information. Remote access tools with built-in features may interact directly with the Windows API to gather information. Information may also be acquired through Windows system management tools such as [Windows Management Instrumentation](https://attack.mitre.org/techniques/T1047) and [PowerShell](https://attack.mitre.org/techniques/T1059/001).','\n',char(10)),0,0,NULL,0,NULL);
INSERT INTO technique VALUES('attack-pattern--2bee5ffb-7a7a-4119-b1f2-158151b19ac0','Application or System Exploitation','Adversaries may exploit software vulnerabilities that can cause an application or system to crash and deny availability to users. (Citation: Sucuri BIND9 August 2015) Some systems may automatically restart critical applications and services when crashes occur, but they can likely be re-exploited to cause a persistent DoS condition.','2020-02-20 15:37:27.052000','2020-03-29 02:07:27.508000','1.0','Attacks targeting web applications may generate logs in the web server, application server, and/or database server that can be used to identify the type of attack. Externally monitor the availability of services that may be targeted by an Endpoint DoS.',0,0,NULL,0,'attack-pattern--c675646d-e204-4aa8-978d-e3d6d65885c4');
INSERT INTO technique VALUES('attack-pattern--53ac20cd-aca3-406e-9aa0-9fc7fdc60a5a','Archive Collected Data',replace('An adversary may compress and/or encrypt data that is collected prior to exfiltration. Compressing the data can help to obfuscate the collected data and minimize the amount of data sent over the network. Encryption can be used to hide information that is being exfiltrated from detection or make exfiltration less conspicuous upon inspection by a defender.\n\nBoth compression and encryption are done prior to exfiltration, and can be performed using a utility, 3rd party library, or custom method.','\n',char(10)),'2020-02-20 20:53:45.725000','2020-10-21 16:36:55.831000','1.0',replace('Archival software and archived files can be detected in many ways. Common utilities that may be present on the system or brought in by an adversary may be detectable through process monitoring and monitoring for command-line arguments for known archival utilities. This may yield a significant number of benign events, depending on how systems in the environment are typically used.\n\nA process that loads the Windows DLL crypt32.dll may be used to perform encryption, decryption, or verification of file signatures.\n\nConsider detecting writing of files with extensions and/or headers associated with compressed or encrypted file types. Detection efforts may focus on follow-on exfiltration activity, where compressed or encrypted files can be detected in transit with a network intrusion detection or data loss prevention system analyzing file headers.(Citation: Wikipedia File Header Signatures)','\n',char(10)),0,0,NULL,0,NULL);
INSERT INTO technique VALUES('attack-pattern--143c0cbb-a297-4142-9624-87ffc778980b','Archive via Custom Method','An adversary may compress or encrypt data that is collected prior to exfiltration using a custom method. Adversaries may choose to use custom archival methods, such as encryption with XOR or stream ciphers implemented with no external library or utility references. Custom implementations of well-known compression algorithms have also been used.(Citation: ESET Sednit Part 2)','2020-02-20 21:09:55.995000','2020-03-25 22:48:14.605000','1.0','Custom archival methods can be very difficult to detect, since many of them use standard programming language concepts, such as bitwise operations.',0,0,NULL,0,'attack-pattern--53ac20cd-aca3-406e-9aa0-9fc7fdc60a5a');
INSERT INTO technique VALUES('attack-pattern--41868330-6ee2-4d0f-b743-9f2294c3c9b6','Archive via Library',replace('An adversary may compress or encrypt data that is collected prior to exfiltration using 3rd party libraries. Many libraries exist that can archive data, including [Python](https://attack.mitre.org/techniques/T1059/006) rarfile (Citation: PyPI RAR), libzip (Citation: libzip), and zlib (Citation: Zlib Github). Most libraries include functionality to encrypt and/or compress data.\n\nSome archival libraries are preinstalled on systems, such as bzip2 on macOS and Linux, and zip on Windows. Note that the libraries are different from the utilities. The libraries can be linked against when compiling, while the utilities require spawning a subshell, or a similar execution mechanism.','\n',char(10)),'2020-02-20 21:08:52.529000','2020-03-29 18:27:30.891000','1.0',replace('Monitor processes for accesses to known archival libraries. This may yield a significant number of benign events, depending on how systems in the environment are typically used.\n\nConsider detecting writing of files with extensions and/or headers associated with compressed or encrypted file types. Detection efforts may focus on follow-on exfiltration activity, where compressed or encrypted files can be detected in transit with a network intrusion detection or data loss prevention system analyzing file headers.(Citation: Wikipedia File Header Signatures)','\n',char(10)),0,0,NULL,0,'attack-pattern--53ac20cd-aca3-406e-9aa0-9fc7fdc60a5a');
INSERT INTO technique VALUES('attack-pattern--00f90846-cbd1-4fc5-9233-df5c2bf2a662','Archive via Utility',replace('An adversary may compress or encrypt data that is collected prior to exfiltration using 3rd party utilities. Many utilities exist that can archive data, including 7-Zip(Citation: 7zip Homepage), WinRAR(Citation: WinRAR Homepage), and WinZip(Citation: WinZip Homepage). Most utilities include functionality to encrypt and/or compress data.\n\nSome 3rd party utilities may be preinstalled, such as `tar` on Linux and macOS or `zip` on Windows systems.','\n',char(10)),'2020-02-20 21:01:25.428000','2020-03-25 21:54:37.374000','1.0',replace('Common utilities that may be present on the system or brought in by an adversary may be detectable through process monitoring and monitoring for command-line arguments for known archival utilities. This may yield a significant number of benign events, depending on how systems in the environment are typically used.\n\nConsider detecting writing of files with extensions and/or headers associated with compressed or encrypted file types. Detection efforts may focus on follow-on exfiltration activity, where compressed or encrypted files can be detected in transit with a network intrusion detection or data loss prevention system analyzing file headers.(Citation: Wikipedia File Header Signatures)','\n',char(10)),0,0,NULL,0,'attack-pattern--53ac20cd-aca3-406e-9aa0-9fc7fdc60a5a');
INSERT INTO technique VALUES('attack-pattern--bf176076-b789-408e-8cba-7275e81c0ada','Asymmetric Cryptography',replace('Adversaries may employ a known asymmetric encryption algorithm to conceal command and control traffic rather than relying on any inherent protections provided by a communication protocol. Asymmetric cryptography, also known as public key cryptography, uses a keypair per party: one public that can be freely distributed, and one private. Due to how the keys are generated, the sender encrypts data with the receiver’s public key and the receiver decrypts the data with their private key. This ensures that only the intended recipient can read the encrypted data. Common public key encryption algorithms include RSA and ElGamal.\n\nFor efficiency, may protocols (including SSL/TLS) use symmetric cryptography once a connection is established, but use asymmetric cryptography to establish or transmit a key. As such, these protocols are classified as [Asymmetric Cryptography](https://attack.mitre.org/techniques/T1573/002).','\n',char(10)),'2020-03-16 15:48:33.882000','2020-03-30 00:37:16.593000','1.0',replace('SSL/TLS inspection is one way of detecting command and control traffic within some encrypted communication channels.(Citation: SANS Decrypting SSL) SSL/TLS inspection does come with certain risks that should be considered before implementing to avoid potential security issues such as incomplete certificate validation.(Citation: SEI SSL Inspection Risks)\n\nIn general, analyze network data for uncommon data flows (e.g., a client sending significantly more data than it receives from a server). Processes utilizing the network that do not normally have network communication or have never been seen before are suspicious. Analyze packet contents to detect communications that do not follow the expected protocol behavior for the port that is being used.(Citation: University of Birmingham C2)','\n',char(10)),0,0,NULL,0,'attack-pattern--b8902400-e6c5-4ba2-95aa-2d35b442b118');
INSERT INTO technique VALUES('attack-pattern--7c0f17c9-1af6-4628-9cbd-9e45482dd605','Asynchronous Procedure Call',replace('Adversaries may inject malicious code into processes via the asynchronous procedure call (APC) queue in order to evade process-based defenses as well as possibly elevate privileges. APC injection is a method of executing arbitrary code in the address space of a separate live process. \n\nAPC injection is commonly performed by attaching malicious code to the APC Queue (Citation: Microsoft APC) of a process''s thread. Queued APC functions are executed when the thread enters an alterable state.(Citation: Microsoft APC) A handle to an existing victim process is first created with native Windows API calls such as <code>OpenThread</code>. At this point <code>QueueUserAPC</code> can be used to invoke a function (such as <code>LoadLibrayA</code> pointing to a malicious DLL). \n\nA variation of APC injection, dubbed "Early Bird injection", involves creating a suspended process in which malicious code can be written and executed before the process'' entry point (and potentially subsequent anti-malware hooks) via an APC. (Citation: CyberBit Early Bird Apr 2018) AtomBombing (Citation: ENSIL AtomBombing Oct 2016) is another variation that utilizes APCs to invoke malicious code previously written to the global atom table.(Citation: Microsoft Atom Table)\n\nRunning code in the context of another process may allow access to the process''s memory, system/network resources, and possibly elevated privileges. Execution via APC injection may also evade detection from security products since the execution is masked under a legitimate process. ','\n',char(10)),'2020-01-14 01:29:43.786000','2020-06-20 22:17:05.394000','1.0',replace('Monitoring Windows API calls indicative of the various types of code injection may generate a significant amount of data and may not be directly useful for defense unless collected under specific circumstances for known bad sequences of calls, since benign use of API functions may be common and difficult to distinguish from malicious behavior. Windows API calls such as <code>SuspendThread</code>/<code>SetThreadContext</code>/<code>ResumeThread</code>, <code>QueueUserAPC</code>/<code>NtQueueApcThread</code>, and those that can be used to modify memory within another process, such as <code>VirtualAllocEx</code>/<code>WriteProcessMemory</code>, may be used for this technique.(Citation: Endgame Process Injection July 2017)\n\nAnalyze process behavior to determine if a process is performing actions it usually does not, such as opening network connections, reading files, or other suspicious actions that could relate to post-compromise behavior. ','\n',char(10)),0,0,NULL,0,'attack-pattern--43e7dc91-05b2-474c-b9ac-2ed4fe101f4d');
INSERT INTO technique VALUES('attack-pattern--6636bc83-0611-45a6-b74f-1f3daf635b8e','At (Linux)',replace('Adversaries may abuse the [at](https://attack.mitre.org/software/S0110) utility to perform task scheduling for initial or recurring execution of malicious code. The [at](https://attack.mitre.org/software/S0110) command within Linux operating systems enables administrators to schedule tasks.(Citation: Kifarunix - Task Scheduling in Linux)\n\nAn adversary may use [at](https://attack.mitre.org/software/S0110) in Linux environments to execute programs at system startup or on a scheduled basis for persistence. [at](https://attack.mitre.org/software/S0110) can also be abused to conduct remote Execution as part of Lateral Movement and or to run a process under the context of a specified account.','\n',char(10)),'2019-12-03 12:59:36.749000','2020-03-23 22:35:13.112000','1.0',replace('Monitor scheduled task creation using command-line invocation. Legitimate scheduled tasks may be created during installation of new software or through system administration functions. Look for changes to tasks that do not correlate with known software, patch cycles, etc. \n\nSuspicious program execution through scheduled tasks may show up as outlier processes that have not been seen before when compared against historical data. Data and events should not be viewed in isolation, but as part of a chain of behavior that could lead to other activities, such as network connections made for Command and Control, learning details about the environment through Discovery, and Lateral Movement.','\n',char(10)),0,1,NULL,0,'attack-pattern--35dd844a-b219-4e2b-a6bb-efa9a75995a9');
INSERT INTO technique VALUES('attack-pattern--f3d95a1f-bba2-44ce-9af7-37866cd63fd0','At (Windows)',replace('Adversaries may abuse the <code>at.exe</code> utility to perform task scheduling for initial or recurring execution of malicious code. The [at](https://attack.mitre.org/software/S0110) utility exists as an executable within Windows for scheduling tasks at a specified time and date. Using [at](https://attack.mitre.org/software/S0110) requires that the Task Scheduler service be running, and the user to be logged on as a member of the local Administrators group. \n\nAn adversary may use <code>at.exe</code> in Windows environments to execute programs at system startup or on a scheduled basis for persistence. [at](https://attack.mitre.org/software/S0110) can also be abused to conduct remote Execution as part of Lateral Movement and or to run a process under the context of a specified account (such as SYSTEM).\n\nNote: The <code>at.exe</code> command line utility has been deprecated in current versions of Windows in favor of <code>schtasks</code>.','\n',char(10)),'2019-11-27 13:52:45.853000','2020-03-24 13:43:40.776000','1.0',replace('Monitor process execution from the svchost.exe in Windows 10 and the Windows Task Scheduler taskeng.exe for older versions of Windows. (Citation: Twitter Leoloobeek Scheduled Task) If scheduled tasks are not used for persistence, then the adversary is likely to remove the task when the action is complete. Monitor Windows Task Scheduler stores in %systemroot%\System32\Tasks for change entries related to scheduled tasks that do not correlate with known software, patch cycles, etc.\n\nConfigure event logging for scheduled task creation and changes by enabling the "Microsoft-Windows-TaskScheduler/Operational" setting within the event logging service. (Citation: TechNet Forum Scheduled Task Operational Setting) Several events will then be logged on scheduled task activity, including: (Citation: TechNet Scheduled Task Events)(Citation: Microsoft Scheduled Task Events Win10)\n\n* Event ID 106 on Windows 7, Server 2008 R2 - Scheduled task registered\n* Event ID 140 on Windows 7, Server 2008 R2 / 4702 on Windows 10, Server 2016 - Scheduled task updated\n* Event ID 141 on Windows 7, Server 2008 R2 / 4699 on Windows 10, Server 2016 - Scheduled task deleted\n* Event ID 4698 on Windows 10, Server 2016 - Scheduled task created\n* Event ID 4700 on Windows 10, Server 2016 - Scheduled task enabled\n* Event ID 4701 on Windows 10, Server 2016 - Scheduled task disabled\n\nTools such as Sysinternals Autoruns may also be used to detect system changes that could be attempts at persistence, including listing current scheduled tasks. (Citation: TechNet Autoruns)\n\nRemote access tools with built-in features may interact directly with the Windows API to perform these functions outside of typical system utilities. Tasks may also be created through Windows system management tools such as [Windows Management Instrumentation](https://attack.mitre.org/techniques/T1047) and [PowerShell](https://attack.mitre.org/techniques/T1059/001), so additional logging may need to be configured to gather the appropriate data.','\n',char(10)),0,1,NULL,0,'attack-pattern--35dd844a-b219-4e2b-a6bb-efa9a75995a9');
INSERT INTO technique VALUES('attack-pattern--1035cdf2-3e5f-446f-a7a7-e8f6d7925967','Audio Capture',replace('An adversary can leverage a computer''s peripheral devices (e.g., microphones and webcams) or applications (e.g., voice and video call services) to capture audio recordings for the purpose of listening into sensitive conversations to gather information.\n\nMalware or scripts may be used to interact with the devices through an available API provided by the operating system or an application to capture audio. Audio files may be written to disk and exfiltrated later.','\n',char(10)),'2017-05-31 21:31:34.528000','2020-07-14 19:42:10.235000','1.0',replace('Detection of this technique may be difficult due to the various APIs that may be used. Telemetry data regarding API use may not be useful depending on how a system is normally used, but may provide context to other potentially malicious activity occurring on a system.\n\nBehavior that could indicate technique use include an unknown or unusual process accessing APIs associated with devices or software that interact with the microphone, recording devices, or recording software, and a process periodically writing files to disk that contain audio data.','\n',char(10)),0,0,NULL,0,NULL);
INSERT INTO technique VALUES('attack-pattern--52d40641-c480-4ad5-81a3-c80ccaddf82d','Authentication Package',NULL,'2017-05-31 21:31:43.135000','2020-01-24 15:43:25.280000',NULL,NULL,0,0,'attack-pattern--b8cfed42-6a8a-4989-ad72-541af74475ec',0,NULL);
INSERT INTO technique VALUES('attack-pattern--b8cfed42-6a8a-4989-ad72-541af74475ec','Authentication Package',replace('Adversaries may abuse authentication packages to execute DLLs when the system boots. Windows authentication package DLLs are loaded by the Local Security Authority (LSA) process at system start. They provide support for multiple logon processes and multiple security protocols to the operating system. (Citation: MSDN Authentication Packages)\n\nAdversaries can use the autostart mechanism provided by LSA authentication packages for persistence by placing a reference to a binary in the Windows Registry location <code>HKLM\SYSTEM\CurrentControlSet\Control\Lsa\</code> with the key value of <code>"Authentication Packages"=&lt;target binary&gt;</code>. The binary will then be executed by the system when the authentication packages are loaded.','\n',char(10)),'2020-01-24 14:54:42.757000','2020-03-25 15:11:25.821000','1.0','Monitor the Registry for changes to the LSA Registry keys. Monitor the LSA process for DLL loads. Windows 8.1 and Windows Server 2012 R2 may generate events when unsigned DLLs try to load into the LSA by setting the Registry key <code>HKLM\SOFTWARE\Microsoft\Windows NT\CurrentVersion\Image File Execution Options\LSASS.exe</code> with AuditLevel = 8. (Citation: Graeber 2014) (Citation: Microsoft Configure LSA)',0,0,NULL,0,'attack-pattern--1ecb2399-e8ba-4f6b-8ba7-5c27d49405cf');
INSERT INTO technique VALUES('attack-pattern--30208d3e-0d6b-43c8-883e-44462a514619','Automated Collection',replace('Once established within a system or network, an adversary may use automated techniques for collecting internal data. Methods for performing this technique could include use of a [Command and Scripting Interpreter](https://attack.mitre.org/techniques/T1059) to search for and copy information fitting set criteria such as file type, location, or name at specific time intervals. This functionality could also be built into remote access tools. \n\nThis technique may incorporate use of other techniques such as [File and Directory Discovery](https://attack.mitre.org/techniques/T1083) and [Lateral Tool Transfer](https://attack.mitre.org/techniques/T1570) to identify and move files.','\n',char(10)),'2017-05-31 21:31:27.985000','2020-03-31 22:18:43.019000','1.0','Depending on the method used, actions could include common file system commands and parameters on the command-line interface within batch files or scripts. A sequence of actions like this may be unusual, depending on the system and network environment. Automated collection may occur along with other techniques such as [Data Staged](https://attack.mitre.org/techniques/T1074). As such, file access monitoring that shows an unusual process performing sequential file opens and potentially copy actions to another location on the file system for many files at once may indicate automated collection behavior. Remote access tools with built-in features may interact directly with the Windows API to gather data. Data may also be acquired through Windows system management tools such as [Windows Management Instrumentation](https://attack.mitre.org/techniques/T1047) and [PowerShell](https://attack.mitre.org/techniques/T1059/001).',0,0,NULL,0,NULL);
INSERT INTO technique VALUES('attack-pattern--774a3188-6ba9-4dc4-879d-d54ee48a5ce9','Automated Exfiltration',replace('Adversaries may exfiltrate data, such as sensitive documents, through the use of automated processing after being gathered during Collection. \n\nWhen automated exfiltration is used, other exfiltration techniques likely apply as well to transfer the information out of the network, such as [Exfiltration Over C2 Channel](https://attack.mitre.org/techniques/T1041) and [Exfiltration Over Alternative Protocol](https://attack.mitre.org/techniques/T1048).','\n',char(10)),'2017-05-31 21:30:29.458000','2020-10-22 02:24:54.881000','1.2','Monitor process file access patterns and network behavior. Unrecognized processes or scripts that appear to be traversing file systems and sending network traffic may be suspicious.',1,0,NULL,0,NULL);
INSERT INTO technique VALUES('attack-pattern--c8e87b83-edbb-48d4-9295-4974897525b7','BITS Jobs',replace('Adversaries may abuse BITS jobs to persistently execute or clean up after malicious payloads. Windows Background Intelligent Transfer Service (BITS) is a low-bandwidth, asynchronous file transfer mechanism exposed through [Component Object Model](https://attack.mitre.org/techniques/T1559/001) (COM). (Citation: Microsoft COM) (Citation: Microsoft BITS) BITS is commonly used by updaters, messengers, and other applications preferred to operate in the background (using available idle bandwidth) without interrupting other networked applications. File transfer tasks are implemented as BITS jobs, which contain a queue of one or more file operations.\n\nThe interface to create and manage BITS jobs is accessible through [PowerShell](https://attack.mitre.org/techniques/T1059/001)  (Citation: Microsoft BITS) and the [BITSAdmin](https://attack.mitre.org/software/S0190) tool. (Citation: Microsoft BITSAdmin)\n\nAdversaries may abuse BITS to download, execute, and even clean up after running malicious code. BITS tasks are self-contained in the BITS job database, without new files or registry modifications, and often permitted by host firewalls. (Citation: CTU BITS Malware June 2016) (Citation: Mondok Windows PiggyBack BITS May 2007) (Citation: Symantec BITS May 2007) BITS enabled execution may also enable persistence by creating long-standing jobs (the default maximum lifetime is 90 days and extendable) or invoking an arbitrary program when a job completes or errors (including after system reboots). (Citation: PaloAlto UBoatRAT Nov 2017) (Citation: CTU BITS Malware June 2016)\n\nBITS upload functionalities can also be used to perform [Exfiltration Over Alternative Protocol](https://attack.mitre.org/techniques/T1048). (Citation: CTU BITS Malware June 2016)','\n',char(10)),'2018-04-18 17:59:24.739000','2020-03-25 23:28:10.049000','1.1',replace('BITS runs as a service and its status can be checked with the Sc query utility (<code>sc query bits</code>). (Citation: Microsoft Issues with BITS July 2011) Active BITS tasks can be enumerated using the [BITSAdmin](https://attack.mitre.org/software/S0190) tool (<code>bitsadmin /list /allusers /verbose</code>). (Citation: Microsoft BITS)\n\nMonitor usage of the [BITSAdmin](https://attack.mitre.org/software/S0190) tool (especially the ‘Transfer’, ''Create'', ''AddFile'', ''SetNotifyFlags'', ''SetNotifyCmdLine'', ''SetMinRetryDelay'', ''SetCustomHeaders'', and ''Resume'' command options)  (Citation: Microsoft BITS)Admin and the Windows Event log for BITS activity. Also consider investigating more detailed information about jobs by parsing the BITS job database. (Citation: CTU BITS Malware June 2016)\n\nMonitor and analyze network activity generated by BITS. BITS jobs use HTTP(S) and SMB for remote connections and are tethered to the creating user and will only function when that user is logged on (this rule applies even if a user attaches the job to a service account). (Citation: Microsoft BITS)','\n',char(10)),0,0,NULL,0,NULL);
INSERT INTO technique VALUES('attack-pattern--44dca04b-808d-46ca-b25f-d85236d4b9f8','Bash History',NULL,'2017-12-14 16:46:06.044000','2020-02-04 13:03:03.354000',NULL,NULL,0,0,'attack-pattern--8187bd2a-866f-4457-9009-86b0ddedffa3',0,NULL);
INSERT INTO technique VALUES('attack-pattern--8187bd2a-866f-4457-9009-86b0ddedffa3','Bash History','Adversaries may search the bash command history on compromised systems for insecurely stored credentials. Bash keeps track of the commands users type on the command-line with the "history" utility. Once a user logs out, the history is flushed to the user’s <code>.bash_history</code> file. For each user, this file resides at the same location: <code>~/.bash_history</code>. Typically, this file keeps track of the user’s last 500 commands. Users often type usernames and passwords on the command-line as parameters to programs, which then get saved to this file when they log out. Attackers can abuse this by looking through the file for potential credentials. (Citation: External to DA, the OS X Way)','2020-02-04 13:02:11.685000','2020-02-07 20:48:49.878000','1.0','Monitoring when the user''s <code>.bash_history</code> is read can help alert to suspicious activity. While users do typically rely on their history of commands, they often access this history through other utilities like "history" instead of commands like <code>cat ~/.bash_history</code>.',0,0,NULL,0,'attack-pattern--435dfb86-2697-4867-85b5-2fef496c0517');
INSERT INTO technique VALUES('attack-pattern--be055942-6e63-49d7-9fa1-9cb7d8a8f3f4','Bidirectional Communication',replace('Adversaries may use an existing, legitimate external Web service as a means for sending commands to and receiving output from a compromised system over the Web service channel. Compromised systems may leverage popular websites and social media to host command and control (C2) instructions. Those infected systems can then send the output from those commands back over that Web service channel. The return traffic may occur in a variety of ways, depending on the Web service being utilized. For example, the return traffic may take the form of the compromised system posting a comment on a forum, issuing a pull request to development project, updating a document hosted on a Web service, or by sending a Tweet. \n\nPopular websites and social media acting as a mechanism for C2 may give a significant amount of cover due to the likelihood that hosts within a network are already communicating with them prior to a compromise. Using common services, such as those offered by Google or Twitter, makes it easier for adversaries to hide in expected noise. Web service providers commonly use SSL/TLS encryption, giving adversaries an added level of protection. ','\n',char(10)),'2020-03-14 22:34:03.024000','2020-03-26 23:15:47.861000','1.0','Host data that can relate unknown or suspicious process activity using a network connection is important to supplement any existing indicators of compromise based on malware command and control signatures and infrastructure or the presence of strong encryption. Packet capture analysis will require SSL/TLS inspection if data is encrypted. Analyze network data for uncommon data flows (e.g., a client sending significantly more data than it receives from a server). User behavior monitoring may help to detect abnormal patterns of activity.(Citation: University of Birmingham C2)',0,0,NULL,0,'attack-pattern--830c9528-df21-472c-8c14-a036bf17d665');
INSERT INTO technique VALUES('attack-pattern--519630c5-f03f-4882-825c-3af924935817','Binary Padding',NULL,'2017-05-31 21:30:22.096000','2020-09-17 18:25:33.796000',NULL,NULL,0,0,'attack-pattern--5bfccc3f-2326-4112-86cc-c1ece9d8a2b5',0,NULL);
INSERT INTO technique VALUES('attack-pattern--5bfccc3f-2326-4112-86cc-c1ece9d8a2b5','Binary Padding',replace('Adversaries may use binary padding to add junk data and change the on-disk representation of malware. This can be done without affecting the functionality or behavior of a binary, but can increase the size of the binary beyond what some security tools are capable of handling due to file size limitations. \n\nBinary padding effectively changes the checksum of the file and can also be used to avoid hash-based blocklists and static anti-virus signatures.(Citation: ESET OceanLotus) The padding used is commonly generated by a function to create junk data and then appended to the end or applied to sections of malware.(Citation: Securelist Malware Tricks April 2017) Increasing the file size may decrease the effectiveness of certain tools and detection capabilities that are not designed or configured to scan large files. This may also reduce the likelihood of being collected for analysis. Public file scanning services, such as VirusTotal, limits the maximum size of an uploaded file to be analyzed.(Citation: VirusTotal FAQ) ','\n',char(10)),'2020-02-05 14:04:25.865000','2020-09-17 18:25:33.828000','1.1','Depending on the method used to pad files, a file-based signature may be capable of detecting padding using a scanning or on-access based tool.  When executed, the resulting process from padded files may also exhibit other behavior characteristics of being used to conduct an intrusion such as system and network information Discovery or Lateral Movement, which could be used as event indicators that point to the source file. ',0,0,NULL,0,'attack-pattern--b3d682b6-98f2-4fb0-aa3b-b4df007ca70a');
CREATE TABLE IF NOT EXISTS "group" (
	id VARCHAR NOT NULL,
	name VARCHAR NOT NULL,
	description VARCHAR,
	created_time DATETIME,
	modified_time DATETIME,
	mitre_version VARCHAR,
	revoked_by VARCHAR,
	deprecated BOOLEAN,
	PRIMARY KEY (id),
	CHECK (deprecated IN (0, 1))
);
INSERT INTO "group" VALUES('intrusion-set--c4d50cdf-87ce-407d-86d8-862883485842','APT-C-36','[APT-C-36](https://attack.mitre.org/groups/G0099) is a suspected South America espionage group that has been active since at least 2018. The group mainly targets Colombian government institutions as well as important corporations in the financial sector, petroleum industry, and professional manufacturing.(Citation: QiAnXin APT-C-36 Feb2019)','2020-05-05 18:53:08.166000','2020-10-14 14:40:36.467000','1.0',NULL,0);
INSERT INTO "group" VALUES('intrusion-set--6a2e693f-24e5-451a-9f88-b36a108e5662','APT1','[APT1](https://attack.mitre.org/groups/G0006) is a Chinese threat group that has been attributed to the 2nd Bureau of the People’s Liberation Army (PLA) General Staff Department’s (GSD) 3rd Department, commonly known by its Military Unit Cover Designator (MUCD) as Unit 61398. (Citation: Mandiant APT1)','2017-05-31 21:31:47.955000','2020-10-22 18:35:55.290000','1.3',NULL,0);
INSERT INTO "group" VALUES('intrusion-set--c47f937f-1022-4f42-8525-e7a4779a14cb','APT12','[APT12](https://attack.mitre.org/groups/G0005) is a threat group that has been attributed to China. The group has targeted a variety of victims including but not limited to media outlets, high-tech companies, and multiple governments.(Citation: Meyers Numbered Panda)','2017-05-31 21:31:47.537000','2020-03-30 18:44:59.268000','2.1',NULL,0);
INSERT INTO "group" VALUES('intrusion-set--d6e88e18-81e8-4709-82d8-973095da1e70','APT16','[APT16](https://attack.mitre.org/groups/G0023) is a China-based threat group that has launched spearphishing campaigns targeting Japanese and Taiwanese organizations. (Citation: FireEye EPS Awakens Part 2)','2017-05-31 21:31:56.270000','2020-10-12 19:54:58.537000','1.1',NULL,0);
INSERT INTO "group" VALUES('intrusion-set--090242d7-73fc-4738-af68-20162f7a5aae','APT17','[APT17](https://attack.mitre.org/groups/G0025) is a China-based threat group that has conducted network intrusions against U.S. government entities, the defense industry, law firms, information technology companies, mining companies, and non-government organizations. (Citation: FireEye APT17)','2017-05-31 21:31:57.307000','2020-10-13 22:33:14.018000','1.1',NULL,0);
INSERT INTO "group" VALUES('intrusion-set--38fd6a28-3353-4f2b-bb2b-459fecd5c648','APT18','[APT18](https://attack.mitre.org/groups/G0026) is a threat group that has operated since at least 2009 and has targeted a range of industries, including technology, manufacturing, human rights groups, government, and medical. (Citation: Dell Lateral Movement)','2017-05-31 21:31:57.733000','2020-03-30 18:46:16.853000','2.1',NULL,0);
INSERT INTO "group" VALUES('intrusion-set--fe8796a4-2a02-41a0-9d27-7aa1e995feb6','APT19','[APT19](https://attack.mitre.org/groups/G0073) is a Chinese-based threat group that has targeted a variety of industries, including defense, finance, energy, pharmaceutical, telecommunications, high tech, education, manufacturing, and legal services. In 2017, a phishing campaign was used to target seven law and investment firms. (Citation: FireEye APT19) Some analysts track [APT19](https://attack.mitre.org/groups/G0073) and [Deep Panda](https://attack.mitre.org/groups/G0009) as the same group, but it is unclear from open source information if the groups are the same. (Citation: ICIT China''s Espionage Jul 2016) (Citation: FireEye APT Groups) (Citation: Unit 42 C0d0so0 Jan 2016)','2018-10-17 00:14:20.652000','2020-06-20 22:48:29.397000','1.3',NULL,0);
INSERT INTO "group" VALUES('intrusion-set--bef4c620-0787-42a8-a96d-b7eb6e85917c','APT28',replace('[APT28](https://attack.mitre.org/groups/G0007) is a threat group that has been attributed to Russia''s General Staff Main Intelligence Directorate (GRU) 85th Main Special Service Center (GTsSS) military unit 26165.(Citation: NSA/FBI Drovorub August 2020) This group has been active since at least 2004.(Citation: DOJ GRU Indictment Jul 2018) (Citation: Ars Technica GRU indictment Jul 2018) (Citation: Crowdstrike DNC June 2016) (Citation: FireEye APT28) (Citation: SecureWorks TG-4127) (Citation: FireEye APT28 January 2017) (Citation: GRIZZLY STEPPE JAR) (Citation: Sofacy DealersChoice) (Citation: Palo Alto Sofacy 06-2018) (Citation: Symantec APT28 Oct 2018) (Citation: ESET Zebrocy May 2019)\n\n[APT28](https://attack.mitre.org/groups/G0007) reportedly compromised the Hillary Clinton campaign, the Democratic National Committee, and the Democratic Congressional Campaign Committee in 2016 in an attempt to interfere with the U.S. presidential election. (Citation: Crowdstrike DNC June 2016) In 2018, the US indicted five GRU Unit 26165 officers associated with [APT28](https://attack.mitre.org/groups/G0007) for cyber operations (including close-access operations) conducted between 2014 and 2018 against the World Anti-Doping Agency (WADA), the US Anti-Doping Agency, a US nuclear facility, the Organization for the Prohibition of Chemical Weapons (OPCW), the Spiez Swiss Chemicals Laboratory, and other organizations.(Citation: US District Court Indictment GRU Oct 2018) Some of these were conducted with the assistance of GRU Unit 74455, which is also referred to as [Sandworm Team](https://attack.mitre.org/groups/G0034). ','\n',char(10)),'2017-05-31 21:31:48.664000','2020-10-06 23:32:21.793000','3.0',NULL,0);
INSERT INTO "group" VALUES('intrusion-set--899ce53f-13a0-479b-a0e4-67d46e241542','APT29','[APT29](https://attack.mitre.org/groups/G0016) is threat group that has been attributed to the Russian government and has operated since at least 2008. (Citation: F-Secure The Dukes) (Citation: GRIZZLY STEPPE JAR) This group reportedly compromised the Democratic National Committee starting in the summer of 2015. (Citation: Crowdstrike DNC June 2016)','2017-05-31 21:31:52.748000','2020-10-22 19:06:15.392000','1.4',NULL,0);
INSERT INTO "group" VALUES('intrusion-set--0bbdf25b-30ff-4894-a1cd-49260d0dd2d9','APT3',replace('[APT3](https://attack.mitre.org/groups/G0022) is a China-based threat group that researchers have attributed to China''s Ministry of State Security. (Citation: FireEye Clandestine Wolf) (Citation: Recorded Future APT3 May 2017) This group is responsible for the campaigns known as Operation Clandestine Fox, Operation Clandestine Wolf, and Operation Double Tap. (Citation: FireEye Clandestine Wolf) (Citation: FireEye Operation Double Tap) As of June 2015, the group appears to have shifted from targeting primarily US victims to primarily political organizations in Hong Kong. (Citation: Symantec Buckeye)\n\nMITRE has also developed an APT3 Adversary Emulation Plan.(Citation: APT3 Adversary Emulation Plan)','\n',char(10)),'2017-05-31 21:31:55.853000','2020-03-30 01:47:03.155000','1.3',NULL,0);
INSERT INTO "group" VALUES('intrusion-set--f047ee18-7985-4946-8bfb-4ed754d3a0dd','APT30','[APT30](https://attack.mitre.org/groups/G0013) is a threat group suspected to be associated with the Chinese government. While [Naikon](https://attack.mitre.org/groups/G0019) shares some characteristics with [APT30](https://attack.mitre.org/groups/G0013), the two groups do not appear to be exact matches.(Citation: FireEye APT30)(Citation: Baumgartner Golovkin Naikon 2015)','2017-05-31 21:31:51.026000','2020-07-29 19:34:28.999000','1.1',NULL,0);
INSERT INTO "group" VALUES('intrusion-set--247cb30b-955f-42eb-97a5-a89fef69341e','APT32','[APT32](https://attack.mitre.org/groups/G0050) is a threat group that has been active since at least 2014. The group has targeted multiple private sector industries as well as with foreign governments, dissidents, and journalists with a strong focus on Southeast Asian countries like Vietnam, the Philippines, Laos, and Cambodia. They have extensively used strategic web compromises to compromise victims. The group is believed to be Vietnam-based.(Citation: FireEye APT32 May 2017)(Citation: Volexity OceanLotus Nov 2017)(Citation: ESET OceanLotus)','2017-12-14 16:46:06.044000','2020-06-29 21:45:34.984000','2.3',NULL,0);
INSERT INTO "group" VALUES('intrusion-set--fbd29c89-18ba-4c2d-b792-51c0adee049f','APT33','[APT33](https://attack.mitre.org/groups/G0064) is a suspected Iranian threat group that has carried out operations since at least 2013. The group has targeted organizations across multiple industries in the United States, Saudi Arabia, and South Korea, with a particular interest in the aviation and energy sectors. (Citation: FireEye APT33 Sept 2017) (Citation: FireEye APT33 Webinar Sept 2017)','2018-04-18 17:59:24.739000','2020-07-01 15:48:20.759000','1.3',NULL,0);
INSERT INTO "group" VALUES('intrusion-set--68ba94ab-78b8-43e7-83e2-aed3466882c6','APT34',NULL,'2018-01-16 16:13:52.465000','2018-10-17 00:17:13.469000',NULL,'intrusion-set--4ca1929c-7d64-4aab-b849-badbfc0c760d',0);
INSERT INTO "group" VALUES('intrusion-set--4a2ce82e-1a74-468a-a6fb-bbead541383c','APT37',replace('[APT37](https://attack.mitre.org/groups/G0067) is a suspected North Korean cyber espionage group that has been active since at least 2012. The group has targeted victims primarily in South Korea, but also in Japan, Vietnam, Russia, Nepal, China, India, Romania, Kuwait, and other parts of the Middle East. [APT37](https://attack.mitre.org/groups/G0067) has also been linked to following campaigns between 2016-2018: Operation Daybreak, Operation Erebus, Golden Time, Evil New Year, Are you Happy?, FreeMilk, Northern Korean Human Rights, and Evil New Year 2018. (Citation: FireEye APT37 Feb 2018) (Citation: Securelist ScarCruft Jun 2016) (Citation: Talos Group123)\n\nNorth Korean group definitions are known to have significant overlap, and the name [Lazarus Group](https://attack.mitre.org/groups/G0032) is known to encompass a broad range of activity. Some organizations use the name Lazarus Group to refer to any activity attributed to North Korea.(Citation: US-CERT HIDDEN COBRA June 2017) Some organizations track North Korean clusters or groups such as Bluenoroff,(Citation: Kaspersky Lazarus Under The Hood Blog 2017) [APT37](https://attack.mitre.org/groups/G0067), and [APT38](https://attack.mitre.org/groups/G0082) separately, while other organizations may track some activity associated with those group names by the name Lazarus Group.','\n',char(10)),'2018-04-18 17:59:24.739000','2020-10-21 18:55:20.925000','1.5',NULL,0);
INSERT INTO "group" VALUES('intrusion-set--00f67a77-86a4-4adf-be26-1a54fc713340','APT38',replace('[APT38](https://attack.mitre.org/groups/G0082) is a financially-motivated threat group that is backed by the North Korean regime. The group mainly targets banks and financial institutions and has targeted more than 16 organizations in at least 13 countries since at least 2014.(Citation: FireEye APT38 Oct 2018)\n\nNorth Korean group definitions are known to have significant overlap, and the name [Lazarus Group](https://attack.mitre.org/groups/G0032) is known to encompass a broad range of activity. Some organizations use the name Lazarus Group to refer to any activity attributed to North Korea.(Citation: US-CERT HIDDEN COBRA June 2017) Some organizations track North Korean clusters or groups such as Bluenoroff,(Citation: Kaspersky Lazarus Under The Hood Blog 2017) [APT37](https://attack.mitre.org/groups/G0067), and [APT38](https://attack.mitre.org/groups/G0082) separately, while other organizations may track some activity associated with those group names by the name Lazarus Group.','\n',char(10)),'2019-01-29 21:27:24.793000','2020-03-30 18:50:43.737000','1.2',NULL,0);
INSERT INTO "group" VALUES('intrusion-set--44e43fad-ffcb-4210-abcf-eaaed9735f80','APT39','[APT39](https://attack.mitre.org/groups/G0087) is an Iranian cyber espionage group that has been active since at least 2014. They have targeted the telecommunication and travel industries to collect personal information that aligns with Iran''s national priorities. (Citation: FireEye APT39 Jan 2019)(Citation: Symantec Chafer Dec 2015)','2019-02-19 16:01:38.585000','2020-08-11 15:46:26.496000','2.3',NULL,0);
INSERT INTO "group" VALUES('intrusion-set--18854f55-ac7c-4634-bd9a-352dd07613b7','APT41','[APT41](https://attack.mitre.org/groups/G0096) is a group that carries out Chinese state-sponsored espionage activity in addition to financially motivated activity. [APT41](https://attack.mitre.org/groups/G0096) has been active since as early as 2012. The group has been observed targeting healthcare, telecom, technology, and video game industries in 14 countries.(Citation: FireEye APT41 Aug 2019)','2019-09-23 13:43:36.945000','2020-06-24 00:51:25.764000','1.1',NULL,0);
INSERT INTO "group" VALUES('intrusion-set--a0cb9370-e39b-44d5-9f50-ef78e412b973','Axiom','[Axiom](https://attack.mitre.org/groups/G0001) is a cyber espionage group suspected to be associated with the Chinese government. It is responsible for the Operation SMN campaign. (Citation: Novetta-Axiom) Though both this group and [Winnti Group](https://attack.mitre.org/groups/G0044) use the malware [Winnti for Windows](https://attack.mitre.org/software/S0141), the two groups appear to be distinct based on differences in reporting on the groups'' TTPs and targeting. (Citation: Kaspersky Winnti April 2013) (Citation: Kaspersky Winnti June 2015) (Citation: Novetta Winnti April 2015)','2017-05-31 21:31:45.629000','2020-03-30 18:52:38.226000','1.2',NULL,0);
INSERT INTO "group" VALUES('intrusion-set--93f52415-0fe4-4d3d-896c-fc9b8e88ab90','BRONZE BUTLER','[BRONZE BUTLER](https://attack.mitre.org/groups/G0060) is a cyber espionage group with likely Chinese origins that has been active since at least 2008. The group primarily targets Japanese organizations, particularly those in government, biotechnology, electronics manufacturing, and industrial chemistry.(Citation: Trend Micro Daserf Nov 2017)(Citation: Secureworks BRONZE BUTLER Oct 2017)(Citation: Trend Micro Tick November 2019)','2018-01-16 16:13:52.465000','2020-06-25 20:54:52.793000','1.1',NULL,0);
INSERT INTO "group" VALUES('intrusion-set--da49b9f1-ca99-443f-9728-0a074db66850','BlackOasis','[BlackOasis](https://attack.mitre.org/groups/G0063) is a Middle Eastern threat group that is believed to be a customer of Gamma Group. The group has shown interest in prominent figures in the United Nations, as well as opposition bloggers, activists, regional news correspondents, and think tanks. (Citation: Securelist BlackOasis Oct 2017) (Citation: Securelist APT Trends Q2 2017) A group known by Microsoft as [NEODYMIUM](https://attack.mitre.org/groups/G0055) is reportedly associated closely with [BlackOasis](https://attack.mitre.org/groups/G0063) operations, but evidence that the group names are aliases has not been identified. (Citation: CyberScoop BlackOasis Oct 2017)','2018-04-18 17:59:24.739000','2018-10-17 00:14:20.652000','1.0',NULL,0);
INSERT INTO "group" VALUES('intrusion-set--6fe8a2a1-a1b0-4af8-953d-4babd329f8f8','BlackTech','[BlackTech](https://attack.mitre.org/groups/G0098) is a cyber espionage group operating against targets in East Asia, particularly Taiwan, and occasionally, Japan and Hong Kong.(Citation: TrendMicro BlackTech June 2017)','2020-05-05 18:36:45.970000','2020-05-06 18:12:23.832000','1.0',NULL,0);
INSERT INTO "group" VALUES('intrusion-set--73a80fab-2aa3-48e0-a4d0-3a4828200aee','Blue Mockingbird','[Blue Mockingbird](https://attack.mitre.org/groups/G0108) is a cluster of observed activity involving Monero cryptocurrency-mining payloads in dynamic-link library (DLL) form on Windows systems. The earliest observed Blue Mockingbird tools were created in December 2019.(Citation: RedCanary Mockingbird May 2020)','2020-05-26 20:09:39.139000','2020-06-25 13:59:09.596000','1.0',NULL,0);
INSERT INTO "group" VALUES('intrusion-set--55033a4d-3ffe-46b2-99b4-2c1541e9ce1c','Carbanak','[Carbanak](https://attack.mitre.org/groups/G0008) is a threat group that mainly targets banks. It also refers to malware of the same name ([Carbanak](https://attack.mitre.org/software/S0030)). It is sometimes referred to as [FIN7](https://attack.mitre.org/groups/G0046), but these appear to be two groups using the same [Carbanak](https://attack.mitre.org/software/S0030) malware and are therefore tracked separately. (Citation: Kaspersky Carbanak) (Citation: FireEye FIN7 April 2017)','2017-05-31 21:31:49.021000','2020-03-28 00:22:39.895000','1.1',NULL,0);
INSERT INTO "group" VALUES('intrusion-set--92d5b3fd-3b39-438e-af68-770e447beada','Charming Kitten','[Charming Kitten](https://attack.mitre.org/groups/G0058) is an Iranian cyber espionage group that has been active since approximately 2014. They appear to focus on targeting individuals of interest to Iran who work in academic research, human rights, and media, with most victims having been located in Iran, the US, Israel, and the UK. [[Charming Kitten](https://attack.mitre.org/groups/G0058) often tries to access private email and Facebook accounts, and sometimes establishes a foothold on victim computers as a secondary objective. The group''s TTPs overlap extensively with another group, [Magic Hound](https://attack.mitre.org/groups/G0059), resulting in reporting that may not distinguish between the two groups'' activities.(Citation: ClearSky Charming Kitten Dec 2017)','2018-01-16 00:14:20.562000','2020-07-04 23:15:31.278000','1.0',NULL,0);
INSERT INTO "group" VALUES('intrusion-set--8c1f0187-0826-4320-bddc-5f326cfcfe2c','Chimera','[Chimera](https://attack.mitre.org/groups/G0114) is a suspected China-based threat group, targeting the semiconductor industry in Taiwan since at least 2018.(Citation: Cycraft Chimera April 2020)','2020-08-24 17:01:55.842000','2020-10-05 20:59:57.694000','1.0',NULL,0);
INSERT INTO "group" VALUES('intrusion-set--8f5e8dc7-739d-4f5e-a8a1-a66e004d7063','Cleaver','[Cleaver](https://attack.mitre.org/groups/G0003) is a threat group that has been attributed to Iranian actors and is responsible for activity tracked as Operation Cleaver. (Citation: Cylance Cleaver) Strong circumstantial evidence suggests Cleaver is linked to Threat Group 2889 (TG-2889). (Citation: Dell Threat Group 2889)','2017-05-31 21:31:46.390000','2020-10-15 16:59:26.732000','1.2',NULL,0);
INSERT INTO "group" VALUES('intrusion-set--dc6fe6ee-04c2-49be-ba3d-f38d2463c02a','Cobalt Group','[Cobalt Group](https://attack.mitre.org/groups/G0080) is a financially motivated threat group that has primarily targeted financial institutions. The group has conducted intrusions to steal money via targeting ATM systems, card processing, payment systems and SWIFT systems. [Cobalt Group](https://attack.mitre.org/groups/G0080) has mainly targeted banks in Eastern Europe, Central Asia, and Southeast Asia. One of the alleged leaders was arrested in Spain in early 2018, but the group still appears to be active. The group has been known to target organizations in order to use their access to then compromise additional victims. (Citation: Talos Cobalt Group July 2018) (Citation: PTSecurity Cobalt Group Aug 2017) (Citation: PTSecurity Cobalt Dec 2016) (Citation: Group IB Cobalt Aug 2017) (Citation: Proofpoint Cobalt June 2017) (Citation: RiskIQ Cobalt Nov 2017) (Citation: RiskIQ Cobalt Jan 2018) Reporting indicates there may be links between [Cobalt Group](https://attack.mitre.org/groups/G0080) and both the malware [Carbanak](https://attack.mitre.org/software/S0030) and the group [Carbanak](https://attack.mitre.org/groups/G0008). (Citation: Europol Cobalt Mar 2018)','2018-10-17 00:14:20.652000','2020-06-23 19:41:51.510000','1.2',NULL,0);
INSERT INTO "group" VALUES('intrusion-set--dcd81c6e-ebf7-4a16-93e0-9a97fa49c88a','CopyKittens','[CopyKittens](https://attack.mitre.org/groups/G0052) is an Iranian cyber espionage group that has been operating since at least 2013. It has targeted countries including Israel, Saudi Arabia, Turkey, the U.S., Jordan, and Germany. The group is responsible for the campaign known as Operation Wilted Tulip. (Citation: ClearSky CopyKittens March 2017) (Citation: ClearSky Wilted Tulip July 2017) (Citation: CopyKittens Nov 2015)','2018-01-16 16:13:52.465000','2020-03-31 12:43:39.280000','1.3',NULL,0);
INSERT INTO "group" VALUES('intrusion-set--8a831aaa-f3e0-47a3-bed8-a9ced744dd12','Dark Caracal','[Dark Caracal](https://attack.mitre.org/groups/G0070) is threat group that has been attributed to the Lebanese General Directorate of General Security (GDGS) and has operated since at least 2012. (Citation: Lookout Dark Caracal Jan 2018)','2018-10-17 00:14:20.652000','2020-06-03 20:22:40.401000','1.2',NULL,0);
INSERT INTO "group" VALUES('intrusion-set--6b9ebeb5-20bf-48b0-afb7-988d769a2f01','DarkHydrus','[DarkHydrus](https://attack.mitre.org/groups/G0079) is a threat group that has targeted government agencies and educational institutions in the Middle East since at least 2016. The group heavily leverages open-source tools and custom payloads for carrying out attacks. (Citation: Unit 42 DarkHydrus July 2018) (Citation: Unit 42 Playbook Dec 2017)','2018-10-17 00:14:20.652000','2020-05-15 15:44:47.629000','1.2',NULL,0);
INSERT INTO "group" VALUES('intrusion-set--813636db-3939-4a45-bea9-6113e970c029','DarkVishnya','[DarkVishnya](https://attack.mitre.org/groups/G0105) is a financially motivated threat actor targeting financial institutions in Eastern Europe. In 2017-2018 the group attacked at least 8 banks in this region.(Citation: Securelist DarkVishnya Dec 2018)','2020-05-15 13:07:26.651000','2020-05-15 15:08:55.062000','1.0',NULL,0);
INSERT INTO "group" VALUES('intrusion-set--9e729a7e-0dd6-4097-95bf-db8d64911383','Darkhotel','[Darkhotel](https://attack.mitre.org/groups/G0012) is a threat group that has been active since at least 2004. The group has conducted activity on hotel and business center Wi‑Fi and physical connections as well as peer-to-peer and file sharing networks. The actors have also conducted spearphishing. (Citation: Kaspersky Darkhotel)','2017-05-31 21:31:50.624000','2020-03-30 18:56:21.706000','1.2',NULL,0);
INSERT INTO "group" VALUES('intrusion-set--a653431d-6a5e-4600-8ad3-609b5af57064','Deep Panda','[Deep Panda](https://attack.mitre.org/groups/G0009) is a suspected Chinese threat group known to target many industries, including government, defense, financial, and telecommunications. (Citation: Alperovitch 2014) The intrusion into healthcare company Anthem has been attributed to [Deep Panda](https://attack.mitre.org/groups/G0009). (Citation: ThreatConnect Anthem) This group is also known as Shell Crew, WebMasters, KungFu Kittens, and PinkPanther. (Citation: RSA Shell Crew) [Deep Panda](https://attack.mitre.org/groups/G0009) also appears to be known as Black Vine based on the attribution of both group names to the Anthem intrusion. (Citation: Symantec Black Vine) Some analysts track [Deep Panda](https://attack.mitre.org/groups/G0009) and [APT19](https://attack.mitre.org/groups/G0073) as the same group, but it is unclear from open source information if the groups are the same. (Citation: ICIT China''s Espionage Jul 2016)','2017-05-31 21:31:49.412000','2020-04-17 21:11:30.305000','1.2',NULL,0);
INSERT INTO "group" VALUES('intrusion-set--f3bdec95-3d62-42d9-a840-29630f6cdc1a','DragonOK','[DragonOK](https://attack.mitre.org/groups/G0017) is a threat group that has targeted Japanese organizations with phishing emails. Due to overlapping TTPs, including similar custom tools, [DragonOK](https://attack.mitre.org/groups/G0017) is thought to have a direct or indirect relationship with the threat group [Moafee](https://attack.mitre.org/groups/G0002). (Citation: Operation Quantum Entanglement) It is known to use a variety of malware, including Sysget/HelloBridge, PlugX, PoisonIvy, FormerFirstRat, NFlog, and NewCT. (Citation: New DragonOK)','2017-05-31 21:31:53.197000','2019-03-22 20:10:32.917000','1.0',NULL,0);
INSERT INTO "group" VALUES('intrusion-set--1c63d4ec-0a75-4daa-b1df-0d11af3d3cc1','Dragonfly',replace('[Dragonfly](https://attack.mitre.org/groups/G0035) Dragonfly is a cyber espionage group that has been active since at least 2011. They initially targeted defense and aviation companies but shifted to focus on the energy sector in early 2013. They have also targeted companies related to industrial control systems. (Citation: Symantec Dragonfly)(Citation: Secureworks IRON LIBERTY July 2019)\n\nA similar group emerged in 2015 and was identified by Symantec as [Dragonfly 2.0](https://attack.mitre.org/groups/G0074). There is debate over the extent of the overlap between [Dragonfly](https://attack.mitre.org/groups/G0035) and [Dragonfly 2.0](https://attack.mitre.org/groups/G0074), but there is sufficient evidence to lead to these being tracked as two separate groups. (Citation: Symantec Dragonfly Sept 2017)(Citation: Fortune Dragonfly 2.0 Sept 2017)(Citation: Dragos DYMALLOY )','\n',char(10)),'2017-05-31 21:32:05.217000','2020-10-14 22:42:00.531000','2.0',NULL,0);
INSERT INTO "group" VALUES('intrusion-set--76d59913-1d24-4992-a8ac-05a3eb093f71','Dragonfly 2.0','[Dragonfly 2.0](https://attack.mitre.org/groups/G0074) is a suspected Russian group that has targeted government entities and multiple U.S. critical infrastructure sectors since at least March 2016. (Citation: US-CERT TA18-074A) (Citation: Symantec Dragonfly Sept 2017) There is debate over the extent of overlap between [Dragonfly 2.0](https://attack.mitre.org/groups/G0074) and [Dragonfly](https://attack.mitre.org/groups/G0035), but there is sufficient evidence to lead to these being tracked as two separate groups. (Citation: Fortune Dragonfly 2.0 Sept 2017)(Citation: Dragos DYMALLOY )','2018-10-17 00:14:20.652000','2020-10-15 20:14:58.980000','1.3',NULL,0);
INSERT INTO "group" VALUES('intrusion-set--ae41895a-243f-4a65-b99b-d85022326c31','Dust Storm','[Dust Storm](https://attack.mitre.org/groups/G0031) is a threat group that has targeted multiple industries in Japan, South Korea, the United States, Europe, and several Southeast Asian countries. (Citation: Cylance Dust Storm)','2017-05-31 21:32:03.306000','2019-03-22 20:14:29.575000','1.0',NULL,0);
INSERT INTO "group" VALUES('intrusion-set--03506554-5f37-4f8f-9ce4-0e9f01a1b484','Elderwood','[Elderwood](https://attack.mitre.org/groups/G0066) is a suspected Chinese cyber espionage group that was reportedly responsible for the 2009 Google intrusion known as Operation Aurora. (Citation: Security Affairs Elderwood Sept 2012) The group has targeted defense organizations, supply chain manufacturers, human rights and nongovernmental organizations (NGOs), and IT service providers. (Citation: Symantec Elderwood Sept 2012) (Citation: CSM Elderwood Sept 2012)','2018-04-18 17:59:24.739000','2020-03-30 18:58:36.955000','1.1',NULL,0);
INSERT INTO "group" VALUES('intrusion-set--96e239be-ad99-49eb-b127-3007b8c1bec9','Equation','[Equation](https://attack.mitre.org/groups/G0020) is a sophisticated threat group that employs multiple remote access tools. The group is known to use zero-day exploits and has developed the capability to overwrite the firmware of hard disk drives. (Citation: Kaspersky Equation QA)','2017-05-31 21:31:54.697000','2020-06-29 01:39:22.044000','1.2',NULL,0);
INSERT INTO "group" VALUES('intrusion-set--fbe9387f-34e6-4828-ac28-3080020c597b','FIN10','[FIN10](https://attack.mitre.org/groups/G0051) is a financially motivated threat group that has targeted organizations in North America since at least 2013 through 2016. The group uses stolen data exfiltrated from victims to extort organizations. (Citation: FireEye FIN10 June 2017)','2017-12-14 16:46:06.044000','2020-03-28 21:26:28.528000','1.2',NULL,0);
INSERT INTO "group" VALUES('intrusion-set--d0b3393b-3bec-4ba3-bda9-199d30db47b6','FIN4','[FIN4](https://attack.mitre.org/groups/G0085) is a financially-motivated threat group that has targeted confidential information related to the public financial market, particularly regarding healthcare and pharmaceutical companies, since at least 2013.(Citation: FireEye Hacking FIN4 Dec 2014)(Citation: FireEye FIN4 Stealing Insider NOV 2014) [FIN4](https://attack.mitre.org/groups/G0085) is unique in that they do not infect victims with typical persistent malware, but rather they focus on capturing credentials authorized to access email and other non-public correspondence.(Citation: FireEye Hacking FIN4 Dec 2014)(Citation: FireEye Hacking FIN4 Video Dec 2014)','2019-01-31 02:01:45.129000','2020-06-23 19:52:35.625000','1.1',NULL,0);
INSERT INTO "group" VALUES('intrusion-set--85403903-15e0-4f9f-9be4-a259ecad4022','FIN5','[FIN5](https://attack.mitre.org/groups/G0053) is a financially motivated threat group that has targeted personally identifiable information and payment card information. The group has been active since at least 2008 and has targeted the restaurant, gaming, and hotel industries. The group is made up of actors who likely speak Russian. (Citation: FireEye Respond Webinar July 2017) (Citation: Mandiant FIN5 GrrCON Oct 2016) (Citation: DarkReading FireEye FIN5 Oct 2015)','2018-01-16 16:13:52.465000','2020-03-19 22:54:59.268000','1.1',NULL,0);
INSERT INTO "group" VALUES('intrusion-set--2a7914cf-dff3-428d-ab0f-1014d1c28aeb','FIN6','[FIN6](https://attack.mitre.org/groups/G0037) is a cyber crime group that has stolen payment card data and sold it for profit on underground marketplaces. This group has aggressively targeted and compromised point of sale (PoS) systems in the hospitality and retail sectors.(Citation: FireEye FIN6 April 2016)(Citation: FireEye FIN6 Apr 2019)','2017-05-31 21:32:06.015000','2020-10-21 00:44:24.198000','3.0',NULL,0);
INSERT INTO "group" VALUES('intrusion-set--3753cc21-2dae-4dfb-8481-d004e74502cc','FIN7','[FIN7](https://attack.mitre.org/groups/G0046) is a financially-motivated threat group that has primarily targeted the U.S. retail, restaurant, and hospitality sectors since mid-2015. They often use point-of-sale malware. A portion of [FIN7](https://attack.mitre.org/groups/G0046) was run out of a front company called Combi Security. [FIN7](https://attack.mitre.org/groups/G0046) is sometimes referred to as [Carbanak](https://attack.mitre.org/groups/G0008) Group, but these appear to be two groups using the same [Carbanak](https://attack.mitre.org/software/S0030) malware and are therefore tracked separately. (Citation: FireEye FIN7 March 2017) (Citation: FireEye FIN7 April 2017) (Citation: FireEye CARBANAK June 2017) (Citation: FireEye FIN7 Aug 2018)','2017-05-31 21:32:09.460000','2020-10-22 18:47:28.215000','1.5',NULL,0);
INSERT INTO "group" VALUES('intrusion-set--fd19bd82-1b14-49a1-a176-6cdc46b8a826','FIN8','[FIN8](https://attack.mitre.org/groups/G0061) is a financially motivated threat group known to launch tailored spearphishing campaigns targeting the retail, restaurant, and hospitality industries. (Citation: FireEye Obfuscation June 2017) (Citation: FireEye Fin8 May 2016)','2018-04-18 17:59:24.739000','2020-03-30 02:15:00.553000','1.1',NULL,0);
INSERT INTO "group" VALUES('intrusion-set--6b1b551c-d770-4f95-8cfc-3cd253c4c04e','Frankenstein','[Frankenstein](https://attack.mitre.org/groups/G0101) is a campaign carried out between January and April 2019 by unknown threat actors. The campaign name comes from the actors'' ability to piece together several unrelated components.(Citation: Talos Frankenstein June 2019)  ','2020-05-11 15:21:09.438000','2020-05-28 00:01:09.384000','1.0',NULL,0);
INSERT INTO "group" VALUES('intrusion-set--0ea72cd5-ca30-46ba-bc04-378f701c658f','GCMAN','[GCMAN](https://attack.mitre.org/groups/G0036) is a threat group that focuses on targeting banks for the purpose of transferring money to e-currency services. (Citation: Securelist GCMAN)','2017-05-31 21:32:05.611000','2020-03-30 19:03:44.853000','1.1',NULL,0);
INSERT INTO "group" VALUES('intrusion-set--c77c5576-ca19-42ed-a36f-4b4486a84133','GOLD SOUTHFIELD','[GOLD SOUTHFIELD](https://attack.mitre.org/groups/G0115) is a financially motivated threat group active since at least 2019 that operates the [REvil](https://attack.mitre.org/software/S0496) Ransomware-as-a Service (RaaS). [GOLD SOUTHFIELD](https://attack.mitre.org/groups/G0115) provides backend infrastructure for affiliates recruited on underground forums to perpetrate high value deployments.(Citation: Secureworks REvil September 2019)(Citation: Secureworks GandCrab and REvil September 2019)(Citation: Secureworks GOLD SOUTHFIELD)','2020-09-22 19:41:27.845000','2020-10-06 15:32:20.089000','1.0',NULL,0);
INSERT INTO "group" VALUES('intrusion-set--2fd2be6a-d3a2-4a65-b499-05ea2693abee','Gallmaker','[Gallmaker](https://attack.mitre.org/groups/G0084) is a cyberespionage group that has targeted victims in the Middle East and has been active since at least December 2017. The group has mainly targeted victims in the defense, military, and government sectors.(Citation: Symantec Gallmaker Oct 2018)','2019-01-30 14:26:42.897000','2020-03-30 19:04:47.798000','1.1',NULL,0);
CREATE TABLE software (
	id VARCHAR NOT NULL,
	name VARCHAR NOT NULL,
	description VARCHAR,
	created_time DATETIME,
	modified_time DATETIME,
	mitre_version VARCHAR,
	revoked_by VARCHAR,
	deprecated BOOLEAN,
	PRIMARY KEY (id),
	CHECK (deprecated IN (0, 1))
);
INSERT INTO software VALUES('malware--7bec698a-7e20-4fd3-bb6a-12787770fb1a','3PARA RAT','[3PARA RAT](https://attack.mitre.org/software/S0066) is a remote access tool (RAT) programmed in C++ that has been used by [Putter Panda](https://attack.mitre.org/groups/G0024). (Citation: CrowdStrike Putter Panda)','2017-05-31 21:32:44.131000','2020-03-30 18:34:04.031000','1.1',NULL,0);
INSERT INTO software VALUES('malware--8e461ca3-0996-4e6e-a0df-e2a5bbc51ebc','4H RAT','[4H RAT](https://attack.mitre.org/software/S0065) is malware that has been used by [Putter Panda](https://attack.mitre.org/groups/G0024) since at least 2007. (Citation: CrowdStrike Putter Panda)','2017-05-31 21:32:43.664000','2020-03-30 14:46:14.131000','1.1',NULL,0);
INSERT INTO software VALUES('malware--a0ebedca-d558-4e48-8ff7-4bf76208d90c','ABK','[ABK](https://attack.mitre.org/software/S0469) is a downloader that has been used by [BRONZE BUTLER](https://attack.mitre.org/groups/G0060) since at least 2019.(Citation: Trend Micro Tick November 2019)','2020-06-10 16:58:56.032000','2020-06-24 15:34:14.618000','1.0',NULL,0);
INSERT INTO software VALUES('malware--fb575479-14ef-41e9-bfab-0b7cf10bec73','ADVSTORESHELL','[ADVSTORESHELL](https://attack.mitre.org/software/S0045) is a spying backdoor that has been used by [APT28](https://attack.mitre.org/groups/G0007) from at least 2012 to 2016. It is generally used for long-term espionage and is deployed on targets deemed interesting after a reconnaissance phase. (Citation: Kaspersky Sofacy) (Citation: ESET Sednit Part 2)','2017-05-31 21:32:34.648000','2020-03-30 01:44:19.899000','1.1',NULL,0);
INSERT INTO software VALUES('malware--56f46b17-8cfa-46c0-b501-dd52fef394e2','ASPXSpy','[ASPXSpy](https://attack.mitre.org/software/S0073) is a Web shell. It has been modified by [Threat Group-3390](https://attack.mitre.org/groups/G0027) actors to create the ASPXTool version. (Citation: Dell TG-3390)','2017-05-31 21:32:47.879000','2020-03-30 14:48:21.994000','1.1',NULL,0);
INSERT INTO software VALUES('malware--e7a5229f-05eb-440e-b982-9a6d2b2b87c8','Agent Tesla','[Agent Tesla](https://attack.mitre.org/software/S0331) is a spyware Trojan written for the .NET framework that has been observed since at least 2014.(Citation: Fortinet Agent Tesla April 2018)(Citation: Bitdefender Agent Tesla April 2020)(Citation: Malwarebytes Agent Tesla April 2020)','2019-01-29 18:44:04.748000','2020-05-28 23:41:03.616000','1.1',NULL,0);
INSERT INTO software VALUES('malware--40d3e230-ed32-469f-ba89-be70cc08ab39','Agent.btz','[Agent.btz](https://attack.mitre.org/software/S0092) is a worm that primarily spreads itself via removable devices such as USB drives. It reportedly infected U.S. military networks in 2008. (Citation: Securelist Agent.btz)','2017-05-31 21:32:59.153000','2020-03-30 14:50:51.213000','1.1',NULL,0);
INSERT INTO software VALUES('malware--5f1d4579-4e8f-48e7-860e-2da773ae432e','Anchor','[Anchor](https://attack.mitre.org/software/S0504) is one of a family of backdoor malware that has been used in conjunction with [TrickBot](https://attack.mitre.org/software/S0266) on selected high profile targets since at least 2018.(Citation: Cyberreason Anchor December 2019)(Citation: Medium Anchor DNS July 2020)','2020-09-10 15:54:21.805000','2020-10-05 17:54:53.991000','1.0',NULL,0);
INSERT INTO software VALUES('malware--3161d76a-e2b2-4b97-9906-24909b735386','Aria-body','[Aria-body](https://attack.mitre.org/software/S0456) is a custom backdoor that has been used by [Naikon](https://attack.mitre.org/groups/G0019).(Citation: CheckPoint Naikon May 2020)','2020-05-26 19:36:04.663000','2020-07-03 21:52:44.685000','1.0',NULL,0);
INSERT INTO software VALUES('malware--edb24a93-1f7a-4bbf-a738-1397a14662c6','Astaroth','[Astaroth](https://attack.mitre.org/software/S0373) is a Trojan and information stealer known to affect companies in Europe and Brazil. It has been known publicly since at least late 2017. (Citation: Cybereason Astaroth Feb 2019) (Citation: Cofense Astaroth Sept 2018)','2019-04-17 13:46:38.565000','2020-06-23 19:38:54.935000','1.2',NULL,0);
INSERT INTO software VALUES('malware--8f423bd7-6ca7-4303-9e85-008c7ad5fdaa','Attor','[Attor](https://attack.mitre.org/software/S0438) is a Windows-based espionage platform that has been seen in use since 2013. [Attor](https://attack.mitre.org/software/S0438) has a loadable plugin architecture to customize functionality for specific targets.(Citation: ESET Attor Oct 2019)','2020-05-06 20:26:15.141000','2020-07-07 12:35:11.897000','1.0',NULL,0);
INSERT INTO software VALUES('malware--24b4ce59-eaac-4c8b-8634-9b093b7ccd92','AuditCred','[AuditCred](https://attack.mitre.org/software/S0347) is a malicious DLL that has been used by [Lazarus Group](https://attack.mitre.org/groups/G0032) during their 2018 attacks.(Citation: TrendMicro Lazarus Nov 2018)','2019-01-30 15:47:41.018000','2020-03-30 14:51:50.371000','1.1',NULL,0);
INSERT INTO software VALUES('malware--f5352566-1a64-49ac-8f7f-97e1d1a03300','AutoIt backdoor','[AutoIt backdoor](https://attack.mitre.org/software/S0129) is malware that has been used by the actors responsible for the MONSOON campaign. The actors frequently used it in weaponized .pps files exploiting CVE-2014-6352. (Citation: Forcepoint Monsoon) This malware makes use of the legitimate scripting language for Windows GUI automation with the same name.','2017-05-31 21:33:14.551000','2020-03-30 14:52:48.605000','1.1',NULL,0);
INSERT INTO software VALUES('malware--36ede314-7db4-4d09-b53d-81bbfbe5f6f8','Avenger','[Avenger](https://attack.mitre.org/software/S0473) is a downloader that has been used by [BRONZE BUTLER](https://attack.mitre.org/groups/G0060) since at least 2019.(Citation: Trend Micro Tick November 2019)','2020-06-11 15:24:48.709000','2020-06-24 17:44:18.663000','1.0',NULL,0);
INSERT INTO software VALUES('malware--f9b05f33-d45d-4e4d-aafe-c208d38a0080','Azorult',replace('[Azorult](https://attack.mitre.org/software/S0344) is a commercial Trojan that is used to steal information from compromised hosts. [Azorult](https://attack.mitre.org/software/S0344) has been observed in the wild as early as 2016.\nIn July 2018, [Azorult](https://attack.mitre.org/software/S0344) was seen used in a spearphishing campaign against targets in North America. [Azorult](https://attack.mitre.org/software/S0344) has been seen used for cryptocurrency theft. (Citation: Unit42 Azorult Nov 2018)(Citation: Proofpoint Azorult July 2018)','\n',char(10)),'2019-01-30 15:19:14.309000','2020-03-30 14:56:50.733000','1.2',NULL,0);
INSERT INTO software VALUES('malware--fb261c56-b80e-43a9-8351-c84081e7213d','BACKSPACE','[BACKSPACE](https://attack.mitre.org/software/S0031) is a backdoor used by [APT30](https://attack.mitre.org/groups/G0013) that dates back to at least 2005. (Citation: FireEye APT30)','2017-05-31 21:32:24.428000','2020-03-30 14:54:21.256000','1.1',NULL,0);
INSERT INTO software VALUES('malware--9dbdadb6-fdbf-490f-a35f-38762d06a0d2','BADCALL','[BADCALL](https://attack.mitre.org/software/S0245) is a Trojan malware variant used by the group [Lazarus Group](https://attack.mitre.org/groups/G0032). (Citation: US-CERT BADCALL)','2018-10-17 00:14:20.652000','2020-03-30 18:32:03.328000','1.1',NULL,0);
INSERT INTO software VALUES('malware--e9595678-d269-469e-ae6b-75e49259de63','BADNEWS','[BADNEWS](https://attack.mitre.org/software/S0128) is malware that has been used by the actors responsible for the [Patchwork](https://attack.mitre.org/groups/G0040) campaign. Its name was given due to its use of RSS feeds, forums, and blogs for command and control. (Citation: Forcepoint Monsoon) (Citation: TrendMicro Patchwork Dec 2017)','2017-05-31 21:33:14.118000','2020-10-21 18:22:52.183000','1.2',NULL,0);
INSERT INTO software VALUES('malware--f0fc920e-57a3-4af5-89be-9ea594c8b1ea','BBK','[BBK](https://attack.mitre.org/software/S0470) is a downloader that has been used by [BRONZE BUTLER](https://attack.mitre.org/groups/G0060) since at least 2019.(Citation: Trend Micro Tick November 2019)','2020-06-10 18:00:28.497000','2020-06-24 15:36:00.792000','1.0',NULL,0);
INSERT INTO software VALUES('malware--64d76fa5-cf8f-469c-b78c-1a4f7c5bad80','BBSRAT','[BBSRAT](https://attack.mitre.org/software/S0127) is malware with remote access tool functionality that has been used in targeted compromises. (Citation: Palo Alto Networks BBSRAT)','2017-05-31 21:33:13.664000','2020-03-30 14:55:06.553000','1.2',NULL,0);
INSERT INTO software VALUES('malware--b8eb28e4-48a6-40ae-951a-328714f75eda','BISCUIT','[BISCUIT](https://attack.mitre.org/software/S0017) is a backdoor that has been used by [APT1](https://attack.mitre.org/groups/G0006) since as early as 2007. (Citation: Mandiant APT1)','2017-05-31 21:32:17.147000','2020-03-30 14:57:52.169000','1.2',NULL,0);
INSERT INTO software VALUES('malware--d69c8146-ab35-4d50-8382-6fc80e641d43','BLACKCOFFEE','[BLACKCOFFEE](https://attack.mitre.org/software/S0069) is malware that has been used by several Chinese groups since at least 2013. (Citation: FireEye APT17) (Citation: FireEye Periscope March 2018)','2017-05-31 21:32:45.892000','2020-03-30 14:58:42.298000','1.1',NULL,0);
INSERT INTO software VALUES('malware--d5268dfb-ae2b-4e0e-ac07-02a460613d8a','BONDUPDATER','[BONDUPDATER](https://attack.mitre.org/software/S0360) is a PowerShell backdoor used by [OilRig](https://attack.mitre.org/groups/G0049). It was first observed in November 2017 during targeting of a Middle Eastern government organization, and an updated version was observed in August 2018 being used to target a government organization with spearphishing emails.(Citation: FireEye APT34 Dec 2017)(Citation: Palo Alto OilRig Sep 2018)','2019-02-18 20:16:12.119000','2020-03-30 15:22:05.356000','1.2',NULL,0);
INSERT INTO software VALUES('malware--56d10a7f-bb42-4267-9b4c-63abb9c06010','BOOSTWRITE','[BOOSTWRITE](https://attack.mitre.org/software/S0415) is a loader crafted to be launched via abuse of the DLL search order of applications used by [FIN7](https://attack.mitre.org/groups/G0046).(Citation: FireEye FIN7 Oct 2019)','2019-10-11 16:04:31.994000','2019-10-15 17:07:57.638000','1.0',NULL,0);
INSERT INTO software VALUES('malware--da2ef4a9-7cbe-400a-a379-e2f230f28db3','BOOTRASH','[BOOTRASH](https://attack.mitre.org/software/S0114) is a [Bootkit](https://attack.mitre.org/techniques/T1067) that targets Windows operating systems. It has been used by threat actors that target the financial sector.(Citation: Mandiant M Trends 2016)(Citation: FireEye Bootkits)(Citation: FireEye BOOTRASH SANS)','2017-05-31 21:33:08.292000','2020-06-29 01:35:30.160000','1.1',NULL,0);
INSERT INTO software VALUES('malware--67fc172a-36fa-4a35-88eb-4ba730ed52a6','BS2005','[BS2005](https://attack.mitre.org/software/S0014) is malware that was used by [Ke3chang](https://attack.mitre.org/groups/G0004) in spearphishing campaigns since at least 2011. (Citation: Villeneuve et al 2014)','2017-05-31 21:32:15.994000','2020-03-30 15:02:35.427000','1.1',NULL,0);
INSERT INTO software VALUES('malware--123bd7b3-675c-4b1a-8482-c55782b20e2b','BUBBLEWRAP','[BUBBLEWRAP](https://attack.mitre.org/software/S0043) is a full-featured, second-stage backdoor used by the [admin@338](https://attack.mitre.org/groups/G0018) group. It is set to run when the system boots and includes functionality to check, upload, and register plug-ins that can further enhance its capabilities. (Citation: FireEye admin@338)','2017-05-31 21:32:33.738000','2020-03-30 15:03:26.307000','1.1',NULL,0);
INSERT INTO software VALUES('malware--d1b7830a-fced-4be3-a99c-f495af9d9e1b','BabyShark','[BabyShark](https://attack.mitre.org/software/S0414) is a Microsoft Visual Basic (VB) script-based malware family that is believed to be associated with several North Korean campaigns. (Citation: Unit42 BabyShark Feb 2019)','2019-10-07 19:05:48.886000','2020-03-30 15:06:31.915000','1.1',NULL,0);
INSERT INTO software VALUES('malware--c13d9621-aca7-436b-ab3d-3a95badb3d00','BackConfig','[BackConfig](https://attack.mitre.org/software/S0475) is a custom Trojan with a flexible plugin architecture that has been used by [Patchwork](https://attack.mitre.org/groups/G0040).(Citation: Unit 42 BackConfig May 2020)','2020-06-17 20:17:37.168000','2020-06-29 15:59:07.478000','1.0',NULL,0);
INSERT INTO software VALUES('malware--083bb47b-02c8-4423-81a2-f9ef58572974','Backdoor.Oldrea','[Backdoor.Oldrea](https://attack.mitre.org/software/S0093) is a backdoor used by [Dragonfly](https://attack.mitre.org/groups/G0035). It appears to be custom malware authored by the group or specifically for it. (Citation: Symantec Dragonfly)','2017-05-31 21:32:59.661000','2020-03-30 02:49:50.902000','1.1',NULL,0);
INSERT INTO software VALUES('malware--9af05de0-bc09-4511-a350-5eb8b06185c1','BadPatch','[BadPatch](https://attack.mitre.org/software/S0337) is a Windows Trojan that was used in a Gaza Hackers-linked campaign.(Citation: Unit 42 BadPatch Oct 2017)','2019-01-29 21:33:34.082000','2020-03-17 00:22:32.796000','1.1',NULL,0);
INSERT INTO software VALUES('malware--835a79f1-842d-472d-b8f4-d54b545c341b','Bandook','[Bandook](https://attack.mitre.org/software/S0234) is a commercially available RAT, written in Delphi, which has been available since roughly 2007  (Citation: EFF Manul Aug 2016) (Citation: Lookout Dark Caracal Jan 2018).','2018-10-17 00:14:20.652000','2020-03-30 15:08:51.834000','1.1',NULL,0);
INSERT INTO software VALUES('malware--1f6e3702-7ca1-4582-b2e7-4591297d05a8','Bankshot','[Bankshot](https://attack.mitre.org/software/S0239) is a remote access tool (RAT) that was first reported by the Department of Homeland Security in December of 2017. In 2018, [Lazarus Group](https://attack.mitre.org/groups/G0032) used the [Bankshot](https://attack.mitre.org/software/S0239) implant in attacks against the Turkish financial sector. (Citation: McAfee Bankshot)','2018-10-17 00:14:20.652000','2020-03-30 20:41:17.223000','1.1',NULL,0);
INSERT INTO software VALUES('malware--65ffc206-d7c1-45b3-b543-f6b726e7840d','Bisonal','[Bisonal](https://attack.mitre.org/software/S0268) is malware that has been used in attacks against targets in Russia, South Korea, and Japan. It has been observed in the wild since 2014. (Citation: Unit 42 Bisonal July 2018)','2018-10-17 00:14:20.652000','2020-03-30 18:38:49.119000','1.1',NULL,0);
INSERT INTO software VALUES('malware--54cc1d4f-5c53-4f0e-9ef5-11b4998e82e4','BlackEnergy','[BlackEnergy](https://attack.mitre.org/software/S0089) is a malware toolkit that has been used by both criminal and APT actors. It dates back to at least 2007 and was originally designed to create botnets for use in conducting Distributed Denial of Service (DDoS) attacks, but its use has evolved to support various plug-ins. It is well known for being used during the confrontation between Georgia and Russia in 2008, as well as in targeting Ukrainian institutions. Variants include BlackEnergy 2 and BlackEnergy 3. (Citation: F-Secure BlackEnergy 2014)','2017-05-31 21:32:57.807000','2020-06-18 20:43:34.069000','1.2',NULL,0);
INSERT INTO software VALUES('malware--4c6d62c2-89f5-4159-8fab-0190b1f9d328','Bonadan','[Bonadan](https://attack.mitre.org/software/S0486) is a malicious version of OpenSSH which acts as a custom backdoor. [Bonadan](https://attack.mitre.org/software/S0486) has been active since at least 2018 and combines a new cryptocurrency-mining module with the same credential-stealing module used by the Onderon family of backdoors.(Citation: ESET ForSSHe December 2018)','2020-07-16 14:59:40.051000','2020-08-10 19:17:14.766000','1.0',NULL,0);
INSERT INTO software VALUES('malware--28b97733-ef07-4414-aaa5-df50b2d30cc5','Brave Prince','[Brave Prince](https://attack.mitre.org/software/S0252) is a Korean-language implant that was first observed in the wild in December 2017. It contains similar code and behavior to [Gold Dragon](https://attack.mitre.org/software/S0249), and was seen along with [Gold Dragon](https://attack.mitre.org/software/S0249) and [RunningRAT](https://attack.mitre.org/software/S0253) in operations surrounding the 2018 Pyeongchang Winter Olympics. (Citation: McAfee Gold Dragon)','2018-10-17 00:14:20.652000','2020-04-21 23:09:30.781000','1.1',NULL,0);
INSERT INTO software VALUES('malware--79499993-a8d6-45eb-b343-bf58dea5bdde','Briba','[Briba](https://attack.mitre.org/software/S0204) is a trojan used by [Elderwood](https://attack.mitre.org/groups/G0066) to open a backdoor and download files on to compromised hosts. (Citation: Symantec Elderwood Sept 2012) (Citation: Symantec Briba May 2012)','2018-04-18 17:59:24.739000','2018-10-17 00:14:20.652000','1.0',NULL,0);
INSERT INTO software VALUES('malware--7bef1b56-4870-4e74-b32a-7dd88c390c44','Bundlore','[Bundlore](https://attack.mitre.org/software/S0482) is adware written for macOS that has been in use since at least 2015. Though categorized as adware, [Bundlore](https://attack.mitre.org/software/S0482) has many features associated with more traditional backdoors.(Citation: MacKeeper Bundlore Apr 2019)','2020-07-01 19:34:28.366000','2020-07-06 15:18:53.409000','1.0',NULL,0);
INSERT INTO software VALUES('malware--5a84dc36-df0d-4053-9b7c-f0c388a57283','CALENDAR','[CALENDAR](https://attack.mitre.org/software/S0025) is malware used by [APT1](https://attack.mitre.org/groups/G0006) that mimics legitimate Gmail Calendar traffic. (Citation: Mandiant APT1)','2017-05-31 21:32:20.137000','2020-03-30 15:12:21.836000','1.2',NULL,0);
INSERT INTO software VALUES('malware--1b9f0800-035e-4ed1-9648-b18294cc5bc8','CARROTBAT','[CARROTBAT](https://attack.mitre.org/software/S0462) is a customized dropper that has been in use since at least 2017. [CARROTBAT](https://attack.mitre.org/software/S0462) has been used to install [SYSCON](https://attack.mitre.org/software/S0464) and has infrastructure overlap with [KONNI](https://attack.mitre.org/software/S0356).(Citation: Unit 42 CARROTBAT November 2018)(Citation: Unit 42 CARROTBAT January 2020)','2020-06-02 14:11:40.581000','2020-06-15 15:13:27.660000','1.0',NULL,0);
INSERT INTO software VALUES('malware--b0f13390-cec7-4814-b37c-ccec01887faa','CCBkdr','[CCBkdr](https://attack.mitre.org/software/S0222) is malware that was injected into a signed version of CCleaner and distributed from CCleaner''s distribution website. (Citation: Talos CCleanup 2017) (Citation: Intezer Aurora Sept 2017)','2018-04-18 17:59:24.739000','2020-03-20 20:01:55.457000','1.2',NULL,0);
INSERT INTO software VALUES('malware--ccd61dfc-b03f-4689-8c18-7c97eab08472','CHOPSTICK','[CHOPSTICK](https://attack.mitre.org/software/S0023) is a malware family of modular backdoors used by [APT28](https://attack.mitre.org/groups/G0007). It has been used since at least 2012 and is usually dropped on victims as second-stage malware, though it has been used as first-stage malware in several cases. It has both Windows and Linux variants. (Citation: FireEye APT28) (Citation: ESET Sednit Part 2) (Citation: FireEye APT28 January 2017) (Citation: DOJ GRU Indictment Jul 2018) It is tracked separately from the [X-Agent for Android](https://attack.mitre.org/software/S0314).','2017-05-31 21:32:19.389000','2020-03-30 15:21:18.086000','2.1',NULL,0);
INSERT INTO software VALUES('malware--8ab98e25-1672-4b5f-a2fb-e60f08a5ea9e','CORALDECK','[CORALDECK](https://attack.mitre.org/software/S0212) is an exfiltration tool used by [APT37](https://attack.mitre.org/groups/G0067). (Citation: FireEye APT37 Feb 2018)','2018-04-18 17:59:24.739000','2020-03-30 15:13:24.829000','1.1',NULL,0);
INSERT INTO software VALUES('malware--60c18d06-7b91-4742-bae3-647845cd9d81','CORESHELL','[CORESHELL](https://attack.mitre.org/software/S0137) is a downloader used by [APT28](https://attack.mitre.org/groups/G0007). The older versions of this malware are known as SOURFACE and newer versions as CORESHELL.(Citation: FireEye APT28) (Citation: FireEye APT28 January 2017)','2017-05-31 21:33:18.506000','2020-03-30 15:14:36.623000','2.1',NULL,0);
INSERT INTO software VALUES('malware--a705b085-1eae-455e-8f4d-842483d814eb','Cadelspy','[Cadelspy](https://attack.mitre.org/software/S0454) is a backdoor that has been used by [APT39](https://attack.mitre.org/groups/G0087).(Citation: Symantec Chafer Dec 2015)','2020-05-22 20:07:15.628000','2020-05-29 13:13:22.064000','1.0',NULL,0);
INSERT INTO software VALUES('malware--b8fdef82-d2cf-4948-8949-6466357b1be1','Calisto','[Calisto](https://attack.mitre.org/software/S0274) is a macOS Trojan that opens a backdoor on the compromised machine. [Calisto](https://attack.mitre.org/software/S0274) is believed to have first been developed in 2016. (Citation: Securelist Calisto July 2018) (Citation: Symantec Calisto July 2018)','2018-10-17 00:14:20.652000','2020-03-30 01:58:55.849000','1.1',NULL,0);
INSERT INTO software VALUES('malware--cb7bcf6f-085f-41db-81ee-4b68481661b5','CallMe','[CallMe](https://attack.mitre.org/software/S0077) is a Trojan designed to run on Apple OSX. It is based on a publicly available tool called Tiny SHell. (Citation: Scarlet Mimic Jan 2016)','2017-05-31 21:32:52.875000','2020-03-30 15:16:18.880000','1.1',NULL,0);
INSERT INTO software VALUES('malware--d20b397a-ea47-48a9-b503-2e2a3551e11d','Cannon','[Cannon](https://attack.mitre.org/software/S0351) is a Trojan with variants written in C# and Delphi. It was first observed in April 2018. (Citation: Unit42 Cannon Nov 2018)(Citation: Unit42 Sofacy Dec 2018)','2019-01-30 18:58:03.614000','2020-03-30 15:17:24.834000','1.1',NULL,0);
INSERT INTO software VALUES('malware--72f54d66-675d-4587-9bd3-4ed09f9522e4','Carbanak','[Carbanak](https://attack.mitre.org/software/S0030) is a full-featured, remote backdoor used by a group of the same name ([Carbanak](https://attack.mitre.org/groups/G0008)). It is intended for espionage, data exfiltration, and providing remote access to infected machines. (Citation: Kaspersky Carbanak) (Citation: FireEye CARBANAK June 2017)','2017-05-31 21:32:22.213000','2020-03-30 18:46:57.986000','1.1',NULL,0);
CREATE TABLE mitigation (
	id VARCHAR NOT NULL,
	name VARCHAR NOT NULL,
	description VARCHAR,
	created_time DATETIME,
	modified_time DATETIME,
	mitre_version VARCHAR,
	revoked_by VARCHAR,
	deprecated BOOLEAN,
	PRIMARY KEY (id),
	CHECK (deprecated IN (0, 1))
);
INSERT INTO mitigation VALUES('course-of-action--4f170666-7edb-4489-85c2-9affa28a72e0','.bash_profile and .bashrc Mitigation','Making these files immutable and only changeable by certain administrators will limit the ability for adversaries to easily create user level persistence.','2018-10-17 00:14:20.652000','2019-07-24 14:02:53.251000','1.0',NULL,1);
INSERT INTO mitigation VALUES('course-of-action--c61fee9f-16fb-4f8c-bbf0-869093fcd4a6','Access Token Manipulation Mitigation',replace('Access tokens are an integral part of the security system within Windows and cannot be turned off. However, an attacker must already have administrator level access on the local system to make full use of this technique; be sure to restrict users and accounts to the least privileges they require to do their job.\n\nAny user can also spoof access tokens if they have legitimate credentials. Follow mitigation guidelines for preventing adversary use of [Valid Accounts](https://attack.mitre.org/techniques/T1078). Limit permissions so that users and user groups cannot create tokens. This setting should be defined for the local system account only. GPO: Computer Configuration > [Policies] > Windows Settings > Security Settings > Local Policies > User Rights Assignment: Create a token object. (Citation: Microsoft Create Token) Also define who can create a process level token to only the local and network service through GPO: Computer Configuration > [Policies] > Windows Settings > Security Settings > Local Policies > User Rights Assignment: Replace a process level token. (Citation: Microsoft Replace Process Token)\n\nAlso limit opportunities for adversaries to increase privileges by limiting Privilege Escalation opportunities.','\n',char(10)),'2018-10-17 00:14:20.652000','2019-07-24 14:29:27.367000','1.0',NULL,1);
INSERT INTO mitigation VALUES('course-of-action--c085476e-1964-4d7f-86e1-d8657a7741e8','Accessibility Features Mitigation',replace('To use this technique remotely, an adversary must use it in conjunction with RDP. Ensure that Network Level Authentication is enabled to force the remote desktop session to authenticate before the session is created and the login screen displayed. It is enabled by default on Windows Vista and later. (Citation: TechNet RDP NLA)\n\nIf possible, use a Remote Desktop Gateway to manage connections and security configuration of RDP within a network. (Citation: TechNet RDP Gateway)\n\nIdentify and block potentially malicious software that may be executed by an adversary with this technique by using whitelisting (Citation: Beechey 2010) tools, like AppLocker, (Citation: Windows Commands JPCERT) (Citation: NSA MS AppLocker) or Software Restriction Policies (Citation: Corio 2008) where appropriate. (Citation: TechNet Applocker vs SRP)','\n',char(10)),'2018-10-17 00:14:20.652000','2020-01-17 16:45:23.252000','1.0',NULL,1);
INSERT INTO mitigation VALUES('course-of-action--5c49bc54-9929-48ca-b581-7018219b5a97','Account Discovery Mitigation',replace('Prevent administrator accounts from being enumerated when an application is elevating through UAC since it can lead to the disclosure of account names. The Registry key is located <code>HKLM\ SOFTWARE\Microsoft\Windows\CurrentVersion\Policies\CredUI\EnumerateAdministrators</code>. It can be disabled through GPO: Computer Configuration > [Policies] > Administrative Templates > Windows Components > Credential User Interface: E numerate administrator accounts on elevation. (Citation: UCF STIG Elevation Account Enumeration)\n\nIdentify unnecessary system utilities or potentially malicious software that may be used to acquire information about system and domain accounts, and audit and/or block them by using whitelisting (Citation: Beechey 2010) tools, like AppLocker, (Citation: Windows Commands JPCERT) (Citation: NSA MS AppLocker) or Software Restriction Policies (Citation: Corio 2008) where appropriate. (Citation: TechNet Applocker vs SRP)','\n',char(10)),'2018-10-17 00:14:20.652000','2020-01-17 16:45:22.989000','1.0',NULL,1);
INSERT INTO mitigation VALUES('course-of-action--f9f9e6ef-bc0a-41ad-ba11-0924e5e84c4c','Account Use Policies','Configure features related to account use like login attempt lockouts, specific login times, etc.','2019-06-11 16:32:21.854000','2019-06-13 16:07:21.233000','1.0',NULL,0);
INSERT INTO mitigation VALUES('course-of-action--e3388c78-2a8d-47c2-8422-c1398b324462','Active Directory Configuration','Configure Active Directory to prevent use of certain techniques; use SID Filtering, etc.','2019-06-06 16:39:58.291000','2020-05-29 16:34:40.344000','1.1',NULL,0);
INSERT INTO mitigation VALUES('course-of-action--a6a47a06-08fc-4ec4-bdc3-20373375ebb9','Antivirus/Antimalware','Use signatures or heuristics to detect malicious software.','2019-06-11 17:08:33.055000','2020-03-31 13:07:15.684000','1.1',NULL,0);
INSERT INTO mitigation VALUES('course-of-action--95c29444-49f9-49f7-8b20-bcd68d8fcaa6','AppCert DLLs Mitigation','Identify and block potentially malicious software that may be executed through AppCert DLLs by using whitelisting (Citation: Beechey 2010) tools, like AppLocker, (Citation: Windows Commands JPCERT) (Citation: NSA MS AppLocker) that are capable of auditing and/or blocking unknown DLLs.','2018-10-17 00:14:20.652000','2020-01-17 16:45:23.701000','1.0',NULL,1);
INSERT INTO mitigation VALUES('course-of-action--10571bf2-8073-4edf-a71c-23bad225532e','AppInit DLLs Mitigation',replace('Upgrade to Windows 8 or later and enable secure boot.\n\nIdentify and block potentially malicious software that may be executed through AppInit DLLs by using whitelisting (Citation: Beechey 2010) tools, like AppLocker, (Citation: Windows Commands JPCERT) (Citation: NSA MS AppLocker) that are capable of auditing and/or blocking unknown DLLs.','\n',char(10)),'2018-10-17 00:14:20.652000','2020-01-17 16:45:23.250000','1.0',NULL,1);
INSERT INTO mitigation VALUES('course-of-action--1e4ef2c7-ee96-4484-9baa-3b5777561301','AppleScript Mitigation','Require that all AppleScript be signed by a trusted developer ID before being executed - this will prevent random AppleScript code from executing  (Citation: applescript signing). This subjects AppleScript code to the same scrutiny as other .app files passing through Gatekeeper.','2018-10-17 00:14:20.652000','2019-07-24 14:31:55.409000','1.0',NULL,1);
INSERT INTO mitigation VALUES('course-of-action--c88151a5-fe3f-4773-8147-d801587065a4','Application Deployment Software Mitigation',replace('Grant access to application deployment systems only to a limited number of authorized administrators. Ensure proper system and access isolation for critical network systems through use of firewalls, account privilege separation, group policy, and multifactor authentication. Verify that account credentials that may be used to access deployment systems are unique and not used throughout the enterprise network. Patch deployment systems regularly to prevent potential remote access through [Exploitation for Privilege Escalation](https://attack.mitre.org/techniques/T1068). \n\nIf the application deployment system can be configured to deploy only signed binaries, then ensure that the trusted signing certificates are not co-located with the application deployment system and are instead located on a system that cannot be accessed remotely or to which remote access is tightly controlled.','\n',char(10)),'2018-10-17 00:14:20.652000','2019-07-24 14:05:33.227000','1.0',NULL,1);
INSERT INTO mitigation VALUES('course-of-action--25dc1ce8-eb55-4333-ae30-a7cb4f5894a1','Application Developer Guidance','This mitigation describes any guidance or training given to developers of applications to avoid introducing security weaknesses that an adversary may be able to take advantage of.','2017-10-25 14:48:53.732000','2018-10-17 00:14:20.652000','1.0',NULL,0);
INSERT INTO mitigation VALUES('course-of-action--b9f0c069-abbe-4a07-a245-2481219a1463','Application Isolation and Sandboxing','Restrict execution of code to a virtual environment on or in transit to an endpoint system.','2019-06-11 17:06:56.230000','2020-03-31 13:08:03.851000','1.1',NULL,0);
INSERT INTO mitigation VALUES('course-of-action--cfc2d2fc-14ff-495f-bd99-585be47b804f','Application Shimming Mitigation',replace('There currently aren''t a lot of ways to mitigate application shimming. Disabling the Shim Engine isn''t recommended because Windows depends on shimming for interoperability and software may become unstable or not work. Microsoft released an optional patch update - KB3045645 - that will remove the "auto-elevate" flag within the sdbinst.exe. This will prevent use of application shimming to bypass UAC. \n\nChanging UAC settings to "Always Notify" will give the user more visibility when UAC elevation is requested, however, this option will not be popular among users due to the constant UAC interruptions.','\n',char(10)),'2018-10-17 00:14:20.652000','2019-07-24 14:32:52.325000','1.0',NULL,1);
INSERT INTO mitigation VALUES('course-of-action--25d5e1d8-c6fb-4735-bc57-115a21222f4b','Application Window Discovery Mitigation','Identify unnecessary system utilities or potentially malicious software that may be used to acquire information, and audit and/or block them by using whitelisting (Citation: Beechey 2010) tools, like AppLocker, (Citation: Windows Commands JPCERT) (Citation: NSA MS AppLocker) or Software Restriction Policies (Citation: Corio 2008) where appropriate. (Citation: TechNet Applocker vs SRP)','2018-10-17 00:14:20.652000','2020-01-17 16:45:23.664000','1.0',NULL,1);
INSERT INTO mitigation VALUES('course-of-action--16dd03c6-0dfb-4d77-89cd-9ff3ee6e533d','Audio Capture Mitigation',replace('Mitigating this technique specifically may be difficult as it requires fine-grained API control. Efforts should be focused on preventing unwanted or unknown code from executing on a system.\n\nIdentify and block potentially malicious software that may be used to record audio by using whitelisting (Citation: Beechey 2010) tools, like AppLocker, (Citation: Windows Commands JPCERT) (Citation: NSA MS AppLocker) or Software Restriction Policies (Citation: Corio 2008) where appropriate. (Citation: TechNet Applocker vs SRP)','\n',char(10)),'2018-10-17 00:14:20.652000','2020-01-17 16:45:23.317000','1.0',NULL,1);
INSERT INTO mitigation VALUES('course-of-action--cc2399fd-3cd3-4319-8d0a-fbd6420cdaf8','Audit','Perform audits or scans of systems, permissions, insecure software, insecure configurations, etc. to identify potential weaknesses.','2019-06-11 17:06:14.029000','2020-03-31 13:08:45.966000','1.1',NULL,0);
INSERT INTO mitigation VALUES('course-of-action--943d370b-2054-44df-8be2-ab4139bde1c5','Authentication Package Mitigation','Windows 8.1, Windows Server 2012 R2, and later versions, may make LSA run as a Protected Process Light (PPL) by setting the Registry key <code>HKLM\SYSTEM\CurrentControlSet\Control\Lsa\RunAsPPL</code>, which requires all DLLs loaded by LSA to be signed by Microsoft. (Citation: Graeber 2014) (Citation: Microsoft Configure LSA)','2018-10-17 00:14:20.652000','2019-07-24 14:34:11.298000','1.0',NULL,1);
INSERT INTO mitigation VALUES('course-of-action--8bd1ae32-a686-48f4-a6f8-470287f76152','Automated Collection Mitigation',replace('Encryption and off-system storage of sensitive information may be one way to mitigate collection of files, but may not stop an adversary from acquiring the information if an intrusion persists over a long period of time and the adversary is able to discover and access the data through other means. A keylogger installed on a system may be able to intercept passwords through [Input Capture](https://attack.mitre.org/techniques/T1056) and be used to decrypt protected documents that an adversary may have collected. Strong passwords should be used to prevent offline cracking of encrypted documents through [Brute Force](https://attack.mitre.org/techniques/T1110) techniques.\n\nIdentify unnecessary system utilities, third-party tools, or potentially malicious software that may be used to collect files and audit and/or block them by using whitelisting (Citation: Beechey 2010) tools, like AppLocker, (Citation: Windows Commands JPCERT) (Citation: NSA MS AppLocker) or Software Restriction Policies (Citation: Corio 2008) where appropriate. (Citation: TechNet Applocker vs SRP)','\n',char(10)),'2018-10-17 00:14:20.652000','2020-01-17 16:45:23.017000','1.0',NULL,1);
INSERT INTO mitigation VALUES('course-of-action--2497ac92-e751-4391-82c6-1b86e34d0294','Automated Exfiltration Mitigation','Identify unnecessary system utilities, scripts, or potentially malicious software that may be used to transfer data outside of a network, and audit and/or block them by using whitelisting (Citation: Beechey 2010) tools, like AppLocker, (Citation: Windows Commands JPCERT) (Citation: NSA MS AppLocker) or Software Restriction Policies (Citation: Corio 2008) where appropriate. (Citation: TechNet Applocker vs SRP)','2018-10-17 00:14:20.652000','2020-01-17 16:45:23.670000','1.0',NULL,1);
INSERT INTO mitigation VALUES('course-of-action--cb825b86-3f3b-4686-ba99-44878f5d3173','BITS Jobs Mitigation',replace('This type of attack technique cannot be easily mitigated with preventive controls since it is based on the abuse of operating system design features. For example, disabling all BITS functionality will likely have unintended side effects, such as preventing legitimate software patching and updating. Efforts should be focused on preventing adversary tools from running earlier in the chain of activity and on identification of subsequent malicious behavior. (Citation: Mondok Windows PiggyBack BITS May 2007)\n\nModify network and/or host firewall rules, as well as other network controls, to only allow legitimate BITS traffic.\n\nConsider limiting access to the BITS interface to specific users or groups. (Citation: Symantec BITS May 2007)\n\nConsider reducing the default BITS job lifetime in Group Policy or by editing the <code>JobInactivityTimeout</code> and <code>MaxDownloadTime</code> Registry values in <code> HKEY_LOCAL_MACHINE\Software\Policies\Microsoft\Windows\BITS</code>. (Citation: Microsoft BITS)','\n',char(10)),'2018-10-17 00:14:20.652000','2019-07-24 14:08:16.317000','1.0',NULL,1);
INSERT INTO mitigation VALUES('course-of-action--ace4daee-f914-4707-be75-843f16da2edf','Bash History Mitigation',replace('There are multiple methods of preventing a user''s command history from being flushed to their .bash_history file, including use of the following commands:\n<code>set +o history</code> and <code>set -o history</code> to start logging again;\n<code>unset HISTFILE</code> being added to a user''s .bash_rc file; and\n<code>ln -s /dev/null ~/.bash_history</code> to write commands to <code>/dev/null</code>instead.','\n',char(10)),'2018-10-17 00:14:20.652000','2019-07-24 14:37:14.608000','1.0',NULL,1);
INSERT INTO mitigation VALUES('course-of-action--90f39ee1-d5a3-4aaa-9f28-3b42815b0d46','Behavior Prevention on Endpoint','Use capabilities to prevent suspicious behavior patterns from occurring on endpoint systems. This could include suspicious process, file, API call, etc. behavior.','2019-06-11 16:43:05.712000','2019-06-11 16:43:05.712000','1.0',NULL,0);
INSERT INTO mitigation VALUES('course-of-action--16a8ac85-a06f-460f-ad22-910167bd7332','Binary Padding Mitigation','Identify potentially malicious software that may be executed from a padded or otherwise obfuscated binary, and audit and/or block it by using whitelisting (Citation: Beechey 2010) tools, like AppLocker, (Citation: Windows Commands JPCERT) (Citation: NSA MS AppLocker) or Software Restriction Policies (Citation: Corio 2008) where appropriate. (Citation: TechNet Applocker vs SRP)','2018-10-17 00:14:20.652000','2020-01-17 16:45:23.322000','1.0',NULL,1);
INSERT INTO mitigation VALUES('course-of-action--7da0387c-ba92-4553-b291-b636ee42b2eb','Boot Integrity','Use secure methods to boot a system and verify the integrity of the operating system and loading mechanisms.','2019-06-11 17:02:36.984000','2020-05-19 12:28:50.603000','1.0',NULL,0);
INSERT INTO mitigation VALUES('course-of-action--96150c35-466f-4f0a-97a9-ae87ee27f751','Bootkit Mitigation','Ensure proper permissions are in place to help prevent adversary access to privileged accounts necessary to perform this action. Use Trusted Platform Module technology and a secure or trusted boot process to prevent system integrity from being compromised. (Citation: TCG Trusted Platform Module) (Citation: TechNet Secure Boot Process)','2018-10-17 00:14:20.652000','2020-04-23 19:10:28.284000','1.0',NULL,1);
INSERT INTO mitigation VALUES('course-of-action--1c0b39f9-a0c5-42b2-abd8-dc8f1eb74e67','Browser Bookmark Discovery Mitigation','File system activity is a common part of an operating system, so it is unlikely that mitigation would be appropriate for this technique. For example, mitigating accesses to browser bookmark files will likely have unintended side effects such as preventing legitimate software from operating properly. Efforts should be focused on preventing adversary tools from running earlier in the chain of activity and on identification of subsequent malicious behavior. It may still be beneficial to identify and block unnecessary system utilities or potentially malicious software by using whitelisting (Citation: Beechey 2010) tools, like AppLocker, (Citation: Windows Commands JPCERT) (Citation: NSA MS AppLocker) or Software Restriction Policies (Citation: Corio 2008) where appropriate. (Citation: TechNet Applocker vs SRP)','2018-10-17 00:14:20.652000','2020-01-17 16:45:23.672000','1.0',NULL,1);
INSERT INTO mitigation VALUES('course-of-action--b52f41b9-ccf6-4da7-a6c0-167eeb71fbd8','Browser Extensions Mitigation',replace('Only install browser extensions from trusted sources that can be verified. Ensure extensions that are installed are the intended ones as many malicious extensions will masquerade as legitimate ones.\n\nBrowser extensions for some browsers can be controlled through Group Policy. Set a browser extension white or black list as appropriate for your security policy. (Citation: Technospot Chrome Extensions GP)\n\nChange settings to prevent the browser from installing extensions without sufficient permissions.\n\nClose out all browser sessions when finished using them.','\n',char(10)),'2018-10-17 00:14:20.652000','2019-07-24 14:41:17.903000','1.0',NULL,1);
INSERT INTO mitigation VALUES('course-of-action--4a99fecc-680b-448e-8fe7-8144c60d272c','Brute Force Mitigation',replace('Set account lockout policies after a certain number of failed login attempts to prevent passwords from being guessed. \nToo strict a policy can create a denial of service condition and render environments un-usable, with all accounts being locked-out permanently. Use multifactor authentication. Follow best practices for mitigating access to [Valid Accounts](https://attack.mitre.org/techniques/T1078)\n\nRefer to NIST guidelines when creating passwords.(Citation: NIST 800-63-3)\n\nWhere possible, also enable multi factor authentication on external facing services.','\n',char(10)),'2018-10-17 00:14:20.652000','2019-07-24 18:03:10.785000','1.1',NULL,1);
INSERT INTO mitigation VALUES('course-of-action--beb45abb-11e8-4aef-9778-1f9ac249784f','Bypass User Account Control Mitigation',replace('Remove users from the local administrator group on systems. Although UAC bypass techniques exist, it is still prudent to use the highest enforcement level for UAC when possible and mitigate bypass opportunities that exist with techniques such as [DLL Search Order Hijacking](https://attack.mitre.org/techniques/T1038). \n\nCheck for common UAC bypass weaknesses on Windows systems to be aware of the risk posture and address issues where appropriate. (Citation: Github UACMe)','\n',char(10)),'2018-10-17 00:14:20.652000','2019-07-24 14:13:23.637000','1.0',NULL,1);
INSERT INTO mitigation VALUES('course-of-action--91816292-3686-4a6e-83c4-4c08513b9b57','CMSTP Mitigation','CMSTP.exe may not be necessary within a given environment (unless using it for VPN connection installation). Consider using application whitelisting configured to block execution of CMSTP.exe if it is not required for a given system or network to prevent potential misuse by adversaries. (Citation: MSitPros CMSTP Aug 2017)','2018-10-17 00:14:20.652000','2019-07-24 18:04:13.126000','1.0',NULL,1);
INSERT INTO mitigation VALUES('course-of-action--d7c49196-b40e-42bc-8eed-b803113692ed','Change Default File Association Mitigation',replace('Direct mitigation of this technique is not recommended since it is a legitimate function that can be performed by users for software preferences. Follow Microsoft''s best practices for file associations. (Citation: MSDN File Associations)\n\nIdentify and block potentially malicious software that may be executed by this technique using whitelisting (Citation: Beechey 2010) tools, like AppLocker, (Citation: Windows Commands JPCERT) (Citation: NSA MS AppLocker) or Software Restriction Policies (Citation: Corio 2008) where appropriate. (Citation: TechNet Applocker vs SRP)','\n',char(10)),'2018-10-17 00:14:20.652000','2020-01-17 16:45:23.675000','1.0',NULL,1);
INSERT INTO mitigation VALUES('course-of-action--3e7018e9-7389-48e7-9208-0bdbcbba9483','Clear Command History Mitigation','Preventing users from deleting or writing to certain files can stop adversaries from maliciously altering their <code>~/.bash_history</code> files. Additionally, making these environment variables readonly can make sure that the history is preserved   (Citation: Securing bash history).','2018-10-17 00:14:20.652000','2019-07-24 18:05:00.492000','1.0',NULL,1);
INSERT INTO mitigation VALUES('course-of-action--19edfa02-1a5f-47e4-ad82-3288f57f64cf','Clipboard Data Mitigation','Instead of blocking software based on clipboard capture behavior, identify potentially malicious software that may contain this functionality, and audit and/or block it by using whitelisting (Citation: Beechey 2010) tools, like AppLocker, (Citation: Windows Commands JPCERT) (Citation: NSA MS AppLocker) or Software Restriction Policies (Citation: Corio 2008) where appropriate. (Citation: TechNet Applocker vs SRP)','2018-10-17 00:14:20.652000','2020-01-17 16:45:23.314000','1.0',NULL,1);
INSERT INTO mitigation VALUES('course-of-action--590777b3-b475-4c7c-aaf8-f4a73b140312','Code Signing','Enforce binary and application integrity with digital signature verification to prevent untrusted code from executing.','2019-06-11 17:01:25.405000','2020-05-20 13:12:02.881000','1.1',NULL,0);
INSERT INTO mitigation VALUES('course-of-action--82fbc58b-171d-4a2d-9a20-c6b2a716bd08','Code Signing Mitigation','Process whitelisting and trusted publishers to verify authenticity of software can help prevent signed malicious or untrusted code from executing on a system. (Citation: NSA MS AppLocker) (Citation: TechNet Trusted Publishers) (Citation: Securelist Digital Certificates)','2018-10-17 00:14:20.652000','2020-01-17 16:45:23.319000','1.0',NULL,1);
INSERT INTO mitigation VALUES('course-of-action--7c1796c7-9fc3-4c3e-9416-527295bf5d95','Commonly Used Port Mitigation','Network intrusion detection and prevention systems that use network signatures to identify traffic for specific adversary malware can be used to mitigate activity at the network level. Signatures are often for unique indicators within protocols and may be based on the specific protocol used by a particular adversary or tool, and will likely be different across various malware families and versions. Adversaries will likely change tool C2 signatures over time or construct protocols in such a way as to avoid detection by common defensive tools. (Citation: University of Birmingham C2)','2018-10-17 00:14:20.652000','2019-07-24 14:17:58.966000','1.0',NULL,1);
INSERT INTO mitigation VALUES('course-of-action--b8d57b16-d8e2-428c-a645-1083795b3445','Communication Through Removable Media Mitigation','Disable Autorun if it is unnecessary. (Citation: Microsoft Disable Autorun) Disallow or restrict removable media at an organizational policy level if they are not required for business operations. (Citation: TechNet Removable Media Control)','2018-10-17 00:14:20.652000','2019-07-24 18:09:33.072000','1.0',NULL,1);
INSERT INTO mitigation VALUES('course-of-action--ae56a49d-5281-45c5-ab95-70a1439c338e','Compile After Delivery Mitigation',replace('This type of technique cannot be easily mitigated with preventive controls or patched since it is based on the abuse of operating system design features. For example, blocking all file compilation may have unintended side effects, such as preventing legitimate OS frameworks and code development mechanisms from operating properly. Consider removing compilers if not needed, otherwise efforts should be focused on preventing adversary tools from running earlier in the chain of activity and on identifying subsequent malicious behavior.\n\nIdentify unnecessary system utilities or potentially malicious software that may be used to decrypt, deobfuscate, decode, and compile files or information, and audit and/or block them by using whitelisting (Citation: Beechey 2010) tools, like AppLocker, (Citation: Windows Commands JPCERT) (Citation: NSA MS AppLocker) or Software Restriction Policies (Citation: Corio 2008) where appropriate. (Citation: TechNet Applocker vs SRP)','\n',char(10)),'2019-04-25 20:53:07.814000','2020-01-17 16:45:24.252000','1.0',NULL,1);
INSERT INTO mitigation VALUES('course-of-action--08e02f67-ea09-4f77-a70b-414963c29fc2','Compiled HTML File Mitigation','Consider blocking download/transfer and execution of potentially uncommon file types known to be used in adversary campaigns, such as CHM files. (Citation: PaloAlto Preventing Opportunistic Attacks Apr 2016) Also consider using application whitelisting to prevent execution of hh.exe if it is not required for a given system or network to prevent potential misuse by adversaries.','2018-10-17 00:14:20.652000','2019-07-24 14:19:23.148000','1.0',NULL,1);
INSERT INTO mitigation VALUES('course-of-action--676975b9-7e8e-463d-a31e-4ed2ecbfed81','Component Firmware Mitigation',replace('Prevent adversary access to privileged accounts or access necessary to perform this technique.\n\nConsider removing and replacing system components suspected of being compromised.','\n',char(10)),'2018-10-17 00:14:20.652000','2019-07-24 18:10:06.475000','1.0',NULL,1);
INSERT INTO mitigation VALUES('course-of-action--ff5d862a-ae6b-4833-8c15-e235d654d28e','Component Object Model Hijacking Mitigation',replace('Direct mitigation of this technique may not be recommended for a particular environment since COM objects are a legitimate part of the operating system and installed software. Blocking COM object changes may have unforeseen side effects to legitimate functionality.\n\nInstead, identify and block potentially malicious software that may execute, or be executed by, this technique using whitelisting (Citation: Beechey 2010) tools, like AppLocker, (Citation: Windows Commands JPCERT) (Citation: NSA MS AppLocker) or Software Restriction Policies (Citation: Corio 2008) where appropriate. (Citation: TechNet Applocker vs SRP)','\n',char(10)),'2018-10-17 00:14:20.652000','2020-01-17 16:45:23.056000','1.0',NULL,1);
INSERT INTO mitigation VALUES('course-of-action--3a41b366-cfd6-4af2-a6e7-3c6e3c4ebcef','Control Panel Items Mitigation',replace('This type of attack technique cannot be easily mitigated with preventive controls since it is based on the abuse of operating system design features. For example, mitigating specific Windows API calls and/or execution of particular file extensions will likely have unintended side effects, such as preventing legitimate software (i.e., drivers and configuration tools) from operating properly. Efforts should be focused on preventing adversary tools from running earlier in the chain of activity and on identification of subsequent malicious behavior.\n\nRestrict storage and execution of Control Panel items to protected directories, such as <code>C:\Windows</code>, rather than user directories.\n\nIndex known safe Control Panel items and block potentially malicious software using whitelisting (Citation: Beechey 2010) tools like AppLocker (Citation: Windows Commands JPCERT) (Citation: NSA MS AppLocker) that are capable of auditing and/or blocking unknown executable files.\n\nConsider fully enabling User Account Control (UAC) to impede system-wide changes from illegitimate administrators. (Citation: Microsoft UAC)','\n',char(10)),'2018-10-17 00:14:20.652000','2020-01-17 16:45:23.678000','1.0',NULL,1);
INSERT INTO mitigation VALUES('course-of-action--9a5b7194-88e0-4579-b82f-e3c27b8cca80','Create Account Mitigation',replace('Use and enforce multifactor authentication. Follow guidelines to prevent or limit adversary access to [Valid Accounts](https://attack.mitre.org/techniques/T1078) that may be used to create privileged accounts within an environment.\n\nAdversaries that create local accounts on systems may have limited access within a network if access levels are properly locked down. These accounts may only be needed for persistence on individual systems and their usefulness depends on the utility of the system they reside on.\n\nProtect domain controllers by ensuring proper security configuration for critical servers. Configure access controls and firewalls to limit access to these systems. Do not allow domain administrator accounts to be used for day-to-day operations that may expose them to potential adversaries on unprivileged systems.','\n',char(10)),'2018-10-17 00:14:20.652000','2019-07-24 18:11:24.572000','1.0',NULL,1);
INSERT INTO mitigation VALUES('course-of-action--49c06d54-9002-491d-9147-8efb537fbd26','Credential Access Protection','Use capabilities to prevent successful credential access by adversaries; including blocking forms of credential dumping.','2019-06-11 16:47:12.859000','2020-03-31 13:09:22.442000','1.1',NULL,0);
INSERT INTO mitigation VALUES('course-of-action--aeff5887-8f9e-48d5-a523-9b395e2ce80a','Credential Dumping Mitigation',replace('### Windows\nMonitor/harden access to LSASS and SAM table with tools that allow process whitelisting. Limit credential overlap across systems to prevent lateral movement opportunities using [Valid Accounts](https://attack.mitre.org/techniques/T1078) if passwords and hashes are obtained. Ensure that local administrator accounts have complex, unique passwords across all systems on the network. Do not put user or admin domain accounts in the local administrator groups across systems unless they are tightly controlled, as this is often equivalent to having a local administrator account with the same password on all systems. Follow best practices for design and administration of an enterprise network to limit privileged account use across administrative tiers. (Citation: Microsoft Securing Privileged Access)\n\nOn Windows 8.1 and Windows Server 2012 R2, enable Protected Process Light for LSA. (Citation: Microsoft LSA)\n\nIdentify and block potentially malicious software that may be used to dump credentials by using whitelisting (Citation: Beechey 2010) tools, like AppLocker, (Citation: Windows Commands JPCERT) (Citation: NSA MS AppLocker) or Software Restriction Policies (Citation: Corio 2008) where appropriate. (Citation: TechNet Applocker vs SRP)\n\nWith Windows 10, Microsoft implemented new protections called Credential Guard to protect the LSA secrets that can be used to obtain credentials through forms of credential dumping. It is not configured by default and has hardware and firmware system requirements. (Citation: TechNet Credential Guard) It also does not protect against all forms of credential dumping. (Citation: GitHub SHB Credential Guard)\n\nManage the access control list for “Replicating Directory Changes” and other permissions associated with domain controller replication. (Citation: AdSecurity DCSync Sept 2015) (Citation: Microsoft Replication ACL)\n\nConsider disabling or restricting NTLM traffic. (Citation: Microsoft Disable NTLM Nov 2012)\n\n### Linux\nScraping the passwords from memory requires root privileges. Follow best practices in restricting access to escalated privileges to avoid hostile programs from accessing such sensitive regions of memory.','\n',char(10)),'2018-10-17 00:14:20.652000','2020-01-17 16:45:23.442000','1.0',NULL,1);
INSERT INTO mitigation VALUES('course-of-action--0472af99-f25c-4abe-9fce-010fa3450e72','Credentials in Files Mitigation','Establish an organizational policy that prohibits password storage in files. Ensure that developers and system administrators are aware of the risk associated with having plaintext passwords in software configuration files that may be left on endpoint systems or servers. Preemptively search for files containing passwords and remove when found. Restrict file shares to specific directories with access only to necessary users. Remove vulnerable Group Policy Preferences. (Citation: Microsoft MS14-025)','2018-10-17 00:14:20.652000','2019-07-24 18:12:19.081000','1.0',NULL,1);
INSERT INTO mitigation VALUES('course-of-action--4490fee2-5c70-4db3-8db5-8d88767dbd55','Credentials in Registry Mitigation','Do not store credentials within the Registry. Proactively search for credentials within Registry keys and attempt to remediate the risk. If necessary software must store credentials, then ensure those accounts have limited permissions so they cannot be abused if obtained by an adversary.','2018-10-17 00:14:20.652000','2019-07-24 14:22:57.902000','1.0',NULL,1);
INSERT INTO mitigation VALUES('course-of-action--f3d0c735-330f-43c2-8e8e-51bcfa51e8c3','Custom Command and Control Protocol Mitigation',replace('Properly configure firewalls and proxies to limit outgoing traffic to only necessary ports and through proper network gateway systems. Also ensure hosts are only provisioned to communicate over authorized interfaces.\n\nNetwork intrusion detection and prevention systems that use network signatures to identify traffic for specific adversary malware can be used to mitigate activity at the network level. Signatures are often for unique indicators within protocols and may be based on the specific protocol used by a particular adversary or tool, and will likely be different across various malware families and versions. Adversaries will likely change tool C2 signatures over time or construct protocols in such a way as to avoid detection by common defensive tools. (Citation: University of Birmingham C2)','\n',char(10)),'2018-10-17 00:14:20.652000','2019-07-24 18:13:22.017000','1.0',NULL,1);
INSERT INTO mitigation VALUES('course-of-action--a569295c-a093-4db4-9fb4-7105edef85ad','Custom Cryptographic Protocol Mitigation','Network intrusion detection and prevention systems that use network signatures to identify traffic for specific adversary malware can be used to mitigate activity at the network level. Since the custom protocol used may not adhere to typical protocol standards, there may be opportunities to signature the traffic on a network level for detection. Signatures are often for unique indicators within protocols and may be based on the specific protocol used by a particular adversary or tool, and will likely be different across various malware families and versions. Adversaries will likely change tool C2 signatures over time or construct protocols in such a way as to avoid detection by common defensive tools. (Citation: University of Birmingham C2)','2018-10-17 00:14:20.652000','2019-07-24 18:14:14.227000','1.0',NULL,1);
CREATE TABLE alias (
	id VARCHAR NOT NULL,
	alias VARCHAR NOT NULL,
	type VARCHAR NOT NULL,
	PRIMARY KEY (id, alias)
);
INSERT INTO alias VALUES('malware--7bec698a-7e20-4fd3-bb6a-12787770fb1a','3PARA RAT','software');
INSERT INTO alias VALUES('malware--8e461ca3-0996-4e6e-a0df-e2a5bbc51ebc','4H RAT','software');
INSERT INTO alias VALUES('malware--a0ebedca-d558-4e48-8ff7-4bf76208d90c','ABK','software');
INSERT INTO alias VALUES('malware--fb575479-14ef-41e9-bfab-0b7cf10bec73','ADVSTORESHELL','software');
INSERT INTO alias VALUES('malware--fb575479-14ef-41e9-bfab-0b7cf10bec73','AZZY','software');
INSERT INTO alias VALUES('malware--fb575479-14ef-41e9-bfab-0b7cf10bec73','EVILTOSS','software');
INSERT INTO alias VALUES('malware--fb575479-14ef-41e9-bfab-0b7cf10bec73','NETUI','software');
INSERT INTO alias VALUES('malware--fb575479-14ef-41e9-bfab-0b7cf10bec73','Sedreco','software');
INSERT INTO alias VALUES('malware--56f46b17-8cfa-46c0-b501-dd52fef394e2','ASPXSpy','software');
INSERT INTO alias VALUES('malware--56f46b17-8cfa-46c0-b501-dd52fef394e2','ASPXTool','software');
INSERT INTO alias VALUES('malware--e7a5229f-05eb-440e-b982-9a6d2b2b87c8','Agent Tesla','software');
INSERT INTO alias VALUES('malware--40d3e230-ed32-469f-ba89-be70cc08ab39','Agent.btz','software');
INSERT INTO alias VALUES('malware--5f1d4579-4e8f-48e7-860e-2da773ae432e','Anchor','software');
INSERT INTO alias VALUES('malware--5f1d4579-4e8f-48e7-860e-2da773ae432e','Anchor_DNS','software');
INSERT INTO alias VALUES('malware--3161d76a-e2b2-4b97-9906-24909b735386','Aria-body','software');
INSERT INTO alias VALUES('malware--edb24a93-1f7a-4bbf-a738-1397a14662c6','Astaroth','software');
INSERT INTO alias VALUES('malware--8f423bd7-6ca7-4303-9e85-008c7ad5fdaa','Attor','software');
INSERT INTO alias VALUES('malware--24b4ce59-eaac-4c8b-8634-9b093b7ccd92','AuditCred','software');
INSERT INTO alias VALUES('malware--24b4ce59-eaac-4c8b-8634-9b093b7ccd92','Roptimizer','software');
INSERT INTO alias VALUES('malware--f5352566-1a64-49ac-8f7f-97e1d1a03300','AutoIt backdoor','software');
INSERT INTO alias VALUES('malware--36ede314-7db4-4d09-b53d-81bbfbe5f6f8','Avenger','software');
INSERT INTO alias VALUES('malware--f9b05f33-d45d-4e4d-aafe-c208d38a0080','Azorult','software');
INSERT INTO alias VALUES('malware--fb261c56-b80e-43a9-8351-c84081e7213d','BACKSPACE','software');
INSERT INTO alias VALUES('malware--fb261c56-b80e-43a9-8351-c84081e7213d','Lecna','software');
INSERT INTO alias VALUES('malware--9dbdadb6-fdbf-490f-a35f-38762d06a0d2','BADCALL','software');
INSERT INTO alias VALUES('malware--e9595678-d269-469e-ae6b-75e49259de63','BADNEWS','software');
INSERT INTO alias VALUES('malware--f0fc920e-57a3-4af5-89be-9ea594c8b1ea','BBK','software');
INSERT INTO alias VALUES('malware--64d76fa5-cf8f-469c-b78c-1a4f7c5bad80','BBSRAT','software');
INSERT INTO alias VALUES('malware--b8eb28e4-48a6-40ae-951a-328714f75eda','BISCUIT','software');
INSERT INTO alias VALUES('malware--d69c8146-ab35-4d50-8382-6fc80e641d43','BLACKCOFFEE','software');
INSERT INTO alias VALUES('malware--d5268dfb-ae2b-4e0e-ac07-02a460613d8a','BONDUPDATER','software');
INSERT INTO alias VALUES('malware--56d10a7f-bb42-4267-9b4c-63abb9c06010','BOOSTWRITE','software');
INSERT INTO alias VALUES('malware--da2ef4a9-7cbe-400a-a379-e2f230f28db3','BOOTRASH','software');
INSERT INTO alias VALUES('malware--67fc172a-36fa-4a35-88eb-4ba730ed52a6','BS2005','software');
INSERT INTO alias VALUES('malware--123bd7b3-675c-4b1a-8482-c55782b20e2b','BUBBLEWRAP','software');
INSERT INTO alias VALUES('malware--123bd7b3-675c-4b1a-8482-c55782b20e2b','Backdoor.APT.FakeWinHTTPHelper','software');
INSERT INTO alias VALUES('malware--d1b7830a-fced-4be3-a99c-f495af9d9e1b','BabyShark','software');
INSERT INTO alias VALUES('malware--c13d9621-aca7-436b-ab3d-3a95badb3d00','BackConfig','software');
INSERT INTO alias VALUES('malware--083bb47b-02c8-4423-81a2-f9ef58572974','Backdoor.Oldrea','software');
INSERT INTO alias VALUES('malware--083bb47b-02c8-4423-81a2-f9ef58572974','Havex','software');
INSERT INTO alias VALUES('malware--9af05de0-bc09-4511-a350-5eb8b06185c1','BadPatch','software');
INSERT INTO alias VALUES('malware--835a79f1-842d-472d-b8f4-d54b545c341b','Bandook','software');
INSERT INTO alias VALUES('malware--1f6e3702-7ca1-4582-b2e7-4591297d05a8','Bankshot','software');
INSERT INTO alias VALUES('malware--1f6e3702-7ca1-4582-b2e7-4591297d05a8','Trojan Manuscript','software');
INSERT INTO alias VALUES('malware--65ffc206-d7c1-45b3-b543-f6b726e7840d','Bisonal','software');
INSERT INTO alias VALUES('malware--54cc1d4f-5c53-4f0e-9ef5-11b4998e82e4','BlackEnergy','software');
INSERT INTO alias VALUES('malware--54cc1d4f-5c53-4f0e-9ef5-11b4998e82e4','Black Energy','software');
INSERT INTO alias VALUES('malware--4c6d62c2-89f5-4159-8fab-0190b1f9d328','Bonadan','software');
INSERT INTO alias VALUES('malware--28b97733-ef07-4414-aaa5-df50b2d30cc5','Brave Prince','software');
INSERT INTO alias VALUES('malware--79499993-a8d6-45eb-b343-bf58dea5bdde','Briba','software');
CREATE TABLE contributor (
	id VARCHAR NOT NULL,
	contributor VARCHAR NOT NULL,
	type VARCHAR NOT NULL,
	PRIMARY KEY (id, contributor)
);
INSERT INTO contributor VALUES('attack-pattern--cabe189c-a0e3-4965-a473-dcff00f17213','Jon Sternstein, Stern Security','technique');
INSERT INTO contributor VALUES('attack-pattern--3986e7fd-a8e9-4ecb-bfc6-55920855912b','James Dunn, @jamdunnDFW, EY','technique');
INSERT INTO contributor VALUES('attack-pattern--3986e7fd-a8e9-4ecb-bfc6-55920855912b','Swapnil Kumbhar','technique');
INSERT INTO contributor VALUES('attack-pattern--3986e7fd-a8e9-4ecb-bfc6-55920855912b','Jacques Pluviose, @Jacqueswildy_IT','technique');
INSERT INTO contributor VALUES('attack-pattern--3986e7fd-a8e9-4ecb-bfc6-55920855912b','Dan Nutting, @KerberToast','technique');
INSERT INTO contributor VALUES('attack-pattern--dcaa092b-7de9-4a21-977f-7fcb77e89c48','Tom Ueltschi @c_APT_ure','technique');
INSERT INTO contributor VALUES('attack-pattern--dcaa092b-7de9-4a21-977f-7fcb77e89c48','Travis Smith, Tripwire','technique');
INSERT INTO contributor VALUES('attack-pattern--dcaa092b-7de9-4a21-977f-7fcb77e89c48','Robby Winchester, @robwinchester3','technique');
INSERT INTO contributor VALUES('attack-pattern--dcaa092b-7de9-4a21-977f-7fcb77e89c48','Jared Atkinson, @jaredcatkinson','technique');
INSERT INTO contributor VALUES('attack-pattern--70e52b04-2a0c-4cea-9d18-7149f1df9dc5','Paul Speulstra, AECOM Global Security Operations Center','technique');
INSERT INTO contributor VALUES('attack-pattern--72b74d71-8169-42aa-92e0-e7b04b9f5a08','Microsoft Threat Intelligence Center (MSTIC)','technique');
INSERT INTO contributor VALUES('attack-pattern--72b74d71-8169-42aa-92e0-e7b04b9f5a08','Travis Smith, Tripwire','technique');
INSERT INTO contributor VALUES('attack-pattern--a10641f4-87b4-45a3-a906-92a149cb2c27','Jannie Li, Microsoft Threat Intelligence Center (MSTIC)','technique');
INSERT INTO contributor VALUES('attack-pattern--a10641f4-87b4-45a3-a906-92a149cb2c27','Praetorian','technique');
INSERT INTO contributor VALUES('attack-pattern--a10641f4-87b4-45a3-a906-92a149cb2c27','Tim MalcomVetter','technique');
INSERT INTO contributor VALUES('attack-pattern--2dbbdcd5-92cf-44c0-aea2-fe24783a6bc3','Microsoft Threat Intelligence Center (MSTIC)','technique');
INSERT INTO contributor VALUES('attack-pattern--37b11151-1776-4f8f-b328-30939fbf2ceb','Phil Stokes, SentinelOne','technique');
INSERT INTO contributor VALUES('attack-pattern--f005e783-57d4-4837-88ad-dbe7faee1c51','Shailesh Tiwary (Indian Army)','technique');
INSERT INTO contributor VALUES('attack-pattern--f005e783-57d4-4837-88ad-dbe7faee1c51','Saisha Agrawal, Microsoft Threat Intelligent Center (MSTIC)','technique');
INSERT INTO contributor VALUES('attack-pattern--f005e783-57d4-4837-88ad-dbe7faee1c51','Jeff Sakowicz, Microsoft Identity Developer Platform Services (IDPM Services)','technique');
INSERT INTO contributor VALUES('attack-pattern--f005e783-57d4-4837-88ad-dbe7faee1c51','Mark Wee','technique');
INSERT INTO contributor VALUES('attack-pattern--c8e87b83-edbb-48d4-9295-4974897525b7','Ricardo Dias','technique');
INSERT INTO contributor VALUES('attack-pattern--c8e87b83-edbb-48d4-9295-4974897525b7','Red Canary','technique');
INSERT INTO contributor VALUES('attack-pattern--5bfccc3f-2326-4112-86cc-c1ece9d8a2b5','Martin Jirkal, ESET','technique');
INSERT INTO contributor VALUES('attack-pattern--5e4a2073-9643-44cb-a0b5-e7f4048446c7','Mike Kemmerer','technique');
INSERT INTO contributor VALUES('attack-pattern--389735f1-f21c-4208-b8f0-f8031e7169b8','Justin Warner, ICEBRG','technique');
INSERT INTO contributor VALUES('attack-pattern--a93494bb-4b80-4ea1-8695-3236a49916fd','Ed Williams, Trustwave, SpiderLabs','technique');
INSERT INTO contributor VALUES('attack-pattern--120d5519-3098-4e1c-9191-2aa61232f073','Stefan Kanthak','technique');
INSERT INTO contributor VALUES('attack-pattern--120d5519-3098-4e1c-9191-2aa61232f073','Casey Smith','technique');
INSERT INTO contributor VALUES('attack-pattern--4cbc6a62-9e34-4f94-8a19-5c1a11392a49','Nik Seetharaman, Palantir','technique');
INSERT INTO contributor VALUES('attack-pattern--4cbc6a62-9e34-4f94-8a19-5c1a11392a49','Ye Yint Min Thu Htut, Offensive Security Team, DBS Bank','technique');
INSERT INTO contributor VALUES('attack-pattern--ffeb0780-356e-4261-b036-cfb6bd234335','Jesse Brown, Red Canary','technique');
INSERT INTO contributor VALUES('attack-pattern--6add2ab5-2711-4e9d-87c8-7a0be8531530','Ed Williams, Trustwave, SpiderLabs','technique');
INSERT INTO contributor VALUES('attack-pattern--98034fef-d9fb-4667-8dc4-2eab6231724c','Travis Smith, Tripwire','technique');
INSERT INTO contributor VALUES('attack-pattern--98034fef-d9fb-4667-8dc4-2eab6231724c','Stefan Kanthak','technique');
INSERT INTO contributor VALUES('attack-pattern--3aef9463-9a7a-43ba-8957-a867e07c1e6a','Vikas Singh, Sophos','technique');
INSERT INTO contributor VALUES('attack-pattern--3aef9463-9a7a-43ba-8957-a867e07c1e6a','Emile Kenning, Sophos','technique');
INSERT INTO contributor VALUES('attack-pattern--a009cb25-4801-4116-9105-80a91cf15c1b','Praetorian','technique');
INSERT INTO contributor VALUES('attack-pattern--a009cb25-4801-4116-9105-80a91cf15c1b','Microsoft Threat Intelligence Center (MSTIC)','technique');
INSERT INTO contributor VALUES('attack-pattern--8f104855-e5b7-4077-b1f5-bc3103b41abe','Praetorian','technique');
INSERT INTO contributor VALUES('attack-pattern--57a3d31a-d04f-4663-b2da-7df8ec3f8c9d','Praetorian','technique');
INSERT INTO contributor VALUES('attack-pattern--19bf235b-8620-4997-b5b4-94e0659ed7c3','Praetorian','technique');
INSERT INTO contributor VALUES('attack-pattern--e49920b0-6c54-40c1-9571-73723653205f','Praetorian','technique');
INSERT INTO contributor VALUES('attack-pattern--e24fcba8-2557-4442-a139-1ee2f2e784db','Suzy Schapperle - Microsoft Azure Red Team','technique');
INSERT INTO contributor VALUES('attack-pattern--e24fcba8-2557-4442-a139-1ee2f2e784db','Praetorian','technique');
INSERT INTO contributor VALUES('attack-pattern--c726e0a2-a57a-4b7b-a973-d0f013246617','Praetorian','technique');
INSERT INTO contributor VALUES('attack-pattern--c726e0a2-a57a-4b7b-a973-d0f013246617','Ye Yint Min Thu Htut, Offensive Security Team, DBS Bank','technique');
INSERT INTO contributor VALUES('attack-pattern--a6937325-9321-4e2e-bb2b-3ed2d40b2a9d','Rahmat Nurfauzi, @infosecn1nja, PT Xynexis International','technique');
INSERT INTO contributor VALUES('attack-pattern--bc0f5e80-91c0-4e04-9fbb-e4e332c85dae','Elastic','technique');
INSERT INTO contributor VALUES('attack-pattern--960c3c86-1480-4d72-b4e0-8c242e84a5c5','CrowdStrike Falcon OverWatch','technique');
CREATE TABLE platform (
	id VARCHAR NOT NULL,
	platform VARCHAR NOT NULL,
	type VARCHAR NOT NULL,
	PRIMARY KEY (id, platform)
);
INSERT INTO platform VALUES('attack-pattern--b63a34e8-0a61-4c97-a23b-bf8a2ed812e2','Linux','technique');
INSERT INTO platform VALUES('attack-pattern--b63a34e8-0a61-4c97-a23b-bf8a2ed812e2','macOS','technique');
INSERT INTO platform VALUES('attack-pattern--d0b4fcdb-d67d-4ed2-99ce-788b12f8c0f4','Linux','technique');
INSERT INTO platform VALUES('attack-pattern--cabe189c-a0e3-4965-a473-dcff00f17213','Linux','technique');
INSERT INTO platform VALUES('attack-pattern--cabe189c-a0e3-4965-a473-dcff00f17213','Windows','technique');
INSERT INTO platform VALUES('attack-pattern--cabe189c-a0e3-4965-a473-dcff00f17213','macOS','technique');
INSERT INTO platform VALUES('attack-pattern--3986e7fd-a8e9-4ecb-bfc6-55920855912b','Windows','technique');
INSERT INTO platform VALUES('attack-pattern--67720091-eee3-4d2d-ae16-8264567f6f5b','Linux','technique');
INSERT INTO platform VALUES('attack-pattern--67720091-eee3-4d2d-ae16-8264567f6f5b','macOS','technique');
INSERT INTO platform VALUES('attack-pattern--67720091-eee3-4d2d-ae16-8264567f6f5b','Windows','technique');
INSERT INTO platform VALUES('attack-pattern--dcaa092b-7de9-4a21-977f-7fcb77e89c48','Windows','technique');
INSERT INTO platform VALUES('attack-pattern--70e52b04-2a0c-4cea-9d18-7149f1df9dc5','Windows','technique');
INSERT INTO platform VALUES('attack-pattern--b24e2a20-3b3d-4bf0-823b-1ed765398fb0','Linux','technique');
INSERT INTO platform VALUES('attack-pattern--b24e2a20-3b3d-4bf0-823b-1ed765398fb0','macOS','technique');
INSERT INTO platform VALUES('attack-pattern--b24e2a20-3b3d-4bf0-823b-1ed765398fb0','Windows','technique');
INSERT INTO platform VALUES('attack-pattern--72b74d71-8169-42aa-92e0-e7b04b9f5a08','Linux','technique');
INSERT INTO platform VALUES('attack-pattern--72b74d71-8169-42aa-92e0-e7b04b9f5a08','macOS','technique');
INSERT INTO platform VALUES('attack-pattern--72b74d71-8169-42aa-92e0-e7b04b9f5a08','Windows','technique');
INSERT INTO platform VALUES('attack-pattern--72b74d71-8169-42aa-92e0-e7b04b9f5a08','Office 365','technique');
INSERT INTO platform VALUES('attack-pattern--72b74d71-8169-42aa-92e0-e7b04b9f5a08','Azure AD','technique');
INSERT INTO platform VALUES('attack-pattern--72b74d71-8169-42aa-92e0-e7b04b9f5a08','AWS','technique');
INSERT INTO platform VALUES('attack-pattern--72b74d71-8169-42aa-92e0-e7b04b9f5a08','GCP','technique');
INSERT INTO platform VALUES('attack-pattern--72b74d71-8169-42aa-92e0-e7b04b9f5a08','Azure','technique');
INSERT INTO platform VALUES('attack-pattern--72b74d71-8169-42aa-92e0-e7b04b9f5a08','SaaS','technique');
INSERT INTO platform VALUES('attack-pattern--a10641f4-87b4-45a3-a906-92a149cb2c27','Windows','technique');
INSERT INTO platform VALUES('attack-pattern--a10641f4-87b4-45a3-a906-92a149cb2c27','Office 365','technique');
INSERT INTO platform VALUES('attack-pattern--a10641f4-87b4-45a3-a906-92a149cb2c27','Azure','technique');
INSERT INTO platform VALUES('attack-pattern--a10641f4-87b4-45a3-a906-92a149cb2c27','GCP','technique');
INSERT INTO platform VALUES('attack-pattern--a10641f4-87b4-45a3-a906-92a149cb2c27','Azure AD','technique');
INSERT INTO platform VALUES('attack-pattern--a10641f4-87b4-45a3-a906-92a149cb2c27','AWS','technique');
INSERT INTO platform VALUES('attack-pattern--a10641f4-87b4-45a3-a906-92a149cb2c27','Linux','technique');
INSERT INTO platform VALUES('attack-pattern--a10641f4-87b4-45a3-a906-92a149cb2c27','macOS','technique');
INSERT INTO platform VALUES('attack-pattern--0458aab9-ad42-4eac-9e22-706a95bafee2','PRE','technique');
INSERT INTO platform VALUES('attack-pattern--67073dde-d720-45ae-83da-b12d5e73ca3b','PRE','technique');
INSERT INTO platform VALUES('attack-pattern--2dbbdcd5-92cf-44c0-aea2-fe24783a6bc3','Office 365','technique');
INSERT INTO platform VALUES('attack-pattern--34f1d81d-fe88-4f97-bd3b-a3164536255d','Windows','technique');
INSERT INTO platform VALUES('attack-pattern--34f1d81d-fe88-4f97-bd3b-a3164536255d','Office 365','technique');
INSERT INTO platform VALUES('attack-pattern--7d57b371-10c2-45e5-b3cc-83a8fb380e4c','Windows','technique');
INSERT INTO platform VALUES('attack-pattern--cc89ecbd-3d33-4a41-bcca-001e702d18fd','Windows','technique');
INSERT INTO platform VALUES('attack-pattern--37b11151-1776-4f8f-b328-30939fbf2ceb','macOS','technique');
INSERT INTO platform VALUES('attack-pattern--f005e783-57d4-4837-88ad-dbe7faee1c51','Office 365','technique');
INSERT INTO platform VALUES('attack-pattern--f005e783-57d4-4837-88ad-dbe7faee1c51','SaaS','technique');
INSERT INTO platform VALUES('attack-pattern--18cffc21-3260-437e-80e4-4ab8bf2ba5e9','Linux','technique');
INSERT INTO platform VALUES('attack-pattern--18cffc21-3260-437e-80e4-4ab8bf2ba5e9','macOS','technique');
INSERT INTO platform VALUES('attack-pattern--18cffc21-3260-437e-80e4-4ab8bf2ba5e9','Windows','technique');
INSERT INTO platform VALUES('attack-pattern--18cffc21-3260-437e-80e4-4ab8bf2ba5e9','AWS','technique');
INSERT INTO platform VALUES('attack-pattern--18cffc21-3260-437e-80e4-4ab8bf2ba5e9','GCP','technique');
INSERT INTO platform VALUES('attack-pattern--18cffc21-3260-437e-80e4-4ab8bf2ba5e9','Azure','technique');
INSERT INTO platform VALUES('attack-pattern--18cffc21-3260-437e-80e4-4ab8bf2ba5e9','Office 365','technique');
INSERT INTO platform VALUES('attack-pattern--18cffc21-3260-437e-80e4-4ab8bf2ba5e9','Azure AD','technique');
CREATE TABLE reference (
	id VARCHAR NOT NULL,
	source VARCHAR NOT NULL,
	external_id VARCHAR,
	url VARCHAR NOT NULL,
	description VARCHAR,
	type VARCHAR NOT NULL,
	PRIMARY KEY (id, source, url)
);
INSERT INTO reference VALUES('attack-pattern--01df3350-ce05-4bdf-bdf8-0a919a66d4a8','mitre-attack','T1156','https://attack.mitre.org/techniques/T1156',NULL,'technique');
INSERT INTO reference VALUES('attack-pattern--01df3350-ce05-4bdf-bdf8-0a919a66d4a8','amnesia malware',NULL,'https://researchcenter.paloaltonetworks.com/2017/04/unit42-new-iotlinux-malware-targets-dvrs-forms-botnet/','Claud Xiao, Cong Zheng, Yanhui Jia. (2017, April 6). New IoT/Linux Malware Targets DVRs, Forms Botnet. Retrieved February 19, 2018.','technique');
INSERT INTO reference VALUES('attack-pattern--b63a34e8-0a61-4c97-a23b-bf8a2ed812e2','mitre-attack','T1546.004','https://attack.mitre.org/techniques/T1546/004',NULL,'technique');
INSERT INTO reference VALUES('attack-pattern--b63a34e8-0a61-4c97-a23b-bf8a2ed812e2','amnesia malware',NULL,'https://researchcenter.paloaltonetworks.com/2017/04/unit42-new-iotlinux-malware-targets-dvrs-forms-botnet/','Claud Xiao, Cong Zheng, Yanhui Jia. (2017, April 6). New IoT/Linux Malware Targets DVRs, Forms Botnet. Retrieved February 19, 2018.','technique');
INSERT INTO reference VALUES('attack-pattern--d0b4fcdb-d67d-4ed2-99ce-788b12f8c0f4','mitre-attack','T1003.008','https://attack.mitre.org/techniques/T1003/008',NULL,'technique');
INSERT INTO reference VALUES('attack-pattern--d0b4fcdb-d67d-4ed2-99ce-788b12f8c0f4','Linux Password and Shadow File Formats',NULL,'https://www.tldp.org/LDP/lame/LAME/linux-admin-made-easy/shadow-file-formats.html','The Linux Documentation Project. (n.d.). Linux Password and Shadow File Formats. Retrieved February 19, 2020.','technique');
INSERT INTO reference VALUES('attack-pattern--d0b4fcdb-d67d-4ed2-99ce-788b12f8c0f4','nixCraft - John the Ripper',NULL,'https://www.cyberciti.biz/faq/unix-linux-password-cracking-john-the-ripper/','Vivek Gite. (2014, September 17). Linux Password Cracking: Explain unshadow and john Commands (John the Ripper Tool). Retrieved February 19, 2020.','technique');
INSERT INTO reference VALUES('attack-pattern--cabe189c-a0e3-4965-a473-dcff00f17213','mitre-attack','T1557.002','https://attack.mitre.org/techniques/T1557/002',NULL,'technique');
INSERT INTO reference VALUES('attack-pattern--cabe189c-a0e3-4965-a473-dcff00f17213','RFC826 ARP',NULL,'https://tools.ietf.org/html/rfc826','Plummer, D. (1982, November). An Ethernet Address Resolution Protocol. Retrieved October 15, 2020.','technique');
INSERT INTO reference VALUES('attack-pattern--cabe189c-a0e3-4965-a473-dcff00f17213','Sans ARP Spoofing Aug 2003',NULL,'https://pen-testing.sans.org/resources/papers/gcih/real-world-arp-spoofing-105411','Siles, R. (2003, August). Real World ARP Spoofing. Retrieved October 15, 2020.','technique');
INSERT INTO reference VALUES('attack-pattern--cabe189c-a0e3-4965-a473-dcff00f17213','Cylance Cleaver',NULL,'https://www.cylance.com/content/dam/cylance/pages/operation-cleaver/Cylance_Operation_Cleaver_Report.pdf','Cylance. (2014, December). Operation Cleaver. Retrieved September 14, 2017.','technique');
INSERT INTO reference VALUES('attack-pattern--3986e7fd-a8e9-4ecb-bfc6-55920855912b','mitre-attack','T1558.004','https://attack.mitre.org/techniques/T1558/004',NULL,'technique');
INSERT INTO reference VALUES('attack-pattern--3986e7fd-a8e9-4ecb-bfc6-55920855912b','Harmj0y Roasting AS-REPs Jan 2017',NULL,'http://www.harmj0y.net/blog/activedirectory/roasting-as-reps/','HarmJ0y. (2017, January 17). Roasting AS-REPs. Retrieved August 24, 2020.','technique');
INSERT INTO reference VALUES('attack-pattern--3986e7fd-a8e9-4ecb-bfc6-55920855912b','Microsoft Kerberos Preauth 2014',NULL,'https://social.technet.microsoft.com/wiki/contents/articles/23559.kerberos-pre-authentication-why-it-should-not-be-disabled.aspx','Sanyal, M.. (2014, March 18). Kerberos Pre-Authentication: Why It Should Not Be Disabled. Retrieved August 25, 2020.','technique');
INSERT INTO reference VALUES('attack-pattern--3986e7fd-a8e9-4ecb-bfc6-55920855912b','Stealthbits Cracking AS-REP Roasting Jun 2019',NULL,'https://blog.stealthbits.com/cracking-active-directory-passwords-with-as-rep-roasting/','Jeff Warren. (2019, June 27). Cracking Active Directory Passwords with AS-REP Roasting. Retrieved August 24, 2020.','technique');
INSERT INTO reference VALUES('attack-pattern--3986e7fd-a8e9-4ecb-bfc6-55920855912b','SANS Attacking Kerberos Nov 2014',NULL,'https://redsiege.com/kerberoast-slides','Medin, T. (2014, November). Attacking Kerberos - Kicking the Guard Dog of Hades. Retrieved March 22, 2018.','technique');
INSERT INTO reference VALUES('attack-pattern--3986e7fd-a8e9-4ecb-bfc6-55920855912b','AdSecurity Cracking Kerberos Dec 2015',NULL,'https://adsecurity.org/?p=2293','Metcalf, S. (2015, December 31). Cracking Kerberos TGS Tickets Using Kerberoast – Exploiting Kerberos to Compromise the Active Directory Domain. Retrieved March 22, 2018.','technique');
INSERT INTO reference VALUES('attack-pattern--3986e7fd-a8e9-4ecb-bfc6-55920855912b','Microsoft Detecting Kerberoasting Feb 2018',NULL,'https://blogs.technet.microsoft.com/motiba/2018/02/23/detecting-kerberoasting-activity-using-azure-security-center/','Bani, M. (2018, February 23). Detecting Kerberoasting activity using Azure Security Center. Retrieved March 23, 2018.','technique');
INSERT INTO reference VALUES('attack-pattern--3986e7fd-a8e9-4ecb-bfc6-55920855912b','Microsoft 4768 TGT 2017',NULL,'https://docs.microsoft.com/en-us/windows/security/threat-protection/auditing/event-4768','Microsoft. (2017, April 19). 4768(S, F): A Kerberos authentication ticket (TGT) was requested. Retrieved August 24, 2020.','technique');
INSERT INTO reference VALUES('attack-pattern--67720091-eee3-4d2d-ae16-8264567f6f5b','mitre-attack','T1548','https://attack.mitre.org/techniques/T1548',NULL,'technique');
INSERT INTO reference VALUES('attack-pattern--dcaa092b-7de9-4a21-977f-7fcb77e89c48','mitre-attack','T1134','https://attack.mitre.org/techniques/T1134',NULL,'technique');
INSERT INTO reference VALUES('attack-pattern--dcaa092b-7de9-4a21-977f-7fcb77e89c48','capec','CAPEC-633','https://capec.mitre.org/data/definitions/633.html',NULL,'technique');
INSERT INTO reference VALUES('attack-pattern--dcaa092b-7de9-4a21-977f-7fcb77e89c48','Pentestlab Token Manipulation',NULL,'https://pentestlab.blog/2017/04/03/token-manipulation/','netbiosX. (2017, April 3). Token Manipulation. Retrieved April 21, 2017.','technique');
INSERT INTO reference VALUES('attack-pattern--dcaa092b-7de9-4a21-977f-7fcb77e89c48','Microsoft Command-line Logging',NULL,'https://technet.microsoft.com/en-us/windows-server-docs/identity/ad-ds/manage/component-updates/command-line-process-auditing','Mathers, B. (2017, March 7). Command line process auditing. Retrieved April 21, 2017.','technique');
INSERT INTO reference VALUES('attack-pattern--dcaa092b-7de9-4a21-977f-7fcb77e89c48','Microsoft LogonUser',NULL,'https://msdn.microsoft.com/en-us/library/windows/desktop/aa378184(v=vs.85).aspx','Microsoft TechNet. (n.d.). Retrieved April 25, 2017.','technique');
INSERT INTO reference VALUES('attack-pattern--dcaa092b-7de9-4a21-977f-7fcb77e89c48','Microsoft DuplicateTokenEx',NULL,'https://msdn.microsoft.com/en-us/library/windows/desktop/aa446617(v=vs.85).aspx','Microsoft TechNet. (n.d.). Retrieved April 25, 2017.','technique');
INSERT INTO reference VALUES('attack-pattern--dcaa092b-7de9-4a21-977f-7fcb77e89c48','Microsoft ImpersonateLoggedOnUser',NULL,'https://msdn.microsoft.com/en-us/library/windows/desktop/aa378612(v=vs.85).aspx','Microsoft TechNet. (n.d.). Retrieved April 25, 2017.','technique');
INSERT INTO reference VALUES('attack-pattern--dcaa092b-7de9-4a21-977f-7fcb77e89c48','BlackHat Atkinson Winchester Token Manipulation',NULL,'https://www.blackhat.com/docs/eu-17/materials/eu-17-Atkinson-A-Process-Is-No-One-Hunting-For-Token-Manipulation.pdf','Atkinson, J., Winchester, R. (2017, December 7). A Process is No One: Hunting for Token Manipulation. Retrieved December 21, 2017.','technique');
INSERT INTO reference VALUES('attack-pattern--9b99b83a-1aac-4e29-b975-b374950551a3','mitre-attack','T1015','https://attack.mitre.org/techniques/T1015',NULL,'technique');
INSERT INTO reference VALUES('attack-pattern--9b99b83a-1aac-4e29-b975-b374950551a3','capec','CAPEC-558','https://capec.mitre.org/data/definitions/558.html',NULL,'technique');
INSERT INTO reference VALUES('attack-pattern--9b99b83a-1aac-4e29-b975-b374950551a3','FireEye Hikit Rootkit',NULL,'https://www.fireeye.com/blog/threat-research/2012/08/hikit-rootkit-advanced-persistent-attack-techniques-part-1.html','Glyer, C., Kazanciyan, R. (2012, August 20). The “Hikit” Rootkit: Advanced and Persistent Attack Techniques (Part 1). Retrieved June 6, 2016.','technique');
INSERT INTO reference VALUES('attack-pattern--9b99b83a-1aac-4e29-b975-b374950551a3','DEFCON2016 Sticky Keys',NULL,'https://www.slideshare.net/DennisMaldonado5/sticky-keys-to-the-kingdom','Maldonado, D., McGuffin, T. (2016, August 6). Sticky Keys to the Kingdom. Retrieved July 5, 2017.','technique');
INSERT INTO reference VALUES('attack-pattern--9b99b83a-1aac-4e29-b975-b374950551a3','Tilbury 2014',NULL,'http://blog.crowdstrike.com/registry-analysis-with-crowdresponse/','Tilbury, C. (2014, August 28). Registry Analysis with CrowdResponse. Retrieved November 12, 2014.','technique');
INSERT INTO reference VALUES('attack-pattern--70e52b04-2a0c-4cea-9d18-7149f1df9dc5','mitre-attack','T1546.008','https://attack.mitre.org/techniques/T1546/008',NULL,'technique');
INSERT INTO reference VALUES('attack-pattern--70e52b04-2a0c-4cea-9d18-7149f1df9dc5','capec','CAPEC-558','https://capec.mitre.org/data/definitions/558.html',NULL,'technique');
INSERT INTO reference VALUES('attack-pattern--70e52b04-2a0c-4cea-9d18-7149f1df9dc5','FireEye Hikit Rootkit',NULL,'https://www.fireeye.com/blog/threat-research/2012/08/hikit-rootkit-advanced-persistent-attack-techniques-part-1.html','Glyer, C., Kazanciyan, R. (2012, August 20). The “Hikit” Rootkit: Advanced and Persistent Attack Techniques (Part 1). Retrieved June 6, 2016.','technique');
INSERT INTO reference VALUES('attack-pattern--70e52b04-2a0c-4cea-9d18-7149f1df9dc5','DEFCON2016 Sticky Keys',NULL,'https://www.slideshare.net/DennisMaldonado5/sticky-keys-to-the-kingdom','Maldonado, D., McGuffin, T. (2016, August 6). Sticky Keys to the Kingdom. Retrieved July 5, 2017.','technique');
INSERT INTO reference VALUES('attack-pattern--70e52b04-2a0c-4cea-9d18-7149f1df9dc5','Tilbury 2014',NULL,'http://blog.crowdstrike.com/registry-analysis-with-crowdresponse/','Tilbury, C. (2014, August 28). Registry Analysis with CrowdResponse. Retrieved November 12, 2014.','technique');
INSERT INTO reference VALUES('attack-pattern--70e52b04-2a0c-4cea-9d18-7149f1df9dc5','Narrator Accessibility Abuse',NULL,'https://giuliocomi.blogspot.com/2019/10/abusing-windows-10-narrators-feedback.html','Comi, G. (2019, October 19). Abusing Windows 10 Narrator''s ''Feedback-Hub'' URI for Fileless Persistence. Retrieved April 28, 2020.','technique');
INSERT INTO reference VALUES('attack-pattern--b24e2a20-3b3d-4bf0-823b-1ed765398fb0','mitre-attack','T1531','https://attack.mitre.org/techniques/T1531',NULL,'technique');
INSERT INTO reference VALUES('attack-pattern--b24e2a20-3b3d-4bf0-823b-1ed765398fb0','CarbonBlack LockerGoga 2019',NULL,'https://www.carbonblack.com/2019/03/22/tau-threat-intelligence-notification-lockergoga-ransomware/','CarbonBlack Threat Analysis Unit. (2019, March 22). TAU Threat Intelligence Notification – LockerGoga Ransomware. Retrieved April 16, 2019.','technique');
INSERT INTO reference VALUES('attack-pattern--b24e2a20-3b3d-4bf0-823b-1ed765398fb0','Unit42 LockerGoga 2019',NULL,'https://unit42.paloaltonetworks.com/born-this-way-origins-of-lockergoga/','Harbison, M.. (2019, March 26). Born This Way? Origins of LockerGoga. Retrieved April 16, 2019.','technique');
INSERT INTO reference VALUES('attack-pattern--72b74d71-8169-42aa-92e0-e7b04b9f5a08','mitre-attack','T1087','https://attack.mitre.org/techniques/T1087',NULL,'technique');
INSERT INTO reference VALUES('attack-pattern--72b74d71-8169-42aa-92e0-e7b04b9f5a08','capec','CAPEC-575','https://capec.mitre.org/data/definitions/575.html',NULL,'technique');
INSERT INTO reference VALUES('attack-pattern--a10641f4-87b4-45a3-a906-92a149cb2c27','mitre-attack','T1098','https://attack.mitre.org/techniques/T1098',NULL,'technique');
INSERT INTO reference VALUES('attack-pattern--a10641f4-87b4-45a3-a906-92a149cb2c27','Microsoft User Modified Event',NULL,'https://docs.microsoft.com/en-us/windows/security/threat-protection/auditing/event-4738','Lich, B., Miroshnikov, A. (2017, April 5). 4738(S): A user account was changed. Retrieved June 30, 2017.','technique');
INSERT INTO reference VALUES('attack-pattern--a10641f4-87b4-45a3-a906-92a149cb2c27','Microsoft Security Event 4670',NULL,'https://www.ultimatewindowssecurity.com/securitylog/encyclopedia/event.aspx?eventID=4670','Franklin Smith, R. (n.d.). Windows Security Log Event ID 4670. Retrieved November 4, 2019.','technique');
INSERT INTO reference VALUES('attack-pattern--a10641f4-87b4-45a3-a906-92a149cb2c27','InsiderThreat ChangeNTLM July 2017',NULL,'https://blog.stealthbits.com/manipulating-user-passwords-with-mimikatz-SetNTLM-ChangeNTLM','Warren, J. (2017, July 11). Manipulating User Passwords with Mimikatz. Retrieved December 4, 2017.','technique');
INSERT INTO reference VALUES('attack-pattern--a10641f4-87b4-45a3-a906-92a149cb2c27','GitHub Mimikatz Issue 92 June 2017',NULL,'https://github.com/gentilkiwi/mimikatz/issues/92','Warren, J. (2017, June 22). lsadump::changentlm and lsadump::setntlm work, but generate Windows events #92. Retrieved December 4, 2017.','technique');
INSERT INTO reference VALUES('attack-pattern--0458aab9-ad42-4eac-9e22-706a95bafee2','mitre-attack','T1583','https://attack.mitre.org/techniques/T1583',NULL,'technique');
CREATE TABLE use (
	id VARCHAR NOT NULL,
	source_id VARCHAR NOT NULL,
	source_type VARCHAR NOT NULL,
	target_id VARCHAR NOT NULL,
	target_type VARCHAR NOT NULL,
	description VARCHAR,
	created_time DATETIME,
	modified_time DATETIME,
	PRIMARY KEY (id)
);
INSERT INTO use VALUES('relationship--f24d37c0-283d-4f37-8278-07fc75cc0e94','intrusion-set--0bbdf25b-30ff-4894-a1cd-49260d0dd2d9','group','malware--4e6b9625-bbda-4d96-a652-b3bb45453f26','software','(Citation: Symantec Buckeye)','2018-01-16 16:13:52.465000','2019-04-29 18:01:20.637000');
INSERT INTO use VALUES('relationship--857b5ce3-0eb0-4dae-a034-42cfa306a447','intrusion-set--4a2ce82e-1a74-468a-a6fb-bbead541383c','group','attack-pattern--be2dcee9-a7a7-4e38-afd6-21b31ecc3d63','technique','[APT37](https://attack.mitre.org/groups/G0067) has used Flash Player (CVE-2016-4117, CVE-2018-4878) and Word (CVE-2017-0199) exploits for execution.(Citation: Securelist ScarCruft Jun 2016)(Citation: FireEye APT37 Feb 2018)(Citation: Talos Group123)','2018-04-18 17:59:24.739000','2019-09-09 19:12:32.476000');
INSERT INTO use VALUES('relationship--91926031-c1b4-462b-8597-20b10e7211ef','intrusion-set--3753cc21-2dae-4dfb-8481-d004e74502cc','group','attack-pattern--6faf650d-bf31-4eb4-802d-1000cf38efaf','technique','[FIN7](https://attack.mitre.org/groups/G0046) created a custom video recording capability that could be used to monitor operations in the victim''s environment.(Citation: FireEye FIN7 Aug 2018)(Citation: DOJ FIN7 Aug 2018)','2018-10-17 00:14:20.652000','2019-06-30 23:13:18.234000');
INSERT INTO use VALUES('relationship--d26b3aeb-972f-471e-ab59-dc1ee2aa532e','intrusion-set--bef4c620-0787-42a8-a96d-b7eb6e85917c','group','malware--af2ad3b7-ab6a-4807-91fd-51bcaff9acbb','software','(Citation: ESET Sednit Part 3)','2017-05-31 21:33:27.041000','2019-09-09 17:44:35.665000');
INSERT INTO use VALUES('relationship--d8e375a3-f455-4c66-bc63-251f320ec8b1','intrusion-set--4ca1929c-7d64-4aab-b849-badbfc0c760d','group','attack-pattern--8f4a33ec-8b1f-4b80-a2f6-642b2e479580','technique','[OilRig](https://attack.mitre.org/groups/G0049) has run <code>tasklist</code> on a victim''s machine.(Citation: Palo Alto OilRig May 2016)','2018-01-16 16:13:52.465000','2019-09-04 22:55:40.600000');
INSERT INTO use VALUES('relationship--0f76d408-be8a-478e-8a5a-aab1d1f96572','malware--e9595678-d269-469e-ae6b-75e49259de63','software','attack-pattern--df8b2a25-8bdf-4856-953c-a04372b1c161','technique','[BADNEWS](https://attack.mitre.org/software/S0128) establishes a backdoor over HTTP.(Citation: PaloAlto Patchwork Mar 2018)','2018-04-18 17:59:24.739000','2020-03-17 00:19:57.934000');
INSERT INTO use VALUES('relationship--343d285a-e910-487b-8e85-dc87cdb63be3','intrusion-set--899ce53f-13a0-479b-a0e4-67d46e241542','group','attack-pattern--9efb1ea7-c37b-4595-9640-b7680cd84279','technique','[APT29](https://attack.mitre.org/groups/G0016) added Registry Run keys to establish persistence.(Citation: Mandiant No Easy Breach)','2017-05-31 21:33:27.049000','2019-07-25 14:25:53.037000');
INSERT INTO use VALUES('relationship--04203d88-5fe1-4e63-be65-51a17705716b','intrusion-set--222fbd21-fc4f-4b7e-9f85-0e6e3a76c33f','group','attack-pattern--01a5a209-b94c-450b-b7f9-946497d91055','technique','[menuPass](https://attack.mitre.org/groups/G0045) uses a modified version of pentesting script wmiexec.vbs, which logs into a remote machine using WMI.(Citation: PWC Cloud Hopper Technical Annex April 2017)(Citation: Github AD-Pentest-Script)','2017-12-14 16:46:06.044000','2019-09-03 18:50:16.264000');
INSERT INTO use VALUES('relationship--8c763d80-4c50-4ebd-b2c6-3cad22c55bfa','intrusion-set--6713ab67-e25b-49cc-808d-2b36d4fbc35c','group','attack-pattern--3c4a2599-71ee-4405-ba1e-0e28414b4bc5','technique','[Ke3chang](https://attack.mitre.org/groups/G0004) gathered information and files from local directories for exfiltration.(Citation: Villeneuve et al 2014)','2017-05-31 21:33:27.034000','2019-04-22 15:08:48.239000');
INSERT INTO use VALUES('relationship--119cfc44-4018-4ce0-b223-b0f353d1c113','malware--1f6e3702-7ca1-4582-b2e7-4591297d05a8','software','attack-pattern--3c4a2599-71ee-4405-ba1e-0e28414b4bc5','technique','[Bankshot](https://attack.mitre.org/software/S0239) collects files from the local system.(Citation: McAfee Bankshot)','2018-10-17 00:14:20.652000','2020-02-18 03:40:29.805000');
INSERT INTO use VALUES('relationship--1ccf8b65-7f3e-4db7-933c-0a9bfae9602e','intrusion-set--17862c7d-9e60-48a0-b48e-da4dc4c3f6b0','group','malware--d1183cb9-258e-4f2f-8415-50ac8252c49e','software','(Citation: TrendMicro Patchwork Dec 2017)','2018-10-17 00:14:20.652000','2019-07-11 13:53:06.089000');
INSERT INTO use VALUES('relationship--7a2f70b7-7b6e-4c05-8f71-42a494b055ce','intrusion-set--1f21da59-6a13-455b-afd0-d58d0a5a7d27','group','attack-pattern--2e34237d-8574-43f6-aace-ae2915de8597','technique','[Gorgon Group](https://attack.mitre.org/groups/G0078) sent emails to victims with malicious Microsoft Office documents attached.(Citation: Unit 42 Gorgon Group Aug 2018)','2018-10-17 00:14:20.652000','2019-07-25 14:56:46.618000');
INSERT INTO use VALUES('relationship--87ddc052-0933-4722-9fb2-4653c4a3663c','intrusion-set--0bbdf25b-30ff-4894-a1cd-49260d0dd2d9','group','attack-pattern--1c34f7aa-9341-4a48-bfab-af22e51aca6c','technique','[APT3](https://attack.mitre.org/groups/G0022) has been known to stage files for exfiltration in a single location.(Citation: aptsim)','2018-01-16 16:13:52.465000','2020-03-16 23:31:14.695000');
INSERT INTO use VALUES('relationship--558d3835-3bcc-4c07-92b1-626a89d79e76','intrusion-set--76d59913-1d24-4992-a8ac-05a3eb093f71','group','attack-pattern--42e8de7b-37b2-4258-905a-6897815e58e0','technique','[Dragonfly 2.0](https://attack.mitre.org/groups/G0074) created accounts disguised as legitimate backup and service accounts as well as an email administration account.(Citation: US-CERT TA18-074A)(Citation: US-CERT APT Energy Oct 2017)','2018-10-17 00:14:20.652000','2019-03-22 20:13:49.252000');
INSERT INTO use VALUES('relationship--0e89ca75-b73e-476e-b56d-1cf815fa7868','intrusion-set--17862c7d-9e60-48a0-b48e-da4dc4c3f6b0','group','attack-pattern--7bc57495-ea59-4380-be31-a64af124ef18','technique','A [Patchwork](https://attack.mitre.org/groups/G0040) payload has searched all fixed drives on the victim for files matching a specified list of extensions.(Citation: Cymmetria Patchwork)(Citation: TrendMicro Patchwork Dec 2017)','2017-05-31 21:33:27.077000','2020-06-29 15:54:17.677000');
INSERT INTO use VALUES('relationship--99e9583f-433d-437d-bf37-7ea2b3f1b613','intrusion-set--93f52415-0fe4-4d3d-896c-fc9b8e88ab90','group','attack-pattern--00f90846-cbd1-4fc5-9233-df5c2bf2a662','technique','[BRONZE BUTLER](https://attack.mitre.org/groups/G0060) has compressed data into password-protected RAR archives prior to exfiltration.(Citation: Secureworks BRONZE BUTLER Oct 2017)(Citation: Trend Micro Tick November 2019)','2018-01-16 16:13:52.465000','2020-06-24 01:27:31.804000');
INSERT INTO use VALUES('relationship--dc10e96f-1d3c-4ab9-8df6-acdc8238ec6c','intrusion-set--bef4c620-0787-42a8-a96d-b7eb6e85917c','group','attack-pattern--f7c0689c-4dbd-489b-81be-7cb7c7079ade','technique','[APT28](https://attack.mitre.org/groups/G0007) added "junk data" to each encoded string, preventing trivial decoding without knowledge of the junk removal algorithm. Each implant was given a "junk length" value when created, tracked by the controller software to allow seamless communication but prevent analysis of the command protocol on the wire.(Citation: FireEye APT28)','2017-05-31 21:33:27.038000','2020-03-19 21:17:42.990000');
INSERT INTO use VALUES('relationship--322703cc-c8f9-4046-8a61-e165a2d11bc7','intrusion-set--fe8796a4-2a02-41a0-9d27-7aa1e995feb6','group','attack-pattern--707399d6-ab3e-4963-9315-d9d3818cd6a0','technique','[APT19](https://attack.mitre.org/groups/G0073) used an HTTP malware variant and a Port 22 malware variant to collect the MAC address and IP address from the victim’s machine.(Citation: Unit 42 C0d0so0 Jan 2016)','2018-10-17 00:14:20.652000','2019-04-25 11:39:52.002000');
INSERT INTO use VALUES('relationship--d848b82c-1b72-4054-b4c1-f31d6d09af6c','intrusion-set--76d59913-1d24-4992-a8ac-05a3eb093f71','group','attack-pattern--2aed01ad-3df3-4410-a8cb-11ea4ded587c','technique','[Dragonfly 2.0](https://attack.mitre.org/groups/G0074) used batch scripts to enumerate administrators and users in the domain.(Citation: US-CERT TA18-074A)','2018-10-17 00:14:20.652000','2020-03-18 20:00:35.578000');
INSERT INTO use VALUES('relationship--7b529102-f95c-4ca1-a5c4-5a3497ab3674','intrusion-set--6713ab67-e25b-49cc-808d-2b36d4fbc35c','group','attack-pattern--322bad5a-1c49-4d23-ab79-76d641794afa','technique','[Ke3chang](https://attack.mitre.org/groups/G0004) performs service discovery using <code>net start</code> commands.(Citation: Villeneuve et al 2014)','2017-05-31 21:33:27.033000','2019-04-22 15:08:48.241000');
INSERT INTO use VALUES('relationship--edaf0203-4959-4e1e-9240-3d20cf0f3b6a','intrusion-set--bef4c620-0787-42a8-a96d-b7eb6e85917c','group','attack-pattern--3b744087-9945-4a6f-91e8-9dbceda417a4','technique','[APT28](https://attack.mitre.org/groups/G0007) uses a tool to infect connected USB devices and transmit itself to air-gapped computers when the infected USB device is inserted.(Citation: Microsoft SIR Vol 19)','2017-05-31 21:33:27.039000','2019-12-20 14:26:00.560000');
INSERT INTO use VALUES('relationship--9a615c7f-986d-4769-bea6-af9ffe0d575e','intrusion-set--0bbdf25b-30ff-4894-a1cd-49260d0dd2d9','group','attack-pattern--25659dd6-ea12-45c4-97e6-381e3e4b593e','technique','[APT3](https://attack.mitre.org/groups/G0022) has used a tool that can obtain info about local and global group users, power users, and administrators.(Citation: Symantec Buckeye)','2017-05-31 21:33:27.057000','2020-03-18 19:31:03.701000');
INSERT INTO use VALUES('relationship--cc2099fb-4785-4884-b274-4f3e8a3b8d99','malware--fb575479-14ef-41e9-bfab-0b7cf10bec73','software','attack-pattern--391d824f-0ef1-47a0-b0ee-c59a75e27670','technique','[ADVSTORESHELL](https://attack.mitre.org/software/S0045) is capable of starting a process using CreateProcess.(Citation: Bitdefender APT28 Dec 2015)','2017-12-14 16:46:06.044000','2020-03-16 15:53:20.350000');
INSERT INTO use VALUES('relationship--87f59d5f-cfda-4e5a-aa0d-6a7cb395528b','intrusion-set--76d59913-1d24-4992-a8ac-05a3eb093f71','group','attack-pattern--232b7f21-adf9-4b42-b936-b9d6f7df856e','technique','[Dragonfly 2.0](https://attack.mitre.org/groups/G0074) has used various forms of spearphishing in attempts to get users to open attachments.(Citation: US-CERT TA18-074A)(Citation: US-CERT APT Energy Oct 2017)','2018-10-17 00:14:20.652000','2020-03-17 14:14:37.025000');
INSERT INTO use VALUES('relationship--1a40426a-355c-4d7e-b51c-e95a102b31e2','intrusion-set--c93fccb1-e8e8-42cf-ae33-2ad1d183913a','group','attack-pattern--677569f9-a8b0-459e-ab24-7f18091fa7bf','technique','[Lazarus Group](https://attack.mitre.org/groups/G0032) keylogger KiloAlfa obtains user tokens from interactive sessions to execute itself with API call <code>CreateProcessAsUserA</code> under that user''s context.(Citation: Novetta Blockbuster)(Citation: Novetta Blockbuster Tools)','2017-12-14 16:46:06.044000','2020-03-16 17:05:14.620000');
INSERT INTO use VALUES('relationship--ed283e07-a029-4d23-aa8f-55f92abb5203','intrusion-set--0bbdf25b-30ff-4894-a1cd-49260d0dd2d9','group','attack-pattern--09a60ea3-a8d1-4ae5-976e-5783248b72a4','technique','[APT3](https://attack.mitre.org/groups/G0022) has used a keylogging tool that records keystrokes in encrypted files.(Citation: Symantec Buckeye)','2017-05-31 21:33:27.057000','2020-03-16 20:00:05.044000');
INSERT INTO use VALUES('relationship--e9011839-ca57-434d-a0cc-007594247110','malware--196f1f32-e0c2-4d46-99cd-234d4b6befe1','software','attack-pattern--e6919abc-99f9-4c6c-95a5-14761e7b2add','technique','[Felismus](https://attack.mitre.org/software/S0171) can download files from remote servers.(Citation: Forcepoint Felismus Mar 2017)','2018-01-16 16:13:52.465000','2020-03-17 01:16:15.749000');
INSERT INTO use VALUES('relationship--639e7b8d-57d7-4c1d-8f42-1496606ea666','intrusion-set--fbd29c89-18ba-4c2d-b792-51c0adee049f','group','attack-pattern--2b742742-28c3-4e1b-bab7-8350d6300fa7','technique','[APT33](https://attack.mitre.org/groups/G0064) has sent spearphishing emails containing links to .hta files.(Citation: FireEye APT33 Sept 2017)(Citation: Symantec Elfin Mar 2019)','2018-04-18 17:59:24.739000','2019-06-28 15:05:33.081000');
INSERT INTO use VALUES('relationship--4fe3f0a3-1330-4b51-be17-b38a54b6e605','intrusion-set--7113eaa5-ba79-4fb3-b68a-398ee9cd698e','group','malware--94379dec-5c87-49db-b36e-66abc0b81344','software','(Citation: FireEye Periscope March 2018)','2018-04-18 17:59:24.739000','2019-04-25 12:24:57.475000');
INSERT INTO use VALUES('relationship--b033e131-e448-46c6-815b-b86e4bd6d638','intrusion-set--fe8796a4-2a02-41a0-9d27-7aa1e995feb6','group','attack-pattern--232b7f21-adf9-4b42-b936-b9d6f7df856e','technique','[APT19](https://attack.mitre.org/groups/G0073) attempted to get users to launch malicious attachments delivered via spearphishing emails.(Citation: FireEye APT19)','2018-10-17 00:14:20.652000','2020-03-12 00:28:05.750000');
INSERT INTO use VALUES('relationship--78865095-f63e-461c-9e32-e202d514747d','intrusion-set--dc6fe6ee-04c2-49be-ba3d-f38d2463c02a','group','attack-pattern--eb062747-2193-45de-8fa2-e62549c37ddf','technique','[Cobalt Group](https://attack.mitre.org/groups/G0080) has used Remote Desktop Protocol to conduct lateral movement.(Citation: Group IB Cobalt Aug 2017)','2018-10-17 00:14:20.652000','2019-07-26 23:38:33.260000');
INSERT INTO use VALUES('relationship--09c957e3-f89b-4b41-93af-bde08f00ce36','intrusion-set--c93fccb1-e8e8-42cf-ae33-2ad1d183913a','group','malware--069af411-9b24-4e85-b26c-623d035bbe84','software','(Citation: McAfee GhostSecret)','2018-10-17 00:14:20.652000','2019-09-09 19:15:45.694000');
INSERT INTO use VALUES('relationship--93b12e1a-7f21-4fa0-9b2a-c96c7c270625','intrusion-set--fe98767f-9df8-42b9-83c9-004b1dec8647','group','attack-pattern--b17a1a56-e99c-403c-8948-561df0cffe81','technique','[PittyTiger](https://attack.mitre.org/groups/G0011) attempts to obtain legitimate credentials during operations.(Citation: Bizeul 2014)','2017-05-31 21:33:27.046000','2020-03-16 18:54:08.794000');
INSERT INTO use VALUES('relationship--cdf73653-b2d7-422f-b433-b6a428ff12d4','intrusion-set--894aab42-3371-47b1-8859-a4a074c804c8','group','attack-pattern--3c4a2599-71ee-4405-ba1e-0e28414b4bc5','technique','[Stealth Falcon](https://attack.mitre.org/groups/G0038) malware gathers data from the local victim system.(Citation: Citizen Lab Stealth Falcon May 2016)','2017-05-31 21:33:27.074000','2020-03-17 02:38:48.465000');
INSERT INTO use VALUES('relationship--82ce1bf6-b1a1-4a82-9c86-8ff372d6a873','intrusion-set--fd19bd82-1b14-49a1-a176-6cdc46b8a826','group','attack-pattern--b3d682b6-98f2-4fb0-aa3b-b4df007ca70a','technique','[FIN8](https://attack.mitre.org/groups/G0061) has used environment variables and standard input (stdin) to obfuscate command-line arguments. [FIN8](https://attack.mitre.org/groups/G0061) also obfuscates malicious macros delivered as payloads.(Citation: FireEye Obfuscation June 2017)(Citation: FireEye Know Your Enemy FIN8 Aug 2016)','2018-04-18 17:59:24.739000','2019-03-22 20:21:57.541000');
INSERT INTO use VALUES('relationship--720c211e-2219-496d-8a34-c3f37dfbe5bf','intrusion-set--bef4c620-0787-42a8-a96d-b7eb6e85917c','group','malware--e669bb87-f773-4c7b-bfcc-a9ffebfdd8d4','software','(Citation: ESET Sednit Part 3)','2017-05-31 21:33:27.041000','2019-09-09 17:44:35.686000');
INSERT INTO use VALUES('relationship--d164d443-6298-47eb-b767-8f1136f6b6b5','intrusion-set--76d59913-1d24-4992-a8ac-05a3eb093f71','group','attack-pattern--7385dfaf-6886-4229-9ecd-6fd678040830','technique','[Dragonfly 2.0](https://attack.mitre.org/groups/G0074) used command line for execution.(Citation: US-CERT TA18-074A)','2018-10-17 00:14:20.652000','2019-03-22 20:13:49.301000');
INSERT INTO use VALUES('relationship--08d91d3c-b7c7-4cbc-a4eb-29edd3be3e3a','intrusion-set--f047ee18-7985-4946-8bfb-4ed754d3a0dd','group','malware--b1de6916-7a22-4460-8d26-6b5483ffaa2a','software','(Citation: FireEye APT30)','2017-05-31 21:33:27.048000','2019-03-22 18:44:28.612000');
INSERT INTO use VALUES('relationship--4fab8d06-e6fb-472f-91ee-f2fd29ef444e','intrusion-set--a653431d-6a5e-4600-8ad3-609b5af57064','group','attack-pattern--b97f1d35-4249-4486-a6b5-ee60ccf24fab','technique','[Deep Panda](https://attack.mitre.org/groups/G0009) has used regsvr32.exe to execute a server variant of [Derusbi](https://attack.mitre.org/software/S0021) in victim networks.(Citation: RSA Shell Crew)','2017-05-31 21:33:27.043000','2020-04-17 21:11:30.416000');
INSERT INTO use VALUES('relationship--7af4c9fb-72c2-437e-b0f2-caf3f07d493b','intrusion-set--76d59913-1d24-4992-a8ac-05a3eb093f71','group','attack-pattern--355be19c-ffc9-46d5-8d50-d6a036c675b6','technique','[Dragonfly 2.0](https://attack.mitre.org/groups/G0074) used SMB for C2.(Citation: US-CERT TA18-074A)','2018-10-17 00:14:20.652000','2019-03-22 20:13:49.303000');
INSERT INTO use VALUES('relationship--a9b92f28-c273-48cf-8ee6-dfa6c240cda9','intrusion-set--4a2ce82e-1a74-468a-a6fb-bbead541383c','group','attack-pattern--8f4a33ec-8b1f-4b80-a2f6-642b2e479580','technique','[APT37](https://attack.mitre.org/groups/G0067)''s Freenki malware lists running processes using the Microsoft Windows API.(Citation: Talos Group123)','2018-10-17 00:14:20.652000','2019-09-09 19:12:32.509000');
INSERT INTO use VALUES('relationship--5dbd040e-0433-42ca-8e03-b3741be31cb2','intrusion-set--dc6fe6ee-04c2-49be-ba3d-f38d2463c02a','group','attack-pattern--4cbc6a62-9e34-4f94-8a19-5c1a11392a49','technique','[Cobalt Group](https://attack.mitre.org/groups/G0080) has used the command <code>cmstp.exe /s /ns C:\Users\ADMINI~W\AppData\Local\Temp\XKNqbpzl.txt</code> to bypass AppLocker and launch a malicious script.(Citation: Talos Cobalt Group July 2018)(Citation: Morphisec Cobalt Gang Oct 2018)(Citation: Unit 42 Cobalt Gang Oct 2018)','2018-10-17 00:14:20.652000','2019-07-26 23:38:33.286000');
INSERT INTO use VALUES('relationship--21ff06b5-022f-40bf-821b-3e08dc9f08a3','intrusion-set--7ecc3b4f-5cdb-457e-b55a-df376b359446','group','attack-pattern--7e150503-88e7-4861-866b-ff1ac82c4475','technique','[Poseidon Group](https://attack.mitre.org/groups/G0033) obtains and saves information about victim network interfaces and addresses.(Citation: Kaspersky Poseidon Group)','2017-05-31 21:33:27.070000','2020-03-18 15:34:54.747000');
INSERT INTO use VALUES('relationship--42897880-fe55-4f54-a42c-f85ba19fb39a','intrusion-set--93f52415-0fe4-4d3d-896c-fc9b8e88ab90','group','tool--bba595da-b73a-4354-aa6c-224d4de7cb4e','software','(Citation: Secureworks BRONZE BUTLER Oct 2017)','2018-01-16 16:13:52.465000','2019-03-22 19:57:37.438000');
INSERT INTO use VALUES('relationship--c87a8320-8705-4de3-93ee-2db5c00ea461','intrusion-set--93f52415-0fe4-4d3d-896c-fc9b8e88ab90','group','attack-pattern--232b7f21-adf9-4b42-b936-b9d6f7df856e','technique','[BRONZE BUTLER](https://attack.mitre.org/groups/G0060) has attempted to get users to launch malicious Microsoft Word attachments delivered via spearphishing emails.(Citation: Symantec Tick Apr 2016)(Citation: Trend Micro Tick November 2019)','2018-10-17 00:14:20.652000','2020-06-24 01:27:31.920000');
INSERT INTO use VALUES('relationship--a9c1c589-b5c6-4231-982f-cae0aa41f349','malware--b8fdef82-d2cf-4948-8949-6466357b1be1','software','attack-pattern--d63a3fb8-9452-4e9d-a60a-54be68d5998c','technique','[Calisto](https://attack.mitre.org/software/S0274) has the capability to use <code>rm -rf</code> to remove folders and files from the victim''s machine.(Citation: Securelist Calisto July 2018)','2018-10-17 00:14:20.652000','2020-01-17 19:33:17.969000');
INSERT INTO use VALUES('relationship--592324b4-064a-430c-8ffc-7f7acd537778','malware--d186c1d6-e3ac-4c3d-a534-9ddfeb8c57bb','software','attack-pattern--045d0922-2310-4e60-b5e4-3302302cb3c5','technique','[DDKONG](https://attack.mitre.org/software/S0255) uses Rundll32 to ensure only a single instance of itself is running at once.(Citation: Rancor Unit42 June 2018)','2018-10-17 00:14:20.652000','2018-10-17 00:14:20.652000');
INSERT INTO use VALUES('relationship--145634d0-cd6c-4184-965a-fe77a158ff97','intrusion-set--ebb73863-fa44-4617-b4cb-b9ed3414eb87','group','attack-pattern--120d5519-3098-4e1c-9191-2aa61232f073','technique','[Honeybee](https://attack.mitre.org/groups/G0072) uses a combination of NTWDBLIB.dll and cliconfg.exe to bypass UAC protections using DLL hijacking.(Citation: McAfee Honeybee)','2018-10-17 00:14:20.652000','2020-04-16 19:41:40.396000');
INSERT INTO use VALUES('relationship--f67c1df3-9a03-456e-9de9-74a2ac3a2061','intrusion-set--76d59913-1d24-4992-a8ac-05a3eb093f71','group','attack-pattern--635cbe30-392d-4e27-978e-66774357c762','technique','[Dragonfly 2.0](https://attack.mitre.org/groups/G0074) created accounts on victims, including administrator accounts, some of which appeared to be tailored to each individual staging target.(Citation: US-CERT TA18-074A)(Citation: US-CERT APT Energy Oct 2017)','2018-10-17 00:14:20.652000','2020-02-11 15:53:07.865000');
INSERT INTO use VALUES('relationship--48042284-2fde-43f0-a3dc-f64e9f16bd77','intrusion-set--0bbdf25b-30ff-4894-a1cd-49260d0dd2d9','group','attack-pattern--707399d6-ab3e-4963-9315-d9d3818cd6a0','technique','A keylogging tool used by [APT3](https://attack.mitre.org/groups/G0022) gathers network information from the victim, including the MAC address, IP address, WINS, DHCP server, and gateway.(Citation: Symantec Buckeye)(Citation: evolution of pirpi)','2017-05-31 21:33:27.057000','2019-04-29 18:01:20.204000');
CREATE TABLE tactic (
	id VARCHAR NOT NULL,
	name VARCHAR NOT NULL,
	description VARCHAR,
	created_time DATETIME,
	modified_time DATETIME,
	short_name VARCHAR,
	PRIMARY KEY (id)
);
INSERT INTO tactic VALUES('x-mitre-tactic--d108ce10-2419-4cf9-a774-46161d6c6cfe','Collection',replace('The adversary is trying to gather data of interest to their goal.\n\nCollection consists of techniques adversaries may use to gather information and the sources information is collected from that are relevant to following through on the adversary''s objectives. Frequently, the next goal after collecting data is to steal (exfiltrate) the data. Common target sources include various drive types, browsers, audio, video, and email. Common collection methods include capturing screenshots and keyboard input.','\n',char(10)),'2018-10-17 00:14:20.652000','2019-07-19 17:44:53.176000','collection');
INSERT INTO tactic VALUES('x-mitre-tactic--f72804c5-f15a-449e-a5da-2eecd181f813','Command and Control',replace('The adversary is trying to communicate with compromised systems to control them.\n\nCommand and Control consists of techniques that adversaries may use to communicate with systems under their control within a victim network. Adversaries commonly attempt to mimic normal, expected traffic to avoid detection. There are many ways an adversary can establish command and control with various levels of stealth depending on the victim’s network structure and defenses.','\n',char(10)),'2018-10-17 00:14:20.652000','2019-07-19 17:45:30.644000','command-and-control');
INSERT INTO tactic VALUES('x-mitre-tactic--2558fd61-8c75-4730-94c4-11926db2a263','Credential Access',replace('The adversary is trying to steal account names and passwords.\n\nCredential Access consists of techniques for stealing credentials like account names and passwords. Techniques used to get credentials include keylogging or credential dumping. Using legitimate credentials can give adversaries access to systems, make them harder to detect, and provide the opportunity to create more accounts to help achieve their goals.','\n',char(10)),'2018-10-17 00:14:20.652000','2019-07-19 17:43:41.967000','credential-access');
INSERT INTO tactic VALUES('x-mitre-tactic--78b23412-0651-46d7-a540-170a1ce8bd5a','Defense Evasion',replace('The adversary is trying to avoid being detected.\n\nDefense Evasion consists of techniques that adversaries use to avoid detection throughout their compromise. Techniques used for defense evasion include uninstalling/disabling security software or obfuscating/encrypting data and scripts. Adversaries also leverage and abuse trusted processes to hide and masquerade their malware. Other tactics’ techniques are cross-listed here when those techniques include the added benefit of subverting defenses. ','\n',char(10)),'2018-10-17 00:14:20.652000','2019-07-19 17:43:23.473000','defense-evasion');
INSERT INTO tactic VALUES('x-mitre-tactic--c17c5845-175e-4421-9713-829d0573dbc9','Discovery',replace('The adversary is trying to figure out your environment.\n\nDiscovery consists of techniques an adversary may use to gain knowledge about the system and internal network. These techniques help adversaries observe the environment and orient themselves before deciding how to act. They also allow adversaries to explore what they can control and what’s around their entry point in order to discover how it could benefit their current objective. Native operating system tools are often used toward this post-compromise information-gathering objective. ','\n',char(10)),'2018-10-17 00:14:20.652000','2019-07-19 17:44:13.228000','discovery');
INSERT INTO tactic VALUES('x-mitre-tactic--4ca45d45-df4d-4613-8980-bac22d278fa5','Execution',replace('The adversary is trying to run malicious code.\n\nExecution consists of techniques that result in adversary-controlled code running on a local or remote system. Techniques that run malicious code are often paired with techniques from all other tactics to achieve broader goals, like exploring a network or stealing data. For example, an adversary might use a remote access tool to run a PowerShell script that does Remote System Discovery. ','\n',char(10)),'2018-10-17 00:14:20.652000','2019-07-19 17:42:06.909000','execution');
INSERT INTO tactic VALUES('x-mitre-tactic--9a4e74ab-5008-408c-84bf-a10dfbc53462','Exfiltration',replace('The adversary is trying to steal data.\n\nExfiltration consists of techniques that adversaries may use to steal data from your network. Once they’ve collected data, adversaries often package it to avoid detection while removing it. This can include compression and encryption. Techniques for getting data out of a target network typically include transferring it over their command and control channel or an alternate channel and may also include putting size limits on the transmission.','\n',char(10)),'2018-10-17 00:14:20.652000','2019-07-19 17:45:12.806000','exfiltration');
INSERT INTO tactic VALUES('x-mitre-tactic--5569339b-94c2-49ee-afb3-2222936582c8','Impact',replace('The adversary is trying to manipulate, interrupt, or destroy your systems and data.\n \nImpact consists of techniques that adversaries use to disrupt availability or compromise integrity by manipulating business and operational processes. Techniques used for impact can include destroying or tampering with data. In some cases, business processes can look fine, but may have been altered to benefit the adversaries’ goals. These techniques might be used by adversaries to follow through on their end goal or to provide cover for a confidentiality breach.','\n',char(10)),'2019-03-14 18:44:44.639000','2019-07-25 18:42:23.222000','impact');
INSERT INTO tactic VALUES('x-mitre-tactic--ffd5bcee-6e16-4dd2-8eca-7b3beedf33ca','Initial Access',replace('The adversary is trying to get into your network.\n\nInitial Access consists of techniques that use various entry vectors to gain their initial foothold within a network. Techniques used to gain a foothold include targeted spearphishing and exploiting weaknesses on public-facing web servers. Footholds gained through initial access may allow for continued access, like valid accounts and use of external remote services, or may be limited-use due to changing passwords.','\n',char(10)),'2018-10-17 00:14:20.652000','2019-07-19 17:41:41.425000','initial-access');
INSERT INTO tactic VALUES('x-mitre-tactic--7141578b-e50b-4dcc-bfa4-08a8dd689e9e','Lateral Movement',replace('The adversary is trying to move through your environment.\n\nLateral Movement consists of techniques that adversaries use to enter and control remote systems on a network. Following through on their primary objective often requires exploring the network to find their target and subsequently gaining access to it. Reaching their objective often involves pivoting through multiple systems and accounts to gain. Adversaries might install their own remote access tools to accomplish Lateral Movement or use legitimate credentials with native network and operating system tools, which may be stealthier. ','\n',char(10)),'2018-10-17 00:14:20.652000','2019-07-19 17:44:36.953000','lateral-movement');
INSERT INTO tactic VALUES('x-mitre-tactic--5bc1d813-693e-4823-9961-abf9af4b0e92','Persistence',replace('The adversary is trying to maintain their foothold.\n\nPersistence consists of techniques that adversaries use to keep access to systems across restarts, changed credentials, and other interruptions that could cut off their access. Techniques used for persistence include any access, action, or configuration changes that let them maintain their foothold on systems, such as replacing or hijacking legitimate code or adding startup code. ','\n',char(10)),'2018-10-17 00:14:20.652000','2019-07-19 17:42:33.899000','persistence');
INSERT INTO tactic VALUES('x-mitre-tactic--5e29b093-294e-49e9-a803-dab3d73b77dd','Privilege Escalation',replace('The adversary is trying to gain higher-level permissions.\n\nPrivilege Escalation consists of techniques that adversaries use to gain higher-level permissions on a system or network. Adversaries can often enter and explore a network with unprivileged access but require elevated permissions to follow through on their objectives. Common approaches are to take advantage of system weaknesses, misconfigurations, and vulnerabilities. Examples of elevated access include: \n•	SYSTEM/root level\n•	local administrator\n•	user account with admin-like access \n•	user accounts with access to specific system or perform specific function\nThese techniques often overlap with Persistence techniques, as OS features that let an adversary persist can execute in an elevated context.  ','\n',char(10)),'2018-10-17 00:14:20.652000','2019-07-19 17:43:00.594000','privilege-escalation');
INSERT INTO tactic VALUES('x-mitre-tactic--daa4cbb1-b4f4-4723-a824-7f1efd6e0592','Reconnaissance',replace('The adversary is trying to gather information they can use to plan future operations.\n\nReconnaissance consists of techniques that involve adversaries actively or passively gathering information that can be used to support targeting. Such information may include details of the victim organization, infrastructure, or staff/personnel. This information can be leveraged by the adversary to aid in other phases of the adversary lifecycle, such as using gathered information to plan and execute Initial Access, to scope and prioritize post-compromise objectives, or to drive and lead further Reconnaissance efforts.','\n',char(10)),'2020-10-02 14:48:41.809000','2020-10-18 02:04:50.842000','reconnaissance');
INSERT INTO tactic VALUES('x-mitre-tactic--d679bca2-e57d-4935-8650-8031c87a4400','Resource Development',replace('The adversary is trying to establish resources they can use to support operations.\n\nResource Development consists of techniques that involve adversaries creating, purchasing, or compromising/stealing resources that can be used to support targeting. Such resources include infrastructure, accounts, or capabilities. These resources can be leveraged by the adversary to aid in other phases of the adversary lifecycle, such as using purchased domains to support Command and Control, email accounts for phishing as a part of Initial Access, or stealing code signing certificates to help with Defense Evasion.','\n',char(10)),'2020-09-30 16:11:59.650000','2020-09-30 16:31:36.322000','resource-development');
CREATE TABLE data_source (
	id VARCHAR NOT NULL,
	source VARCHAR NOT NULL,
	PRIMARY KEY (id, source),
	FOREIGN KEY(id) REFERENCES technique (id) ON DELETE CASCADE
);
INSERT INTO data_source VALUES('attack-pattern--b63a34e8-0a61-4c97-a23b-bf8a2ed812e2','File monitoring');
INSERT INTO data_source VALUES('attack-pattern--b63a34e8-0a61-4c97-a23b-bf8a2ed812e2','Process monitoring');
INSERT INTO data_source VALUES('attack-pattern--b63a34e8-0a61-4c97-a23b-bf8a2ed812e2','Process command-line parameters');
INSERT INTO data_source VALUES('attack-pattern--b63a34e8-0a61-4c97-a23b-bf8a2ed812e2','Process use of network');
INSERT INTO data_source VALUES('attack-pattern--cabe189c-a0e3-4965-a473-dcff00f17213','Packet capture');
INSERT INTO data_source VALUES('attack-pattern--cabe189c-a0e3-4965-a473-dcff00f17213','Netflow/Enclave netflow');
INSERT INTO data_source VALUES('attack-pattern--3986e7fd-a8e9-4ecb-bfc6-55920855912b','Authentication logs');
INSERT INTO data_source VALUES('attack-pattern--3986e7fd-a8e9-4ecb-bfc6-55920855912b','Windows event logs');
INSERT INTO data_source VALUES('attack-pattern--67720091-eee3-4d2d-ae16-8264567f6f5b','Process command-line parameters');
INSERT INTO data_source VALUES('attack-pattern--67720091-eee3-4d2d-ae16-8264567f6f5b','API monitoring');
INSERT INTO data_source VALUES('attack-pattern--67720091-eee3-4d2d-ae16-8264567f6f5b','File monitoring');
INSERT INTO data_source VALUES('attack-pattern--67720091-eee3-4d2d-ae16-8264567f6f5b','Process monitoring');
INSERT INTO data_source VALUES('attack-pattern--67720091-eee3-4d2d-ae16-8264567f6f5b','Windows Registry');
INSERT INTO data_source VALUES('attack-pattern--dcaa092b-7de9-4a21-977f-7fcb77e89c48','Process command-line parameters');
INSERT INTO data_source VALUES('attack-pattern--dcaa092b-7de9-4a21-977f-7fcb77e89c48','API monitoring');
INSERT INTO data_source VALUES('attack-pattern--dcaa092b-7de9-4a21-977f-7fcb77e89c48','Authentication logs');
INSERT INTO data_source VALUES('attack-pattern--dcaa092b-7de9-4a21-977f-7fcb77e89c48','Windows event logs');
INSERT INTO data_source VALUES('attack-pattern--dcaa092b-7de9-4a21-977f-7fcb77e89c48','Process monitoring');
INSERT INTO data_source VALUES('attack-pattern--dcaa092b-7de9-4a21-977f-7fcb77e89c48','Access tokens');
INSERT INTO data_source VALUES('attack-pattern--70e52b04-2a0c-4cea-9d18-7149f1df9dc5','File monitoring');
INSERT INTO data_source VALUES('attack-pattern--70e52b04-2a0c-4cea-9d18-7149f1df9dc5','Windows Registry');
INSERT INTO data_source VALUES('attack-pattern--70e52b04-2a0c-4cea-9d18-7149f1df9dc5','Process command-line parameters');
INSERT INTO data_source VALUES('attack-pattern--70e52b04-2a0c-4cea-9d18-7149f1df9dc5','Process monitoring');
INSERT INTO data_source VALUES('attack-pattern--b24e2a20-3b3d-4bf0-823b-1ed765398fb0','Process command-line parameters');
INSERT INTO data_source VALUES('attack-pattern--b24e2a20-3b3d-4bf0-823b-1ed765398fb0','Process monitoring');
INSERT INTO data_source VALUES('attack-pattern--b24e2a20-3b3d-4bf0-823b-1ed765398fb0','Windows event logs');
INSERT INTO data_source VALUES('attack-pattern--72b74d71-8169-42aa-92e0-e7b04b9f5a08','Process command-line parameters');
INSERT INTO data_source VALUES('attack-pattern--72b74d71-8169-42aa-92e0-e7b04b9f5a08','API monitoring');
INSERT INTO data_source VALUES('attack-pattern--72b74d71-8169-42aa-92e0-e7b04b9f5a08','Azure activity logs');
INSERT INTO data_source VALUES('attack-pattern--72b74d71-8169-42aa-92e0-e7b04b9f5a08','Office 365 account logs');
INSERT INTO data_source VALUES('attack-pattern--72b74d71-8169-42aa-92e0-e7b04b9f5a08','Process monitoring');
INSERT INTO data_source VALUES('attack-pattern--a10641f4-87b4-45a3-a906-92a149cb2c27','Authentication logs');
INSERT INTO data_source VALUES('attack-pattern--a10641f4-87b4-45a3-a906-92a149cb2c27','Windows event logs');
INSERT INTO data_source VALUES('attack-pattern--67073dde-d720-45ae-83da-b12d5e73ca3b','Packet capture');
INSERT INTO data_source VALUES('attack-pattern--67073dde-d720-45ae-83da-b12d5e73ca3b','Network device logs');
INSERT INTO data_source VALUES('attack-pattern--2dbbdcd5-92cf-44c0-aea2-fe24783a6bc3','Office 365 audit logs');
INSERT INTO data_source VALUES('attack-pattern--34f1d81d-fe88-4f97-bd3b-a3164536255d','Windows Registry');
INSERT INTO data_source VALUES('attack-pattern--34f1d81d-fe88-4f97-bd3b-a3164536255d','Process command-line parameters');
INSERT INTO data_source VALUES('attack-pattern--34f1d81d-fe88-4f97-bd3b-a3164536255d','Process monitoring');
INSERT INTO data_source VALUES('attack-pattern--34f1d81d-fe88-4f97-bd3b-a3164536255d','File monitoring');
INSERT INTO data_source VALUES('attack-pattern--7d57b371-10c2-45e5-b3cc-83a8fb380e4c','Process monitoring');
INSERT INTO data_source VALUES('attack-pattern--7d57b371-10c2-45e5-b3cc-83a8fb380e4c','Process command-line parameters');
INSERT INTO data_source VALUES('attack-pattern--7d57b371-10c2-45e5-b3cc-83a8fb380e4c','Loaded DLLs');
INSERT INTO data_source VALUES('attack-pattern--7d57b371-10c2-45e5-b3cc-83a8fb380e4c','Windows Registry');
INSERT INTO data_source VALUES('attack-pattern--cc89ecbd-3d33-4a41-bcca-001e702d18fd','Process monitoring');
INSERT INTO data_source VALUES('attack-pattern--cc89ecbd-3d33-4a41-bcca-001e702d18fd','Process command-line parameters');
INSERT INTO data_source VALUES('attack-pattern--cc89ecbd-3d33-4a41-bcca-001e702d18fd','Loaded DLLs');
INSERT INTO data_source VALUES('attack-pattern--cc89ecbd-3d33-4a41-bcca-001e702d18fd','Windows Registry');
INSERT INTO data_source VALUES('attack-pattern--37b11151-1776-4f8f-b328-30939fbf2ceb','Process command-line parameters');
INSERT INTO data_source VALUES('attack-pattern--37b11151-1776-4f8f-b328-30939fbf2ceb','API monitoring');
CREATE TABLE defense_bypassed (
	id VARCHAR NOT NULL,
	defense VARCHAR NOT NULL,
	PRIMARY KEY (id, defense),
	FOREIGN KEY(id) REFERENCES technique (id) ON DELETE CASCADE
);
INSERT INTO defense_bypassed VALUES('attack-pattern--dcaa092b-7de9-4a21-977f-7fcb77e89c48','File system access controls');
INSERT INTO defense_bypassed VALUES('attack-pattern--dcaa092b-7de9-4a21-977f-7fcb77e89c48','Host forensic analysis');
INSERT INTO defense_bypassed VALUES('attack-pattern--dcaa092b-7de9-4a21-977f-7fcb77e89c48','Heuristic Detection');
INSERT INTO defense_bypassed VALUES('attack-pattern--dcaa092b-7de9-4a21-977f-7fcb77e89c48','System access controls');
INSERT INTO defense_bypassed VALUES('attack-pattern--dcaa092b-7de9-4a21-977f-7fcb77e89c48','Windows User Account Control');
INSERT INTO defense_bypassed VALUES('attack-pattern--f005e783-57d4-4837-88ad-dbe7faee1c51','System Access Controls');
INSERT INTO defense_bypassed VALUES('attack-pattern--7c0f17c9-1af6-4628-9cbd-9e45482dd605','Application control');
INSERT INTO defense_bypassed VALUES('attack-pattern--7c0f17c9-1af6-4628-9cbd-9e45482dd605','Anti-virus');
INSERT INTO defense_bypassed VALUES('attack-pattern--c8e87b83-edbb-48d4-9295-4974897525b7','Host forensic analysis');
INSERT INTO defense_bypassed VALUES('attack-pattern--c8e87b83-edbb-48d4-9295-4974897525b7','Firewall');
INSERT INTO defense_bypassed VALUES('attack-pattern--5bfccc3f-2326-4112-86cc-c1ece9d8a2b5','Signature-based detection');
INSERT INTO defense_bypassed VALUES('attack-pattern--5bfccc3f-2326-4112-86cc-c1ece9d8a2b5','Anti-virus');
INSERT INTO defense_bypassed VALUES('attack-pattern--1b7b1806-7746-41a1-a35d-e48dae25ddba','Anti-virus');
INSERT INTO defense_bypassed VALUES('attack-pattern--1b7b1806-7746-41a1-a35d-e48dae25ddba','Host intrusion prevention systems');
INSERT INTO defense_bypassed VALUES('attack-pattern--1b7b1806-7746-41a1-a35d-e48dae25ddba','File monitoring');
INSERT INTO defense_bypassed VALUES('attack-pattern--120d5519-3098-4e1c-9191-2aa61232f073','Windows User Account Control');
INSERT INTO defense_bypassed VALUES('attack-pattern--4cbc6a62-9e34-4f94-8a19-5c1a11392a49','Application control');
INSERT INTO defense_bypassed VALUES('attack-pattern--4cbc6a62-9e34-4f94-8a19-5c1a11392a49','Anti-virus');
INSERT INTO defense_bypassed VALUES('attack-pattern--3aef9463-9a7a-43ba-8957-a867e07c1e6a','Host forensic analysis');
INSERT INTO defense_bypassed VALUES('attack-pattern--3aef9463-9a7a-43ba-8957-a867e07c1e6a','Log analysis');
INSERT INTO defense_bypassed VALUES('attack-pattern--6495ae23-3ab4-43c5-a94f-5638a2c31fd2','Anti Virus');
INSERT INTO defense_bypassed VALUES('attack-pattern--6495ae23-3ab4-43c5-a94f-5638a2c31fd2','Log Analysis');
INSERT INTO defense_bypassed VALUES('attack-pattern--6495ae23-3ab4-43c5-a94f-5638a2c31fd2','Host Intrusion Prevention Systems');
INSERT INTO defense_bypassed VALUES('attack-pattern--32901740-b42c-4fdd-bc02-345b5dc57082','Windows User Account Control');
INSERT INTO defense_bypassed VALUES('attack-pattern--c726e0a2-a57a-4b7b-a973-d0f013246617','Anti-virus');
INSERT INTO defense_bypassed VALUES('attack-pattern--c726e0a2-a57a-4b7b-a973-d0f013246617','Host intrusion prevention systems');
INSERT INTO defense_bypassed VALUES('attack-pattern--c726e0a2-a57a-4b7b-a973-d0f013246617','Signature-based detection');
INSERT INTO defense_bypassed VALUES('attack-pattern--c726e0a2-a57a-4b7b-a973-d0f013246617','Static File Analysis');
INSERT INTO defense_bypassed VALUES('attack-pattern--c726e0a2-a57a-4b7b-a973-d0f013246617','Binary Analysis');
INSERT INTO defense_bypassed VALUES('attack-pattern--a6937325-9321-4e2e-bb2b-3ed2d40b2a9d','Application control');
INSERT INTO defense_bypassed VALUES('attack-pattern--a6937325-9321-4e2e-bb2b-3ed2d40b2a9d','Digital Certificate Validation');
INSERT INTO defense_bypassed VALUES('attack-pattern--791481f8-e96a-41be-b089-a088763083d4','Anti-virus');
INSERT INTO defense_bypassed VALUES('attack-pattern--791481f8-e96a-41be-b089-a088763083d4','Host intrusion prevention systems');
INSERT INTO defense_bypassed VALUES('attack-pattern--791481f8-e96a-41be-b089-a088763083d4','File monitoring');
INSERT INTO defense_bypassed VALUES('attack-pattern--4ff5d6a8-c062-4c68-a778-36fc5edd564f','Application control');
INSERT INTO defense_bypassed VALUES('attack-pattern--677569f9-a8b0-459e-ab24-7f18091fa7bf','Windows User Account Control');
INSERT INTO defense_bypassed VALUES('attack-pattern--677569f9-a8b0-459e-ab24-7f18091fa7bf','File system access controls');
INSERT INTO defense_bypassed VALUES('attack-pattern--677569f9-a8b0-459e-ab24-7f18091fa7bf','System access controls');
INSERT INTO defense_bypassed VALUES('attack-pattern--e64c62cf-9cd7-4a14-94ec-cdaac43ab44b','Application control');
INSERT INTO defense_bypassed VALUES('attack-pattern--e64c62cf-9cd7-4a14-94ec-cdaac43ab44b','Anti-virus');
INSERT INTO defense_bypassed VALUES('attack-pattern--3ccef7ae-cb5e-48f6-8302-897105fbf55c','Signature-based detection');
INSERT INTO defense_bypassed VALUES('attack-pattern--3ccef7ae-cb5e-48f6-8302-897105fbf55c','Anti-virus');
INSERT INTO defense_bypassed VALUES('attack-pattern--3ccef7ae-cb5e-48f6-8302-897105fbf55c','Host intrusion prevention systems');
INSERT INTO defense_bypassed VALUES('attack-pattern--3ccef7ae-cb5e-48f6-8302-897105fbf55c','Network intrusion detection system');
INSERT INTO defense_bypassed VALUES('attack-pattern--0c8ab3eb-df48-4b9c-ace7-beacaac81cc5','File system access controls');
INSERT INTO defense_bypassed VALUES('attack-pattern--0c8ab3eb-df48-4b9c-ace7-beacaac81cc5','File monitoring');
INSERT INTO defense_bypassed VALUES('attack-pattern--4eb28bed-d11a-4641-9863-c2ac017d910a','Log analysis');
INSERT INTO defense_bypassed VALUES('attack-pattern--5372c5fe-f424-4def-bcd5-d3a8e770f07b','Firewall');
INSERT INTO defense_bypassed VALUES('attack-pattern--ac08589e-ee59-4935-8667-d845e38fe579','Anti-virus');
INSERT INTO defense_bypassed VALUES('attack-pattern--ac08589e-ee59-4935-8667-d845e38fe579','Host intrusion prevention systems');
CREATE TABLE effective_permission (
	id VARCHAR NOT NULL,
	permission VARCHAR NOT NULL,
	PRIMARY KEY (id, permission),
	FOREIGN KEY(id) REFERENCES technique (id) ON DELETE CASCADE
);
INSERT INTO effective_permission VALUES('attack-pattern--dcaa092b-7de9-4a21-977f-7fcb77e89c48','SYSTEM');
INSERT INTO effective_permission VALUES('attack-pattern--70e52b04-2a0c-4cea-9d18-7149f1df9dc5','SYSTEM');
INSERT INTO effective_permission VALUES('attack-pattern--7d57b371-10c2-45e5-b3cc-83a8fb380e4c','Administrator');
INSERT INTO effective_permission VALUES('attack-pattern--7d57b371-10c2-45e5-b3cc-83a8fb380e4c','SYSTEM');
INSERT INTO effective_permission VALUES('attack-pattern--cc89ecbd-3d33-4a41-bcca-001e702d18fd','Administrator');
INSERT INTO effective_permission VALUES('attack-pattern--cc89ecbd-3d33-4a41-bcca-001e702d18fd','SYSTEM');
INSERT INTO effective_permission VALUES('attack-pattern--120d5519-3098-4e1c-9191-2aa61232f073','Administrator');
INSERT INTO effective_permission VALUES('attack-pattern--b84903f0-c7d5-435d-a69e-de47cc3578c0','root');
INSERT INTO effective_permission VALUES('attack-pattern--70d81154-b187-45f9-8ec5-295d01255979','Administrator');
INSERT INTO effective_permission VALUES('attack-pattern--70d81154-b187-45f9-8ec5-295d01255979','SYSTEM');
INSERT INTO effective_permission VALUES('attack-pattern--70d81154-b187-45f9-8ec5-295d01255979','User');
INSERT INTO effective_permission VALUES('attack-pattern--b21c3b2d-02e6-45b1-980b-e69051040839','User');
INSERT INTO effective_permission VALUES('attack-pattern--573ad264-1371-4ae0-8482-d2673b719dba','root');
INSERT INTO effective_permission VALUES('attack-pattern--8cdeb020-e31e-4f88-a582-f53dcfbda819','SYSTEM');
INSERT INTO effective_permission VALUES('attack-pattern--c4ad009b-6e13-4419-8d21-918a1652de02','Administrator');
INSERT INTO effective_permission VALUES('attack-pattern--c4ad009b-6e13-4419-8d21-918a1652de02','SYSTEM');
INSERT INTO effective_permission VALUES('attack-pattern--c4ad009b-6e13-4419-8d21-918a1652de02','User');
INSERT INTO effective_permission VALUES('attack-pattern--58af3705-8740-4c68-9329-ec015a7013c2','Administrator');
INSERT INTO effective_permission VALUES('attack-pattern--58af3705-8740-4c68-9329-ec015a7013c2','SYSTEM');
INSERT INTO effective_permission VALUES('attack-pattern--58af3705-8740-4c68-9329-ec015a7013c2','User');
INSERT INTO effective_permission VALUES('attack-pattern--43881e51-ac74-445b-b4c6-f9f9e9bf23fe','SYSTEM');
INSERT INTO effective_permission VALUES('attack-pattern--35dd844a-b219-4e2b-a6bb-efa9a75995a9','Administrator');
INSERT INTO effective_permission VALUES('attack-pattern--35dd844a-b219-4e2b-a6bb-efa9a75995a9','SYSTEM');
INSERT INTO effective_permission VALUES('attack-pattern--35dd844a-b219-4e2b-a6bb-efa9a75995a9','User');
INSERT INTO effective_permission VALUES('attack-pattern--9e8b28c9-35fe-48ac-a14d-e6cc032dcbcd','Administrator');
INSERT INTO effective_permission VALUES('attack-pattern--9e8b28c9-35fe-48ac-a14d-e6cc032dcbcd','SYSTEM');
INSERT INTO effective_permission VALUES('attack-pattern--9e8b28c9-35fe-48ac-a14d-e6cc032dcbcd','User');
INSERT INTO effective_permission VALUES('attack-pattern--17cc750b-e95b-4d7d-9dde-49e0de24148c','SYSTEM');
INSERT INTO effective_permission VALUES('attack-pattern--1365fe3b-0f50-455d-b4da-266ce31c23b0','root');
INSERT INTO effective_permission VALUES('attack-pattern--b17a1a56-e99c-403c-8948-561df0cffe81','Administrator');
INSERT INTO effective_permission VALUES('attack-pattern--b17a1a56-e99c-403c-8948-561df0cffe81','User');
INSERT INTO effective_permission VALUES('attack-pattern--2959d63f-73fd-46a1-abd2-109d7dcede32','Administrator');
INSERT INTO effective_permission VALUES('attack-pattern--2959d63f-73fd-46a1-abd2-109d7dcede32','SYSTEM');
CREATE TABLE impact (
	id VARCHAR NOT NULL,
	impact VARCHAR NOT NULL,
	PRIMARY KEY (id, impact),
	FOREIGN KEY(id) REFERENCES technique (id) ON DELETE CASCADE
);
INSERT INTO impact VALUES('attack-pattern--b24e2a20-3b3d-4bf0-823b-1ed765398fb0','Availability');
INSERT INTO impact VALUES('attack-pattern--18cffc21-3260-437e-80e4-4ab8bf2ba5e9','Availability');
INSERT INTO impact VALUES('attack-pattern--2bee5ffb-7a7a-4119-b1f2-158151b19ac0','Availability');
INSERT INTO impact VALUES('attack-pattern--d45a3d09-b3cf-48f4-9f0f-f521ee5cb05c','Availability');
INSERT INTO impact VALUES('attack-pattern--b80d107d-fa0d-4b60-9684-b0433e8bdba0','Availability');
INSERT INTO impact VALUES('attack-pattern--ac9e6b22-11bf-45d7-9181-c1cb08360931','Integrity');
INSERT INTO impact VALUES('attack-pattern--5909f20f-3c39-4795-be06-ef1ea40d350b','Integrity');
INSERT INTO impact VALUES('attack-pattern--0bda01d5-4c1d-4062-8ee2-6872334383c3','Availability');
INSERT INTO impact VALUES('attack-pattern--fb640c43-aa6b-431e-a961-a279010424ac','Availability');
INSERT INTO impact VALUES('attack-pattern--0af0ca99-357d-4ba1-805f-674fdfb7bef9','Availability');
INSERT INTO impact VALUES('attack-pattern--1988cc35-ced8-4dad-b2d1-7628488fa967','Availability');
INSERT INTO impact VALUES('attack-pattern--c675646d-e204-4aa8-978d-e3d6d65885c4','Availability');
INSERT INTO impact VALUES('attack-pattern--0cfe31a7-81fc-472c-bc45-e2808d1066a3','Integrity');
INSERT INTO impact VALUES('attack-pattern--f5bb433e-bdf6-4781-84bc-35e97e43be89','Availability');
INSERT INTO impact VALUES('attack-pattern--f5d8eed6-48a9-4cdf-a3d7-d1ffa99c3d2a','Availability');
INSERT INTO impact VALUES('attack-pattern--8c41090b-aa47-4331-986b-8c9a51a91103','Integrity');
INSERT INTO impact VALUES('attack-pattern--d74c4a7e-ffbf-432f-9365-7ebf1f787cab','Availability');
INSERT INTO impact VALUES('attack-pattern--0df05477-c572-4ed6-88a9-47c581f548f7','Availability');
INSERT INTO impact VALUES('attack-pattern--36b2a1d7-e09e-49bf-b45e-477076c2ec01','Availability');
INSERT INTO impact VALUES('attack-pattern--cd25c1b4-935c-4f0e-ba8d-552f28bc4783','Availability');
INSERT INTO impact VALUES('attack-pattern--32ad5c86-2bcf-47d8-8fdc-d7f3d79a7490','Integrity');
INSERT INTO impact VALUES('attack-pattern--38eb0c22-6caf-46ce-8869-5964bd735858','Availability');
INSERT INTO impact VALUES('attack-pattern--20fb2507-d71c-455d-9b6d-6104461cf26b','Availability');
INSERT INTO impact VALUES('attack-pattern--1cfcb312-b8d7-47a4-b560-4b16cc677292','Integrity');
INSERT INTO impact VALUES('attack-pattern--ff73aa03-0090-4464-83ac-f89e233c02bc','Availability');
INSERT INTO impact VALUES('attack-pattern--d0613359-5781-4fd2-b5be-c269270be1f6','Integrity');
CREATE TABLE permission (
	id VARCHAR NOT NULL,
	permission VARCHAR NOT NULL,
	PRIMARY KEY (id, permission),
	FOREIGN KEY(id) REFERENCES technique (id) ON DELETE CASCADE
);
INSERT INTO permission VALUES('attack-pattern--b63a34e8-0a61-4c97-a23b-bf8a2ed812e2','Administrator');
INSERT INTO permission VALUES('attack-pattern--b63a34e8-0a61-4c97-a23b-bf8a2ed812e2','User');
INSERT INTO permission VALUES('attack-pattern--d0b4fcdb-d67d-4ed2-99ce-788b12f8c0f4','root');
INSERT INTO permission VALUES('attack-pattern--cabe189c-a0e3-4965-a473-dcff00f17213','User');
INSERT INTO permission VALUES('attack-pattern--3986e7fd-a8e9-4ecb-bfc6-55920855912b','User');
INSERT INTO permission VALUES('attack-pattern--67720091-eee3-4d2d-ae16-8264567f6f5b','Administrator');
INSERT INTO permission VALUES('attack-pattern--67720091-eee3-4d2d-ae16-8264567f6f5b','User');
INSERT INTO permission VALUES('attack-pattern--dcaa092b-7de9-4a21-977f-7fcb77e89c48','Administrator');
INSERT INTO permission VALUES('attack-pattern--dcaa092b-7de9-4a21-977f-7fcb77e89c48','User');
INSERT INTO permission VALUES('attack-pattern--70e52b04-2a0c-4cea-9d18-7149f1df9dc5','Administrator');
INSERT INTO permission VALUES('attack-pattern--b24e2a20-3b3d-4bf0-823b-1ed765398fb0','root');
INSERT INTO permission VALUES('attack-pattern--b24e2a20-3b3d-4bf0-823b-1ed765398fb0','Administrator');
INSERT INTO permission VALUES('attack-pattern--b24e2a20-3b3d-4bf0-823b-1ed765398fb0','SYSTEM');
INSERT INTO permission VALUES('attack-pattern--b24e2a20-3b3d-4bf0-823b-1ed765398fb0','User');
INSERT INTO permission VALUES('attack-pattern--72b74d71-8169-42aa-92e0-e7b04b9f5a08','User');
INSERT INTO permission VALUES('attack-pattern--2dbbdcd5-92cf-44c0-aea2-fe24783a6bc3','Administrator');
INSERT INTO permission VALUES('attack-pattern--34f1d81d-fe88-4f97-bd3b-a3164536255d','Administrator');
INSERT INTO permission VALUES('attack-pattern--34f1d81d-fe88-4f97-bd3b-a3164536255d','User');
INSERT INTO permission VALUES('attack-pattern--7d57b371-10c2-45e5-b3cc-83a8fb380e4c','Administrator');
INSERT INTO permission VALUES('attack-pattern--7d57b371-10c2-45e5-b3cc-83a8fb380e4c','SYSTEM');
INSERT INTO permission VALUES('attack-pattern--cc89ecbd-3d33-4a41-bcca-001e702d18fd','Administrator');
INSERT INTO permission VALUES('attack-pattern--37b11151-1776-4f8f-b328-30939fbf2ceb','User');
INSERT INTO permission VALUES('attack-pattern--42fe883a-21ea-4cfb-b94a-78b6476dcc83','Administrator');
INSERT INTO permission VALUES('attack-pattern--4ae4f953-fe58-4cc8-a327-33257e30a830','User');
INSERT INTO permission VALUES('attack-pattern--f3d95a1f-bba2-44ce-9af7-37866cd63fd0','Administrator');
INSERT INTO permission VALUES('attack-pattern--1035cdf2-3e5f-446f-a7a7-e8f6d7925967','User');
INSERT INTO permission VALUES('attack-pattern--b8cfed42-6a8a-4989-ad72-541af74475ec','Administrator');
INSERT INTO permission VALUES('attack-pattern--30208d3e-0d6b-43c8-883e-44462a514619','User');
INSERT INTO permission VALUES('attack-pattern--c8e87b83-edbb-48d4-9295-4974897525b7','Administrator');
INSERT INTO permission VALUES('attack-pattern--c8e87b83-edbb-48d4-9295-4974897525b7','SYSTEM');
INSERT INTO permission VALUES('attack-pattern--c8e87b83-edbb-48d4-9295-4974897525b7','User');
INSERT INTO permission VALUES('attack-pattern--8187bd2a-866f-4457-9009-86b0ddedffa3','User');
INSERT INTO permission VALUES('attack-pattern--be055942-6e63-49d7-9fa1-9cb7d8a8f3f4','User');
INSERT INTO permission VALUES('attack-pattern--1ecb2399-e8ba-4f6b-8ba7-5c27d49405cf','root');
INSERT INTO permission VALUES('attack-pattern--1ecb2399-e8ba-4f6b-8ba7-5c27d49405cf','Administrator');
INSERT INTO permission VALUES('attack-pattern--1ecb2399-e8ba-4f6b-8ba7-5c27d49405cf','User');
INSERT INTO permission VALUES('attack-pattern--1b7b1806-7746-41a1-a35d-e48dae25ddba','Administrator');
INSERT INTO permission VALUES('attack-pattern--1b7b1806-7746-41a1-a35d-e48dae25ddba','SYSTEM');
INSERT INTO permission VALUES('attack-pattern--5e4a2073-9643-44cb-a0b5-e7f4048446c7','User');
INSERT INTO permission VALUES('attack-pattern--389735f1-f21c-4208-b8f0-f8031e7169b8','User');
INSERT INTO permission VALUES('attack-pattern--a93494bb-4b80-4ea1-8695-3236a49916fd','User');
INSERT INTO permission VALUES('attack-pattern--120d5519-3098-4e1c-9191-2aa61232f073','Administrator');
INSERT INTO permission VALUES('attack-pattern--120d5519-3098-4e1c-9191-2aa61232f073','User');
INSERT INTO permission VALUES('attack-pattern--4cbc6a62-9e34-4f94-8a19-5c1a11392a49','User');
INSERT INTO permission VALUES('attack-pattern--ffeb0780-356e-4261-b036-cfb6bd234335','Administrator');
INSERT INTO permission VALUES('attack-pattern--ffeb0780-356e-4261-b036-cfb6bd234335','User');
INSERT INTO permission VALUES('attack-pattern--6add2ab5-2711-4e9d-87c8-7a0be8531530','SYSTEM');
INSERT INTO permission VALUES('attack-pattern--98034fef-d9fb-4667-8dc4-2eab6231724c','Administrator');
INSERT INTO permission VALUES('attack-pattern--98034fef-d9fb-4667-8dc4-2eab6231724c','SYSTEM');
INSERT INTO permission VALUES('attack-pattern--98034fef-d9fb-4667-8dc4-2eab6231724c','User');
CREATE TABLE system_requirement (
	id VARCHAR NOT NULL,
	requirement VARCHAR NOT NULL,
	PRIMARY KEY (id, requirement),
	FOREIGN KEY(id) REFERENCES technique (id) ON DELETE CASCADE
);
INSERT INTO system_requirement VALUES('attack-pattern--3986e7fd-a8e9-4ecb-bfc6-55920855912b','Valid domain account');
INSERT INTO system_requirement VALUES('attack-pattern--cc89ecbd-3d33-4a41-bcca-001e702d18fd','Secure boot disabled on systems running Windows 8 and later');
INSERT INTO system_requirement VALUES('attack-pattern--30208d3e-0d6b-43c8-883e-44462a514619','Permissions to access directories and files that store information of interest.');
INSERT INTO system_requirement VALUES('attack-pattern--6495ae23-3ab4-43c5-a94f-5638a2c31fd2','Clearing the Windows event logs requires Administrator permissions');
INSERT INTO system_requirement VALUES('attack-pattern--c726e0a2-a57a-4b7b-a973-d0f013246617','Compiler software (either native to the system or delivered by the adversary)');
INSERT INTO system_requirement VALUES('attack-pattern--791481f8-e96a-41be-b089-a088763083d4','Ability to update component device firmware from the host operating system.');
INSERT INTO system_requirement VALUES('attack-pattern--837f9164-50af-4ac0-8219-379d8a74cefc','Access to files');
INSERT INTO system_requirement VALUES('attack-pattern--341e222a-a6e3-4f6f-b69c-831d792b1580','Ability to query some Registry locations depends on the adversary''s level of access. User permissions are usually limited to access of user-related Registry keys.');
INSERT INTO system_requirement VALUES('attack-pattern--3c4a2599-71ee-4405-ba1e-0e28414b4bc5','Privileges to access certain files and directories');
INSERT INTO system_requirement VALUES('attack-pattern--ae676644-d2d2-41b7-af7e-9bed1b55898c','Privileges to access network shared drive');
INSERT INTO system_requirement VALUES('attack-pattern--1b7ba276-eedc-4951-a762-0ceea2c030ec','Privileges to access removable media drive and files');
INSERT INTO system_requirement VALUES('attack-pattern--e6415f09-df0e-48de-9aba-928c902b7549','Presence of physical medium or device');
INSERT INTO system_requirement VALUES('attack-pattern--a3e1e6c5-9c74-4fc0-a16c-a9d228c17829','Presence of physical medium or device');
INSERT INTO system_requirement VALUES('attack-pattern--be2dcee9-a7a7-4e38-afd6-21b31ecc3d63','Remote exploitation for execution requires a remotely accessible service reachable over the network or other vector of access such as spearphishing or drive-by compromise.');
INSERT INTO system_requirement VALUES('attack-pattern--9db0cf3a-a3c9-4012-8268-123b9db6fd82','Unpatched software or otherwise vulnerable target. Depending on the target and goal, the system and exploitable service may need to be remotely accessible from the internal network.');
INSERT INTO system_requirement VALUES('attack-pattern--7bc57495-ea59-4380-be31-a64af124ef18','Some folders may require Administrator, SYSTEM or specific user depending on permission levels and access controls');
INSERT INTO system_requirement VALUES('attack-pattern--f2877f7f-9a4c-4251-879f-1224e3006bee','Valid domain account or the ability to sniff traffic within a domain');
INSERT INTO system_requirement VALUES('attack-pattern--c92e3d68-2349-49e4-a341-7edca2deff96',' .NET Framework version 4 or higher');
INSERT INTO system_requirement VALUES('attack-pattern--edf91964-b26e-4b4a-9600-ccacd7d7df24','Access to Domain Controller or backup');
INSERT INTO system_requirement VALUES('attack-pattern--f2857333-11d4-45bf-b064-2c28d8525be5','NTFS partitioned hard drive');
INSERT INTO system_requirement VALUES('attack-pattern--a750a9f6-0bde-4bb3-9aae-1e2786e9780c','Established network share connection to a remote system. Level of access depends on permissions of the account used.');
INSERT INTO system_requirement VALUES('attack-pattern--3257eb21-f9a7-4430-8de1-d8b6e288f529','Network interface access and packet capture driver');
INSERT INTO system_requirement VALUES('attack-pattern--ed7efd4d-ce28-4a19-a8e6-c58011eb2c7a','Office 2007, 2010, 2013, and 2016');
INSERT INTO system_requirement VALUES('attack-pattern--7b211ac6-c815-4189-93a9-ab415deca926','Kerberos authentication enabled');
INSERT INTO system_requirement VALUES('attack-pattern--8f4a33ec-8b1f-4b80-a2f6-642b2e479580','Administrator, SYSTEM may provide better process ownership details');
INSERT INTO system_requirement VALUES('attack-pattern--cc3502b5-30cc-4473-ad48-42d51a6ef6d1','Python is installed.');
INSERT INTO system_requirement VALUES('attack-pattern--eb062747-2193-45de-8fa2-e62549c37ddf','RDP service enabled, account in the Remote Desktop Users group');
INSERT INTO system_requirement VALUES('attack-pattern--54a649ff-439a-41a4-9856-8d144a2551ba','Active remote service accepting connections and valid credentials');
INSERT INTO system_requirement VALUES('attack-pattern--3b744087-9945-4a6f-91e8-9dbceda417a4','Removable media allowed, Autorun enabled or vulnerability present that allows for code execution');
INSERT INTO system_requirement VALUES('attack-pattern--4f9ca633-15c5-463c-9724-bdcd54fde541','SMB enabled; Host/network firewalls not blocking SMB ports between source and destination; Use of domain account in administrator group on remote system or default system admin account.');
INSERT INTO system_requirement VALUES('attack-pattern--2db31dcd-54da-405d-acef-b9129b816ed6','An SSH server is configured and running.');
INSERT INTO system_requirement VALUES('attack-pattern--4d2a5b3e-340d-4600-9123-309dd63c9bf8','SSH service enabled, trust relationships configured, established connections');
INSERT INTO system_requirement VALUES('attack-pattern--804c042c-cfe6-449e-bc1a-ba0a998a70db','Shared webroot directory on remote system');
INSERT INTO system_requirement VALUES('attack-pattern--3fc01293-ef5e-41c6-86ce-61f10706b64a','Kerberos authentication enabled');
INSERT INTO system_requirement VALUES('attack-pattern--246fd3c7-f5e3-466d-8787-4c13d9e3b61c','Access to shared folders and content with write permissions');
INSERT INTO system_requirement VALUES('attack-pattern--dd43c543-bb85-4a6f-aa6e-160d90d06a49',replace('Smart card Proxy: Use of smart cards for single or multifactor authentication to access to network resources. Attached smart card reader with card inserted.\n\nOut-of-band one-time code: Access to the device, service, or communications to intercept the one-time code.\n\nHardware token: Access to the seed and algorithm of generating one-time codes.','\n',char(10)));
INSERT INTO system_requirement VALUES('attack-pattern--c898c4b5-bf36-4e6e-a4ad-5b8c4c13e35b','MS Office version specified in <code>_VBA_PROJECT</code> stream must match host');
INSERT INTO system_requirement VALUES('attack-pattern--01327cde-66c4-4123-bf34-5f258d59457b','VNC server installed and listening for connections.');
INSERT INTO system_requirement VALUES('attack-pattern--69e5226d-05dc-4f15-95d7-44f5ed78d06e','An externally facing login portal is configured.');
INSERT INTO system_requirement VALUES('attack-pattern--5d0d3609-d06d-49e1-b9c9-b544e0c618cb','Adversary access to Web server with vulnerability or account to upload and serve the Web shell file.');
INSERT INTO system_requirement VALUES('attack-pattern--01a5a209-b94c-450b-b7f9-946497d91055',replace('WMI service, winmgmt, running.\nHost/network firewalls allowing SMB and WMI ports from source to destination.\nSMB authentication.','\n',char(10)));
INSERT INTO system_requirement VALUES('attack-pattern--ebbe170d-aa74-4946-8511-9921243415a3','Microsoft Core XML Services (MSXML) or access to wmic.exe');
CREATE TABLE mitigate (
	id VARCHAR NOT NULL,
	source_id VARCHAR NOT NULL,
	target_id VARCHAR NOT NULL,
	description VARCHAR,
	created_time DATETIME,
	modified_time DATETIME,
	PRIMARY KEY (id),
	FOREIGN KEY(source_id) REFERENCES mitigation (id),
	FOREIGN KEY(target_id) REFERENCES technique (id)
);
INSERT INTO mitigate VALUES('relationship--483a70b9-eae9-4d5f-925c-95c2dd7b9fa5','course-of-action--beb45abb-11e8-4aef-9778-1f9ac249784f','attack-pattern--ca1a3f50-5ebd-41f8-8320-2c7d6a6e88be',NULL,'2017-05-31 21:33:27.026000','2020-02-05 20:08:34.059000');
INSERT INTO mitigate VALUES('relationship--6e466b8c-5798-4976-becb-9237f6154aab','course-of-action--676975b9-7e8e-463d-a31e-4ed2ecbfed81','attack-pattern--10d5f3b7-6be6-4da5-9a77-0f1e2bbfcc44',NULL,'2018-10-17 00:14:20.652000','2020-10-23 15:04:39.981000');
INSERT INTO mitigate VALUES('relationship--142800a5-62e9-48e9-97ef-186cfb68ffa1','course-of-action--9e57c770-5a39-49a2-bb91-253ba629e3ac','attack-pattern--6c174520-beea-43d9-aac6-28fb77f3e446',NULL,'2017-05-31 21:33:27.028000','2020-01-24 17:35:47.835000');
INSERT INTO mitigate VALUES('relationship--36adf5c8-2426-41e1-807d-f4d7958b9d54','course-of-action--84d633a4-dd93-40ca-8510-40238c021931','attack-pattern--dc27c2ec-c5f9-4228-ba57-d67b590bda93',NULL,'2017-12-14 16:46:06.044000','2020-03-13 21:01:23.214000');
INSERT INTO mitigate VALUES('relationship--4b12c645-96fc-45ac-b515-8333d6e254ef','course-of-action--d0fcf37a-b6c4-4745-9c43-4fcdb8bfc88e','attack-pattern--ad255bfe-a9e6-4b52-a258-8d3462abe842',NULL,'2017-05-31 21:33:27.017000','2020-03-15 00:27:20.188000');
INSERT INTO mitigate VALUES('relationship--02f28dfb-4e72-47e2-a390-2ec3fa67d26d','course-of-action--3e7018e9-7389-48e7-9208-0bdbcbba9483','attack-pattern--d3046a90-580c-4004-8208-66915bc29830',NULL,'2017-12-14 16:46:06.044000','2020-01-31 12:32:52.511000');
INSERT INTO mitigate VALUES('relationship--c593abb1-54ce-4196-a11f-f1dd65fed9aa','course-of-action--82d8e990-c901-4aed-8596-cc002e7eb307','attack-pattern--f3c544dc-673c-4ef3-accb-53229f1ae077',NULL,'2017-05-31 21:33:27.030000','2020-03-15 01:07:42.903000');
INSERT INTO mitigate VALUES('relationship--162d2d29-7810-499c-a9ab-08b60cd0e713','course-of-action--91816292-3686-4a6e-83c4-4c08513b9b57','attack-pattern--7d6f590f-544b-45b4-9a42-e0805f342af3',NULL,'2018-04-18 17:59:24.739000','2020-01-31 18:58:17.429000');
INSERT INTO mitigate VALUES('relationship--47639246-6268-4a7e-9670-965873bdfb42','course-of-action--1a7f5bd3-f6ee-4bd7-b949-2f3632ad6158','attack-pattern--6fb6408c-0db3-41d9-a3a1-a32e5f16454e',NULL,'2017-12-14 16:46:06.044000','2020-02-05 16:23:01.832000');
INSERT INTO mitigate VALUES('relationship--130275cb-368e-4168-a4bf-60b39566bc50','course-of-action--1c0711c8-2a73-48a1-893d-ff88bcd23824','attack-pattern--4eeaf8a9-c86b-4954-a663-9555fb406466',NULL,'2017-05-31 21:33:27.020000','2020-03-28 00:26:48.868000');
INSERT INTO mitigate VALUES('relationship--52b6181e-881e-4b96-93a3-1292bc2f1352','course-of-action--9378f139-10ef-4e4b-b679-2255a0818902','attack-pattern--39a130e1-6ab7-434a-8bd2-418e7d9d6427',NULL,'2017-05-31 21:33:27.023000','2020-03-19 15:12:13.273000');
INSERT INTO mitigate VALUES('relationship--13f5fad8-1b6f-4b65-9803-155f93b5d357','course-of-action--7c39ebbf-244e-4d1c-b0ac-b282453ece43','attack-pattern--1c338d0f-a65e-4073-a5c1-c06878849f21',NULL,'2017-05-31 21:33:27.027000','2020-01-14 17:24:47.567000');
INSERT INTO mitigate VALUES('relationship--e1275bcd-0462-4f79-b18f-2132b0bb74ec','course-of-action--c88151a5-fe3f-4773-8147-d801587065a4','attack-pattern--327f3cc5-eea1-42d4-a6cd-ed34b7ce8f61',NULL,'2017-05-31 21:33:27.019000','2020-03-23 15:40:51.383000');
INSERT INTO mitigate VALUES('relationship--b6f00052-49e3-48c5-8f5e-492be4e67acf','course-of-action--c1676218-c16a-41c9-8f7a-023779916e39','attack-pattern--7e150503-88e7-4861-866b-ff1ac82c4475',NULL,'2017-05-31 21:33:27.022000','2020-03-15 14:15:33.165000');
INSERT INTO mitigate VALUES('relationship--67bde2b2-49d1-4a61-8fe7-1a48c58089e6','course-of-action--8a61f6b9-6b7a-4cf2-8e08-f1e26434f6df','attack-pattern--91ce1ede-107f-4d8b-bf4c-735e8789c94b',NULL,'2017-12-14 16:46:06.044000','2020-02-12 16:34:07.059000');
INSERT INTO mitigate VALUES('relationship--ab524992-5666-466b-8c12-ec79b269901b','course-of-action--25e53928-6f33-49b7-baee-8180578286f6','attack-pattern--6856ddd6-2df3-4379-8b87-284603c189c3',NULL,'2017-05-31 21:33:27.019000','2020-03-30 19:16:29.726000');
INSERT INTO mitigate VALUES('relationship--8467ea5f-cb0d-4eb6-b524-8bfd01e58721','course-of-action--80c91478-ac87-434f-bee7-11f37aec4d74','attack-pattern--edbe24e9-aec4-4994-ac75-6a6bc7f1ddd0',NULL,'2018-01-16 16:13:52.465000','2020-03-09 13:00:19.403000');
INSERT INTO mitigate VALUES('relationship--a93e5f9f-5c8c-4832-93db-a6c180840a43','course-of-action--d4fd04e0-d1a4-4b5a-a5bb-16683cdbcce2','attack-pattern--10d51417-ee35-4589-b1ff-b6df1c334e8d',NULL,'2017-05-31 21:33:27.031000','2020-06-19 20:07:09.769000');
INSERT INTO mitigate VALUES('relationship--49faeaac-7118-4705-8c29-9ce2a88ae958','course-of-action--37a3f3f5-76e6-43fe-b935-f1f494c95725','attack-pattern--fe926152-f431-4baf-956c-4ad3cb0bf23b',NULL,'2018-04-18 17:59:24.739000','2020-03-29 20:00:46.982000');
INSERT INTO mitigate VALUES('relationship--bdac804d-9081-4dae-8bc7-877c771e5d72','course-of-action--97d8eadb-0459-4c1d-bf1a-e053bd75df61','attack-pattern--3f18edba-28f4-4bb9-82c3-8aa60dcac5f7',NULL,'2018-04-18 17:59:24.739000','2020-03-11 14:19:57.047000');
INSERT INTO mitigate VALUES('relationship--efa2ae6b-8942-4ea2-80ca-b4181dd01427','course-of-action--94f6b4f5-b528-4f50-91d5-f66457c2f8f7','attack-pattern--544b0346-29ad-41e1-a808-501bb4193f47',NULL,'2018-01-16 16:13:52.465000','2020-07-14 19:39:44.767000');
INSERT INTO mitigate VALUES('relationship--3e89d94b-5e6f-48b3-ba80-d366940fa968','course-of-action--25d5e1d8-c6fb-4735-bc57-115a21222f4b','attack-pattern--4ae4f953-fe58-4cc8-a327-33257e30a830',NULL,'2017-05-31 21:33:27.018000','2020-03-26 15:44:27.175000');
INSERT INTO mitigate VALUES('relationship--5ab4f110-5fd1-466b-9437-0132ae390ac9','course-of-action--65da1eb6-d35d-4853-b280-98a76c0aef53','attack-pattern--3f886f2a-874f-4333-b794-aa6075009b1c',NULL,'2018-04-18 17:59:24.739000','2020-10-21 01:10:54.561000');
INSERT INTO mitigate VALUES('relationship--77fad92a-72ba-44d2-b4cb-a3079fbdb256','course-of-action--902286b2-96cc-4dd7-931f-e7340c9961da','attack-pattern--0c8ab3eb-df48-4b9c-ace7-beacaac81cc5',NULL,'2017-05-31 21:33:27.018000','2020-01-30 22:27:40.243000');
INSERT INTO mitigate VALUES('relationship--2083aef8-4d72-4bef-8cbc-33f2c5f4a176','course-of-action--e547ed6a-f1ca-40df-8613-2ce27927f145','attack-pattern--e6415f09-df0e-48de-9aba-928c902b7549',NULL,'2017-05-31 21:33:27.022000','2020-03-28 00:31:48.788000');
INSERT INTO mitigate VALUES('relationship--65a4317d-86b2-40c1-9d27-a067bcc2ad80','course-of-action--910482b1-6749-4934-abcb-3e34d58294fc','attack-pattern--772bc7a8-a157-42cc-8728-d648e25c7fe7',NULL,'2018-01-16 16:13:52.465000','2020-03-30 13:36:10.384000');
INSERT INTO mitigate VALUES('relationship--bb283a5e-7d61-4b33-aa30-e7c2f0bacbe6','course-of-action--39706d54-0d06-4a25-816a-78cc43455100','attack-pattern--1b7ba276-eedc-4951-a762-0ceea2c030ec',NULL,'2017-05-31 21:33:27.020000','2020-03-24 15:44:46.688000');
INSERT INTO mitigate VALUES('relationship--918956f2-db79-4721-8741-3b461a280e51','course-of-action--77fd4d73-6b79-4593-82e7-e4a439cc7604','attack-pattern--04ef4356-8926-45e2-9441-634b6f3dcecb',NULL,'2017-12-14 16:46:06.044000','2020-01-24 14:22:27.719000');
INSERT INTO mitigate VALUES('relationship--cf699238-7091-4d79-9741-d792152f37c1','course-of-action--b8d57b16-d8e2-428c-a645-1083795b3445','attack-pattern--64196062-5210-42c3-9a02-563a0d1797ef',NULL,'2017-05-31 21:33:27.027000','2020-07-14 19:44:50.970000');
INSERT INTO mitigate VALUES('relationship--c57efd0b-817e-45c2-9f11-e8e7ac11b44c','course-of-action--da987565-27b6-4b31-bbcd-74b909847116','attack-pattern--99709758-2b96-48f2-a68a-ad7fbd828091',NULL,'2017-05-31 21:33:27.020000','2020-03-30 13:59:11.395000');
INSERT INTO mitigate VALUES('relationship--ab6dbf38-dfed-4bfa-9d7d-bbe6864f82d3','course-of-action--06824aa2-94a5-474c-97f6-57c2e983d885','attack-pattern--36675cd3-fe00-454c-8516-aebecacbe9d9',NULL,'2017-12-14 16:46:06.044000','2020-03-19 19:17:58.670000');
INSERT INTO mitigate VALUES('relationship--395468bb-f42a-4e60-8f2f-d998c9d85675','course-of-action--3a41b366-cfd6-4af2-a6e7-3c6e3c4ebcef','attack-pattern--8df54627-376c-487c-a09c-7d2b5620f56e',NULL,'2018-04-18 17:59:24.739000','2020-01-31 18:59:22.086000');
INSERT INTO mitigate VALUES('relationship--1d0bbeb7-5477-4321-81cd-ef66607d7972','course-of-action--53b3b027-bed3-480c-9101-1247047d0fe6','attack-pattern--51dea151-0898-4a45-967c-3ebee0420484',NULL,'2017-05-31 21:33:27.025000','2020-02-11 18:24:04.781000');
INSERT INTO mitigate VALUES('relationship--5a491b91-739f-498b-b8f2-b14aaea07893','course-of-action--0472af99-f25c-4abe-9fce-010fa3450e72','attack-pattern--ba8e391f-14b5-496f-81f2-2d5ecd646c1c',NULL,'2017-05-31 21:33:27.026000','2020-02-04 12:55:04.030000');
INSERT INTO mitigate VALUES('relationship--03f30a17-095b-4656-a7db-87d98628dfd8','course-of-action--f6469191-1814-4dbe-a081-2a6daf83a10b','attack-pattern--8f4a33ec-8b1f-4b80-a2f6-642b2e479580',NULL,'2017-05-31 21:33:27.023000','2020-03-26 18:05:53.242000');
INSERT INTO mitigate VALUES('relationship--85d0b364-01f8-4ed5-a0dd-a534ccfc7025','course-of-action--c7e49501-6021-414f-bfa1-94519d8ec314','attack-pattern--dc31fe1e-d722-49da-8f5f-92c7b5aff534',NULL,'2018-10-17 00:14:20.652000','2020-04-29 14:37:59.726000');
INSERT INTO mitigate VALUES('relationship--ff93eedd-e788-4541-9a9b-ccead3df0d13','course-of-action--ed202147-4026-4330-b5bd-1e8dfa8cf7cc','attack-pattern--57340c81-c025-4189-8fa0-fc7ede51bae4',NULL,'2017-05-31 21:33:27.029000','2020-08-13 20:02:49.814000');
INSERT INTO mitigate VALUES('relationship--338cf92d-43a8-4fdd-948d-1a3bde10d917','course-of-action--d8787791-d22e-45bb-a9a8-251d8d0a1ff2','attack-pattern--322bad5a-1c49-4d23-ab79-76d641794afa',NULL,'2017-05-31 21:33:27.018000','2020-03-15 01:05:08.927000');
INSERT INTO mitigate VALUES('relationship--701a2767-70f3-44f1-a397-9c04517ece67','course-of-action--9da16278-c6c5-4410-8a6b-9c16ce8005b3','attack-pattern--2892b9ee-ca9f-4723-b332-0dc6e843a8ae',NULL,'2018-01-16 16:13:52.465000','2020-01-24 13:51:48.171000');
INSERT INTO mitigate VALUES('relationship--8d7c04bb-ed5b-4339-a115-62089ea6711b','course-of-action--797312d4-8a84-4daf-9c56-57da4133c322','attack-pattern--9fa07bef-9c81-421e-a8e5-ad4366c5a925',NULL,'2018-04-18 17:59:24.739000','2020-07-14 19:38:14.416000');
INSERT INTO mitigate VALUES('relationship--4c2b4c0f-0ded-4f0f-ad5a-a95241ba927e','course-of-action--94e95eeb-7cdb-4bd7-afba-f32fda303dbb','attack-pattern--e7eab98d-ae11-4491-bd28-a53ba875865a',NULL,'2017-05-31 21:33:27.030000','2020-01-31 12:39:48.366000');
INSERT INTO mitigate VALUES('relationship--f132ff40-9e9d-49b8-a47d-832a21e1e56d','course-of-action--e0703d4f-3972-424a-8277-84004817e024','attack-pattern--c4ad009b-6e13-4419-8d21-918a1652de02',NULL,'2017-05-31 21:33:27.021000','2020-07-06 18:49:35.880000');
INSERT INTO mitigate VALUES('relationship--8ff745b7-9985-4781-a8bc-dae6d71233d3','course-of-action--34efb2fd-4dc2-40d4-a564-0c147c85034d','attack-pattern--56fca983-1cf1-4fd1-bda0-5e170a37ab59',NULL,'2017-05-31 21:33:27.029000','2020-01-31 12:36:39.314000');
INSERT INTO mitigate VALUES('relationship--68936ca4-6cf3-4b81-b1e8-17c28c2fb9ef','course-of-action--dbf0186e-722d-4a0a-af6a-b3460f162f84','attack-pattern--2169ba87-1146-4fc7-a118-12b72251db7e',NULL,'2018-04-18 17:59:24.739000','2020-02-05 20:10:18.885000');
INSERT INTO mitigate VALUES('relationship--620ab17a-3e46-4083-82b0-aeff74d104cd','course-of-action--1e4ef2c7-ee96-4484-9baa-3b5777561301','attack-pattern--5ad95aaa-49c1-4784-821d-2e83f47b079b',NULL,'2017-12-14 16:46:06.044000','2020-03-09 14:09:09.019000');
INSERT INTO mitigate VALUES('relationship--4fa2cbf0-9721-4bbe-86b4-334848cd3dd6','course-of-action--5c167af7-c2cb-42c8-ae67-3fb275bf8488','attack-pattern--128c55d3-aeba-469f-bd3e-c8996ab4112a',NULL,'2017-05-31 21:33:27.028000','2020-02-18 16:56:57.167000');
INSERT INTO mitigate VALUES('relationship--3d635b23-78b7-4de4-9417-8077787c7c0b','course-of-action--5c49bc54-9929-48ca-b581-7018219b5a97','attack-pattern--72b74d71-8169-42aa-92e0-e7b04b9f5a08',NULL,'2017-05-31 21:33:27.026000','2020-09-16 15:10:18.380000');
INSERT INTO mitigate VALUES('relationship--1a3de27b-377c-4390-9911-2da8aaa705e3','course-of-action--16a8ac85-a06f-460f-ad22-910167bd7332','attack-pattern--519630c5-f03f-4882-825c-3af924935817',NULL,'2017-05-31 21:33:27.018000','2020-02-05 15:08:06.008000');
INSERT INTO mitigate VALUES('relationship--87131e3c-9d73-4910-a56d-f917d6660a7d','course-of-action--d5dce4b9-f1fa-4c03-aff9-ce177246cb64','attack-pattern--f44731de-ea9f-406d-9b83-30ecbb9b4392',NULL,'2017-05-31 21:33:27.021000','2020-03-10 18:34:39.928000');
INSERT INTO mitigate VALUES('relationship--bde4d54d-16d7-4a07-a35a-9f0cc6956be2','course-of-action--a0d8db1d-a731-4428-8209-c07175f4b1fe','attack-pattern--c848fcf7-6b62-4bde-8216-b6c157d48da0',NULL,'2017-05-31 21:33:27.024000','2020-03-14 18:19:53.341000');
CREATE TABLE phase (
	tactic_id VARCHAR NOT NULL,
	tech_id VARCHAR NOT NULL,
	PRIMARY KEY (tactic_id, tech_id),
	FOREIGN KEY(tactic_id) REFERENCES tactic (id) ON DELETE CASCADE,
	FOREIGN KEY(tech_id) REFERENCES technique (id) ON DELETE CASCADE
);
INSERT INTO phase VALUES('x-mitre-tactic--5e29b093-294e-49e9-a803-dab3d73b77dd','attack-pattern--b63a34e8-0a61-4c97-a23b-bf8a2ed812e2');
INSERT INTO phase VALUES('x-mitre-tactic--5bc1d813-693e-4823-9961-abf9af4b0e92','attack-pattern--b63a34e8-0a61-4c97-a23b-bf8a2ed812e2');
INSERT INTO phase VALUES('x-mitre-tactic--2558fd61-8c75-4730-94c4-11926db2a263','attack-pattern--d0b4fcdb-d67d-4ed2-99ce-788b12f8c0f4');
INSERT INTO phase VALUES('x-mitre-tactic--2558fd61-8c75-4730-94c4-11926db2a263','attack-pattern--cabe189c-a0e3-4965-a473-dcff00f17213');
INSERT INTO phase VALUES('x-mitre-tactic--d108ce10-2419-4cf9-a774-46161d6c6cfe','attack-pattern--cabe189c-a0e3-4965-a473-dcff00f17213');
INSERT INTO phase VALUES('x-mitre-tactic--2558fd61-8c75-4730-94c4-11926db2a263','attack-pattern--3986e7fd-a8e9-4ecb-bfc6-55920855912b');
INSERT INTO phase VALUES('x-mitre-tactic--5e29b093-294e-49e9-a803-dab3d73b77dd','attack-pattern--67720091-eee3-4d2d-ae16-8264567f6f5b');
INSERT INTO phase VALUES('x-mitre-tactic--78b23412-0651-46d7-a540-170a1ce8bd5a','attack-pattern--67720091-eee3-4d2d-ae16-8264567f6f5b');
INSERT INTO phase VALUES('x-mitre-tactic--78b23412-0651-46d7-a540-170a1ce8bd5a','attack-pattern--dcaa092b-7de9-4a21-977f-7fcb77e89c48');
INSERT INTO phase VALUES('x-mitre-tactic--5e29b093-294e-49e9-a803-dab3d73b77dd','attack-pattern--dcaa092b-7de9-4a21-977f-7fcb77e89c48');
INSERT INTO phase VALUES('x-mitre-tactic--5e29b093-294e-49e9-a803-dab3d73b77dd','attack-pattern--70e52b04-2a0c-4cea-9d18-7149f1df9dc5');
INSERT INTO phase VALUES('x-mitre-tactic--5bc1d813-693e-4823-9961-abf9af4b0e92','attack-pattern--70e52b04-2a0c-4cea-9d18-7149f1df9dc5');
INSERT INTO phase VALUES('x-mitre-tactic--5569339b-94c2-49ee-afb3-2222936582c8','attack-pattern--b24e2a20-3b3d-4bf0-823b-1ed765398fb0');
INSERT INTO phase VALUES('x-mitre-tactic--c17c5845-175e-4421-9713-829d0573dbc9','attack-pattern--72b74d71-8169-42aa-92e0-e7b04b9f5a08');
INSERT INTO phase VALUES('x-mitre-tactic--5bc1d813-693e-4823-9961-abf9af4b0e92','attack-pattern--a10641f4-87b4-45a3-a906-92a149cb2c27');
INSERT INTO phase VALUES('x-mitre-tactic--d679bca2-e57d-4935-8650-8031c87a4400','attack-pattern--0458aab9-ad42-4eac-9e22-706a95bafee2');
INSERT INTO phase VALUES('x-mitre-tactic--daa4cbb1-b4f4-4723-a824-7f1efd6e0592','attack-pattern--67073dde-d720-45ae-83da-b12d5e73ca3b');
INSERT INTO phase VALUES('x-mitre-tactic--5bc1d813-693e-4823-9961-abf9af4b0e92','attack-pattern--2dbbdcd5-92cf-44c0-aea2-fe24783a6bc3');
INSERT INTO phase VALUES('x-mitre-tactic--5bc1d813-693e-4823-9961-abf9af4b0e92','attack-pattern--34f1d81d-fe88-4f97-bd3b-a3164536255d');
INSERT INTO phase VALUES('x-mitre-tactic--5e29b093-294e-49e9-a803-dab3d73b77dd','attack-pattern--7d57b371-10c2-45e5-b3cc-83a8fb380e4c');
INSERT INTO phase VALUES('x-mitre-tactic--5bc1d813-693e-4823-9961-abf9af4b0e92','attack-pattern--7d57b371-10c2-45e5-b3cc-83a8fb380e4c');
INSERT INTO phase VALUES('x-mitre-tactic--5e29b093-294e-49e9-a803-dab3d73b77dd','attack-pattern--cc89ecbd-3d33-4a41-bcca-001e702d18fd');
INSERT INTO phase VALUES('x-mitre-tactic--5bc1d813-693e-4823-9961-abf9af4b0e92','attack-pattern--cc89ecbd-3d33-4a41-bcca-001e702d18fd');
INSERT INTO phase VALUES('x-mitre-tactic--4ca45d45-df4d-4613-8980-bac22d278fa5','attack-pattern--37b11151-1776-4f8f-b328-30939fbf2ceb');
INSERT INTO phase VALUES('x-mitre-tactic--78b23412-0651-46d7-a540-170a1ce8bd5a','attack-pattern--f005e783-57d4-4837-88ad-dbe7faee1c51');
INSERT INTO phase VALUES('x-mitre-tactic--7141578b-e50b-4dcc-bfa4-08a8dd689e9e','attack-pattern--f005e783-57d4-4837-88ad-dbe7faee1c51');
INSERT INTO phase VALUES('x-mitre-tactic--5569339b-94c2-49ee-afb3-2222936582c8','attack-pattern--18cffc21-3260-437e-80e4-4ab8bf2ba5e9');
INSERT INTO phase VALUES('x-mitre-tactic--f72804c5-f15a-449e-a5da-2eecd181f813','attack-pattern--355be19c-ffc9-46d5-8d50-d6a036c675b6');
INSERT INTO phase VALUES('x-mitre-tactic--5e29b093-294e-49e9-a803-dab3d73b77dd','attack-pattern--42fe883a-21ea-4cfb-b94a-78b6476dcc83');
INSERT INTO phase VALUES('x-mitre-tactic--5bc1d813-693e-4823-9961-abf9af4b0e92','attack-pattern--42fe883a-21ea-4cfb-b94a-78b6476dcc83');
INSERT INTO phase VALUES('x-mitre-tactic--c17c5845-175e-4421-9713-829d0573dbc9','attack-pattern--4ae4f953-fe58-4cc8-a327-33257e30a830');
INSERT INTO phase VALUES('x-mitre-tactic--5569339b-94c2-49ee-afb3-2222936582c8','attack-pattern--2bee5ffb-7a7a-4119-b1f2-158151b19ac0');
INSERT INTO phase VALUES('x-mitre-tactic--d108ce10-2419-4cf9-a774-46161d6c6cfe','attack-pattern--53ac20cd-aca3-406e-9aa0-9fc7fdc60a5a');
INSERT INTO phase VALUES('x-mitre-tactic--d108ce10-2419-4cf9-a774-46161d6c6cfe','attack-pattern--143c0cbb-a297-4142-9624-87ffc778980b');
INSERT INTO phase VALUES('x-mitre-tactic--d108ce10-2419-4cf9-a774-46161d6c6cfe','attack-pattern--41868330-6ee2-4d0f-b743-9f2294c3c9b6');
INSERT INTO phase VALUES('x-mitre-tactic--d108ce10-2419-4cf9-a774-46161d6c6cfe','attack-pattern--00f90846-cbd1-4fc5-9233-df5c2bf2a662');
INSERT INTO phase VALUES('x-mitre-tactic--f72804c5-f15a-449e-a5da-2eecd181f813','attack-pattern--bf176076-b789-408e-8cba-7275e81c0ada');
INSERT INTO phase VALUES('x-mitre-tactic--78b23412-0651-46d7-a540-170a1ce8bd5a','attack-pattern--7c0f17c9-1af6-4628-9cbd-9e45482dd605');
INSERT INTO phase VALUES('x-mitre-tactic--5e29b093-294e-49e9-a803-dab3d73b77dd','attack-pattern--7c0f17c9-1af6-4628-9cbd-9e45482dd605');
INSERT INTO phase VALUES('x-mitre-tactic--4ca45d45-df4d-4613-8980-bac22d278fa5','attack-pattern--6636bc83-0611-45a6-b74f-1f3daf635b8e');
INSERT INTO phase VALUES('x-mitre-tactic--5bc1d813-693e-4823-9961-abf9af4b0e92','attack-pattern--6636bc83-0611-45a6-b74f-1f3daf635b8e');
INSERT INTO phase VALUES('x-mitre-tactic--5e29b093-294e-49e9-a803-dab3d73b77dd','attack-pattern--6636bc83-0611-45a6-b74f-1f3daf635b8e');
INSERT INTO phase VALUES('x-mitre-tactic--4ca45d45-df4d-4613-8980-bac22d278fa5','attack-pattern--f3d95a1f-bba2-44ce-9af7-37866cd63fd0');
INSERT INTO phase VALUES('x-mitre-tactic--5bc1d813-693e-4823-9961-abf9af4b0e92','attack-pattern--f3d95a1f-bba2-44ce-9af7-37866cd63fd0');
INSERT INTO phase VALUES('x-mitre-tactic--5e29b093-294e-49e9-a803-dab3d73b77dd','attack-pattern--f3d95a1f-bba2-44ce-9af7-37866cd63fd0');
INSERT INTO phase VALUES('x-mitre-tactic--d108ce10-2419-4cf9-a774-46161d6c6cfe','attack-pattern--1035cdf2-3e5f-446f-a7a7-e8f6d7925967');
INSERT INTO phase VALUES('x-mitre-tactic--5bc1d813-693e-4823-9961-abf9af4b0e92','attack-pattern--b8cfed42-6a8a-4989-ad72-541af74475ec');
INSERT INTO phase VALUES('x-mitre-tactic--5e29b093-294e-49e9-a803-dab3d73b77dd','attack-pattern--b8cfed42-6a8a-4989-ad72-541af74475ec');
INSERT INTO phase VALUES('x-mitre-tactic--d108ce10-2419-4cf9-a774-46161d6c6cfe','attack-pattern--30208d3e-0d6b-43c8-883e-44462a514619');
INSERT INTO phase VALUES('x-mitre-tactic--9a4e74ab-5008-408c-84bf-a10dfbc53462','attack-pattern--774a3188-6ba9-4dc4-879d-d54ee48a5ce9');
COMMIT;
