/*
 * Copyright (C) 2015, Wazuh Inc.
 *
 * This program is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General Public
 * License (version 2) as published by the FSF - Free Software
 * Foundation.
 */

#include <stdarg.h>
#include <stddef.h>
#include <setjmp.h>
#include <cmocka.h>
#include <stdio.h>
#include <stdlib.h>
#include "cJSON.h"


#include "../../wazuh_modules/wmodules.h"
#include "../../wazuh_modules/vulnerability_detector/wm_vuln_detector.h"
#include "../../wrappers/wazuh/shared/debug_op_wrappers.h"
#include "../../wrappers/externals/sqlite/sqlite3_wrappers.h"
#include "../../headers/shared.h"
#include "../../headers/file_op.h"
#include "mocks_wm_vuln_detector.h"
#include "../../wrappers/externals/cJSON/cJSON_wrappers.h"
#include "../../wrappers/libc/stdio_wrappers.h"
#include "../../wrappers/common.h"

void wm_vuldet_free_scan_agent(scan_agent *agent);
void wm_vuldet_free_alas_pkg(vu_alas_pkg **alas_pkg);
int wm_vuldet_json_nvd_parser(FILE * fp, wm_vuldet_db *parsed_vulnerabilities, bool* working);
int wm_vuldet_parse_nvd_configurations(cJSON *configurations, nvd_vulnerability *data);
int wm_vuldet_parse_nvd_metrics(cJSON *metrics, nvd_vulnerability *data);
int wm_vuldet_parse_nvd_weaknesses(cJSON *weaknesses, nvd_vulnerability *data);
int wm_vuldet_parse_nvd_descriptions(cJSON *descriptions, nvd_vulnerability *data);
int wm_vuldet_parse_nvd_references(cJSON *references, nvd_vulnerability *data);
void wm_vuldet_free_nvd_list(nvd_vulnerability *nvd_it);
void wm_vuldet_free_nvd_node(nvd_vulnerability *data);
void wm_vuldet_free_nvd_references(nvd_references *data);
void wm_vuldet_free_nvd_conf_cpe_match(nvd_conf_cpe_match *data);
void wm_vuldet_free_nvd_configuration(nvd_configuration *data);
FILE * w_get_file_pointer(const char * path);
void wm_vuldet_free_nvd_report(vu_nvd_report *report);
void wm_vuldet_free_nvd_report_list(vu_nvd_report *nvd_report_list);
char *wm_vuldet_clean_version(const char *version);
int vw_vuldet_filter_vulnerabilities(vu_feed feed, const char *vendor, const char *target_sw);
int wm_vuldet_check_generic_package(sqlite3 *dbCVE,
                                    vu_feed feed,
                                    char *pkg_name,
                                    char *pkg_source,
                                    char *pkg_version,
                                    char *pkg_arch,
                                    char *pkg_vendor,
                                    char *pkg_reference,
                                    char *pkg_type,
                                    OSHash *cve_table,
                                    int8_t name_type,
                                    int *vuln_count);
int wm_vuldet_check_specific_package(sqlite3 *dbCVE,
                                     vu_feed feed,
                                     char *pkg_name,
                                     char *pkg_source,
                                     char *pkg_version,
                                     char *pkg_arch,
                                     char *pkg_vendor,
                                     char *pkg_reference,
                                     char *pkg_type,
                                     OSHash *cve_table,
                                     int8_t name_type,
                                     int *vuln_count);
int wm_vuldet_get_children(sqlite3 *dbCVE, int configuration_id, int package_id, int *children);
int wm_vuldet_get_siblings(sqlite3 *dbCVE, int parent, int configuration_id, int *siblings);

/* setup */

char *nvd_feed_mock_cve = "{\"resultsPerPage\":1, \
\"startIndex\":0, \
\"totalResults\":1, \
\"format\":\"NVD_CVE\", \
\"version\":\"2.0\", \
\"timestamp\":\"2023-05-30T16:35:57.987\",\
\"vulnerabilities\":[ \
{\"cve\":{ \
\"id\":\"CVE-2020-0504\", \
\"sourceIdentifier\":\"secure@intel.com\", \
\"published\":\"2020-03-12T18:15:12.023\", \
\"lastModified\":\"2021-05-19T17:00:01.097\", \
\"vulnStatus\":\"Analyzed\", \
\"warningCVEData\":\"test\"}, \
\"cpe\":\"CVE\"}]}";

char *nvd_feed_mock_descriptions = "{ \
\"descriptions\":[ \
{\"lang\":\"es\", \
\"value\":\"El desbordamiento del búfer.\"}, \
{\"lang\":\"en\", \
\"value\":\"Buffer overflow.\"}]}";

char *nvd_feed_mock_metrics = "{ \
\"metrics\":{ \
\"cvssMetricV41\":[{ \
\"source\":\"nvd@nist.gov\", \
\"type\":\"Primary\"}], \
\"cvssMetricV31\":[{ \
\"source\":\"nvd@nist.gov\", \
\"type\":\"Primary\", \
\"cvssData\":{ \
\"version\":\"3.1\", \
\"vectorString\":\"CVSS:3.1/AV:L/AC:L/PR:L/UI:N/S:U/C:H/I:H/A:H\", \
\"attackVector\":\"LOCAL\", \
\"attackComplexity\":\"LOW\", \
\"privilegesRequired\":\"LOW\", \
\"userInteraction\":\"NONE\", \
\"scope\":\"UNCHANGED\", \
\"confidentialityImpact\":\"HIGH\", \
\"integrityImpact\":\"HIGH\", \
\"availabilityImpact\":\"HIGH\", \
\"baseScore\":7.8, \
\"baseSeverity\":\"HIGH\"}, \
\"exploitabilityScore\":1.8, \
\"impactScore\":5.9}], \
\"cvssMetricV2\":[{ \
\"source\":\"nvd@nist.gov\", \
\"type\":\"Primary\", \
\"cvssData\":{ \
\"version\":\"2.0\", \
\"vectorString\":\"AV:L/AC:L/Au:N/C:P/I:P/A:P\", \
\"accessVector\":\"LOCAL\", \
\"accessComplexity\":\"LOW\", \
\"authentication\":\"NONE\", \
\"confidentialityImpact\":\"PARTIAL\", \
\"integrityImpact\":\"PARTIAL\", \
\"availabilityImpact\":\"PARTIAL\", \
\"baseScore\":4.6}, \
\"baseSeverity\":\"MEDIUM\", \
\"exploitabilityScore\":3.9, \
\"impactScore\":6.4, \
\"acInsufInfo\":false, \
\"obtainAllPrivilege\":false, \
\"obtainUserPrivilege\":false, \
\"obtainOtherPrivilege\":false, \
\"userInteractionRequired\":false}]}}";

char *nvd_feed_mock_weaknesses = "{ \
\"weaknesses\":[{ \
\"source\":\"nvd@nist.gov\", \
\"type\":\"Primary\", \
\"description\":[{ \
\"lang\":\"en\", \
\"value\":\"CWE-120\"}]}]}";

char *nvd_feed_mock_references = "{ \
\"references\":[{ \
\"url\":\"https://security.netapp.com/advisory/ntap-20200320-0003/\", \
\"source\":\"secure@intel.com\", \
\"tags\":[\"Third Party Advisory\"]}, \
{\"url\":\"https://www.intel.com/content/www/us/en/security-center/advisory/intel-sa-00315.html\", \
\"source\":\"secure@intel.com\", \
\"tags\":[\"Vendor Advisory\"]}]}";

char *nvd_feed_mock_configurations = "{ \
\"configurations\": [{ \
\"operator\": \"AND\", \
\"nodes\": [{ \
\"operator\": \"OR\", \
\"negate\": false, \
\"cpeMatch\": [{ \
\"vulnerable\": true, \
\"criteria\": \"cpe:2.3:a:xpdfreader:xpdf:3.03-17:*:*:*:*:*:*:*\", \
\"versionStartIncluding\":\"15.40\", \
\"versionStartExcluding\":\"10.30\", \
\"versionEndIncluding\":\"11.10\", \
\"versionEndExcluding\":\"15.40.44.5107\", \
\"matchCriteriaId\": \"3BCCCEC9-8F50-4F8E-A51F-B973832C33E2\"}]}, \
{\"operator\": \"OR\", \
\"negate\": false, \
\"cpeMatch\": [{ \
\"vulnerable\": false, \
\"criteria\": \"cpe:2.3:o:debian:debian_linux:8.0:*:*:*:*:*:*:*\", \
\"matchCriteriaId\": \"C11E6FB0-C8C0-4527-9AA0-CB9B316F8F43\"}]}]}, \
{\"operator\": \"AND\", \
\"nodes\": [{ \
\"operator\": \"OR\", \
\"negate\": false, \
\"cpeMatch\": [{ \
\"vulnerable\": true, \
\"criteria\": \"cpe:2.3:a:xpdfreader:xpdf:3.04-4:*:*:*:*:*:*:*\", \
\"matchCriteriaId\": \"0B60B556-E6FF-44CB-98ED-CCAEFDDA3845\"}]}, \
{\"operator\": \"OR\", \
\"negate\": false, \
\"cpeMatch\": [{ \
\"vulnerable\": false, \
\"criteria\": \"cpe:2.3:o:debian:debian_linux:9.0:*:*:*:*:*:*:*\", \
\"matchCriteriaId\": \"DEECE5FC-CACF-4496-A3E7-164736409252\"}]}]}, \
{\"operator\": \"AND\", \
\"nodes\": [{ \
\"operator\": \"OR\", \
\"negate\": false, \
\"cpeMatch\": [{ \
\"vulnerable\": true, \
\"criteria\": \"cpe:2.3:a:xpdfreader:xpdf:3.04-13:*:*:*:*:*:*:*\", \
\"matchCriteriaId\": \"C306DA00-63D2-4750-A690-670BDDA3CF89\"}]}, \
{\"operator\": \"OR\", \
\"negate\": false, \
\"cpeMatch\": [{ \
\"vulnerable\": false, \
\"criteria\": \"cpe:2.3:o:debian:debian_linux:10.0:*:*:*:*:*:*:*\", \
\"matchCriteriaId\": \"07B237A9-69A3-4A9C-9DA0-4E06BD37AE73\"}]}]}]}";

char *nvd_feed_mock_complete = "{\"resultsPerPage\":1, \
\"startIndex\":0, \
\"totalResults\":1, \
\"format\":\"NVD_CVE\", \
\"version\":\"2.0\", \
\"timestamp\":\"2023-05-30T16:35:57.987\", \
\"vulnerabilities\":[ \
{\"cve\":{ \
\"id\":\"CVE-2020-0504\", \
\"sourceIdentifier\":\"secure@intel.com\", \
\"published\":\"2020-03-12T18:15:12.023\", \
\"lastModified\":\"2021-05-19T17:00:01.097\", \
\"vulnStatus\":\"Analyzed\", \
\"descriptions\":[{ \
\"lang\":\"en\", \
\"value\":\"Buffer overflow.\"}, \
{\"lang\":\"es\", \
\"value\":\"El desbordamiento del búfer.\"}], \
\"metrics\":{ \
\"cvssMetricV31\":[{ \
\"source\":\"nvd@nist.gov\", \
\"type\":\"Primary\", \
\"cvssData\":{ \
\"version\":\"3.1\", \
\"vectorString\":\"CVSS:3.1/AV:L/AC:L/PR:L/UI:N/S:U/C:H/I:H/A:H\", \
\"attackVector\":\"LOCAL\", \
\"attackComplexity\":\"LOW\", \
\"privilegesRequired\":\"LOW\", \
\"userInteraction\":\"NONE\", \
\"scope\":\"UNCHANGED\", \
\"confidentialityImpact\":\"HIGH\", \
\"integrityImpact\":\"HIGH\", \
\"availabilityImpact\":\"HIGH\", \
\"baseScore\":7.8, \
\"baseSeverity\":\"HIGH\"}, \
\"exploitabilityScore\":1.8, \
\"impactScore\":5.9}], \
\"cvssMetricV2\":[{ \
\"source\":\"nvd@nist.gov\", \
\"type\":\"Primary\", \
\"cvssData\":{ \
\"version\":\"2.0\", \
\"vectorString\":\"AV:L/AC:L/Au:N/C:P/I:P/A:P\", \
\"accessVector\":\"LOCAL\", \
\"accessComplexity\":\"LOW\", \
\"authentication\":\"NONE\", \
\"confidentialityImpact\":\"PARTIAL\", \
\"integrityImpact\":\"PARTIAL\", \
\"availabilityImpact\":\"PARTIAL\", \
\"baseScore\":4.6}, \
\"baseSeverity\":\"MEDIUM\", \
\"exploitabilityScore\":3.9, \
\"impactScore\":6.4, \
\"acInsufInfo\":false, \
\"obtainAllPrivilege\":false, \
\"obtainUserPrivilege\":false, \
\"obtainOtherPrivilege\":false, \
\"userInteractionRequired\":false}]}, \
\"weaknesses\":[{ \
\"source\":\"nvd@nist.gov\", \
\"type\":\"Primary\", \
\"description\":[{ \
\"lang\":\"en\", \
\"value\":\"CWE-120\"}]}], \
\"configurations\":[{ \
\"nodes\":[{ \
\"operator\":\"OR\", \
\"negate\":false, \
\"cpeMatch\":[{ \
\"vulnerable\":true, \
\"criteria\":\"cpe:2.3:a:intel:graphics_driver:*:*:*:*:*:*:*:*\", \
\"versionStartIncluding\":\"15.40\", \
\"versionEndExcluding\":\"15.40.44.5107\", \
\"matchCriteriaId\":\"A7250C6F-1E48-4E0B-AACB-52D4FC3F494D\"}, \
{\"vulnerable\":true, \
\"criteria\":\"cpe:2.3:a:intel:graphics_driver:*:*:*:*:*:*:*:*\", \
\"versionStartIncluding\":\"15.45\", \
\"versionEndExcluding\":\"15.45.30.5103\", \
\"matchCriteriaId\":\"AB43B202-E327-43C3-8F68-A1CC5E6C8300\"}, \
{\"vulnerable\":true, \
\"criteria\":\"cpe:2.3:a:intel:graphics_driver:*:*:*:*:*:*:*:*\", \
\"versionStartIncluding\":\"26.20\", \
\"versionEndExcluding\":\"26.20.100.7158\", \
\"matchCriteriaId\":\"0819D727-C1B5-410F-93D6-098B388756CE\"}]}]}], \
\"references\":[{ \
\"url\":\"https://security.netapp.com/advisory/ntap-20200320-0003/\", \
\"source\":\"secure@intel.com\", \
\"tags\":[\"Third Party Advisory\"]}, \
{\"url\":\"https://www.intel.com/content/www/us/en/security-center/advisory/intel-sa-00315.html\", \
\"source\":\"secure@intel.com\", \
\"tags\":[\"Vendor Advisory\"]}]}}]}]}";

static int setup_json_parser_decode(void **state)
{
    test_mode = 0;

    wm_vuldet_db *parsed_vulnerabilities = NULL;
    os_calloc(1, sizeof(wm_vuldet_db), parsed_vulnerabilities);

    if (!parsed_vulnerabilities)
    {
        return 0;
    }

    *state = parsed_vulnerabilities;

    return 0;
}

static int setup_json_parser_nvd(void **state)
{
    test_mode = 0;

    nvd_vulnerability  *nvd_vulnerabilities = NULL;
    os_calloc(1, sizeof(nvd_vulnerability), nvd_vulnerabilities);

    if (!nvd_vulnerabilities)
    {
        return 0;
    }

    *state = nvd_vulnerabilities;

    return 0;
}

static int teardown_json_parser_nvd(void **state)
{
    test_mode = 0;

    nvd_vulnerability *nvd_vulnerabilities = *state;

    if (nvd_vulnerabilities)
    {
        wm_vuldet_free_nvd_list(nvd_vulnerabilities);
    }

    return 0;
}

static int teardown_json_parser_decode(void **state)
{
    test_mode = 0;

    wm_vuldet_db *parsed_vulnerabilities = *state;
  
    if (parsed_vulnerabilities)
    {

        if (parsed_vulnerabilities->nvd_vulnerabilities)
        {
            wm_vuldet_free_nvd_list(parsed_vulnerabilities->nvd_vulnerabilities);
        }
       
        os_free(parsed_vulnerabilities);
    }

    return 0;
}

static int setup_version(void **state)
{
    char *version;

    os_calloc(15, sizeof(char*), version);

    if (!version) {
        return -1;
    }

    state[0] = version;

    return 0;
}

static int setup_scan_agent(void **state)
{
    scan_agent *agent = calloc(1, sizeof(scan_agent));

    if (!agent)
    {
        return -1;
    }

    agent->agent_id = strdup("000");
    *state = agent;

    return 0;
}

static int setup_children(void **state)
{

    int *children;

    os_calloc(1, sizeof(int), children);

    if (!children)
        return -1;

    state[0] = children;

    return 0;
}

static int setup_siblings(void **state)
{

    int *siblings;

    os_calloc(1, sizeof(int), siblings);

    if (!siblings)
        return -1;

    state[0] = siblings;

    return 0;
}

/* teardown */

static int teardown_version(void **state)
{
    char *version = state[0];

    if (version)
    {
        free(version);
    }

    return 0;
}

static int teardown_scan_agent(void **state)
{
    scan_agent *agent = *state;

    if (agent)
    {
        wm_vuldet_free_scan_agent(agent);
    }

    return 0;
}

static int teardown_children(void **state)
{
    int *children = state[0];

    if (children)
    {
        free(children);
    }

    return 0;
}

static int teardown_siblings(void **state)
{
    int *siblings = state[0];

    if (siblings)
    {
        free(siblings);
    }

    return 0;
}

/* tests */

/* wm_vuldet_clean_version */

void test_wm_vuldet_clean_version_complete(void **state)
{
    char *version = state[0];

    strcpy(version, "1:test-release");

    char *ret = wm_vuldet_clean_version(version);
    assert_string_equal(ret, "test");

    os_free(ret);
}

void test_wm_vuldet_clean_version_no_epoch(void **state)
{
    char *version = state[0];

    strcpy(version, "test-release");

    char *ret = wm_vuldet_clean_version(version);
    assert_string_equal(ret, "test");

    os_free(ret);
}

void test_wm_vuldet_clean_version_no_release(void **state)
{
    char *version = state[0];

    strcpy(version, "1:test");

    char *ret = wm_vuldet_clean_version(version);
    assert_string_equal(ret, "test");

    os_free(ret);
}

void test_wm_vuldet_clean_version_no_epoch_release(void **state)
{
    char *version = state[0];

    strcpy(version, "test");

    char *ret = wm_vuldet_clean_version(version);
    assert_string_equal(ret, "test");

    os_free(ret);
}

/* wm_vuldet_get_children */

void test_wm_vuldet_get_children_prepare_error(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    int configuration_id = 0;
    int package_id = 0;
    int *children = state[0];

    will_return(__wrap_sqlite3_prepare_v2, NULL);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_ERROR);
    will_return(__wrap_sqlite3_errmsg, "error");
    expect_string(__wrap__mterror, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mterror, formatted_msg, "(5571): Couldn't get from the NVD the 'children' dependencies of the package with ID '0'");
    expect_string(__wrap__mterror, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mterror, formatted_msg, "(5503): SQL error: 'error'");

    int ret = wm_vuldet_get_children(db, configuration_id, package_id, children);
    assert_int_equal(ret, -1);
}

void test_wm_vuldet_get_children_done(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    int configuration_id = 0;
    int package_id = 0;

    int *children = state[0];

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_int, index, 1);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);
    expect_value(__wrap_sqlite3_bind_int, index, 2);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_get_children(db, configuration_id, package_id, children);
    assert_int_equal(ret, 0);
}

void test_wm_vuldet_get_children_OK(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    int configuration_id = 0;
    int package_id = 0;

    int *children = state[0];

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_int, index, 1);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);
    expect_value(__wrap_sqlite3_bind_int, index, 2);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_int, iCol, 0);
    will_return(__wrap_sqlite3_column_int, 1);

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_get_children(db, configuration_id, package_id, children);
    assert_int_equal(ret, 0);
}

/* wm_vuldet_get_siblings */

void test_wm_vuldet_get_siblings_prepare_error(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    int configuration_id = 0;
    int parent = 0;

    int *siblings = state[0];

    will_return(__wrap_sqlite3_prepare_v2, NULL);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_ERROR);
    will_return(__wrap_sqlite3_errmsg, "error");
    expect_string(__wrap__mterror, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mterror, formatted_msg, "(5571): Couldn't get from the NVD the 'siblings' dependencies of the package with ID '0'");
    expect_string(__wrap__mterror, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mterror, formatted_msg, "(5503): SQL error: 'error'");

    int ret = wm_vuldet_get_siblings(db, configuration_id, parent, siblings);
    assert_int_equal(ret, -1);
}

void test_wm_vuldet_get_siblings_done(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    int configuration_id = 0;
    int parent = 0;

    int *siblings = state[0];

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_int, index, 1);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);
    expect_value(__wrap_sqlite3_bind_int, index, 2);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_get_siblings(db, configuration_id, parent, siblings);
    assert_int_equal(ret, 0);
}

void test_wm_vuldet_get_siblings_OK(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    int configuration_id = 0;
    int parent = 0;

    int *siblings = state[0];

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_int, index, 1);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);
    expect_value(__wrap_sqlite3_bind_int, index, 2);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_int, iCol, 0);
    will_return(__wrap_sqlite3_column_int, 1);

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_get_siblings(db, configuration_id, parent, siblings);
    assert_int_equal(ret, 0);
}

/* test vw_vuldet_filter_vulnerabilities */

void test_vw_vuldet_filter_vulnerabilities_vendor_wrong_Ubuntu(void **state)
{
    (void)state;
    vu_feed feed = FEED_UBUNTU;
    const char *vendor = "redhat";
    const char *target_sw = "*";

    int ret = vw_vuldet_filter_vulnerabilities(feed, vendor, target_sw);

    assert_int_equal(ret, 1);
}

void test_vw_vuldet_filter_vulnerabilities_vendor_wrong_Arch(void **state)
{
    (void)state;
    vu_feed feed = FEED_ARCH;
    const char *vendor = "redhat";
    const char *target_sw = "*";

    int ret = vw_vuldet_filter_vulnerabilities(feed, vendor, target_sw);

    assert_int_equal(ret, 1);
}

void test_vw_vuldet_filter_vulnerabilities_vendor_wrong_Arch_vendor(void **state)
{
    (void)state;
    vu_feed feed = FEED_REDHAT;
    const char *vendor = "archlinux";
    const char *target_sw = "*";

    int ret = vw_vuldet_filter_vulnerabilities(feed, vendor, target_sw);

    assert_int_equal(ret, 1);
}

void test_vw_vuldet_filter_vulnerabilities_vendor_wrong_Debian(void **state)
{
    (void)state;
    vu_feed feed = FEED_DEBIAN;
    const char *vendor = "redhat";
    const char *target_sw = "*";

    int ret = vw_vuldet_filter_vulnerabilities(feed, vendor, target_sw);

    assert_int_equal(ret, 1);
}

void test_vw_vuldet_filter_vulnerabilities_vendor_wrong_Redhat(void **state)
{
    (void)state;
    vu_feed feed = FEED_REDHAT;
    const char *vendor = "debian";
    const char *target_sw = "*";

    int ret = vw_vuldet_filter_vulnerabilities(feed, vendor, target_sw);

    assert_int_equal(ret, 1);
}

void test_vw_vuldet_filter_vulnerabilities_vendor_wrong_Amazon(void **state)
{
    (void)state;
    vu_feed feed = FEED_ALAS;
    const char *vendor = "redhat";
    const char *target_sw = "*";

    int ret = vw_vuldet_filter_vulnerabilities(feed, vendor, target_sw);

    assert_int_equal(ret, 1);
}

void test_vw_vuldet_filter_vulnerabilities_vendor_wrong_SUSE(void **state)
{
    (void)state;
    vu_feed feed = FEED_SUSE;
    const char *vendor = "redhat";
    const char *target_sw = "*";

    int ret = vw_vuldet_filter_vulnerabilities(feed, vendor, target_sw);

    assert_int_equal(ret, 1);
}

void test_vw_vuldet_filter_vulnerabilities_target_sw_not_whitelist(void **state)
{
    (void)state;
    vu_feed feed = FEED_DEBIAN;
    const char *vendor = "vendor";
    const char *target_sw = "jenkins";

    int ret = vw_vuldet_filter_vulnerabilities(feed, vendor, target_sw);

    assert_int_equal(ret, 1);
}

void test_vw_vuldet_filter_vulnerabilities_vendor_target_sw_OK(void **state)
{
    (void)state;
    vu_feed feed = FEED_DEBIAN;
    const char *vendor = "vendor";
    const char *target_sw = "*";

    int ret = vw_vuldet_filter_vulnerabilities(feed, vendor, target_sw);

    assert_int_equal(ret, 0);
}

void test_vw_vuldet_filter_vulnerabilities_vendor_target_sw_mac_OK(void **state)
{
    (void)state;
    vu_feed feed = FEED_MAC;
    const char *vendor = "vendor";
    const char *target_sw[] = {
        "*",
        "mac",
        "mac_os",
        "mac_os_x",
        "macos"};
    int size_array = array_size(target_sw);
    int i;
    int ret;

    for (i = 0; i < size_array; i++)
    {
        ret = vw_vuldet_filter_vulnerabilities(feed, vendor, target_sw[i]);
        assert_int_equal(ret, 0);
    }
}

void test_vw_vuldet_filter_vulnerabilities_vendor_target_sw_mac_Err(void **state)
{
    (void)state;
    vu_feed feed = FEED_MAC;
    const char *vendor = "vendor";
    const char *target_sw[] = {
        "linux",
        "linux_kernel",
        "mupdf",
        "gpl_ghostscript"};
    int size_array = array_size(target_sw);
    int i;
    int ret;

    for (i = 0; i < size_array; i++)
    {
        ret = vw_vuldet_filter_vulnerabilities(feed, vendor, target_sw[i]);
        assert_int_equal(ret, 1);
    }
}

/* test wm_vuldet_check_generic_package */

void test_wm_vuldet_check_generic_package_prepare_error(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "test";
    char *pkg_version = NULL;
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = NULL;
    int8_t name_type = PACKAGE_SOURCE;
    int *vuln_count = NULL;

    will_return(__wrap_sqlite3_prepare_v2, NULL);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_ERROR);
    will_return(__wrap_sqlite3_errmsg, "error");
    expect_string(__wrap__mterror, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mterror, formatted_msg, "(5570): Couldn't get from the NVD the 'generic' vulnerabilities of the package 'test'");
    expect_string(__wrap__mterror, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mterror, formatted_msg, "(5503): SQL error: 'error'");

    int ret = wm_vuldet_check_generic_package(db,
                                              FEED_DEBIAN,
                                              pkg_name,
                                              pkg_source,
                                              pkg_version,
                                              pkg_arch,
                                              pkg_vendor,
                                              pkg_reference,
                                              pkg_type,
                                              cve_table,
                                              name_type,
                                              vuln_count);
    assert_int_equal(ret, -1);
}

void test_wm_vuldet_check_generic_package_done(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "test";
    char *pkg_version = NULL;
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = NULL;
    int8_t name_type = PACKAGE_SOURCE;
    int *vuln_count = NULL;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "test");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_generic_package(db,
                                              FEED_DEBIAN,
                                              pkg_name,
                                              pkg_source,
                                              pkg_version,
                                              pkg_arch,
                                              pkg_vendor,
                                              pkg_reference,
                                              pkg_type,
                                              cve_table,
                                              name_type,
                                              vuln_count);
    assert_int_equal(ret, 0);
}

void test_wm_vuldet_check_generic_package_skip(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "test";
    char *pkg_version = NULL;
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = NULL;
    int8_t name_type = PACKAGE_SOURCE;
    int *vuln_count = NULL;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "test");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "1.0.0"); // start_including
    expect_value(__wrap_sqlite3_column_text, iCol, 2);
    will_return(__wrap_sqlite3_column_text, NULL); // start_excluding
    expect_value(__wrap_sqlite3_column_text, iCol, 3);
    will_return(__wrap_sqlite3_column_text, NULL); // end_including
    expect_value(__wrap_sqlite3_column_text, iCol, 4);
    will_return(__wrap_sqlite3_column_text, NULL); // end_excluding
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 6);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 7);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 8);
    will_return(__wrap_sqlite3_column_int, 0); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 9);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 10);
    will_return(__wrap_sqlite3_column_text, "redhat"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 11);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_generic_package(db,
                                              FEED_DEBIAN,
                                              pkg_name,
                                              pkg_source,
                                              pkg_version,
                                              pkg_arch,
                                              pkg_vendor,
                                              pkg_reference,
                                              pkg_type,
                                              cve_table,
                                              name_type,
                                              vuln_count);
    assert_int_equal(ret, 0);
}

void test_wm_vuldet_check_generic_package_start_including_no_vulnerable(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "test";
    char *pkg_version = "0.0.0";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = NULL;
    int8_t name_type = PACKAGE_SOURCE;
    int *vuln_count = NULL;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "test");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "1.0.0"); // start_including
    expect_value(__wrap_sqlite3_column_text, iCol, 2);
    will_return(__wrap_sqlite3_column_text, NULL); // start_excluding
    expect_value(__wrap_sqlite3_column_text, iCol, 3);
    will_return(__wrap_sqlite3_column_text, NULL); // end_including
    expect_value(__wrap_sqlite3_column_text, iCol, 4);
    will_return(__wrap_sqlite3_column_text, NULL); // end_excluding
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 6);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 7);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 8);
    will_return(__wrap_sqlite3_column_int, 0); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 9);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 10);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 11);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, 0);

    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5460): Package 'test' not vulnerable to 'CVE-0000-0000'. Version (0.0.0) not 'greater than or equal' '1.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_generic_package(db,
                                              FEED_DEBIAN,
                                              pkg_name,
                                              pkg_source,
                                              pkg_version,
                                              pkg_arch,
                                              pkg_vendor,
                                              pkg_reference,
                                              pkg_type,
                                              cve_table,
                                              name_type,
                                              vuln_count);
    assert_int_equal(ret, 0);
}

void test_wm_vuldet_check_generic_package_start_including_vulnerable(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "test";
    char *pkg_version = "0.0.0";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = (OSHash *)1;
    int8_t name_type = PACKAGE_SOURCE;
    int i = 1;
    int *vuln_count = &i;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "test");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "0.0.0"); // start_including
    expect_value(__wrap_sqlite3_column_text, iCol, 2);
    will_return(__wrap_sqlite3_column_text, NULL); // start_excluding
    expect_value(__wrap_sqlite3_column_text, iCol, 3);
    will_return(__wrap_sqlite3_column_text, NULL); // end_including
    expect_value(__wrap_sqlite3_column_text, iCol, 4);
    will_return(__wrap_sqlite3_column_text, NULL); // end_excluding
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 6);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 7);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 8);
    will_return(__wrap_sqlite3_column_int, 0); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 9);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 10);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 11);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    will_return(__wrap_wm_vuldet_add_cve_node, 0);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5458): Package 'test' inserted into the vulnerability 'CVE-0000-0000'. Version (0.0.0) 'greater than or equal' '0.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_generic_package(db,
                                              FEED_DEBIAN,
                                              pkg_name,
                                              pkg_source,
                                              pkg_version,
                                              pkg_arch,
                                              pkg_vendor,
                                              pkg_reference,
                                              pkg_type,
                                              cve_table,
                                              name_type,
                                              vuln_count);
    assert_int_equal(ret, 1);
}

void test_wm_vuldet_check_generic_package_start_excluding_no_vulnerable(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "test";
    char *pkg_version = "0.0.0";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = NULL;
    int8_t name_type = PACKAGE_SOURCE;
    int *vuln_count = NULL;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "test");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, NULL); // start_including
    expect_value(__wrap_sqlite3_column_text, iCol, 2);
    will_return(__wrap_sqlite3_column_text, "1.0.0"); // start_excluding
    expect_value(__wrap_sqlite3_column_text, iCol, 3);
    will_return(__wrap_sqlite3_column_text, NULL); // end_including
    expect_value(__wrap_sqlite3_column_text, iCol, 4);
    will_return(__wrap_sqlite3_column_text, NULL); // end_excluding
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 6);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 7);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 8);
    will_return(__wrap_sqlite3_column_int, 0); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 9);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 10);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 11);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, 0);

    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5460): Package 'test' not vulnerable to 'CVE-0000-0000'. Version (0.0.0) not 'greater than' '1.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_generic_package(db,
                                              FEED_DEBIAN,
                                              pkg_name,
                                              pkg_source,
                                              pkg_version,
                                              pkg_arch,
                                              pkg_vendor,
                                              pkg_reference,
                                              pkg_type,
                                              cve_table,
                                              name_type,
                                              vuln_count);
    assert_int_equal(ret, 0);
}

void test_wm_vuldet_check_generic_package_start_excluding_vulnerable(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "test";
    char *pkg_version = "0.0.0";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = (OSHash *)1;
    int8_t name_type = PACKAGE_SOURCE;
    int i = 1;
    int *vuln_count = &i;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "test");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, NULL); // start_including
    expect_value(__wrap_sqlite3_column_text, iCol, 2);
    will_return(__wrap_sqlite3_column_text, "1.0.0"); // start_excluding
    expect_value(__wrap_sqlite3_column_text, iCol, 3);
    will_return(__wrap_sqlite3_column_text, NULL); // end_including
    expect_value(__wrap_sqlite3_column_text, iCol, 4);
    will_return(__wrap_sqlite3_column_text, NULL); // end_excluding
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 6);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 7);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 8);
    will_return(__wrap_sqlite3_column_int, 0); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 9);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 10);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 11);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    will_return(__wrap_wm_vuldet_add_cve_node, 0);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5458): Package 'test' inserted into the vulnerability 'CVE-0000-0000'. Version (0.0.0) 'greater than' '1.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_generic_package(db,
                                              FEED_DEBIAN,
                                              pkg_name,
                                              pkg_source,
                                              pkg_version,
                                              pkg_arch,
                                              pkg_vendor,
                                              pkg_reference,
                                              pkg_type,
                                              cve_table,
                                              name_type,
                                              vuln_count);
    assert_int_equal(ret, 1);
}

void test_wm_vuldet_check_generic_package_no_starts_no_vulnerable(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "test";
    char *pkg_version = "2.0.0";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = NULL;
    int8_t name_type = PACKAGE_SOURCE;
    int i = 1;
    int *vuln_count = &i;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "test");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, NULL); // start_including
    expect_value(__wrap_sqlite3_column_text, iCol, 2);
    will_return(__wrap_sqlite3_column_text, NULL); // start_excluding
    expect_value(__wrap_sqlite3_column_text, iCol, 3);
    will_return(__wrap_sqlite3_column_text, "1.0.0"); // end_including
    expect_value(__wrap_sqlite3_column_text, iCol, 4);
    will_return(__wrap_sqlite3_column_text, NULL); // end_excluding
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 6);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 7);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 8);
    will_return(__wrap_sqlite3_column_int, 0); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 9);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 10);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 11);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, 0);

    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5460): Package 'test' not vulnerable to 'CVE-0000-0000'. Version (2.0.0) not 'less than or equal' '1.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_generic_package(db,
                                              FEED_DEBIAN,
                                              pkg_name,
                                              pkg_source,
                                              pkg_version,
                                              pkg_arch,
                                              pkg_vendor,
                                              pkg_reference,
                                              pkg_type,
                                              cve_table,
                                              name_type,
                                              vuln_count);
    assert_int_equal(ret, 0);
}

void test_wm_vuldet_check_generic_package_no_starts_vulnerable(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "test";
    char *pkg_version = "0.0.0";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = (OSHash *)1;
    int8_t name_type = PACKAGE_SOURCE;
    int i = 1;
    int *vuln_count = &i;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "test");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, NULL); // start_including
    expect_value(__wrap_sqlite3_column_text, iCol, 2);
    will_return(__wrap_sqlite3_column_text, NULL); // start_excluding
    expect_value(__wrap_sqlite3_column_text, iCol, 3);
    will_return(__wrap_sqlite3_column_text, "1.0.0"); // end_including
    expect_value(__wrap_sqlite3_column_text, iCol, 4);
    will_return(__wrap_sqlite3_column_text, NULL); // end_excluding
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 6);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 7);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 8);
    will_return(__wrap_sqlite3_column_int, 0); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 9);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 10);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 11);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    will_return(__wrap_wm_vuldet_add_cve_node, 0);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5458): Package 'test' inserted into the vulnerability 'CVE-0000-0000'. Version (0.0.0) 'less than or equal' '1.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_generic_package(db,
                                              FEED_DEBIAN,
                                              pkg_name,
                                              pkg_source,
                                              pkg_version,
                                              pkg_arch,
                                              pkg_vendor,
                                              pkg_reference,
                                              pkg_type,
                                              cve_table,
                                              name_type,
                                              vuln_count);
    assert_int_equal(ret, 1);
}

void test_wm_vuldet_check_generic_package_end_including_no_vulnerable(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "test";
    char *pkg_version = "2.0.0";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = NULL;
    int8_t name_type = PACKAGE_SOURCE;
    int *vuln_count = NULL;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "test");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, NULL); // start_including
    expect_value(__wrap_sqlite3_column_text, iCol, 2);
    will_return(__wrap_sqlite3_column_text, NULL); // start_excluding
    expect_value(__wrap_sqlite3_column_text, iCol, 3);
    will_return(__wrap_sqlite3_column_text, "1.0.0"); // end_including
    expect_value(__wrap_sqlite3_column_text, iCol, 4);
    will_return(__wrap_sqlite3_column_text, NULL); // end_excluding
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 6);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 7);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 8);
    will_return(__wrap_sqlite3_column_int, 0); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 9);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 10);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 11);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, 0);

    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5460): Package 'test' not vulnerable to 'CVE-0000-0000'. Version (2.0.0) not 'less than or equal' '1.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_generic_package(db,
                                              FEED_DEBIAN,
                                              pkg_name,
                                              pkg_source,
                                              pkg_version,
                                              pkg_arch,
                                              pkg_vendor,
                                              pkg_reference,
                                              pkg_type,
                                              cve_table,
                                              name_type,
                                              vuln_count);
    assert_int_equal(ret, 0);
}

void test_wm_vuldet_check_generic_package_end_including_vulnerable(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "test";
    char *pkg_version = "0.0.0";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = (OSHash *)1;
    int8_t name_type = PACKAGE_SOURCE;
    int i = 1;
    int *vuln_count = &i;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "test");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, NULL); // start_including
    expect_value(__wrap_sqlite3_column_text, iCol, 2);
    will_return(__wrap_sqlite3_column_text, NULL); // start_excluding
    expect_value(__wrap_sqlite3_column_text, iCol, 3);
    will_return(__wrap_sqlite3_column_text, "1.0.0"); // end_including
    expect_value(__wrap_sqlite3_column_text, iCol, 4);
    will_return(__wrap_sqlite3_column_text, NULL); // end_excluding
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 6);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 7);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 8);
    will_return(__wrap_sqlite3_column_int, 0); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 9);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 10);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 11);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    will_return(__wrap_wm_vuldet_add_cve_node, 0);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5458): Package 'test' inserted into the vulnerability 'CVE-0000-0000'. Version (0.0.0) 'less than or equal' '1.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_generic_package(db,
                                              FEED_DEBIAN,
                                              pkg_name,
                                              pkg_source,
                                              pkg_version,
                                              pkg_arch,
                                              pkg_vendor,
                                              pkg_reference,
                                              pkg_type,
                                              cve_table,
                                              name_type,
                                              vuln_count);
    assert_int_equal(ret, 1);
}

void test_wm_vuldet_check_generic_package_end_excluding_no_vulnerable(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "test";
    char *pkg_version = "2.0.0";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = NULL;
    int8_t name_type = PACKAGE_SOURCE;
    int *vuln_count = NULL;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "test");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, NULL); // start_including
    expect_value(__wrap_sqlite3_column_text, iCol, 2);
    will_return(__wrap_sqlite3_column_text, NULL); // start_excluding
    expect_value(__wrap_sqlite3_column_text, iCol, 3);
    will_return(__wrap_sqlite3_column_text, NULL); // end_including
    expect_value(__wrap_sqlite3_column_text, iCol, 4);
    will_return(__wrap_sqlite3_column_text, "1.0.0"); // end_excluding
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 6);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 7);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 8);
    will_return(__wrap_sqlite3_column_int, 0); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 9);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 10);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 11);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, 0);

    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5460): Package 'test' not vulnerable to 'CVE-0000-0000'. Version (2.0.0) not 'less than' '1.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_generic_package(db,
                                              FEED_DEBIAN,
                                              pkg_name,
                                              pkg_source,
                                              pkg_version,
                                              pkg_arch,
                                              pkg_vendor,
                                              pkg_reference,
                                              pkg_type,
                                              cve_table,
                                              name_type,
                                              vuln_count);
    assert_int_equal(ret, 0);
}

void test_wm_vuldet_check_generic_package_end_excluding_vulnerable(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "test";
    char *pkg_version = "0.0.0";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = (OSHash *)1;
    int8_t name_type = PACKAGE_SOURCE;
    int i = 1;
    int *vuln_count = &i;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "test");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, NULL); // start_including
    expect_value(__wrap_sqlite3_column_text, iCol, 2);
    will_return(__wrap_sqlite3_column_text, NULL); // start_excluding
    expect_value(__wrap_sqlite3_column_text, iCol, 3);
    will_return(__wrap_sqlite3_column_text, NULL); // end_including
    expect_value(__wrap_sqlite3_column_text, iCol, 4);
    will_return(__wrap_sqlite3_column_text, "1.0.0"); // end_excluding
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 6);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 7);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 8);
    will_return(__wrap_sqlite3_column_int, 0); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 9);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 10);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 11);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    will_return(__wrap_wm_vuldet_add_cve_node, 0);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5458): Package 'test' inserted into the vulnerability 'CVE-0000-0000'. Version (0.0.0) 'less than' '1.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_generic_package(db,
                                              FEED_DEBIAN,
                                              pkg_name,
                                              pkg_source,
                                              pkg_version,
                                              pkg_arch,
                                              pkg_vendor,
                                              pkg_reference,
                                              pkg_type,
                                              cve_table,
                                              name_type,
                                              vuln_count);
    assert_int_equal(ret, 1);
}

void test_wm_vuldet_check_generic_package_no_ends_no_vulnerable(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "test";
    char *pkg_version = "1.0.0";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = NULL;
    int8_t name_type = PACKAGE_SOURCE;
    int i = 1;
    int *vuln_count = &i;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "test");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "2.0.0"); // start_including
    expect_value(__wrap_sqlite3_column_text, iCol, 2);
    will_return(__wrap_sqlite3_column_text, NULL); // start_excluding
    expect_value(__wrap_sqlite3_column_text, iCol, 3);
    will_return(__wrap_sqlite3_column_text, NULL); // end_including
    expect_value(__wrap_sqlite3_column_text, iCol, 4);
    will_return(__wrap_sqlite3_column_text, NULL); // end_excluding
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 6);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 7);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 8);
    will_return(__wrap_sqlite3_column_int, 0); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 9);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 10);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 11);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, 0);

    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5460): Package 'test' not vulnerable to 'CVE-0000-0000'. Version (1.0.0) not 'greater than or equal' '2.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_generic_package(db,
                                              FEED_DEBIAN,
                                              pkg_name,
                                              pkg_source,
                                              pkg_version,
                                              pkg_arch,
                                              pkg_vendor,
                                              pkg_reference,
                                              pkg_type,
                                              cve_table,
                                              name_type,
                                              vuln_count);
    assert_int_equal(ret, 0);
}

void test_wm_vuldet_check_generic_package_no_ends_vulnerable(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "test";
    char *pkg_version = "1.0.0";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = (OSHash *)1;
    int8_t name_type = PACKAGE_SOURCE;
    int i = 1;
    int *vuln_count = &i;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "test");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "0.0.0"); // start_including
    expect_value(__wrap_sqlite3_column_text, iCol, 2);
    will_return(__wrap_sqlite3_column_text, NULL); // start_excluding
    expect_value(__wrap_sqlite3_column_text, iCol, 3);
    will_return(__wrap_sqlite3_column_text, NULL); // end_including
    expect_value(__wrap_sqlite3_column_text, iCol, 4);
    will_return(__wrap_sqlite3_column_text, NULL); // end_excluding
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 6);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 7);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 8);
    will_return(__wrap_sqlite3_column_int, 0); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 9);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 10);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 11);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    will_return(__wrap_wm_vuldet_add_cve_node, 0);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5458): Package 'test' inserted into the vulnerability 'CVE-0000-0000'. Version (1.0.0) 'greater than or equal' '0.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_generic_package(db,
                                              FEED_DEBIAN,
                                              pkg_name,
                                              pkg_source,
                                              pkg_version,
                                              pkg_arch,
                                              pkg_vendor,
                                              pkg_reference,
                                              pkg_type,
                                              cve_table,
                                              name_type,
                                              vuln_count);
    assert_int_equal(ret, 1);
}

void test_wm_vuldet_check_generic_package_no_starts_no_ends_children(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "test";
    char *pkg_version = "0.0.0";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = (OSHash *)1;
    int8_t name_type = PACKAGE_SOURCE;
    int i = 1;
    int *vuln_count = &i;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "test");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, NULL); // start_including
    expect_value(__wrap_sqlite3_column_text, iCol, 2);
    will_return(__wrap_sqlite3_column_text, NULL); // start_excluding
    expect_value(__wrap_sqlite3_column_text, iCol, 3);
    will_return(__wrap_sqlite3_column_text, NULL); // end_including
    expect_value(__wrap_sqlite3_column_text, iCol, 4);
    will_return(__wrap_sqlite3_column_text, NULL); // end_excluding
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 6);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 7);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 8);
    will_return(__wrap_sqlite3_column_int, 0); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 9);
    will_return(__wrap_sqlite3_column_text, "AND"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 10);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 11);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    // Children
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_int, index, 1);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);
    expect_value(__wrap_sqlite3_bind_int, index, 2);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_int, iCol, 0);
    will_return(__wrap_sqlite3_column_int, 1);

    expect_sqlite3_step_call(SQLITE_DONE);

    will_return(__wrap_wm_vuldet_add_cve_node, 0);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5458): Package 'test' inserted into the vulnerability 'CVE-0000-0000'. Version (0.0.0) 'equals' '*' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_generic_package(db,
                                              FEED_DEBIAN,
                                              pkg_name,
                                              pkg_source,
                                              pkg_version,
                                              pkg_arch,
                                              pkg_vendor,
                                              pkg_reference,
                                              pkg_type,
                                              cve_table,
                                              name_type,
                                              vuln_count);
    assert_int_equal(ret, 1);
}

void test_wm_vuldet_check_generic_package_no_children_no_siblings(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "test";
    char *pkg_version = "1.0.0";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = (OSHash *)1;
    int8_t name_type = PACKAGE_SOURCE;
    int i = 1;
    int *vuln_count = &i;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "test");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "0.0.0"); // start_including
    expect_value(__wrap_sqlite3_column_text, iCol, 2);
    will_return(__wrap_sqlite3_column_text, NULL); // start_excluding
    expect_value(__wrap_sqlite3_column_text, iCol, 3);
    will_return(__wrap_sqlite3_column_text, "1.0.0"); // end_including
    expect_value(__wrap_sqlite3_column_text, iCol, 4);
    will_return(__wrap_sqlite3_column_text, NULL); // end_excluding
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 6);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 7);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 8);
    will_return(__wrap_sqlite3_column_int, 0); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 9);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 10);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 11);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    will_return(__wrap_wm_vuldet_add_cve_node, 0);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5458): Package 'test' inserted into the vulnerability 'CVE-0000-0000'. Version (1.0.0) 'less than or equal' '1.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_generic_package(db,
                                              FEED_DEBIAN,
                                              pkg_name,
                                              pkg_source,
                                              pkg_version,
                                              pkg_arch,
                                              pkg_vendor,
                                              pkg_reference,
                                              pkg_type,
                                              cve_table,
                                              name_type,
                                              vuln_count);
    assert_int_equal(ret, 1);
}

void test_wm_vuldet_check_generic_package_children_invalid(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "test";
    char *pkg_version = "1.0.0";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = NULL;
    int8_t name_type = PACKAGE_SOURCE;
    int i = 1;
    int *vuln_count = &i;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "test");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "0.0.0"); // start_including
    expect_value(__wrap_sqlite3_column_text, iCol, 2);
    will_return(__wrap_sqlite3_column_text, NULL); // start_excluding
    expect_value(__wrap_sqlite3_column_text, iCol, 3);
    will_return(__wrap_sqlite3_column_text, "1.0.0"); // end_including
    expect_value(__wrap_sqlite3_column_text, iCol, 4);
    will_return(__wrap_sqlite3_column_text, NULL); // end_excluding
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 6);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 7);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 8);
    will_return(__wrap_sqlite3_column_int, 0); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 9);
    will_return(__wrap_sqlite3_column_text, "AND"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 10);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 11);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    // Children
    will_return(__wrap_sqlite3_prepare_v2, NULL);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_ERROR);
    will_return(__wrap_sqlite3_errmsg, "error");
    expect_string(__wrap__mterror, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mterror, formatted_msg, "(5571): Couldn't get from the NVD the 'children' dependencies of the package with ID '0'");
    expect_string(__wrap__mterror, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mterror, formatted_msg, "(5503): SQL error: 'error'");

    int ret = wm_vuldet_check_generic_package(db,
                                              FEED_DEBIAN,
                                              pkg_name,
                                              pkg_source,
                                              pkg_version,
                                              pkg_arch,
                                              pkg_vendor,
                                              pkg_reference,
                                              pkg_type,
                                              cve_table,
                                              name_type,
                                              vuln_count);
    assert_int_equal(ret, -1);
}

void test_wm_vuldet_check_generic_package_children_OK(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "test";
    char *pkg_version = "1.0.0";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = (OSHash *)1;
    int8_t name_type = PACKAGE_SOURCE;
    int i = 1;
    int *vuln_count = &i;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "test");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "0.0.0"); // start_including
    expect_value(__wrap_sqlite3_column_text, iCol, 2);
    will_return(__wrap_sqlite3_column_text, NULL); // start_excluding
    expect_value(__wrap_sqlite3_column_text, iCol, 3);
    will_return(__wrap_sqlite3_column_text, "1.0.0"); // end_including
    expect_value(__wrap_sqlite3_column_text, iCol, 4);
    will_return(__wrap_sqlite3_column_text, NULL); // end_excluding
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 6);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 7);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 8);
    will_return(__wrap_sqlite3_column_int, 0); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 9);
    will_return(__wrap_sqlite3_column_text, "AND"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 10);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 11);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    // Children
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_int, index, 1);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);
    expect_value(__wrap_sqlite3_bind_int, index, 2);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_int, iCol, 0);
    will_return(__wrap_sqlite3_column_int, 1);

    expect_sqlite3_step_call(SQLITE_DONE);

    will_return(__wrap_wm_vuldet_add_cve_node, 0);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5458): Package 'test' inserted into the vulnerability 'CVE-0000-0000'. Version (1.0.0) 'less than or equal' '1.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_generic_package(db,
                                              FEED_DEBIAN,
                                              pkg_name,
                                              pkg_source,
                                              pkg_version,
                                              pkg_arch,
                                              pkg_vendor,
                                              pkg_reference,
                                              pkg_type,
                                              cve_table,
                                              name_type,
                                              vuln_count);
    assert_int_equal(ret, 1);
}

void test_wm_vuldet_check_generic_package_siblings_invalid(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "test";
    char *pkg_version = "1.0.0";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = NULL;
    int8_t name_type = PACKAGE_SOURCE;
    int i = 1;
    int *vuln_count = &i;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "test");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "0.0.0"); // start_including
    expect_value(__wrap_sqlite3_column_text, iCol, 2);
    will_return(__wrap_sqlite3_column_text, NULL); // start_excluding
    expect_value(__wrap_sqlite3_column_text, iCol, 3);
    will_return(__wrap_sqlite3_column_text, "1.0.0"); // end_including
    expect_value(__wrap_sqlite3_column_text, iCol, 4);
    will_return(__wrap_sqlite3_column_text, NULL); // end_excluding
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 6);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 7);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 8);
    will_return(__wrap_sqlite3_column_int, 1000); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 9);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 10);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 11);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    // Siblings
    will_return(__wrap_sqlite3_prepare_v2, NULL);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_ERROR);
    will_return(__wrap_sqlite3_errmsg, "error");
    expect_string(__wrap__mterror, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mterror, formatted_msg, "(5571): Couldn't get from the NVD the 'siblings' dependencies of the package with ID '0'");
    expect_string(__wrap__mterror, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mterror, formatted_msg, "(5503): SQL error: 'error'");

    int ret = wm_vuldet_check_generic_package(db,
                                              FEED_DEBIAN,
                                              pkg_name,
                                              pkg_source,
                                              pkg_version,
                                              pkg_arch,
                                              pkg_vendor,
                                              pkg_reference,
                                              pkg_type,
                                              cve_table,
                                              name_type,
                                              vuln_count);
    assert_int_equal(ret, -1);
}

void test_wm_vuldet_check_generic_package_siblings_OK(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "test";
    char *pkg_version = "1.0.0";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = (OSHash *)1;
    int8_t name_type = PACKAGE_SOURCE;
    int i = 1;
    int *vuln_count = &i;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "test");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "0.0.0"); // start_including
    expect_value(__wrap_sqlite3_column_text, iCol, 2);
    will_return(__wrap_sqlite3_column_text, NULL); // start_excluding
    expect_value(__wrap_sqlite3_column_text, iCol, 3);
    will_return(__wrap_sqlite3_column_text, "1.0.0"); // end_including
    expect_value(__wrap_sqlite3_column_text, iCol, 4);
    will_return(__wrap_sqlite3_column_text, NULL); // end_excluding
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 6);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 7);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 8);
    will_return(__wrap_sqlite3_column_int, 1000); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 9);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 10);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 11);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    // Siblings
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_int, index, 1);
    expect_value(__wrap_sqlite3_bind_int, value, 1000);
    will_return(__wrap_sqlite3_bind_int, 0);
    expect_value(__wrap_sqlite3_bind_int, index, 2);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_int, iCol, 0);
    will_return(__wrap_sqlite3_column_int, 1);

    expect_sqlite3_step_call(SQLITE_DONE);

    will_return(__wrap_wm_vuldet_add_cve_node, 0);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5458): Package 'test' inserted into the vulnerability 'CVE-0000-0000'. Version (1.0.0) 'less than or equal' '1.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_generic_package(db,
                                              FEED_DEBIAN,
                                              pkg_name,
                                              pkg_source,
                                              pkg_version,
                                              pkg_arch,
                                              pkg_vendor,
                                              pkg_reference,
                                              pkg_type,
                                              cve_table,
                                              name_type,
                                              vuln_count);
    assert_int_equal(ret, 1);
}

void test_wm_vuldet_check_generic_package_ignore_package(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "test";
    char *pkg_version = "1.0.0";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = NULL;
    int8_t name_type = PACKAGE_SOURCE;
    int i = 1;
    int *vuln_count = &i;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "test");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, NULL); // start_including
    expect_value(__wrap_sqlite3_column_text, iCol, 2);
    will_return(__wrap_sqlite3_column_text, NULL); // start_excluding
    expect_value(__wrap_sqlite3_column_text, iCol, 3);
    will_return(__wrap_sqlite3_column_text, NULL); // end_including
    expect_value(__wrap_sqlite3_column_text, iCol, 4);
    will_return(__wrap_sqlite3_column_text, NULL); // end_excluding
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 6);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 7);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 8);
    will_return(__wrap_sqlite3_column_int, 1000); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 9);
    will_return(__wrap_sqlite3_column_text, "AND"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 10);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 11);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_generic_package(db,
                                              FEED_DEBIAN,
                                              pkg_name,
                                              pkg_source,
                                              pkg_version,
                                              pkg_arch,
                                              pkg_vendor,
                                              pkg_reference,
                                              pkg_type,
                                              cve_table,
                                              name_type,
                                              vuln_count);
    assert_int_equal(ret, 1);
}

void test_wm_vuldet_check_generic_package_insert_package_error(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "test";
    char *pkg_version = "1.0.0";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = NULL;
    int8_t name_type = PACKAGE_SOURCE;
    int i = 1;
    int *vuln_count = &i;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "test");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "0.0.0"); // start_including
    expect_value(__wrap_sqlite3_column_text, iCol, 2);
    will_return(__wrap_sqlite3_column_text, NULL); // start_excluding
    expect_value(__wrap_sqlite3_column_text, iCol, 3);
    will_return(__wrap_sqlite3_column_text, "1.0.0"); // end_including
    expect_value(__wrap_sqlite3_column_text, iCol, 4);
    will_return(__wrap_sqlite3_column_text, NULL); // end_excluding
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 6);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 7);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 8);
    will_return(__wrap_sqlite3_column_int, 1000); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 9);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 10);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 11);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    // Siblings
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_int, index, 1);
    expect_value(__wrap_sqlite3_bind_int, value, 1000);
    will_return(__wrap_sqlite3_bind_int, 0);
    expect_value(__wrap_sqlite3_bind_int, index, 2);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_int, iCol, 0);
    will_return(__wrap_sqlite3_column_int, 1);

    expect_sqlite3_step_call(SQLITE_DONE);

    will_return(__wrap_wm_vuldet_add_cve_node, -1);
    expect_string(__wrap__mterror, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mterror, formatted_msg, "(5572): Couldn't insert the package 'test' into the vulnerability 'CVE-0000-0000'. Version (1.0.0) 'less than or equal' '1.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_generic_package(db,
                                              FEED_DEBIAN,
                                              pkg_name,
                                              pkg_source,
                                              pkg_version,
                                              pkg_arch,
                                              pkg_vendor,
                                              pkg_reference,
                                              pkg_type,
                                              cve_table,
                                              name_type,
                                              vuln_count);
    assert_int_equal(ret, 1);
}

void test_wm_vuldet_check_generic_package_duplicated_package(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "test";
    char *pkg_version = "1.0.0";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = NULL;
    int8_t name_type = PACKAGE_SOURCE;
    int i = 1;
    int *vuln_count = &i;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "test");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "0.0.0"); // start_including
    expect_value(__wrap_sqlite3_column_text, iCol, 2);
    will_return(__wrap_sqlite3_column_text, NULL); // start_excluding
    expect_value(__wrap_sqlite3_column_text, iCol, 3);
    will_return(__wrap_sqlite3_column_text, "1.0.0"); // end_including
    expect_value(__wrap_sqlite3_column_text, iCol, 4);
    will_return(__wrap_sqlite3_column_text, NULL); // end_excluding
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 6);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 7);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 8);
    will_return(__wrap_sqlite3_column_int, 1000); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 9);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 10);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 11);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    // Siblings
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_int, index, 1);
    expect_value(__wrap_sqlite3_bind_int, value, 1000);
    will_return(__wrap_sqlite3_bind_int, 0);
    expect_value(__wrap_sqlite3_bind_int, index, 2);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_int, iCol, 0);
    will_return(__wrap_sqlite3_column_int, 1);

    expect_sqlite3_step_call(SQLITE_DONE);

    will_return(__wrap_wm_vuldet_add_cve_node, 1);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5459): Trying to insert duplicated package 'test' into the vulnerability 'CVE-0000-0000'. Version (1.0.0) 'less than or equal' '1.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_generic_package(db,
                                              FEED_DEBIAN,
                                              pkg_name,
                                              pkg_source,
                                              pkg_version,
                                              pkg_arch,
                                              pkg_vendor,
                                              pkg_reference,
                                              pkg_type,
                                              cve_table,
                                              name_type,
                                              vuln_count);
    assert_int_equal(ret, 1);
}

void test_wm_vuldet_check_generic_package_insert_package_OK(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "test";
    char *pkg_version = "1.0.0";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = (OSHash *)1;
    int8_t name_type = PACKAGE_SOURCE;
    int i = 1;
    int *vuln_count = &i;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "test");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "0.0.0"); // start_including
    expect_value(__wrap_sqlite3_column_text, iCol, 2);
    will_return(__wrap_sqlite3_column_text, NULL); // start_excluding
    expect_value(__wrap_sqlite3_column_text, iCol, 3);
    will_return(__wrap_sqlite3_column_text, "1.0.0"); // end_including
    expect_value(__wrap_sqlite3_column_text, iCol, 4);
    will_return(__wrap_sqlite3_column_text, NULL); // end_excluding
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 6);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 7);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 8);
    will_return(__wrap_sqlite3_column_int, 1000); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 9);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 10);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 11);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    // Siblings
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_int, index, 1);
    expect_value(__wrap_sqlite3_bind_int, value, 1000);
    will_return(__wrap_sqlite3_bind_int, 0);
    expect_value(__wrap_sqlite3_bind_int, index, 2);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_int, iCol, 0);
    will_return(__wrap_sqlite3_column_int, 1);

    expect_sqlite3_step_call(SQLITE_DONE);

    will_return(__wrap_wm_vuldet_add_cve_node, 0);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5458): Package 'test' inserted into the vulnerability 'CVE-0000-0000'. Version (1.0.0) 'less than or equal' '1.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_generic_package(db,
                                              FEED_DEBIAN,
                                              pkg_name,
                                              pkg_source,
                                              pkg_version,
                                              pkg_arch,
                                              pkg_vendor,
                                              pkg_reference,
                                              pkg_type,
                                              cve_table,
                                              name_type,
                                              vuln_count);
    assert_int_equal(ret, 1);
}

void test_wm_vuldet_check_generic_package_insert_package_Ubuntu(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "ubuntu_linux";
    char *pkg_version = "1.0.0";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = (OSHash *)1;
    int8_t name_type = PACKAGE_SOURCE;
    int i = 1;
    int *vuln_count = &i;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "ubuntu_linux");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 2);
    expect_string(__wrap_sqlite3_bind_text, buffer, "canonical");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "0.0.0"); // start_including
    expect_value(__wrap_sqlite3_column_text, iCol, 2);
    will_return(__wrap_sqlite3_column_text, NULL); // start_excluding
    expect_value(__wrap_sqlite3_column_text, iCol, 3);
    will_return(__wrap_sqlite3_column_text, "1.0.0"); // end_including
    expect_value(__wrap_sqlite3_column_text, iCol, 4);
    will_return(__wrap_sqlite3_column_text, NULL); // end_excluding
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 6);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 7);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 8);
    will_return(__wrap_sqlite3_column_int, 1000); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 9);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 10);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 11);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    // Siblings
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_int, index, 1);
    expect_value(__wrap_sqlite3_bind_int, value, 1000);
    will_return(__wrap_sqlite3_bind_int, 0);
    expect_value(__wrap_sqlite3_bind_int, index, 2);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_int, iCol, 0);
    will_return(__wrap_sqlite3_column_int, 1);

    expect_sqlite3_step_call(SQLITE_DONE);

    will_return(__wrap_wm_vuldet_add_cve_node, 0);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5458): Package 'ubuntu_linux' inserted into the vulnerability 'CVE-0000-0000'. Version (1.0.0) 'less than or equal' '1.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_generic_package(db,
                                              FEED_DEBIAN,
                                              pkg_name,
                                              pkg_source,
                                              pkg_version,
                                              pkg_arch,
                                              pkg_vendor,
                                              pkg_reference,
                                              pkg_type,
                                              cve_table,
                                              name_type,
                                              vuln_count);
    assert_int_equal(ret, 1);
}

void test_wm_vuldet_check_generic_package_insert_package_Debian(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "debian_linux";
    char *pkg_version = "1.0.0";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = (OSHash *)1;
    int8_t name_type = PACKAGE_SOURCE;
    int i = 1;
    int *vuln_count = &i;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "debian_linux");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 2);
    expect_string(__wrap_sqlite3_bind_text, buffer, "debian");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "0.0.0"); // start_including
    expect_value(__wrap_sqlite3_column_text, iCol, 2);
    will_return(__wrap_sqlite3_column_text, NULL); // start_excluding
    expect_value(__wrap_sqlite3_column_text, iCol, 3);
    will_return(__wrap_sqlite3_column_text, "1.0.0"); // end_including
    expect_value(__wrap_sqlite3_column_text, iCol, 4);
    will_return(__wrap_sqlite3_column_text, NULL); // end_excluding
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 6);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 7);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 8);
    will_return(__wrap_sqlite3_column_int, 1000); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 9);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 10);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 11);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    // Siblings
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_int, index, 1);
    expect_value(__wrap_sqlite3_bind_int, value, 1000);
    will_return(__wrap_sqlite3_bind_int, 0);
    expect_value(__wrap_sqlite3_bind_int, index, 2);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_int, iCol, 0);
    will_return(__wrap_sqlite3_column_int, 1);

    expect_sqlite3_step_call(SQLITE_DONE);

    will_return(__wrap_wm_vuldet_add_cve_node, 0);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5458): Package 'debian_linux' inserted into the vulnerability 'CVE-0000-0000'. Version (1.0.0) 'less than or equal' '1.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_generic_package(db,
                                              FEED_DEBIAN,
                                              pkg_name,
                                              pkg_source,
                                              pkg_version,
                                              pkg_arch,
                                              pkg_vendor,
                                              pkg_reference,
                                              pkg_type,
                                              cve_table,
                                              name_type,
                                              vuln_count);
    assert_int_equal(ret, 1);
}

void test_wm_vuldet_check_generic_package_insert_package_RedHat(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "enterprise_linux";
    char *pkg_version = "1.0.0";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = (OSHash *)1;
    int8_t name_type = PACKAGE_SOURCE;
    int i = 1;
    int *vuln_count = &i;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "enterprise_linux");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 2);
    expect_string(__wrap_sqlite3_bind_text, buffer, "redhat");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "0.0.0"); // start_including
    expect_value(__wrap_sqlite3_column_text, iCol, 2);
    will_return(__wrap_sqlite3_column_text, NULL); // start_excluding
    expect_value(__wrap_sqlite3_column_text, iCol, 3);
    will_return(__wrap_sqlite3_column_text, "1.0.0"); // end_including
    expect_value(__wrap_sqlite3_column_text, iCol, 4);
    will_return(__wrap_sqlite3_column_text, NULL); // end_excluding
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 6);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 7);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 8);
    will_return(__wrap_sqlite3_column_int, 1000); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 9);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 10);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 11);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    // Siblings
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_int, index, 1);
    expect_value(__wrap_sqlite3_bind_int, value, 1000);
    will_return(__wrap_sqlite3_bind_int, 0);
    expect_value(__wrap_sqlite3_bind_int, index, 2);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_int, iCol, 0);
    will_return(__wrap_sqlite3_column_int, 1);

    expect_sqlite3_step_call(SQLITE_DONE);

    will_return(__wrap_wm_vuldet_add_cve_node, 0);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5458): Package 'enterprise_linux' inserted into the vulnerability 'CVE-0000-0000'. Version (1.0.0) 'less than or equal' '1.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_generic_package(db,
                                              FEED_DEBIAN,
                                              pkg_name,
                                              pkg_source,
                                              pkg_version,
                                              pkg_arch,
                                              pkg_vendor,
                                              pkg_reference,
                                              pkg_type,
                                              cve_table,
                                              name_type,
                                              vuln_count);
    assert_int_equal(ret, 1);
}

void test_wm_vuldet_check_generic_package_insert_package_Arch(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = "arch_linux";
    char *pkg_source = NULL;
    char *pkg_version = "1.0.0";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = (OSHash *)1;
    int8_t name_type = PACKAGE_NAME;
    int i = 1;
    int *vuln_count = &i;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "arch_linux");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 2);
    expect_string(__wrap_sqlite3_bind_text, buffer, "archlinux");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "0.0.0"); // start_including
    expect_value(__wrap_sqlite3_column_text, iCol, 2);
    will_return(__wrap_sqlite3_column_text, NULL); // start_excluding
    expect_value(__wrap_sqlite3_column_text, iCol, 3);
    will_return(__wrap_sqlite3_column_text, "1.0.0"); // end_including
    expect_value(__wrap_sqlite3_column_text, iCol, 4);
    will_return(__wrap_sqlite3_column_text, NULL); // end_excluding
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 6);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 7);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 8);
    will_return(__wrap_sqlite3_column_int, 1000); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 9);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 10);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 11);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    // Siblings
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_int, index, 1);
    expect_value(__wrap_sqlite3_bind_int, value, 1000);
    will_return(__wrap_sqlite3_bind_int, 0);
    expect_value(__wrap_sqlite3_bind_int, index, 2);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_int, iCol, 0);
    will_return(__wrap_sqlite3_column_int, 1);

    expect_sqlite3_step_call(SQLITE_DONE);

    will_return(__wrap_wm_vuldet_add_cve_node, 0);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5458): Package 'arch_linux' inserted into the vulnerability 'CVE-0000-0000'. Version (1.0.0) 'less than or equal' '1.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_generic_package(db,
                                              FEED_ARCH,
                                              pkg_name,
                                              pkg_source,
                                              pkg_version,
                                              pkg_arch,
                                              pkg_vendor,
                                              pkg_reference,
                                              pkg_type,
                                              cve_table,
                                              name_type,
                                              vuln_count);
    assert_int_equal(ret, 1);
}

void test_wm_vuldet_check_generic_package_insert_package_linux_kernel(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "linux_kernel";
    char *pkg_version = "1.0.0";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = (OSHash *)1;
    int8_t name_type = PACKAGE_SOURCE;
    int i = 1;
    int *vuln_count = &i;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "linux_kernel");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 2);
    expect_string(__wrap_sqlite3_bind_text, buffer, "linux");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "0.0.0"); // start_including
    expect_value(__wrap_sqlite3_column_text, iCol, 2);
    will_return(__wrap_sqlite3_column_text, NULL); // start_excluding
    expect_value(__wrap_sqlite3_column_text, iCol, 3);
    will_return(__wrap_sqlite3_column_text, "1.0.0"); // end_including
    expect_value(__wrap_sqlite3_column_text, iCol, 4);
    will_return(__wrap_sqlite3_column_text, NULL); // end_excluding
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 6);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 7);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 8);
    will_return(__wrap_sqlite3_column_int, 1000); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 9);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 10);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 11);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    // Siblings
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_int, index, 1);
    expect_value(__wrap_sqlite3_bind_int, value, 1000);
    will_return(__wrap_sqlite3_bind_int, 0);
    expect_value(__wrap_sqlite3_bind_int, index, 2);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_int, iCol, 0);
    will_return(__wrap_sqlite3_column_int, 1);

    expect_sqlite3_step_call(SQLITE_DONE);

    will_return(__wrap_wm_vuldet_add_cve_node, 0);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5458): Package 'linux_kernel' inserted into the vulnerability 'CVE-0000-0000'. Version (1.0.0) 'less than or equal' '1.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_generic_package(db,
                                              FEED_DEBIAN,
                                              pkg_name,
                                              pkg_source,
                                              pkg_version,
                                              pkg_arch,
                                              pkg_vendor,
                                              pkg_reference,
                                              pkg_type,
                                              cve_table,
                                              name_type,
                                              vuln_count);
    assert_int_equal(ret, 1);
}

void test_wm_vuldet_check_generic_package_insert_package_mac_os_x(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "mac_os_x";
    char *pkg_version = "10.15.1";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = (OSHash *)1;
    int8_t name_type = PACKAGE_SOURCE;
    int i = 1;
    int *vuln_count = &i;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "mac_os_x");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "10.10.0"); // start_including
    expect_value(__wrap_sqlite3_column_text, iCol, 2);
    will_return(__wrap_sqlite3_column_text, NULL); // start_excluding
    expect_value(__wrap_sqlite3_column_text, iCol, 3);
    will_return(__wrap_sqlite3_column_text, "10.15.1"); // end_including
    expect_value(__wrap_sqlite3_column_text, iCol, 4);
    will_return(__wrap_sqlite3_column_text, NULL); // end_excluding
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 6);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 7);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 8);
    will_return(__wrap_sqlite3_column_int, 1000); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 9);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 10);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 11);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    // Siblings
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_int, index, 1);
    expect_value(__wrap_sqlite3_bind_int, value, 1000);
    will_return(__wrap_sqlite3_bind_int, 0);
    expect_value(__wrap_sqlite3_bind_int, index, 2);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_int, iCol, 0);
    will_return(__wrap_sqlite3_column_int, 1);

    expect_sqlite3_step_call(SQLITE_DONE);

    will_return(__wrap_wm_vuldet_add_cve_node, 0);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5458): Package 'mac_os_x' inserted into the vulnerability 'CVE-0000-0000'. Version (10.15.1) 'less than or equal' '10.15.1' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_generic_package(db,
                                              FEED_MAC,
                                              pkg_name,
                                              pkg_source,
                                              pkg_version,
                                              pkg_arch,
                                              pkg_vendor,
                                              pkg_reference,
                                              pkg_type,
                                              cve_table,
                                              name_type,
                                              vuln_count);
    assert_int_equal(ret, 1);
}

void test_wm_vuldet_check_generic_package_insert_package_mac_os_x_server(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "mac_os_x_server";
    char *pkg_version = "10.15.1";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = (OSHash *)1;
    int8_t name_type = PACKAGE_SOURCE;
    int i = 1;
    int *vuln_count = &i;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "mac_os_x_server");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "10.10.0"); // start_including
    expect_value(__wrap_sqlite3_column_text, iCol, 2);
    will_return(__wrap_sqlite3_column_text, NULL); // start_excluding
    expect_value(__wrap_sqlite3_column_text, iCol, 3);
    will_return(__wrap_sqlite3_column_text, "10.15.1"); // end_including
    expect_value(__wrap_sqlite3_column_text, iCol, 4);
    will_return(__wrap_sqlite3_column_text, NULL); // end_excluding
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 6);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 7);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 8);
    will_return(__wrap_sqlite3_column_int, 1000); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 9);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 10);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 11);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    // Siblings
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_int, index, 1);
    expect_value(__wrap_sqlite3_bind_int, value, 1000);
    will_return(__wrap_sqlite3_bind_int, 0);
    expect_value(__wrap_sqlite3_bind_int, index, 2);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_int, iCol, 0);
    will_return(__wrap_sqlite3_column_int, 1);

    expect_sqlite3_step_call(SQLITE_DONE);

    will_return(__wrap_wm_vuldet_add_cve_node, 0);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5458): Package 'mac_os_x_server' inserted into the vulnerability 'CVE-0000-0000'. Version (10.15.1) 'less than or equal' '10.15.1' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_generic_package(db,
                                              FEED_MAC,
                                              pkg_name,
                                              pkg_source,
                                              pkg_version,
                                              pkg_arch,
                                              pkg_vendor,
                                              pkg_reference,
                                              pkg_type,
                                              cve_table,
                                              name_type,
                                              vuln_count);
    assert_int_equal(ret, 1);
}

void test_wm_vuldet_check_generic_package_insert_package_mac_os(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "mac_os";
    char *pkg_version = "10.15.1";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = (OSHash *)1;
    int8_t name_type = PACKAGE_SOURCE;
    int i = 1;
    int *vuln_count = &i;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "mac_os");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "10.10.0"); // start_including
    expect_value(__wrap_sqlite3_column_text, iCol, 2);
    will_return(__wrap_sqlite3_column_text, NULL); // start_excluding
    expect_value(__wrap_sqlite3_column_text, iCol, 3);
    will_return(__wrap_sqlite3_column_text, "10.15.1"); // end_including
    expect_value(__wrap_sqlite3_column_text, iCol, 4);
    will_return(__wrap_sqlite3_column_text, NULL); // end_excluding
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 6);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 7);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 8);
    will_return(__wrap_sqlite3_column_int, 1000); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 9);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 10);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 11);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    // Siblings
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_int, index, 1);
    expect_value(__wrap_sqlite3_bind_int, value, 1000);
    will_return(__wrap_sqlite3_bind_int, 0);
    expect_value(__wrap_sqlite3_bind_int, index, 2);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_int, iCol, 0);
    will_return(__wrap_sqlite3_column_int, 1);

    expect_sqlite3_step_call(SQLITE_DONE);

    will_return(__wrap_wm_vuldet_add_cve_node, 0);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5458): Package 'mac_os' inserted into the vulnerability 'CVE-0000-0000'. Version (10.15.1) 'less than or equal' '10.15.1' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_generic_package(db,
                                              FEED_MAC,
                                              pkg_name,
                                              pkg_source,
                                              pkg_version,
                                              pkg_arch,
                                              pkg_vendor,
                                              pkg_reference,
                                              pkg_type,
                                              cve_table,
                                              name_type,
                                              vuln_count);
    assert_int_equal(ret, 1);
}

void test_wm_vuldet_check_generic_package_insert_package_macos(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "macos";
    char *pkg_version = "10.15.1";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = (OSHash *)1;
    int8_t name_type = PACKAGE_SOURCE;
    int i = 1;
    int *vuln_count = &i;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "macos");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "10.10.0"); // start_including
    expect_value(__wrap_sqlite3_column_text, iCol, 2);
    will_return(__wrap_sqlite3_column_text, NULL); // start_excluding
    expect_value(__wrap_sqlite3_column_text, iCol, 3);
    will_return(__wrap_sqlite3_column_text, "10.15.1"); // end_including
    expect_value(__wrap_sqlite3_column_text, iCol, 4);
    will_return(__wrap_sqlite3_column_text, NULL); // end_excluding
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 6);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 7);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 8);
    will_return(__wrap_sqlite3_column_int, 1000); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 9);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 10);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 11);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    // Siblings
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_int, index, 1);
    expect_value(__wrap_sqlite3_bind_int, value, 1000);
    will_return(__wrap_sqlite3_bind_int, 0);
    expect_value(__wrap_sqlite3_bind_int, index, 2);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_int, iCol, 0);
    will_return(__wrap_sqlite3_column_int, 1);

    expect_sqlite3_step_call(SQLITE_DONE);

    will_return(__wrap_wm_vuldet_add_cve_node, 0);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5458): Package 'macos' inserted into the vulnerability 'CVE-0000-0000'. Version (10.15.1) 'less than or equal' '10.15.1' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_generic_package(db,
                                              FEED_MAC,
                                              pkg_name,
                                              pkg_source,
                                              pkg_version,
                                              pkg_arch,
                                              pkg_vendor,
                                              pkg_reference,
                                              pkg_type,
                                              cve_table,
                                              name_type,
                                              vuln_count);
    assert_int_equal(ret, 1);
}

void test_wm_vuldet_check_generic_package_insert_package_mac_no_vendor(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "safari";
    char *pkg_version = "2.1.3";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = (OSHash *)1;
    int8_t name_type = PACKAGE_SOURCE;
    int i = 1;
    int *vuln_count = &i;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "safari");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 2);
    expect_string(__wrap_sqlite3_bind_text, buffer, "");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 3);
    expect_string(__wrap_sqlite3_bind_text, buffer, "");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "1.0.0"); // start_including
    expect_value(__wrap_sqlite3_column_text, iCol, 2);
    will_return(__wrap_sqlite3_column_text, NULL); // start_excluding
    expect_value(__wrap_sqlite3_column_text, iCol, 3);
    will_return(__wrap_sqlite3_column_text, "2.2.0"); // end_including
    expect_value(__wrap_sqlite3_column_text, iCol, 4);
    will_return(__wrap_sqlite3_column_text, NULL); // end_excluding
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 6);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 7);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 8);
    will_return(__wrap_sqlite3_column_int, 1000); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 9);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 10);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 11);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    // Siblings
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_int, index, 1);
    expect_value(__wrap_sqlite3_bind_int, value, 1000);
    will_return(__wrap_sqlite3_bind_int, 0);
    expect_value(__wrap_sqlite3_bind_int, index, 2);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_int, iCol, 0);
    will_return(__wrap_sqlite3_column_int, 1);

    expect_sqlite3_step_call(SQLITE_DONE);

    will_return(__wrap_wm_vuldet_add_cve_node, 0);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5458): Package 'safari' inserted into the vulnerability 'CVE-0000-0000'. Version (2.1.3) 'less than or equal' '2.2.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_generic_package(db,
                                              FEED_MAC,
                                              pkg_name,
                                              pkg_source,
                                              pkg_version,
                                              pkg_arch,
                                              pkg_vendor,
                                              pkg_reference,
                                              pkg_type,
                                              cve_table,
                                              name_type,
                                              vuln_count);
    assert_int_equal(ret, 1);
}

void test_wm_vuldet_check_generic_package_insert_package_mac_with_vendor(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "safari";
    char *pkg_version = "2.1.3";
    char *pkg_arch = NULL;
    char *pkg_vendor = "apple";
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = (OSHash *)1;
    int8_t name_type = PACKAGE_SOURCE;
    int i = 1;
    int *vuln_count = &i;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "safari");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 2);
    expect_string(__wrap_sqlite3_bind_text, buffer, "apple");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 3);
    expect_string(__wrap_sqlite3_bind_text, buffer, "apple");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "1.0.0"); // start_including
    expect_value(__wrap_sqlite3_column_text, iCol, 2);
    will_return(__wrap_sqlite3_column_text, NULL); // start_excluding
    expect_value(__wrap_sqlite3_column_text, iCol, 3);
    will_return(__wrap_sqlite3_column_text, "2.2.0"); // end_including
    expect_value(__wrap_sqlite3_column_text, iCol, 4);
    will_return(__wrap_sqlite3_column_text, NULL); // end_excluding
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 6);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 7);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 8);
    will_return(__wrap_sqlite3_column_int, 1000); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 9);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 10);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 11);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    // Siblings
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_int, index, 1);
    expect_value(__wrap_sqlite3_bind_int, value, 1000);
    will_return(__wrap_sqlite3_bind_int, 0);
    expect_value(__wrap_sqlite3_bind_int, index, 2);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_int, iCol, 0);
    will_return(__wrap_sqlite3_column_int, 1);

    expect_sqlite3_step_call(SQLITE_DONE);

    will_return(__wrap_wm_vuldet_add_cve_node, 0);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5458): Package 'safari' inserted into the vulnerability 'CVE-0000-0000'. Version (2.1.3) 'less than or equal' '2.2.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_generic_package(db,
                                              FEED_MAC,
                                              pkg_name,
                                              pkg_source,
                                              pkg_version,
                                              pkg_arch,
                                              pkg_vendor,
                                              pkg_reference,
                                              pkg_type,
                                              cve_table,
                                              name_type,
                                              vuln_count);
    assert_int_equal(ret, 1);
}

/* test wm_vuldet_check_specific_package */

void test_wm_vuldet_check_specific_package_pkg_version_NULL(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "test";
    char *pkg_version = NULL;
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = NULL;
    int8_t name_type = PACKAGE_SOURCE;
    int *vuln_count = NULL;

    expect_string(__wrap__mterror, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mterror, formatted_msg, "(5576): Missing version for package 'test' of the inventory.");

    int ret = wm_vuldet_check_specific_package(db,
                                               FEED_DEBIAN,
                                               pkg_name,
                                               pkg_source,
                                               pkg_version,
                                               pkg_arch,
                                               pkg_vendor,
                                               pkg_reference,
                                               pkg_type,
                                               cve_table,
                                               name_type,
                                               vuln_count);
    assert_int_equal(ret, 0);
}

void test_wm_vuldet_check_specific_package_pkg_version_empty(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "test";
    char *pkg_version = "";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = NULL;
    int8_t name_type = PACKAGE_SOURCE;
    int *vuln_count = NULL;

    expect_string(__wrap__mterror, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mterror, formatted_msg, "(5591): Invalid version for package 'test' of the inventory: ''");

    int ret = wm_vuldet_check_specific_package(db,
                                               FEED_DEBIAN,
                                               pkg_name,
                                               pkg_source,
                                               pkg_version,
                                               pkg_arch,
                                               pkg_vendor,
                                               pkg_reference,
                                               pkg_type,
                                               cve_table,
                                               name_type,
                                               vuln_count);
    assert_int_equal(ret, 0);
}

void test_wm_vuldet_check_specific_package_prepare_error(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "test";
    char *pkg_version = "0.0.0";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = NULL;
    int8_t name_type = PACKAGE_SOURCE;
    int *vuln_count = NULL;

    will_return(__wrap_sqlite3_prepare_v2, NULL);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_ERROR);
    will_return(__wrap_sqlite3_errmsg, "error");
    expect_string(__wrap__mterror, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mterror, formatted_msg, "(5570): Couldn't get from the NVD the 'specific' vulnerabilities of the package 'test'");
    expect_string(__wrap__mterror, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mterror, formatted_msg, "(5503): SQL error: 'error'");

    int ret = wm_vuldet_check_specific_package(db,
                                               FEED_DEBIAN,
                                               pkg_name,
                                               pkg_source,
                                               pkg_version,
                                               pkg_arch,
                                               pkg_vendor,
                                               pkg_reference,
                                               pkg_type,
                                               cve_table,
                                               name_type,
                                               vuln_count);
    assert_int_equal(ret, -1);
}

void test_wm_vuldet_check_specific_package_done(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "test";
    char *pkg_version = "0.0.0";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = NULL;
    int8_t name_type = PACKAGE_SOURCE;
    int *vuln_count = NULL;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "test");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 2);
    expect_string(__wrap_sqlite3_bind_text, buffer, "%0.0.0%");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_specific_package(db,
                                               FEED_DEBIAN,
                                               pkg_name,
                                               pkg_source,
                                               pkg_version,
                                               pkg_arch,
                                               pkg_vendor,
                                               pkg_reference,
                                               pkg_type,
                                               cve_table,
                                               name_type,
                                               vuln_count);
    assert_int_equal(ret, 0);
}

void test_wm_vuldet_check_specific_package_skip(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "test";
    char *pkg_version = "0.0.0";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = NULL;
    int8_t name_type = PACKAGE_SOURCE;
    int *vuln_count = NULL;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "test");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 2);
    expect_string(__wrap_sqlite3_bind_text, buffer, "%0.0.0%");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "1.0.0"); // version_cmp
    expect_value(__wrap_sqlite3_column_int, iCol, 2);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 3);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 4);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 6);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 7);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 8);
    will_return(__wrap_sqlite3_column_text, "node.js"); // target_sw

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_specific_package(db,
                                               FEED_DEBIAN,
                                               pkg_name,
                                               pkg_source,
                                               pkg_version,
                                               pkg_arch,
                                               pkg_vendor,
                                               pkg_reference,
                                               pkg_type,
                                               cve_table,
                                               name_type,
                                               vuln_count);
    assert_int_equal(ret, 0);
}

void test_wm_vuldet_check_specific_package_no_vulnerable(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "test";
    char *pkg_version = "0.0.0";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = NULL;
    int8_t name_type = PACKAGE_SOURCE;
    int *vuln_count = NULL;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "test");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 2);
    expect_string(__wrap_sqlite3_bind_text, buffer, "%0.0.0%");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "1.0.0"); // version_cmp
    expect_value(__wrap_sqlite3_column_int, iCol, 2);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 3);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 4);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 6);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 7);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 8);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, 0);

    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5460): Package 'test' not vulnerable to 'CVE-0000-0000'. Version (0.0.0) not 'equals' '1.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_specific_package(db,
                                               FEED_DEBIAN,
                                               pkg_name,
                                               pkg_source,
                                               pkg_version,
                                               pkg_arch,
                                               pkg_vendor,
                                               pkg_reference,
                                               pkg_type,
                                               cve_table,
                                               name_type,
                                               vuln_count);
    assert_int_equal(ret, 0);
}

void test_wm_vuldet_check_specific_package_vulnerable(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "test";
    char *pkg_version = "0.0.0";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = (OSHash *)1;
    int8_t name_type = PACKAGE_SOURCE;
    int i = 1;
    int *vuln_count = &i;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "test");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 2);
    expect_string(__wrap_sqlite3_bind_text, buffer, "%0.0.0%");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "0.0.0"); // version_cmp
    expect_value(__wrap_sqlite3_column_int, iCol, 2);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 3);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 4);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 6);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 7);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 8);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    will_return(__wrap_wm_vuldet_add_cve_node, 0);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5458): Package 'test' inserted into the vulnerability 'CVE-0000-0000'. Version (0.0.0) 'equals' '0.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_specific_package(db,
                                               FEED_DEBIAN,
                                               pkg_name,
                                               pkg_source,
                                               pkg_version,
                                               pkg_arch,
                                               pkg_vendor,
                                               pkg_reference,
                                               pkg_type,
                                               cve_table,
                                               name_type,
                                               vuln_count);
    assert_int_equal(ret, 1);
}

void test_wm_vuldet_check_specific_package_no_children_no_siblings(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "test";
    char *pkg_version = "0.0.0";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = (OSHash *)1;
    int8_t name_type = PACKAGE_SOURCE;
    int i = 1;
    int *vuln_count = &i;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "test");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 2);
    expect_string(__wrap_sqlite3_bind_text, buffer, "%0.0.0%");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "0.0.0"); // version_cmp
    expect_value(__wrap_sqlite3_column_int, iCol, 2);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 3);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 4);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 6);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 7);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 8);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    will_return(__wrap_wm_vuldet_add_cve_node, 0);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5458): Package 'test' inserted into the vulnerability 'CVE-0000-0000'. Version (0.0.0) 'equals' '0.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_specific_package(db,
                                               FEED_DEBIAN,
                                               pkg_name,
                                               pkg_source,
                                               pkg_version,
                                               pkg_arch,
                                               pkg_vendor,
                                               pkg_reference,
                                               pkg_type,
                                               cve_table,
                                               name_type,
                                               vuln_count);
    assert_int_equal(ret, 1);
}

void test_wm_vuldet_check_specific_package_children_invalid(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "test";
    char *pkg_version = "0.0.0";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = NULL;
    int8_t name_type = PACKAGE_SOURCE;
    int i = 1;
    int *vuln_count = &i;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "test");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 2);
    expect_string(__wrap_sqlite3_bind_text, buffer, "%0.0.0%");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "0.0.0"); // version_cmp
    expect_value(__wrap_sqlite3_column_int, iCol, 2);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 3);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 4);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 6);
    will_return(__wrap_sqlite3_column_text, "AND"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 7);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 8);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    // Children
    will_return(__wrap_sqlite3_prepare_v2, NULL);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_ERROR);
    will_return(__wrap_sqlite3_errmsg, "error");
    expect_string(__wrap__mterror, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mterror, formatted_msg, "(5571): Couldn't get from the NVD the 'children' dependencies of the package with ID '0'");
    expect_string(__wrap__mterror, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mterror, formatted_msg, "(5503): SQL error: 'error'");

    int ret = wm_vuldet_check_specific_package(db,
                                               FEED_DEBIAN,
                                               pkg_name,
                                               pkg_source,
                                               pkg_version,
                                               pkg_arch,
                                               pkg_vendor,
                                               pkg_reference,
                                               pkg_type,
                                               cve_table,
                                               name_type,
                                               vuln_count);
    assert_int_equal(ret, -1);
}

void test_wm_vuldet_check_specific_package_children_OK(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "test";
    char *pkg_version = "0.0.0";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = (OSHash *)1;
    int8_t name_type = PACKAGE_SOURCE;
    int i = 1;
    int *vuln_count = &i;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "test");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 2);
    expect_string(__wrap_sqlite3_bind_text, buffer, "%0.0.0%");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "0.0.0"); // version_cmp
    expect_value(__wrap_sqlite3_column_int, iCol, 2);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 3);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 4);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 6);
    will_return(__wrap_sqlite3_column_text, "AND"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 7);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 8);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    // Children
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_int, index, 1);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);
    expect_value(__wrap_sqlite3_bind_int, index, 2);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_int, iCol, 0);
    will_return(__wrap_sqlite3_column_int, 1);

    expect_sqlite3_step_call(SQLITE_DONE);

    will_return(__wrap_wm_vuldet_add_cve_node, 0);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5458): Package 'test' inserted into the vulnerability 'CVE-0000-0000'. Version (0.0.0) 'equals' '0.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_specific_package(db,
                                               FEED_DEBIAN,
                                               pkg_name,
                                               pkg_source,
                                               pkg_version,
                                               pkg_arch,
                                               pkg_vendor,
                                               pkg_reference,
                                               pkg_type,
                                               cve_table,
                                               name_type,
                                               vuln_count);
    assert_int_equal(ret, 1);
}

void test_wm_vuldet_check_specific_package_siblings_invalid(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "test";
    char *pkg_version = "0.0.0";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = NULL;
    int8_t name_type = PACKAGE_SOURCE;
    int i = 1;
    int *vuln_count = &i;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "test");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 2);
    expect_string(__wrap_sqlite3_bind_text, buffer, "%0.0.0%");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "0.0.0"); // version_cmp
    expect_value(__wrap_sqlite3_column_int, iCol, 2);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 3);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 4);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 1000); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 6);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 7);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 8);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    // Siblings
    will_return(__wrap_sqlite3_prepare_v2, NULL);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_ERROR);
    will_return(__wrap_sqlite3_errmsg, "error");
    expect_string(__wrap__mterror, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mterror, formatted_msg, "(5571): Couldn't get from the NVD the 'siblings' dependencies of the package with ID '0'");
    expect_string(__wrap__mterror, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mterror, formatted_msg, "(5503): SQL error: 'error'");

    int ret = wm_vuldet_check_specific_package(db,
                                               FEED_DEBIAN,
                                               pkg_name,
                                               pkg_source,
                                               pkg_version,
                                               pkg_arch,
                                               pkg_vendor,
                                               pkg_reference,
                                               pkg_type,
                                               cve_table,
                                               name_type,
                                               vuln_count);
    assert_int_equal(ret, -1);
}

void test_wm_vuldet_check_specific_package_siblings_OK(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "test";
    char *pkg_version = "0.0.0";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = (OSHash *)1;
    int8_t name_type = PACKAGE_SOURCE;
    int i = 1;
    int *vuln_count = &i;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "test");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 2);
    expect_string(__wrap_sqlite3_bind_text, buffer, "%0.0.0%");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "0.0.0"); // version_cmp
    expect_value(__wrap_sqlite3_column_int, iCol, 2);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 3);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 4);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 1000); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 6);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 7);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 8);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    // Siblings
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_int, index, 1);
    expect_value(__wrap_sqlite3_bind_int, value, 1000);
    will_return(__wrap_sqlite3_bind_int, 0);
    expect_value(__wrap_sqlite3_bind_int, index, 2);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_int, iCol, 0);
    will_return(__wrap_sqlite3_column_int, 1);

    expect_sqlite3_step_call(SQLITE_DONE);

    will_return(__wrap_wm_vuldet_add_cve_node, 0);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5458): Package 'test' inserted into the vulnerability 'CVE-0000-0000'. Version (0.0.0) 'equals' '0.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_specific_package(db,
                                               FEED_DEBIAN,
                                               pkg_name,
                                               pkg_source,
                                               pkg_version,
                                               pkg_arch,
                                               pkg_vendor,
                                               pkg_reference,
                                               pkg_type,
                                               cve_table,
                                               name_type,
                                               vuln_count);
    assert_int_equal(ret, 1);
}

void test_wm_vuldet_check_specific_package_os_package_valid(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "ubuntu_linux";
    char *pkg_version = "0.0.0";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = NULL;
    int8_t name_type = PACKAGE_SOURCE;
    int i = 1;
    int *vuln_count = &i;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "ubuntu_linux");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 2);
    expect_string(__wrap_sqlite3_bind_text, buffer, "%0.0.0%");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 3);
    expect_string(__wrap_sqlite3_bind_text, buffer, "canonical");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "0.0.0"); // version_cmp
    expect_value(__wrap_sqlite3_column_int, iCol, 2);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 3);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 4);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 1000); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 6);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 7);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 8);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    // Siblings
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_int, index, 1);
    expect_value(__wrap_sqlite3_bind_int, value, 1000);
    will_return(__wrap_sqlite3_bind_int, 0);
    expect_value(__wrap_sqlite3_bind_int, index, 2);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_int, iCol, 0);
    will_return(__wrap_sqlite3_column_int, 1);

    expect_sqlite3_step_call(SQLITE_DONE);

    will_return(__wrap_wm_vuldet_add_cve_node, -1);
    expect_string(__wrap__mterror, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mterror, formatted_msg, "(5572): Couldn't insert the package 'ubuntu_linux' into the vulnerability 'CVE-0000-0000'. Version (0.0.0) 'equals' '0.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_specific_package(db,
                                               FEED_DEBIAN,
                                               pkg_name,
                                               pkg_source,
                                               pkg_version,
                                               pkg_arch,
                                               pkg_vendor,
                                               pkg_reference,
                                               pkg_type,
                                               cve_table,
                                               name_type,
                                               vuln_count);
    assert_int_equal(ret, 1);
}

void test_wm_vuldet_check_specific_package_ignore_package(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "ubuntu_linux";
    char *pkg_version = "0.0.0";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = NULL;
    int8_t name_type = PACKAGE_SOURCE;
    int i = 1;
    int *vuln_count = &i;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "ubuntu_linux");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 2);
    expect_string(__wrap_sqlite3_bind_text, buffer, "%0.0.0%");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 3);
    expect_string(__wrap_sqlite3_bind_text, buffer, "canonical");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "0.0.0"); // version_cmp
    expect_value(__wrap_sqlite3_column_int, iCol, 2);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 3);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 4);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 6);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 7);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 8);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_specific_package(db,
                                               FEED_DEBIAN,
                                               pkg_name,
                                               pkg_source,
                                               pkg_version,
                                               pkg_arch,
                                               pkg_vendor,
                                               pkg_reference,
                                               pkg_type,
                                               cve_table,
                                               name_type,
                                               vuln_count);
    assert_int_equal(ret, 1);
}

void test_wm_vuldet_check_specific_package_insert_package_error(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "test";
    char *pkg_version = "0.0.0";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = NULL;
    int8_t name_type = PACKAGE_SOURCE;
    int i = 1;
    int *vuln_count = &i;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "test");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 2);
    expect_string(__wrap_sqlite3_bind_text, buffer, "%0.0.0%");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "0.0.0"); // version_cmp
    expect_value(__wrap_sqlite3_column_int, iCol, 2);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 3);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 4);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 1000); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 6);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 7);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 8);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    // Siblings
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_int, index, 1);
    expect_value(__wrap_sqlite3_bind_int, value, 1000);
    will_return(__wrap_sqlite3_bind_int, 0);
    expect_value(__wrap_sqlite3_bind_int, index, 2);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_int, iCol, 0);
    will_return(__wrap_sqlite3_column_int, 1);

    expect_sqlite3_step_call(SQLITE_DONE);

    will_return(__wrap_wm_vuldet_add_cve_node, -1);
    expect_string(__wrap__mterror, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mterror, formatted_msg, "(5572): Couldn't insert the package 'test' into the vulnerability 'CVE-0000-0000'. Version (0.0.0) 'equals' '0.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_specific_package(db,
                                               FEED_DEBIAN,
                                               pkg_name,
                                               pkg_source,
                                               pkg_version,
                                               pkg_arch,
                                               pkg_vendor,
                                               pkg_reference,
                                               pkg_type,
                                               cve_table,
                                               name_type,
                                               vuln_count);
    assert_int_equal(ret, 1);
}

void test_wm_vuldet_check_specific_package_duplicated_package(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "test";
    char *pkg_version = "0.0.0";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = NULL;
    int8_t name_type = PACKAGE_SOURCE;
    int i = 1;
    int *vuln_count = &i;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "test");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 2);
    expect_string(__wrap_sqlite3_bind_text, buffer, "%0.0.0%");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "0.0.0"); // version_cmp
    expect_value(__wrap_sqlite3_column_int, iCol, 2);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 3);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 4);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 1000); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 6);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 7);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 8);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    // Siblings
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_int, index, 1);
    expect_value(__wrap_sqlite3_bind_int, value, 1000);
    will_return(__wrap_sqlite3_bind_int, 0);
    expect_value(__wrap_sqlite3_bind_int, index, 2);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_int, iCol, 0);
    will_return(__wrap_sqlite3_column_int, 1);

    expect_sqlite3_step_call(SQLITE_DONE);

    will_return(__wrap_wm_vuldet_add_cve_node, 1);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5459): Trying to insert duplicated package 'test' into the vulnerability 'CVE-0000-0000'. Version (0.0.0) 'equals' '0.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_specific_package(db,
                                               FEED_DEBIAN,
                                               pkg_name,
                                               pkg_source,
                                               pkg_version,
                                               pkg_arch,
                                               pkg_vendor,
                                               pkg_reference,
                                               pkg_type,
                                               cve_table,
                                               name_type,
                                               vuln_count);
    assert_int_equal(ret, 1);
}

void test_wm_vuldet_check_specific_package_insert_package_OK(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "test";
    char *pkg_version = "0.0.0";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = (OSHash *)1;
    int8_t name_type = PACKAGE_SOURCE;
    int i = 1;
    int *vuln_count = &i;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "test");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 2);
    expect_string(__wrap_sqlite3_bind_text, buffer, "%0.0.0%");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "0.0.0"); // version_cmp
    expect_value(__wrap_sqlite3_column_int, iCol, 2);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 3);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 4);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 1000); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 6);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 7);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 8);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    // Siblings
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_int, index, 1);
    expect_value(__wrap_sqlite3_bind_int, value, 1000);
    will_return(__wrap_sqlite3_bind_int, 0);
    expect_value(__wrap_sqlite3_bind_int, index, 2);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_int, iCol, 0);
    will_return(__wrap_sqlite3_column_int, 1);

    expect_sqlite3_step_call(SQLITE_DONE);

    will_return(__wrap_wm_vuldet_add_cve_node, 0);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5458): Package 'test' inserted into the vulnerability 'CVE-0000-0000'. Version (0.0.0) 'equals' '0.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_specific_package(db,
                                               FEED_DEBIAN,
                                               pkg_name,
                                               pkg_source,
                                               pkg_version,
                                               pkg_arch,
                                               pkg_vendor,
                                               pkg_reference,
                                               pkg_type,
                                               cve_table,
                                               name_type,
                                               vuln_count);
    assert_int_equal(ret, 1);
}

void test_wm_vuldet_check_specific_package_insert_package_Ubuntu(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "ubuntu_linux";
    char *pkg_version = "0.0.0";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = (OSHash *)1;
    int8_t name_type = PACKAGE_SOURCE;
    int i = 1;
    int *vuln_count = &i;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "ubuntu_linux");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 2);
    expect_string(__wrap_sqlite3_bind_text, buffer, "%0.0.0%");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 3);
    expect_string(__wrap_sqlite3_bind_text, buffer, "canonical");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "0.0.0"); // version_cmp
    expect_value(__wrap_sqlite3_column_int, iCol, 2);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 3);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 4);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 1000); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 6);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 7);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 8);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    // Siblings
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_int, index, 1);
    expect_value(__wrap_sqlite3_bind_int, value, 1000);
    will_return(__wrap_sqlite3_bind_int, 0);
    expect_value(__wrap_sqlite3_bind_int, index, 2);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_int, iCol, 0);
    will_return(__wrap_sqlite3_column_int, 1);

    expect_sqlite3_step_call(SQLITE_DONE);

    will_return(__wrap_wm_vuldet_add_cve_node, 0);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5458): Package 'ubuntu_linux' inserted into the vulnerability 'CVE-0000-0000'. Version (0.0.0) 'equals' '0.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_specific_package(db,
                                               FEED_DEBIAN,
                                               pkg_name,
                                               pkg_source,
                                               pkg_version,
                                               pkg_arch,
                                               pkg_vendor,
                                               pkg_reference,
                                               pkg_type,
                                               cve_table,
                                               name_type,
                                               vuln_count);
    assert_int_equal(ret, 1);
}

void test_wm_vuldet_check_specific_package_insert_package_Debian(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "debian_linux";
    char *pkg_version = "0.0.0";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = (OSHash *)1;
    int8_t name_type = PACKAGE_SOURCE;
    int i = 1;
    int *vuln_count = &i;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "debian_linux");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 2);
    expect_string(__wrap_sqlite3_bind_text, buffer, "%0.0.0%");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 3);
    expect_string(__wrap_sqlite3_bind_text, buffer, "debian");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "0.0.0"); // version_cmp
    expect_value(__wrap_sqlite3_column_int, iCol, 2);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 3);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 4);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 1000); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 6);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 7);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 8);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    // Siblings
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_int, index, 1);
    expect_value(__wrap_sqlite3_bind_int, value, 1000);
    will_return(__wrap_sqlite3_bind_int, 0);
    expect_value(__wrap_sqlite3_bind_int, index, 2);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_int, iCol, 0);
    will_return(__wrap_sqlite3_column_int, 1);

    expect_sqlite3_step_call(SQLITE_DONE);

    will_return(__wrap_wm_vuldet_add_cve_node, 0);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5458): Package 'debian_linux' inserted into the vulnerability 'CVE-0000-0000'. Version (0.0.0) 'equals' '0.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_specific_package(db,
                                               FEED_DEBIAN,
                                               pkg_name,
                                               pkg_source,
                                               pkg_version,
                                               pkg_arch,
                                               pkg_vendor,
                                               pkg_reference,
                                               pkg_type,
                                               cve_table,
                                               name_type,
                                               vuln_count);
    assert_int_equal(ret, 1);
}

void test_wm_vuldet_check_specific_package_insert_package_Arch(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = "arch_linux";
    char *pkg_source = NULL;
    char *pkg_version = "0.0.0";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = (OSHash *)1;
    int8_t name_type = PACKAGE_NAME;
    int i = 1;
    int *vuln_count = &i;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "arch_linux");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 2);
    expect_string(__wrap_sqlite3_bind_text, buffer, "%0.0.0%");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 3);
    expect_string(__wrap_sqlite3_bind_text, buffer, "archlinux");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "0.0.0"); // version_cmp
    expect_value(__wrap_sqlite3_column_int, iCol, 2);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 3);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 4);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 1000); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 6);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 7);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 8);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    // Siblings
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_int, index, 1);
    expect_value(__wrap_sqlite3_bind_int, value, 1000);
    will_return(__wrap_sqlite3_bind_int, 0);
    expect_value(__wrap_sqlite3_bind_int, index, 2);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_int, iCol, 0);
    will_return(__wrap_sqlite3_column_int, 1);

    expect_sqlite3_step_call(SQLITE_DONE);

    will_return(__wrap_wm_vuldet_add_cve_node, 0);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5458): Package 'arch_linux' inserted into the vulnerability 'CVE-0000-0000'. Version (0.0.0) 'equals' '0.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_specific_package(db,
                                               FEED_ARCH,
                                               pkg_name,
                                               pkg_source,
                                               pkg_version,
                                               pkg_arch,
                                               pkg_vendor,
                                               pkg_reference,
                                               pkg_type,
                                               cve_table,
                                               name_type,
                                               vuln_count);
    assert_int_equal(ret, 1);
}

void test_wm_vuldet_check_specific_package_insert_package_RedHat(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "enterprise_linux";
    char *pkg_version = "0.0.0";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = (OSHash *)1;
    int8_t name_type = PACKAGE_SOURCE;
    int i = 1;
    int *vuln_count = &i;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "enterprise_linux");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 2);
    expect_string(__wrap_sqlite3_bind_text, buffer, "%0.0.0%");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 3);
    expect_string(__wrap_sqlite3_bind_text, buffer, "redhat");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "0.0.0"); // version_cmp
    expect_value(__wrap_sqlite3_column_int, iCol, 2);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 3);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 4);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 1000); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 6);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 7);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 8);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    // Siblings
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_int, index, 1);
    expect_value(__wrap_sqlite3_bind_int, value, 1000);
    will_return(__wrap_sqlite3_bind_int, 0);
    expect_value(__wrap_sqlite3_bind_int, index, 2);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_int, iCol, 0);
    will_return(__wrap_sqlite3_column_int, 1);

    expect_sqlite3_step_call(SQLITE_DONE);

    will_return(__wrap_wm_vuldet_add_cve_node, 0);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5458): Package 'enterprise_linux' inserted into the vulnerability 'CVE-0000-0000'. Version (0.0.0) 'equals' '0.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_specific_package(db,
                                               FEED_DEBIAN,
                                               pkg_name,
                                               pkg_source,
                                               pkg_version,
                                               pkg_arch,
                                               pkg_vendor,
                                               pkg_reference,
                                               pkg_type,
                                               cve_table,
                                               name_type,
                                               vuln_count);
    assert_int_equal(ret, 1);
}

void test_wm_vuldet_check_specific_package_insert_package_linux_kernel(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "linux_kernel";
    char *pkg_version = "0.0.0";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = (OSHash *)1;
    int8_t name_type = PACKAGE_SOURCE;
    int i = 1;
    int *vuln_count = &i;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "linux_kernel");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 2);
    expect_string(__wrap_sqlite3_bind_text, buffer, "%0.0.0%");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 3);
    expect_string(__wrap_sqlite3_bind_text, buffer, "linux");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "0.0.0"); // version_cmp
    expect_value(__wrap_sqlite3_column_int, iCol, 2);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 3);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 4);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 1000); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 6);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 7);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 8);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    // Siblings
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_int, index, 1);
    expect_value(__wrap_sqlite3_bind_int, value, 1000);
    will_return(__wrap_sqlite3_bind_int, 0);
    expect_value(__wrap_sqlite3_bind_int, index, 2);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_int, iCol, 0);
    will_return(__wrap_sqlite3_column_int, 1);

    expect_sqlite3_step_call(SQLITE_DONE);

    will_return(__wrap_wm_vuldet_add_cve_node, 0);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5458): Package 'linux_kernel' inserted into the vulnerability 'CVE-0000-0000'. Version (0.0.0) 'equals' '0.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_specific_package(db,
                                               FEED_DEBIAN,
                                               pkg_name,
                                               pkg_source,
                                               pkg_version,
                                               pkg_arch,
                                               pkg_vendor,
                                               pkg_reference,
                                               pkg_type,
                                               cve_table,
                                               name_type,
                                               vuln_count);
    assert_int_equal(ret, 1);
}

void test_wm_vuldet_check_specific_package_insert_package_mac_os_x(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "mac_os_x";
    char *pkg_version = "10.15.1";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = (OSHash *)1;
    int8_t name_type = PACKAGE_SOURCE;
    int i = 1;
    int *vuln_count = &i;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "mac_os_x");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 2);
    expect_string(__wrap_sqlite3_bind_text, buffer, "%10.15.1%");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "10.15.1"); // version_cmp
    expect_value(__wrap_sqlite3_column_int, iCol, 2);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 3);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 4);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 1000); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 6);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 7);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 8);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    // Siblings
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_int, index, 1);
    expect_value(__wrap_sqlite3_bind_int, value, 1000);
    will_return(__wrap_sqlite3_bind_int, 0);
    expect_value(__wrap_sqlite3_bind_int, index, 2);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_int, iCol, 0);
    will_return(__wrap_sqlite3_column_int, 1);

    expect_sqlite3_step_call(SQLITE_DONE);

    will_return(__wrap_wm_vuldet_add_cve_node, 0);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5458): Package 'mac_os_x' inserted into the vulnerability 'CVE-0000-0000'. Version (10.15.1) 'equals' '10.15.1' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_specific_package(db,
                                               FEED_MAC,
                                               pkg_name,
                                               pkg_source,
                                               pkg_version,
                                               pkg_arch,
                                               pkg_vendor,
                                               pkg_reference,
                                               pkg_type,
                                               cve_table,
                                               name_type,
                                               vuln_count);
    assert_int_equal(ret, 1);
}

void test_wm_vuldet_check_specific_package_insert_package_mac_os_x_server(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "mac_os_x_server";
    char *pkg_version = "10.15.1";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = (OSHash *)1;
    int8_t name_type = PACKAGE_SOURCE;
    int i = 1;
    int *vuln_count = &i;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "mac_os_x_server");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 2);
    expect_string(__wrap_sqlite3_bind_text, buffer, "%10.15.1%");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "10.15.1"); // version_cmp
    expect_value(__wrap_sqlite3_column_int, iCol, 2);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 3);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 4);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 1000); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 6);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 7);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 8);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    // Siblings
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_int, index, 1);
    expect_value(__wrap_sqlite3_bind_int, value, 1000);
    will_return(__wrap_sqlite3_bind_int, 0);
    expect_value(__wrap_sqlite3_bind_int, index, 2);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_int, iCol, 0);
    will_return(__wrap_sqlite3_column_int, 1);

    expect_sqlite3_step_call(SQLITE_DONE);

    will_return(__wrap_wm_vuldet_add_cve_node, 0);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5458): Package 'mac_os_x_server' inserted into the vulnerability 'CVE-0000-0000'. Version (10.15.1) 'equals' '10.15.1' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_specific_package(db,
                                               FEED_MAC,
                                               pkg_name,
                                               pkg_source,
                                               pkg_version,
                                               pkg_arch,
                                               pkg_vendor,
                                               pkg_reference,
                                               pkg_type,
                                               cve_table,
                                               name_type,
                                               vuln_count);
    assert_int_equal(ret, 1);
}

void test_wm_vuldet_check_specific_package_insert_package_mac_os(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "mac_os";
    char *pkg_version = "10.15.1";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = (OSHash *)1;
    int8_t name_type = PACKAGE_SOURCE;
    int i = 1;
    int *vuln_count = &i;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "mac_os");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 2);
    expect_string(__wrap_sqlite3_bind_text, buffer, "%10.15.1%");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "10.15.1"); // version_cmp
    expect_value(__wrap_sqlite3_column_int, iCol, 2);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 3);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 4);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 1000); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 6);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 7);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 8);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    // Siblings
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_int, index, 1);
    expect_value(__wrap_sqlite3_bind_int, value, 1000);
    will_return(__wrap_sqlite3_bind_int, 0);
    expect_value(__wrap_sqlite3_bind_int, index, 2);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_int, iCol, 0);
    will_return(__wrap_sqlite3_column_int, 1);

    expect_sqlite3_step_call(SQLITE_DONE);

    will_return(__wrap_wm_vuldet_add_cve_node, 0);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5458): Package 'mac_os' inserted into the vulnerability 'CVE-0000-0000'. Version (10.15.1) 'equals' '10.15.1' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_specific_package(db,
                                               FEED_MAC,
                                               pkg_name,
                                               pkg_source,
                                               pkg_version,
                                               pkg_arch,
                                               pkg_vendor,
                                               pkg_reference,
                                               pkg_type,
                                               cve_table,
                                               name_type,
                                               vuln_count);
    assert_int_equal(ret, 1);
}

void test_wm_vuldet_check_specific_package_insert_package_macos(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "macos";
    char *pkg_version = "10.15.1";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = (OSHash *)1;
    int8_t name_type = PACKAGE_SOURCE;
    int i = 1;
    int *vuln_count = &i;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "macos");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 2);
    expect_string(__wrap_sqlite3_bind_text, buffer, "%10.15.1%");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "10.15.1"); // version_cmp
    expect_value(__wrap_sqlite3_column_int, iCol, 2);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 3);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 4);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 1000); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 6);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 7);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 8);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    // Siblings
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_int, index, 1);
    expect_value(__wrap_sqlite3_bind_int, value, 1000);
    will_return(__wrap_sqlite3_bind_int, 0);
    expect_value(__wrap_sqlite3_bind_int, index, 2);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_int, iCol, 0);
    will_return(__wrap_sqlite3_column_int, 1);

    expect_sqlite3_step_call(SQLITE_DONE);

    will_return(__wrap_wm_vuldet_add_cve_node, 0);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5458): Package 'macos' inserted into the vulnerability 'CVE-0000-0000'. Version (10.15.1) 'equals' '10.15.1' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_specific_package(db,
                                               FEED_MAC,
                                               pkg_name,
                                               pkg_source,
                                               pkg_version,
                                               pkg_arch,
                                               pkg_vendor,
                                               pkg_reference,
                                               pkg_type,
                                               cve_table,
                                               name_type,
                                               vuln_count);
    assert_int_equal(ret, 1);
}

void test_wm_vuldet_check_specific_package_insert_package_mac_no_vendor(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "safari";
    char *pkg_version = "2.1.3";
    char *pkg_arch = NULL;
    char *pkg_vendor = NULL;
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = (OSHash *)1;
    int8_t name_type = PACKAGE_SOURCE;
    int i = 1;
    int *vuln_count = &i;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "safari");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 2);
    expect_string(__wrap_sqlite3_bind_text, buffer, "%2.1.3%");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 3);
    expect_string(__wrap_sqlite3_bind_text, buffer, "");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 4);
    expect_string(__wrap_sqlite3_bind_text, buffer, "");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "2.1.3"); // version_cmp
    expect_value(__wrap_sqlite3_column_int, iCol, 2);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 3);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 4);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 1000); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 6);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 7);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 8);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    // Siblings
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_int, index, 1);
    expect_value(__wrap_sqlite3_bind_int, value, 1000);
    will_return(__wrap_sqlite3_bind_int, 0);
    expect_value(__wrap_sqlite3_bind_int, index, 2);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_int, iCol, 0);
    will_return(__wrap_sqlite3_column_int, 1);

    expect_sqlite3_step_call(SQLITE_DONE);

    will_return(__wrap_wm_vuldet_add_cve_node, 0);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5458): Package 'safari' inserted into the vulnerability 'CVE-0000-0000'. Version (2.1.3) 'equals' '2.1.3' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_specific_package(db,
                                               FEED_MAC,
                                               pkg_name,
                                               pkg_source,
                                               pkg_version,
                                               pkg_arch,
                                               pkg_vendor,
                                               pkg_reference,
                                               pkg_type,
                                               cve_table,
                                               name_type,
                                               vuln_count);
    assert_int_equal(ret, 1);
}

void test_wm_vuldet_check_specific_package_insert_package_mac_with_vendor(void **state)
{
    sqlite3 *db = (sqlite3 *)1;
    sqlite3_stmt *stmt = NULL;

    char *pkg_name = NULL;
    char *pkg_source = "safari";
    char *pkg_version = "2.1.3";
    char *pkg_arch = NULL;
    char *pkg_vendor = "apple";
    char *pkg_reference = NULL;
    char *pkg_type = NULL;
    OSHash *cve_table = (OSHash *)1;
    int8_t name_type = PACKAGE_SOURCE;
    int i = 1;
    int *vuln_count = &i;

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "safari");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 2);
    expect_string(__wrap_sqlite3_bind_text, buffer, "%2.1.3%");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 3);
    expect_string(__wrap_sqlite3_bind_text, buffer, "apple");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 4);
    expect_string(__wrap_sqlite3_bind_text, buffer, "apple");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "2.1.3"); // version_cmp
    expect_value(__wrap_sqlite3_column_int, iCol, 2);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 3);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 4);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 1000); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 6);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 7);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 8);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    // Siblings
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_int, index, 1);
    expect_value(__wrap_sqlite3_bind_int, value, 1000);
    will_return(__wrap_sqlite3_bind_int, 0);
    expect_value(__wrap_sqlite3_bind_int, index, 2);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_int, iCol, 0);
    will_return(__wrap_sqlite3_column_int, 1);

    expect_sqlite3_step_call(SQLITE_DONE);

    will_return(__wrap_wm_vuldet_add_cve_node, 0);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5458): Package 'safari' inserted into the vulnerability 'CVE-0000-0000'. Version (2.1.3) 'equals' '2.1.3' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    int ret = wm_vuldet_check_specific_package(db,
                                               FEED_MAC,
                                               pkg_name,
                                               pkg_source,
                                               pkg_version,
                                               pkg_arch,
                                               pkg_vendor,
                                               pkg_reference,
                                               pkg_type,
                                               cve_table,
                                               name_type,
                                               vuln_count);
    assert_int_equal(ret, 1);
}

/* wm_vuldet_linux_nvd_vulnerabilities */

void test_wm_vuldet_linux_nvd_vulnerabilities_prepare_error(void **state)
{
    scan_agent *agent = *state;
    sqlite3 *db = (sqlite3 *)1;
    OSHash *cve_table = (OSHash *)1;
    sqlite3_stmt *stmt = NULL;
    int *vuln_count = NULL;

    expect_string(__wrap__mtdebug1, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug1, formatted_msg, "(5451): Analyzing NVD vulnerabilities for agent '000'");

    will_return(__wrap_sqlite3_prepare_v2, NULL);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_ERROR);
    will_return(__wrap_sqlite3_errmsg, "error");
    expect_string(__wrap__mterror, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mterror, formatted_msg, "(5569): Couldn't get the packages of the agent '000'");
    expect_string(__wrap__mterror, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mterror, formatted_msg, "(5503): SQL error: 'error'");

    int ret = wm_vuldet_linux_nvd_vulnerabilities(db, agent, cve_table);
    assert_int_equal(ret, -1);
}

void test_wm_vuldet_linux_nvd_vulnerabilities_done(void **state)
{
    scan_agent *agent = *state;
    sqlite3 *db = (sqlite3 *)1;
    OSHash *cve_table = (OSHash *)1;
    sqlite3_stmt *stmt = NULL;
    int *vuln_count = NULL;

    expect_string(__wrap__mtdebug1, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug1, formatted_msg, "(5451): Analyzing NVD vulnerabilities for agent '000'");

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "000");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_DONE);

    expect_string(__wrap__mtdebug1, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug1, formatted_msg, "(5455): The NVD found a total of '0' potential vulnerabilities for agent '000'");

    expect_string(__wrap__mtdebug1, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug1, formatted_msg, "(5470): It took '0' seconds to 'find NVD' vulnerabilities in agent '000'");

    int ret = wm_vuldet_linux_nvd_vulnerabilities(db, agent, cve_table);
    assert_int_equal(ret, 0);
}

void test_wm_vuldet_linux_nvd_vulnerabilities_no_source_no_name(void **state)
{
    scan_agent *agent = *state;
    sqlite3 *db = (sqlite3 *)1;
    OSHash *cve_table = (OSHash *)1;
    sqlite3_stmt *stmt = NULL;
    int *vuln_count = NULL;

    expect_string(__wrap__mtdebug1, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug1, formatted_msg, "(5451): Analyzing NVD vulnerabilities for agent '000'");

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "000");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value_count(__wrap_sqlite3_column_text, iCol, 0, 1);
    will_return_count(__wrap_sqlite3_column_text, NULL, 1); // source
    expect_value_count(__wrap_sqlite3_column_text, iCol, 1, 1);
    will_return_count(__wrap_sqlite3_column_text, NULL, 1); // name
    expect_value_count(__wrap_sqlite3_column_text, iCol, 2, 2);
    will_return_count(__wrap_sqlite3_column_text, "0.0.0", 2); // version
    expect_value_count(__wrap_sqlite3_column_text, iCol, 3, 2);
    will_return_count(__wrap_sqlite3_column_text, "1.1.1", 2); // pkg source version
    expect_value_count(__wrap_sqlite3_column_text, iCol, 4, 2);
    will_return_count(__wrap_sqlite3_column_text, "x32", 2); // architecture
    expect_value_count(__wrap_sqlite3_column_text, iCol, 5, 1);
    will_return_count(__wrap_sqlite3_column_text, NULL, 1); // vendor
    expect_value_count(__wrap_sqlite3_column_text, iCol, 6, 1);
    will_return_count(__wrap_sqlite3_column_text, NULL, 1); // reference
    expect_value_count(__wrap_sqlite3_column_text, iCol, 7, 1);
    will_return_count(__wrap_sqlite3_column_text, NULL, 1); // type

    expect_sqlite3_step_call(SQLITE_DONE);

    expect_string(__wrap__mtdebug1, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug1, formatted_msg, "(5455): The NVD found a total of '0' potential vulnerabilities for agent '000'");

    expect_string(__wrap__mtdebug1, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug1, formatted_msg, "(5470): It took '0' seconds to 'find NVD' vulnerabilities in agent '000'");

    int ret = wm_vuldet_linux_nvd_vulnerabilities(db, agent, cve_table);
    assert_int_equal(ret, 0);
}

void test_wm_vuldet_linux_nvd_vulnerabilities_source_generic_invalid(void **state)
{
    scan_agent *agent = *state;
    sqlite3 *db = (sqlite3 *)1;
    OSHash *cve_table = (OSHash *)1;
    sqlite3_stmt *stmt = NULL;
    int *vuln_count = NULL;

    expect_string(__wrap__mtdebug1, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug1, formatted_msg, "(5451): Analyzing NVD vulnerabilities for agent '000'");

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "000");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value_count(__wrap_sqlite3_column_text, iCol, 0, 2);
    will_return_count(__wrap_sqlite3_column_text, "source", 2); // source
    expect_value_count(__wrap_sqlite3_column_text, iCol, 1, 2);
    will_return_count(__wrap_sqlite3_column_text, "name", 2); // name
    expect_value_count(__wrap_sqlite3_column_text, iCol, 2, 2);
    will_return_count(__wrap_sqlite3_column_text, "0.0.0", 2); // version
    expect_value_count(__wrap_sqlite3_column_text, iCol, 3, 2);
    will_return_count(__wrap_sqlite3_column_text, "1.1.1", 2); // pkg source version
    expect_value_count(__wrap_sqlite3_column_text, iCol, 4, 2);
    will_return_count(__wrap_sqlite3_column_text, "x32", 2); // architecture
    expect_value_count(__wrap_sqlite3_column_text, iCol, 5, 1);
    will_return_count(__wrap_sqlite3_column_text, NULL, 1); // vendor
    expect_value_count(__wrap_sqlite3_column_text, iCol, 6, 1);
    will_return_count(__wrap_sqlite3_column_text, NULL, 1); // reference
    expect_value_count(__wrap_sqlite3_column_text, iCol, 7, 1);
    will_return_count(__wrap_sqlite3_column_text, NULL, 1); // type

    // test_wm_vuldet_check_generic_package_prepare_error
    will_return(__wrap_sqlite3_prepare_v2, NULL);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_ERROR);
    will_return(__wrap_sqlite3_errmsg, "error");
    expect_string(__wrap__mterror, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mterror, formatted_msg, "(5570): Couldn't get from the NVD the 'generic' vulnerabilities of the package 'source'");
    expect_string(__wrap__mterror, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mterror, formatted_msg, "(5503): SQL error: 'error'");

    int ret = wm_vuldet_linux_nvd_vulnerabilities(db, agent, cve_table);
    assert_int_equal(ret, -1);
}

void test_wm_vuldet_linux_nvd_vulnerabilities_source_specific_invalid(void **state)
{
    scan_agent *agent = *state;
    sqlite3 *db = (sqlite3 *)1;
    OSHash *cve_table = (OSHash *)1;
    sqlite3_stmt *stmt = NULL;
    int *vuln_count = NULL;

    expect_string(__wrap__mtdebug1, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug1, formatted_msg, "(5451): Analyzing NVD vulnerabilities for agent '000'");

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "000");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value_count(__wrap_sqlite3_column_text, iCol, 0, 2);
    will_return_count(__wrap_sqlite3_column_text, "source", 2); // source
    expect_value_count(__wrap_sqlite3_column_text, iCol, 1, 2);
    will_return_count(__wrap_sqlite3_column_text, "name", 2); // name
    expect_value_count(__wrap_sqlite3_column_text, iCol, 2, 2);
    will_return_count(__wrap_sqlite3_column_text, "0.0.0", 2); // version
    expect_value_count(__wrap_sqlite3_column_text, iCol, 3, 2);
    will_return_count(__wrap_sqlite3_column_text, "0.1.1", 2); // pkg source version
    expect_value_count(__wrap_sqlite3_column_text, iCol, 4, 2);
    will_return_count(__wrap_sqlite3_column_text, "x32", 2); // architecture
    expect_value_count(__wrap_sqlite3_column_text, iCol, 5, 1);
    will_return_count(__wrap_sqlite3_column_text, NULL, 1); // vendor
    expect_value_count(__wrap_sqlite3_column_text, iCol, 6, 1);
    will_return_count(__wrap_sqlite3_column_text, NULL, 1); // reference
    expect_value_count(__wrap_sqlite3_column_text, iCol, 7, 1);
    will_return_count(__wrap_sqlite3_column_text, NULL, 1); // type

    // test_wm_vuldet_check_generic_package_siblings_OK
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "source");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "0.0.0"); // start_including
    expect_value(__wrap_sqlite3_column_text, iCol, 2);
    will_return(__wrap_sqlite3_column_text, NULL); // start_excluding
    expect_value(__wrap_sqlite3_column_text, iCol, 3);
    will_return(__wrap_sqlite3_column_text, "1.0.0"); // end_including
    expect_value(__wrap_sqlite3_column_text, iCol, 4);
    will_return(__wrap_sqlite3_column_text, NULL); // end_excluding
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 6);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 7);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 8);
    will_return(__wrap_sqlite3_column_int, 1000); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 9);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 10);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 11);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    // Siblings
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_int, index, 1);
    expect_value(__wrap_sqlite3_bind_int, value, 1000);
    will_return(__wrap_sqlite3_bind_int, 0);
    expect_value(__wrap_sqlite3_bind_int, index, 2);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_int, iCol, 0);
    will_return(__wrap_sqlite3_column_int, 1);

    expect_sqlite3_step_call(SQLITE_DONE);

    will_return(__wrap_wm_vuldet_add_cve_node, 0);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5458): Package 'source' inserted into the vulnerability 'CVE-0000-0000'. Version (0.1.1) 'less than or equal' '1.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    // test_wm_vuldet_check_specific_package_prepare_error
    will_return(__wrap_sqlite3_prepare_v2, NULL);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_ERROR);
    will_return(__wrap_sqlite3_errmsg, "error");
    expect_string(__wrap__mterror, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mterror, formatted_msg, "(5570): Couldn't get from the NVD the 'specific' vulnerabilities of the package 'source'");
    expect_string(__wrap__mterror, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mterror, formatted_msg, "(5503): SQL error: 'error'");

    int ret = wm_vuldet_linux_nvd_vulnerabilities(db, agent, cve_table);
    assert_int_equal(ret, -1);
}

void test_wm_vuldet_linux_nvd_vulnerabilities_source_no_vulnerable(void **state)
{
    scan_agent *agent = *state;
    sqlite3 *db = (sqlite3 *)1;
    OSHash *cve_table = (OSHash *)1;
    sqlite3_stmt *stmt = NULL;
    int *vuln_count = NULL;

    expect_string(__wrap__mtdebug1, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug1, formatted_msg, "(5451): Analyzing NVD vulnerabilities for agent '000'");

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "000");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value_count(__wrap_sqlite3_column_text, iCol, 0, 2);
    will_return_count(__wrap_sqlite3_column_text, "source", 2); // source
    expect_value_count(__wrap_sqlite3_column_text, iCol, 1, 2);
    will_return_count(__wrap_sqlite3_column_text, "name", 2); // name
    expect_value_count(__wrap_sqlite3_column_text, iCol, 2, 2);
    will_return_count(__wrap_sqlite3_column_text, "2.0.0", 2); // version
    expect_value_count(__wrap_sqlite3_column_text, iCol, 3, 2);
    will_return_count(__wrap_sqlite3_column_text, "1.1.1", 2); // pkg source version
    expect_value_count(__wrap_sqlite3_column_text, iCol, 4, 2);
    will_return_count(__wrap_sqlite3_column_text, "x32", 2); // architecture
    expect_value_count(__wrap_sqlite3_column_text, iCol, 5, 1);
    will_return_count(__wrap_sqlite3_column_text, NULL, 1); // vendor
    expect_value_count(__wrap_sqlite3_column_text, iCol, 6, 1);
    will_return_count(__wrap_sqlite3_column_text, NULL, 1); // reference
    expect_value_count(__wrap_sqlite3_column_text, iCol, 7, 1);
    will_return_count(__wrap_sqlite3_column_text, NULL, 1); // type

    // test_wm_vuldet_check_generic_package_end_including_no_vulnerable()
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "source");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, NULL); // start_including
    expect_value(__wrap_sqlite3_column_text, iCol, 2);
    will_return(__wrap_sqlite3_column_text, NULL); // start_excluding
    expect_value(__wrap_sqlite3_column_text, iCol, 3);
    will_return(__wrap_sqlite3_column_text, "1.0.0"); // end_including
    expect_value(__wrap_sqlite3_column_text, iCol, 4);
    will_return(__wrap_sqlite3_column_text, NULL); // end_excluding
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 6);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 7);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 8);
    will_return(__wrap_sqlite3_column_int, 0); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 9);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 10);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 11);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, 0);

    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5460): Package 'source' not vulnerable to 'CVE-0000-0000'. Version (1.1.1) not 'less than or equal' '1.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    // test_wm_vuldet_check_specific_package_no_vulnerable()
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "source");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 2);
    expect_string(__wrap_sqlite3_bind_text, buffer, "%1.1.1%");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "1.0.0"); // version_cmp
    expect_value(__wrap_sqlite3_column_int, iCol, 2);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 3);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 4);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 6);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 7);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 8);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, 0);

    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5460): Package 'source' not vulnerable to 'CVE-0000-0000'. Version (1.1.1) not 'equals' '1.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    expect_sqlite3_step_call(SQLITE_DONE);

    expect_string(__wrap__mtdebug1, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug1, formatted_msg, "(5455): The NVD found a total of '0' potential vulnerabilities for agent '000'");

    expect_string(__wrap__mtdebug1, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug1, formatted_msg, "(5470): It took '0' seconds to 'find NVD' vulnerabilities in agent '000'");

    int ret = wm_vuldet_linux_nvd_vulnerabilities(db, agent, cve_table);
    assert_int_equal(ret, 0);
}

void test_wm_vuldet_linux_nvd_vulnerabilities_source_vulnerable(void **state)
{
    scan_agent *agent = *state;
    sqlite3 *db = (sqlite3 *)1;
    OSHash *cve_table = (OSHash *)1;
    sqlite3_stmt *stmt = NULL;
    int *vuln_count = NULL;

    expect_string(__wrap__mtdebug1, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug1, formatted_msg, "(5451): Analyzing NVD vulnerabilities for agent '000'");

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "000");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value_count(__wrap_sqlite3_column_text, iCol, 0, 2);
    will_return_count(__wrap_sqlite3_column_text, "source", 2); // source
    expect_value_count(__wrap_sqlite3_column_text, iCol, 1, 2);
    will_return_count(__wrap_sqlite3_column_text, "name", 2); // name
    expect_value_count(__wrap_sqlite3_column_text, iCol, 2, 2);
    will_return_count(__wrap_sqlite3_column_text, "0.0.0", 2); // version
    expect_value_count(__wrap_sqlite3_column_text, iCol, 3, 2);
    will_return_count(__wrap_sqlite3_column_text, "0.0.0", 2); // pkg source version
    expect_value_count(__wrap_sqlite3_column_text, iCol, 4, 2);
    will_return_count(__wrap_sqlite3_column_text, "x32", 2); // architecture
    expect_value_count(__wrap_sqlite3_column_text, iCol, 5, 1);
    will_return_count(__wrap_sqlite3_column_text, NULL, 1); // vendor
    expect_value_count(__wrap_sqlite3_column_text, iCol, 6, 1);
    will_return_count(__wrap_sqlite3_column_text, NULL, 1); // reference
    expect_value_count(__wrap_sqlite3_column_text, iCol, 7, 1);
    will_return_count(__wrap_sqlite3_column_text, NULL, 1); // type

    // test_wm_vuldet_check_generic_package_siblings_OK
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "source");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "0.0.0"); // start_including
    expect_value(__wrap_sqlite3_column_text, iCol, 2);
    will_return(__wrap_sqlite3_column_text, NULL); // start_excluding
    expect_value(__wrap_sqlite3_column_text, iCol, 3);
    will_return(__wrap_sqlite3_column_text, "1.0.0"); // end_including
    expect_value(__wrap_sqlite3_column_text, iCol, 4);
    will_return(__wrap_sqlite3_column_text, NULL); // end_excluding
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 6);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 7);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 8);
    will_return(__wrap_sqlite3_column_int, 1000); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 9);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 10);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 11);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    // Siblings
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_int, index, 1);
    expect_value(__wrap_sqlite3_bind_int, value, 1000);
    will_return(__wrap_sqlite3_bind_int, 0);
    expect_value(__wrap_sqlite3_bind_int, index, 2);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_int, iCol, 0);
    will_return(__wrap_sqlite3_column_int, 1);

    expect_sqlite3_step_call(SQLITE_DONE);

    will_return(__wrap_wm_vuldet_add_cve_node, 0);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5458): Package 'source' inserted into the vulnerability 'CVE-0000-0000'. Version (0.0.0) 'less than or equal' '1.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    // test_wm_vuldet_check_specific_package_vulnerable()

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "source");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 2);
    expect_string(__wrap_sqlite3_bind_text, buffer, "%0.0.0%");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "0.0.0"); // version_cmp
    expect_value(__wrap_sqlite3_column_int, iCol, 2);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 3);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 4);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 6);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 7);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 8);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    will_return(__wrap_wm_vuldet_add_cve_node, 0);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5458): Package 'source' inserted into the vulnerability 'CVE-0000-0000'. Version (0.0.0) 'equals' '0.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    expect_sqlite3_step_call(SQLITE_DONE);

    expect_string(__wrap__mtdebug1, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug1, formatted_msg, "(5455): The NVD found a total of '2' potential vulnerabilities for agent '000'");

    expect_string(__wrap__mtdebug1, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug1, formatted_msg, "(5470): It took '0' seconds to 'find NVD' vulnerabilities in agent '000'");

    int ret = wm_vuldet_linux_nvd_vulnerabilities(db, agent, cve_table);
    assert_int_equal(ret, 0);
}

void test_wm_vuldet_linux_nvd_vulnerabilities_name_generic_invalid(void **state)
{
    scan_agent *agent = *state;
    sqlite3 *db = (sqlite3 *)1;
    OSHash *cve_table = (OSHash *)1;
    sqlite3_stmt *stmt = NULL;
    int *vuln_count = NULL;

    expect_string(__wrap__mtdebug1, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug1, formatted_msg, "(5451): Analyzing NVD vulnerabilities for agent '000'");

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "000");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value_count(__wrap_sqlite3_column_text, iCol, 0, 1);
    will_return_count(__wrap_sqlite3_column_text, NULL, 1); // source
    expect_value_count(__wrap_sqlite3_column_text, iCol, 1, 2);
    will_return_count(__wrap_sqlite3_column_text, "name", 2); // name
    expect_value_count(__wrap_sqlite3_column_text, iCol, 2, 2);
    will_return_count(__wrap_sqlite3_column_text, "0.0.0", 2); // version
    expect_value_count(__wrap_sqlite3_column_text, iCol, 3, 2);
    will_return_count(__wrap_sqlite3_column_text, "1.1.1", 2); // pkg source version
    expect_value_count(__wrap_sqlite3_column_text, iCol, 4, 2);
    will_return_count(__wrap_sqlite3_column_text, "x32", 2); // architecture
    expect_value_count(__wrap_sqlite3_column_text, iCol, 5, 1);
    will_return_count(__wrap_sqlite3_column_text, NULL, 1); // vendor
    expect_value_count(__wrap_sqlite3_column_text, iCol, 6, 1);
    will_return_count(__wrap_sqlite3_column_text, NULL, 1); // reference
    expect_value_count(__wrap_sqlite3_column_text, iCol, 7, 1);
    will_return_count(__wrap_sqlite3_column_text, NULL, 1); // type

    // test_wm_vuldet_check_generic_package_prepare_error
    will_return(__wrap_sqlite3_prepare_v2, NULL);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_ERROR);
    will_return(__wrap_sqlite3_errmsg, "error");
    expect_string(__wrap__mterror, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mterror, formatted_msg, "(5570): Couldn't get from the NVD the 'generic' vulnerabilities of the package 'name'");
    expect_string(__wrap__mterror, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mterror, formatted_msg, "(5503): SQL error: 'error'");

    int ret = wm_vuldet_linux_nvd_vulnerabilities(db, agent, cve_table);
    assert_int_equal(ret, -1);
}

void test_wm_vuldet_linux_nvd_vulnerabilities_name_specific_invalid(void **state)
{
    scan_agent *agent = *state;
    sqlite3 *db = (sqlite3 *)1;
    OSHash *cve_table = (OSHash *)1;
    sqlite3_stmt *stmt = NULL;
    int *vuln_count = NULL;

    expect_string(__wrap__mtdebug1, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug1, formatted_msg, "(5451): Analyzing NVD vulnerabilities for agent '000'");

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "000");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value_count(__wrap_sqlite3_column_text, iCol, 0, 1);
    will_return_count(__wrap_sqlite3_column_text, NULL, 1); // source
    expect_value_count(__wrap_sqlite3_column_text, iCol, 1, 2);
    will_return_count(__wrap_sqlite3_column_text, "name", 2); // name
    expect_value_count(__wrap_sqlite3_column_text, iCol, 2, 2);
    will_return_count(__wrap_sqlite3_column_text, "0.0.0", 2); // version
    expect_value_count(__wrap_sqlite3_column_text, iCol, 3, 2);
    will_return_count(__wrap_sqlite3_column_text, "1.1.1", 2); // pkg source version
    expect_value_count(__wrap_sqlite3_column_text, iCol, 4, 2);
    will_return_count(__wrap_sqlite3_column_text, "x32", 2); // architecture
    expect_value_count(__wrap_sqlite3_column_text, iCol, 5, 1);
    will_return_count(__wrap_sqlite3_column_text, NULL, 1); // vendor
    expect_value_count(__wrap_sqlite3_column_text, iCol, 6, 1);
    will_return_count(__wrap_sqlite3_column_text, NULL, 1); // reference
    expect_value_count(__wrap_sqlite3_column_text, iCol, 7, 1);
    will_return_count(__wrap_sqlite3_column_text, NULL, 1); // type

    // test_wm_vuldet_check_generic_package_siblings_OK
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "name");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "0.0.0"); // start_including
    expect_value(__wrap_sqlite3_column_text, iCol, 2);
    will_return(__wrap_sqlite3_column_text, NULL); // start_excluding
    expect_value(__wrap_sqlite3_column_text, iCol, 3);
    will_return(__wrap_sqlite3_column_text, "1.0.0"); // end_including
    expect_value(__wrap_sqlite3_column_text, iCol, 4);
    will_return(__wrap_sqlite3_column_text, NULL); // end_excluding
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 6);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 7);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 8);
    will_return(__wrap_sqlite3_column_int, 1000); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 9);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 10);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 11);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    // Siblings
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_int, index, 1);
    expect_value(__wrap_sqlite3_bind_int, value, 1000);
    will_return(__wrap_sqlite3_bind_int, 0);
    expect_value(__wrap_sqlite3_bind_int, index, 2);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_int, iCol, 0);
    will_return(__wrap_sqlite3_column_int, 1);

    expect_sqlite3_step_call(SQLITE_DONE);

    will_return(__wrap_wm_vuldet_add_cve_node, 0);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5458): Package 'name' inserted into the vulnerability 'CVE-0000-0000'. Version (0.0.0) 'less than or equal' '1.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    // test_wm_vuldet_check_specific_package_prepare_error
    will_return(__wrap_sqlite3_prepare_v2, NULL);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_ERROR);
    will_return(__wrap_sqlite3_errmsg, "error");
    expect_string(__wrap__mterror, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mterror, formatted_msg, "(5570): Couldn't get from the NVD the 'specific' vulnerabilities of the package 'name'");
    expect_string(__wrap__mterror, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mterror, formatted_msg, "(5503): SQL error: 'error'");

    int ret = wm_vuldet_linux_nvd_vulnerabilities(db, agent, cve_table);
    assert_int_equal(ret, -1);
}

void test_wm_vuldet_linux_nvd_vulnerabilities_name_no_vulnerable(void **state)
{
    scan_agent *agent = *state;
    sqlite3 *db = (sqlite3 *)1;
    OSHash *cve_table = (OSHash *)1;
    sqlite3_stmt *stmt = NULL;
    int *vuln_count = NULL;

    expect_string(__wrap__mtdebug1, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug1, formatted_msg, "(5451): Analyzing NVD vulnerabilities for agent '000'");

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "000");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value_count(__wrap_sqlite3_column_text, iCol, 0, 1);
    will_return_count(__wrap_sqlite3_column_text, NULL, 1); // source
    expect_value_count(__wrap_sqlite3_column_text, iCol, 1, 2);
    will_return_count(__wrap_sqlite3_column_text, "name", 2); // name
    expect_value_count(__wrap_sqlite3_column_text, iCol, 2, 2);
    will_return_count(__wrap_sqlite3_column_text, "2.0.0", 2); // version
    expect_value_count(__wrap_sqlite3_column_text, iCol, 3, 2);
    will_return_count(__wrap_sqlite3_column_text, "1.1.1", 2); // pkg source version
    expect_value_count(__wrap_sqlite3_column_text, iCol, 4, 2);
    will_return_count(__wrap_sqlite3_column_text, "x32", 2); // architecture
    expect_value_count(__wrap_sqlite3_column_text, iCol, 5, 1);
    will_return_count(__wrap_sqlite3_column_text, NULL, 1); // vendor
    expect_value_count(__wrap_sqlite3_column_text, iCol, 6, 1);
    will_return_count(__wrap_sqlite3_column_text, NULL, 1); // reference
    expect_value_count(__wrap_sqlite3_column_text, iCol, 7, 1);
    will_return_count(__wrap_sqlite3_column_text, NULL, 1); // type

    // test_wm_vuldet_check_generic_package_end_including_no_vulnerable()
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "name");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, NULL); // start_including
    expect_value(__wrap_sqlite3_column_text, iCol, 2);
    will_return(__wrap_sqlite3_column_text, NULL); // start_excluding
    expect_value(__wrap_sqlite3_column_text, iCol, 3);
    will_return(__wrap_sqlite3_column_text, "1.0.0"); // end_including
    expect_value(__wrap_sqlite3_column_text, iCol, 4);
    will_return(__wrap_sqlite3_column_text, NULL); // end_excluding
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 6);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 7);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 8);
    will_return(__wrap_sqlite3_column_int, 0); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 9);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 10);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 11);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, 0);

    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5460): Package 'name' not vulnerable to 'CVE-0000-0000'. Version (2.0.0) not 'less than or equal' '1.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    // test_wm_vuldet_check_specific_package_no_vulnerable()
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "name");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 2);
    expect_string(__wrap_sqlite3_bind_text, buffer, "%2.0.0%");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "1.0.0"); // version_cmp
    expect_value(__wrap_sqlite3_column_int, iCol, 2);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 3);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 4);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 6);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 7);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 8);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, 0);

    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5460): Package 'name' not vulnerable to 'CVE-0000-0000'. Version (2.0.0) not 'equals' '1.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    expect_sqlite3_step_call(SQLITE_DONE);

    expect_string(__wrap__mtdebug1, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug1, formatted_msg, "(5455): The NVD found a total of '0' potential vulnerabilities for agent '000'");

    expect_string(__wrap__mtdebug1, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug1, formatted_msg, "(5470): It took '0' seconds to 'find NVD' vulnerabilities in agent '000'");

    int ret = wm_vuldet_linux_nvd_vulnerabilities(db, agent, cve_table);
    assert_int_equal(ret, 0);
}

void test_wm_vuldet_linux_nvd_vulnerabilities_name_vulnerable(void **state)
{
    scan_agent *agent = *state;
    sqlite3 *db = (sqlite3 *)1;
    OSHash *cve_table = (OSHash *)1;
    sqlite3_stmt *stmt = NULL;
    int *vuln_count = NULL;

    expect_string(__wrap__mtdebug1, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug1, formatted_msg, "(5451): Analyzing NVD vulnerabilities for agent '000'");

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "000");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value_count(__wrap_sqlite3_column_text, iCol, 0, 1);
    will_return_count(__wrap_sqlite3_column_text, NULL, 1); // source
    expect_value_count(__wrap_sqlite3_column_text, iCol, 1, 2);
    will_return_count(__wrap_sqlite3_column_text, "name", 2); // name
    expect_value_count(__wrap_sqlite3_column_text, iCol, 2, 2);
    will_return_count(__wrap_sqlite3_column_text, "0.0.0", 2); // version
    expect_value_count(__wrap_sqlite3_column_text, iCol, 3, 2);
    will_return_count(__wrap_sqlite3_column_text, "1.1.1", 2); // pkg source version
    expect_value_count(__wrap_sqlite3_column_text, iCol, 4, 2);
    will_return_count(__wrap_sqlite3_column_text, "x32", 2); // architecture
    expect_value_count(__wrap_sqlite3_column_text, iCol, 5, 1);
    will_return_count(__wrap_sqlite3_column_text, NULL, 1); // vendor
    expect_value_count(__wrap_sqlite3_column_text, iCol, 6, 1);
    will_return_count(__wrap_sqlite3_column_text, NULL, 1); // reference
    expect_value_count(__wrap_sqlite3_column_text, iCol, 7, 1);
    will_return_count(__wrap_sqlite3_column_text, NULL, 1); // type

    // test_wm_vuldet_check_generic_package_siblings_OK
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "name");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "0.0.0"); // start_including
    expect_value(__wrap_sqlite3_column_text, iCol, 2);
    will_return(__wrap_sqlite3_column_text, NULL); // start_excluding
    expect_value(__wrap_sqlite3_column_text, iCol, 3);
    will_return(__wrap_sqlite3_column_text, "1.0.0"); // end_including
    expect_value(__wrap_sqlite3_column_text, iCol, 4);
    will_return(__wrap_sqlite3_column_text, NULL); // end_excluding
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 6);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 7);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 8);
    will_return(__wrap_sqlite3_column_int, 1000); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 9);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 10);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 11);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    // Siblings
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_int, index, 1);
    expect_value(__wrap_sqlite3_bind_int, value, 1000);
    will_return(__wrap_sqlite3_bind_int, 0);
    expect_value(__wrap_sqlite3_bind_int, index, 2);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_int, iCol, 0);
    will_return(__wrap_sqlite3_column_int, 1);

    expect_sqlite3_step_call(SQLITE_DONE);

    will_return(__wrap_wm_vuldet_add_cve_node, 0);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5458): Package 'name' inserted into the vulnerability 'CVE-0000-0000'. Version (0.0.0) 'less than or equal' '1.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    // test_wm_vuldet_check_specific_package_vulnerable()

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "name");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 2);
    expect_string(__wrap_sqlite3_bind_text, buffer, "%0.0.0%");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "0.0.0"); // version_cmp
    expect_value(__wrap_sqlite3_column_int, iCol, 2);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 3);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 4);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 6);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 7);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 8);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    will_return(__wrap_wm_vuldet_add_cve_node, 0);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5458): Package 'name' inserted into the vulnerability 'CVE-0000-0000'. Version (0.0.0) 'equals' '0.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    expect_sqlite3_step_call(SQLITE_DONE);

    expect_string(__wrap__mtdebug1, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug1, formatted_msg, "(5455): The NVD found a total of '2' potential vulnerabilities for agent '000'");

    expect_string(__wrap__mtdebug1, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug1, formatted_msg, "(5470): It took '0' seconds to 'find NVD' vulnerabilities in agent '000'");

    int ret = wm_vuldet_linux_nvd_vulnerabilities(db, agent, cve_table);
    assert_int_equal(ret, 0);
}

void test_wm_vuldet_linux_nvd_vulnerabilities_pkg_source_version_NULL(void **state)
{
    scan_agent *agent = *state;
    sqlite3 *db = (sqlite3 *)1;
    OSHash *cve_table = (OSHash *)1;
    sqlite3_stmt *stmt = NULL;
    int *vuln_count = NULL;

    expect_string(__wrap__mtdebug1, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug1, formatted_msg, "(5451): Analyzing NVD vulnerabilities for agent '000'");

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "000");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value_count(__wrap_sqlite3_column_text, iCol, 0, 2);
    will_return_count(__wrap_sqlite3_column_text, "source", 2); // source
    expect_value_count(__wrap_sqlite3_column_text, iCol, 1, 2);
    will_return_count(__wrap_sqlite3_column_text, "name", 2); // name
    expect_value_count(__wrap_sqlite3_column_text, iCol, 2, 2);
    will_return_count(__wrap_sqlite3_column_text, "0.0.0", 2); // version
    expect_value_count(__wrap_sqlite3_column_text, iCol, 3, 1);
    will_return_count(__wrap_sqlite3_column_text, NULL, 1); // pkg source version
    expect_value_count(__wrap_sqlite3_column_text, iCol, 4, 2);
    will_return_count(__wrap_sqlite3_column_text, "x32", 2); // architecture
    expect_value_count(__wrap_sqlite3_column_text, iCol, 5, 1);
    will_return_count(__wrap_sqlite3_column_text, NULL, 1); // vendor
    expect_value_count(__wrap_sqlite3_column_text, iCol, 6, 1);
    will_return_count(__wrap_sqlite3_column_text, NULL, 1); // reference
    expect_value_count(__wrap_sqlite3_column_text, iCol, 7, 1);
    will_return_count(__wrap_sqlite3_column_text, NULL, 1); // type

    // test_wm_vuldet_check_generic_package_siblings_OK
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "source");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "0.0.0"); // start_including
    expect_value(__wrap_sqlite3_column_text, iCol, 2);
    will_return(__wrap_sqlite3_column_text, NULL); // start_excluding
    expect_value(__wrap_sqlite3_column_text, iCol, 3);
    will_return(__wrap_sqlite3_column_text, "1.0.0"); // end_including
    expect_value(__wrap_sqlite3_column_text, iCol, 4);
    will_return(__wrap_sqlite3_column_text, NULL); // end_excluding
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 6);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 7);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 8);
    will_return(__wrap_sqlite3_column_int, 1000); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 9);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 10);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 11);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    // Siblings
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_int, index, 1);
    expect_value(__wrap_sqlite3_bind_int, value, 1000);
    will_return(__wrap_sqlite3_bind_int, 0);
    expect_value(__wrap_sqlite3_bind_int, index, 2);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_int, iCol, 0);
    will_return(__wrap_sqlite3_column_int, 1);

    expect_sqlite3_step_call(SQLITE_DONE);

    will_return(__wrap_wm_vuldet_add_cve_node, 0);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5458): Package 'source' inserted into the vulnerability 'CVE-0000-0000'. Version (0.0.0) 'less than or equal' '1.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    // test_wm_vuldet_check_specific_package_vulnerable()

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "source");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 2);
    expect_string(__wrap_sqlite3_bind_text, buffer, "%0.0.0%");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "0.0.0"); // version_cmp
    expect_value(__wrap_sqlite3_column_int, iCol, 2);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 3);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 4);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 6);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 7);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 8);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    will_return(__wrap_wm_vuldet_add_cve_node, 0);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5458): Package 'source' inserted into the vulnerability 'CVE-0000-0000'. Version (0.0.0) 'equals' '0.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    expect_sqlite3_step_call(SQLITE_DONE);

    expect_string(__wrap__mtdebug1, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug1, formatted_msg, "(5455): The NVD found a total of '2' potential vulnerabilities for agent '000'");

    expect_string(__wrap__mtdebug1, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug1, formatted_msg, "(5470): It took '0' seconds to 'find NVD' vulnerabilities in agent '000'");

    int ret = wm_vuldet_linux_nvd_vulnerabilities(db, agent, cve_table);
    assert_int_equal(ret, 0);
}

void test_wm_vuldet_linux_nvd_vulnerabilities_os_pkg_mac(void **state)
{
    scan_agent *agent = *state;
    agent->dist = FEED_MAC;
    sqlite3 *db = (sqlite3 *)1;
    OSHash *cve_table = (OSHash *)1;
    sqlite3_stmt *stmt = NULL;
    int *vuln_count = NULL;

    expect_string(__wrap__mtdebug1, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug1, formatted_msg, "(5451): Analyzing NVD vulnerabilities for agent '000'");

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "000");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value_count(__wrap_sqlite3_column_text, iCol, 0, 1);
    will_return_count(__wrap_sqlite3_column_text, NULL, 1); // source
    expect_value_count(__wrap_sqlite3_column_text, iCol, 1, 2);
    will_return_count(__wrap_sqlite3_column_text, "mac_os_x", 2); // name
    expect_value_count(__wrap_sqlite3_column_text, iCol, 2, 2);
    will_return_count(__wrap_sqlite3_column_text, "0.0.0", 2); // version
    expect_value_count(__wrap_sqlite3_column_text, iCol, 3, 1);
    will_return_count(__wrap_sqlite3_column_text, NULL, 1); // pkg source version
    expect_value_count(__wrap_sqlite3_column_text, iCol, 4, 2);
    will_return_count(__wrap_sqlite3_column_text, "x64", 2); // architecture
    expect_value_count(__wrap_sqlite3_column_text, iCol, 5, 1);
    will_return_count(__wrap_sqlite3_column_text, NULL, 1); // vendor
    expect_value_count(__wrap_sqlite3_column_text, iCol, 6, 1);
    will_return_count(__wrap_sqlite3_column_text, NULL, 1); // reference
    expect_value_count(__wrap_sqlite3_column_text, iCol, 7, 1);
    will_return_count(__wrap_sqlite3_column_text, NULL, 1); // type

    // test_wm_vuldet_check_generic_package_siblings_OK
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "mac_os_x");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "0.0.0"); // start_including
    expect_value(__wrap_sqlite3_column_text, iCol, 2);
    will_return(__wrap_sqlite3_column_text, NULL); // start_excluding
    expect_value(__wrap_sqlite3_column_text, iCol, 3);
    will_return(__wrap_sqlite3_column_text, "1.0.0"); // end_including
    expect_value(__wrap_sqlite3_column_text, iCol, 4);
    will_return(__wrap_sqlite3_column_text, NULL); // end_excluding
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 6);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 7);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 8);
    will_return(__wrap_sqlite3_column_int, 1000); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 9);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 10);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 11);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    // Siblings
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_int, index, 1);
    expect_value(__wrap_sqlite3_bind_int, value, 1000);
    will_return(__wrap_sqlite3_bind_int, 0);
    expect_value(__wrap_sqlite3_bind_int, index, 2);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_int, iCol, 0);
    will_return(__wrap_sqlite3_column_int, 1);

    expect_sqlite3_step_call(SQLITE_DONE);

    will_return(__wrap_wm_vuldet_add_cve_node, 0);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5458): Package 'mac_os_x' inserted into the vulnerability 'CVE-0000-0000'. Version (0.0.0) 'less than or equal' '1.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    // test_wm_vuldet_check_specific_package_vulnerable()

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "mac_os_x");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 2);
    expect_string(__wrap_sqlite3_bind_text, buffer, "%0.0.0%");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "0.0.0"); // version_cmp
    expect_value(__wrap_sqlite3_column_int, iCol, 2);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 3);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 4);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 6);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 7);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 8);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    expect_sqlite3_step_call(SQLITE_DONE);

    expect_sqlite3_step_call(SQLITE_DONE);

    expect_string(__wrap__mtdebug1, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug1, formatted_msg, "(5455): The NVD found a total of '1' potential vulnerabilities for agent '000'");

    expect_string(__wrap__mtdebug1, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug1, formatted_msg, "(5470): It took '0' seconds to 'find NVD' vulnerabilities in agent '000'");

    int ret = wm_vuldet_linux_nvd_vulnerabilities(db, agent, cve_table);
    assert_int_equal(ret, 0);
}

void test_wm_vuldet_linux_nvd_vulnerabilities_app_pkg_mac_no_vendor(void **state)
{
    scan_agent *agent = *state;
    agent->dist = FEED_MAC;
    sqlite3 *db = (sqlite3 *)1;
    OSHash *cve_table = (OSHash *)1;
    sqlite3_stmt *stmt = NULL;
    int *vuln_count = NULL;

    expect_string(__wrap__mtdebug1, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug1, formatted_msg, "(5451): Analyzing NVD vulnerabilities for agent '000'");

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "000");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value_count(__wrap_sqlite3_column_text, iCol, 0, 1);
    will_return_count(__wrap_sqlite3_column_text, NULL, 1); // source
    expect_value_count(__wrap_sqlite3_column_text, iCol, 1, 2);
    will_return_count(__wrap_sqlite3_column_text, "safari", 2); // name
    expect_value_count(__wrap_sqlite3_column_text, iCol, 2, 2);
    will_return_count(__wrap_sqlite3_column_text, "0.0.0", 2); // version
    expect_value_count(__wrap_sqlite3_column_text, iCol, 3, 1);
    will_return_count(__wrap_sqlite3_column_text, NULL, 1); // pkg source version
    expect_value_count(__wrap_sqlite3_column_text, iCol, 4, 2);
    will_return_count(__wrap_sqlite3_column_text, "x64", 2); // architecture
    expect_value_count(__wrap_sqlite3_column_text, iCol, 5, 1);
    will_return_count(__wrap_sqlite3_column_text, NULL, 1); // vendor
    expect_value_count(__wrap_sqlite3_column_text, iCol, 6, 1);
    will_return_count(__wrap_sqlite3_column_text, NULL, 1); // reference
    expect_value_count(__wrap_sqlite3_column_text, iCol, 7, 1);
    will_return_count(__wrap_sqlite3_column_text, NULL, 1); // type

    // test_wm_vuldet_check_generic_package_siblings_OK
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "safari");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 2);
    expect_string(__wrap_sqlite3_bind_text, buffer, "");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 3);
    expect_string(__wrap_sqlite3_bind_text, buffer, "");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "0.0.0"); // start_including
    expect_value(__wrap_sqlite3_column_text, iCol, 2);
    will_return(__wrap_sqlite3_column_text, NULL); // start_excluding
    expect_value(__wrap_sqlite3_column_text, iCol, 3);
    will_return(__wrap_sqlite3_column_text, "1.0.0"); // end_including
    expect_value(__wrap_sqlite3_column_text, iCol, 4);
    will_return(__wrap_sqlite3_column_text, NULL); // end_excluding
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 6);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 7);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 8);
    will_return(__wrap_sqlite3_column_int, 1000); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 9);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 10);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 11);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    // Siblings
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_int, index, 1);
    expect_value(__wrap_sqlite3_bind_int, value, 1000);
    will_return(__wrap_sqlite3_bind_int, 0);
    expect_value(__wrap_sqlite3_bind_int, index, 2);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_int, iCol, 0);
    will_return(__wrap_sqlite3_column_int, 1);

    expect_sqlite3_step_call(SQLITE_DONE);

    will_return(__wrap_wm_vuldet_add_cve_node, 0);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5458): Package 'safari' inserted into the vulnerability 'CVE-0000-0000'. Version (0.0.0) 'less than or equal' '1.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    // test_wm_vuldet_check_specific_package_vulnerable()

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "safari");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 2);
    expect_string(__wrap_sqlite3_bind_text, buffer, "%0.0.0%");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 3);
    expect_string(__wrap_sqlite3_bind_text, buffer, "");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 4);
    expect_string(__wrap_sqlite3_bind_text, buffer, "");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "0.0.0"); // version_cmp
    expect_value(__wrap_sqlite3_column_int, iCol, 2);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 3);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 4);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 6);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 7);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 8);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    will_return(__wrap_wm_vuldet_add_cve_node, 0);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5458): Package 'safari' inserted into the vulnerability 'CVE-0000-0000'. Version (0.0.0) 'equals' '0.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    expect_sqlite3_step_call(SQLITE_DONE);

    expect_string(__wrap__mtdebug1, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug1, formatted_msg, "(5455): The NVD found a total of '2' potential vulnerabilities for agent '000'");

    expect_string(__wrap__mtdebug1, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug1, formatted_msg, "(5470): It took '0' seconds to 'find NVD' vulnerabilities in agent '000'");

    int ret = wm_vuldet_linux_nvd_vulnerabilities(db, agent, cve_table);
    assert_int_equal(ret, 0);
}

void test_wm_vuldet_linux_nvd_vulnerabilities_app_pkg_mac_with_vendor(void **state)
{
    scan_agent *agent = *state;
    agent->dist = FEED_MAC;
    sqlite3 *db = (sqlite3 *)1;
    OSHash *cve_table = (OSHash *)1;
    sqlite3_stmt *stmt = NULL;
    int *vuln_count = NULL;

    expect_string(__wrap__mtdebug1, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug1, formatted_msg, "(5451): Analyzing NVD vulnerabilities for agent '000'");

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "000");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value_count(__wrap_sqlite3_column_text, iCol, 0, 1);
    will_return_count(__wrap_sqlite3_column_text, NULL, 1); // source
    expect_value_count(__wrap_sqlite3_column_text, iCol, 1, 2);
    will_return_count(__wrap_sqlite3_column_text, "safari", 2); // name
    expect_value_count(__wrap_sqlite3_column_text, iCol, 2, 2);
    will_return_count(__wrap_sqlite3_column_text, "0.0.0", 2); // version
    expect_value_count(__wrap_sqlite3_column_text, iCol, 3, 1);
    will_return_count(__wrap_sqlite3_column_text, NULL, 1); // pkg source version
    expect_value_count(__wrap_sqlite3_column_text, iCol, 4, 2);
    will_return_count(__wrap_sqlite3_column_text, "x64", 2); // architecture
    expect_value_count(__wrap_sqlite3_column_text, iCol, 5, 2);
    will_return_count(__wrap_sqlite3_column_text, "apple", 2); // vendor
    expect_value_count(__wrap_sqlite3_column_text, iCol, 6, 1);
    will_return_count(__wrap_sqlite3_column_text, NULL, 1); // reference
    expect_value_count(__wrap_sqlite3_column_text, iCol, 7, 1);
    will_return_count(__wrap_sqlite3_column_text, NULL, 1); // type

    // test_wm_vuldet_check_generic_package_siblings_OK
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "safari");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 2);
    expect_string(__wrap_sqlite3_bind_text, buffer, "apple");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 3);
    expect_string(__wrap_sqlite3_bind_text, buffer, "apple");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "0.0.0"); // start_including
    expect_value(__wrap_sqlite3_column_text, iCol, 2);
    will_return(__wrap_sqlite3_column_text, NULL); // start_excluding
    expect_value(__wrap_sqlite3_column_text, iCol, 3);
    will_return(__wrap_sqlite3_column_text, "1.0.0"); // end_including
    expect_value(__wrap_sqlite3_column_text, iCol, 4);
    will_return(__wrap_sqlite3_column_text, NULL); // end_excluding
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 6);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 7);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 8);
    will_return(__wrap_sqlite3_column_int, 1000); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 9);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 10);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 11);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    // Siblings
    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_int, index, 1);
    expect_value(__wrap_sqlite3_bind_int, value, 1000);
    will_return(__wrap_sqlite3_bind_int, 0);
    expect_value(__wrap_sqlite3_bind_int, index, 2);
    expect_value(__wrap_sqlite3_bind_int, value, 0);
    will_return(__wrap_sqlite3_bind_int, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_int, iCol, 0);
    will_return(__wrap_sqlite3_column_int, 1);

    expect_sqlite3_step_call(SQLITE_DONE);

    will_return(__wrap_wm_vuldet_add_cve_node, 0);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5458): Package 'safari' inserted into the vulnerability 'CVE-0000-0000'. Version (0.0.0) 'less than or equal' '1.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    // test_wm_vuldet_check_specific_package_vulnerable()

    will_return(__wrap_sqlite3_prepare_v2, 1);
    will_return(__wrap_sqlite3_prepare_v2, SQLITE_OK);
    will_return(__wrap_sqlite3_finalize, SQLITE_OK);

    expect_value(__wrap_sqlite3_bind_text, pos, 1);
    expect_string(__wrap_sqlite3_bind_text, buffer, "safari");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 2);
    expect_string(__wrap_sqlite3_bind_text, buffer, "%0.0.0%");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 3);
    expect_string(__wrap_sqlite3_bind_text, buffer, "apple");
    will_return(__wrap_sqlite3_bind_text, 0);
    expect_value(__wrap_sqlite3_bind_text, pos, 4);
    expect_string(__wrap_sqlite3_bind_text, buffer, "apple");
    will_return(__wrap_sqlite3_bind_text, 0);

    expect_sqlite3_step_call(SQLITE_ROW);

    expect_value(__wrap_sqlite3_column_text, iCol, 0);
    will_return(__wrap_sqlite3_column_text, "CVE-0000-0000");
    expect_value(__wrap_sqlite3_column_text, iCol, 1);
    will_return(__wrap_sqlite3_column_text, "0.0.0"); // version_cmp
    expect_value(__wrap_sqlite3_column_int, iCol, 2);
    will_return(__wrap_sqlite3_column_int, 0); // package_id
    expect_value(__wrap_sqlite3_column_int, iCol, 3);
    will_return(__wrap_sqlite3_column_int, 0); // configuration_id
    expect_value(__wrap_sqlite3_column_int, iCol, 4);
    will_return(__wrap_sqlite3_column_int, 0); // package_vulnerable
    expect_value(__wrap_sqlite3_column_int, iCol, 5);
    will_return(__wrap_sqlite3_column_int, 0); // parent
    expect_value(__wrap_sqlite3_column_text, iCol, 6);
    will_return(__wrap_sqlite3_column_text, "OR"); // operator
    expect_value(__wrap_sqlite3_column_text, iCol, 7);
    will_return(__wrap_sqlite3_column_text, "vendor"); // vendor
    expect_value(__wrap_sqlite3_column_text, iCol, 8);
    will_return(__wrap_sqlite3_column_text, "*"); // target_sw

    will_return(__wrap_wm_checks_package_vulnerability, VU_VULNERABLE);

    will_return(__wrap_wm_vuldet_add_cve_node, 0);
    expect_string(__wrap__mtdebug2, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug2, formatted_msg, "(5458): Package 'safari' inserted into the vulnerability 'CVE-0000-0000'. Version (0.0.0) 'equals' '0.0.0' (feed 'NVD').");

    expect_sqlite3_step_call(SQLITE_DONE);

    expect_sqlite3_step_call(SQLITE_DONE);

    expect_string(__wrap__mtdebug1, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug1, formatted_msg, "(5455): The NVD found a total of '2' potential vulnerabilities for agent '000'");

    expect_string(__wrap__mtdebug1, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtdebug1, formatted_msg, "(5470): It took '0' seconds to 'find NVD' vulnerabilities in agent '000'");

    int ret = wm_vuldet_linux_nvd_vulnerabilities(db, agent, cve_table);
    assert_int_equal(ret, 0);
}

// Tests wm_vuldet_json_nvd_parser

void test_wm_vuldet_json_nvd_parser_feed_NULL(void **state)
{
    wm_vuldet_db *parsed_vulnerabilities = *state;
    char *json_feed = "{}";
    bool working = true;
    FILE * fp = NULL;

    fp = fopen("/tmp/feed_invalid.json", "w");
    fprintf(fp, "%s", json_feed);

    will_return(__wrap_strerror, "Could not open file");
    int ret = wm_vuldet_json_nvd_parser(fp, parsed_vulnerabilities, &working);
    assert_int_equal(OS_INVALID, ret);

    fclose(fp);
}

void test_wm_vuldet_json_nvd_parser_vuln_invalid(void **state)
{
    wm_vuldet_db *parsed_vulnerabilities = *state;
    char *json_feed = "{\"vulnerabilities\":[{\"c\":{]}}]}";
    char *cve;
    bool working = true;
    FILE * fp = NULL;

    fp = fopen("/tmp/feed_invalid.json", "w");
    fprintf(fp, "%s", json_feed);
    fclose(fp);
    fp = fopen("/tmp/feed_invalid.json", "r");

    int ret = wm_vuldet_json_nvd_parser(fp, parsed_vulnerabilities, &working);
    assert_int_equal(OS_INVALID, ret);
}

void test_wm_vuldet_json_nvd_parser_vuln_cve_invalid(void **state)
{
    wm_vuldet_db *parsed_vulnerabilities = *state;
    char *json_feed = "{\"vulnerabilities\":[{\"cve\":{]}}]}";
    char *cve;
    const char *match_cve = "{\"cve\":{";
    cve = strstr(json_feed, match_cve);
    bool working = true;
    FILE * fp = NULL;

    fp = fopen("/tmp/feed_invalid.json", "w");
    fprintf(fp, "%s", json_feed);
    fclose(fp);
    fp = fopen("/tmp/feed_invalid.json", "r");

    cJSON *cve_list = __real_cJSON_Parse(cve);
    will_return(__wrap_cJSON_Parse, cve_list);
    int ret = wm_vuldet_json_nvd_parser(fp, parsed_vulnerabilities, &working);
    assert_int_equal(OS_INVALID, ret);
    __real_cJSON_Delete(cve_list);
}

void test_wm_vuldet_json_nvd_parser_vuln_cve_iterations(void **state)
{   
    wm_vuldet_db *parsed_vulnerabilities = *state;
    cJSON *cve_list;
    char *json_feed = "{\"cve\":{\"id\":\"CVE\"},\"cve\":{\"id\":\"CVE\"}}";

    bool working = true;
    char * buffer = NULL;
    FILE * fp = NULL;

    fp = fopen("/tmp/feed_iterations.json", "w");
    fprintf(fp, "%s", json_feed);
    fclose(fp);
    fp = fopen("/tmp/feed_warning.json", "r");
  
    char *cve;
    const char *match_cve = "{\"cve\":{";
    int num_vuln = 2;
 
    cve = strstr(json_feed, match_cve);
    cve_list = __real_cJSON_Parse(cve);
    for (int i = 1; i < num_vuln; i++) {
        will_return(__wrap_cJSON_Parse, cve_list);
        expect_function_call(__wrap_cJSON_Delete);
    }

    int ret = wm_vuldet_json_nvd_parser(fp, parsed_vulnerabilities, &working);
    assert_int_equal(OS_SUCCESS, ret);
    __real_cJSON_Delete(cve_list);

    fp = fopen("/tmp/feed_warning.json", "r");
    cve = strstr(json_feed, match_cve);
    cve_list = __real_cJSON_Parse(cve);
    for (int i = 1; i < num_vuln; i++) {
        will_return(__wrap_cJSON_Parse, cve_list);
        expect_function_call(__wrap_cJSON_Delete);
    }

    ret = wm_vuldet_json_nvd_parser(fp, parsed_vulnerabilities, &working);
    assert_int_equal(OS_SUCCESS, ret);
    __real_cJSON_Delete(cve_list);

}

void test_wm_vuldet_json_nvd_parser_warnings(void **state)
{
    wm_vuldet_db *parsed_vulnerabilities = *state;
    char *json_feed = nvd_feed_mock_cve;
 
    bool working = true;
    char * buffer = NULL;
    FILE * fp = NULL;

    fp = fopen("/tmp/feed_warning.json", "w");
    fprintf(fp, "%s", json_feed);
    fclose(fp);
    fp = fopen("/tmp/feed_warning.json", "r");

    char *cve;
    const char *match_cve = "{\"cve\":{";
    cve = strstr(json_feed, match_cve);
    cJSON *cve_list = __real_cJSON_Parse(cve);
    will_return(__wrap_cJSON_Parse, cve_list);
    expect_string(__wrap__mtwarn, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtwarn, formatted_msg, "(5536): An unexpected tag was found when processing the CVE list 'warningCVEData'");
    expect_function_call(__wrap_cJSON_Delete);
    expect_string(__wrap__mtwarn, tag, "wazuh-modulesd:vulnerability-detector");
    expect_string(__wrap__mtwarn, formatted_msg, "(5536): An unexpected tag was found when processing the CVE list 'cpe'");
  
    int ret = wm_vuldet_json_nvd_parser(fp, parsed_vulnerabilities, &working);

    assert_string_equal("CVE-2020-0504", parsed_vulnerabilities->nvd_vulnerabilities->id);
    assert_string_equal("secure@intel.com", parsed_vulnerabilities->nvd_vulnerabilities->assigner);
    assert_string_equal("2020-03-12T18:15:12.023", parsed_vulnerabilities->nvd_vulnerabilities->published);
    assert_string_equal("2021-05-19T17:00:01.097", parsed_vulnerabilities->nvd_vulnerabilities->last_modified);
    assert_int_equal(OS_SUCCESS, ret);
    __real_cJSON_Delete(cve_list);
    
}

void test_wm_vuldet_json_nvd_parser_node_descriptions(void **state)
{
    nvd_vulnerability  *nvd_vulnerabilities = *state;

    cJSON *cve_list = __real_cJSON_Parse(nvd_feed_mock_descriptions);

    int ret = wm_vuldet_parse_nvd_descriptions(cve_list->child, nvd_vulnerabilities);
    assert_string_equal("Buffer overflow.", nvd_vulnerabilities->description);
    assert_int_equal(OS_SUCCESS, ret);
    __real_cJSON_Delete(cve_list);
}

void test_wm_vuldet_json_nvd_parser_node_metrics(void **state)
{
    nvd_vulnerability  *nvd_vulnerabilities = *state;

    cJSON *cve_list = __real_cJSON_Parse(nvd_feed_mock_metrics);

    int ret = wm_vuldet_parse_nvd_metrics(cve_list->child, nvd_vulnerabilities);

    assert_string_equal("3.1", nvd_vulnerabilities->cvss3->version);
    assert_string_equal("/AV:L/AC:L/PR:L/UI:N/S:U/C:H/I:H/A:H", nvd_vulnerabilities->cvss3->vector_string);
    assert_float_equal(7.8, nvd_vulnerabilities->cvss3->base_score, 1);
    assert_float_equal(1.8, nvd_vulnerabilities->cvss3->exploitability_score, 1);
    assert_float_equal(5.9, nvd_vulnerabilities->cvss3->impact_score, 1);
    assert_string_equal("2.0", nvd_vulnerabilities->cvss2->version);
    assert_string_equal("AV:L/AC:L/Au:N/C:P/I:P/A:P", nvd_vulnerabilities->cvss2->vector_string);
    assert_float_equal(4.6, nvd_vulnerabilities->cvss2->base_score, 1);
    assert_float_equal(3.9, nvd_vulnerabilities->cvss2->exploitability_score, 1);
    assert_float_equal(6.4, nvd_vulnerabilities->cvss2->impact_score, 1);
    
    assert_int_equal(OS_SUCCESS, ret);
    __real_cJSON_Delete(cve_list);
}

void test_wm_vuldet_json_nvd_parser_node_weaknesses(void **state)
{
    nvd_vulnerability  *nvd_vulnerabilities = *state;

    cJSON *cve_list = __real_cJSON_Parse(nvd_feed_mock_weaknesses);

    int ret = wm_vuldet_parse_nvd_weaknesses(cve_list->child, nvd_vulnerabilities);
    assert_string_equal("CWE-120", nvd_vulnerabilities->cwe);
    
    assert_int_equal(OS_SUCCESS, ret);
    __real_cJSON_Delete(cve_list);
}

void test_wm_vuldet_json_nvd_parser_node_configurations(void **state)
{
    nvd_vulnerability  *nvd_vulnerabilities = *state;
    cJSON *cve_list = __real_cJSON_Parse(nvd_feed_mock_configurations);

    int ret = wm_vuldet_parse_nvd_configurations(cve_list->child, nvd_vulnerabilities);

    assert_string_equal("AND", nvd_vulnerabilities->configuration->operator);
    assert_string_equal("OR", nvd_vulnerabilities->configuration->children->operator);
    assert_int_equal(1, nvd_vulnerabilities->configuration->children->cpe_matches->vulnerable);
    assert_string_equal("cpe:2.3:a:xpdfreader:xpdf:3.03-17:*:*:*:*:*:*:*", nvd_vulnerabilities->configuration->children->cpe_matches->cpe_uri23);
    assert_string_equal("15.40", nvd_vulnerabilities->configuration->children->cpe_matches->version_start_including);
    assert_string_equal("11.10", nvd_vulnerabilities->configuration->children->cpe_matches->version_end_including);
    assert_string_equal("10.30", nvd_vulnerabilities->configuration->children->cpe_matches->version_start_excluding);
    assert_string_equal("15.40.44.5107", nvd_vulnerabilities->configuration->children->cpe_matches->version_end_excluding);
    assert_string_equal("OR", nvd_vulnerabilities->configuration->children->next->operator);
    assert_int_equal(0, nvd_vulnerabilities->configuration->children->next->cpe_matches->vulnerable);
    assert_string_equal("cpe:2.3:o:debian:debian_linux:8.0:*:*:*:*:*:*:*", nvd_vulnerabilities->configuration->children->next->cpe_matches->cpe_uri23);
    
    assert_int_equal(OS_SUCCESS, ret);
    __real_cJSON_Delete(cve_list);
}

void test_wm_vuldet_json_nvd_parser_node_references(void **state)
{
    nvd_vulnerability  *nvd_vulnerabilities = *state;

    cJSON *cve_list = __real_cJSON_Parse(nvd_feed_mock_references);

    int ret = wm_vuldet_parse_nvd_references(cve_list->child, nvd_vulnerabilities);
    assert_string_equal("https://security.netapp.com/advisory/ntap-20200320-0003/", nvd_vulnerabilities->references->url);
    assert_string_equal("secure@intel.com", nvd_vulnerabilities->references->refsource);
    assert_string_equal("https://www.intel.com/content/www/us/en/security-center/advisory/intel-sa-00315.html", nvd_vulnerabilities->references->next->url);
    assert_string_equal("secure@intel.com", nvd_vulnerabilities->references->next->refsource);
    assert_int_equal(OS_SUCCESS, ret);
    __real_cJSON_Delete(cve_list);
}

void test_wm_vuldet_json_nvd_parser_complete(void **state)
{

    wm_vuldet_db *parsed_vulnerabilities = *state;
    char *json_feed = nvd_feed_mock_complete;
    bool working = true;
    char * buffer = NULL;
    FILE * fp = NULL;

    fp = fopen("/tmp/feed_complete.json", "w");
    fprintf(fp, "%s", json_feed);
    fclose(fp);
    fp = fopen("/tmp/feed_complete.json", "r");

    char *cve;
    const char *match_cve = "{\"cve\":{";
    cve = strstr(json_feed, match_cve);
    cJSON *cve_list = __real_cJSON_Parse(cve);
    will_return(__wrap_cJSON_Parse, cve_list);
    expect_function_call(__wrap_cJSON_Delete);
    int ret = wm_vuldet_json_nvd_parser(fp, parsed_vulnerabilities, &working);

    assert_string_equal("CVE-2020-0504", parsed_vulnerabilities->nvd_vulnerabilities->id);
    assert_string_equal("secure@intel.com", parsed_vulnerabilities->nvd_vulnerabilities->assigner);
    assert_string_equal("2020-03-12T18:15:12.023", parsed_vulnerabilities->nvd_vulnerabilities->published);
    assert_string_equal("2021-05-19T17:00:01.097", parsed_vulnerabilities->nvd_vulnerabilities->last_modified);
    assert_string_equal("Buffer overflow.", parsed_vulnerabilities->nvd_vulnerabilities->description);
    assert_string_equal("3.1", parsed_vulnerabilities->nvd_vulnerabilities->cvss3->version);
    assert_string_equal("/AV:L/AC:L/PR:L/UI:N/S:U/C:H/I:H/A:H", parsed_vulnerabilities->nvd_vulnerabilities->cvss3->vector_string);
    assert_float_equal(7.8, parsed_vulnerabilities->nvd_vulnerabilities->cvss3->base_score, 1);
    assert_float_equal(1.8, parsed_vulnerabilities->nvd_vulnerabilities->cvss3->exploitability_score, 1);
    assert_float_equal(5.9, parsed_vulnerabilities->nvd_vulnerabilities->cvss3->impact_score, 1);
    assert_string_equal("2.0", parsed_vulnerabilities->nvd_vulnerabilities->cvss2->version);
    assert_string_equal("AV:L/AC:L/Au:N/C:P/I:P/A:P", parsed_vulnerabilities->nvd_vulnerabilities->cvss2->vector_string);
    assert_float_equal(4.6, parsed_vulnerabilities->nvd_vulnerabilities->cvss2->base_score, 1);
    assert_float_equal(3.9, parsed_vulnerabilities->nvd_vulnerabilities->cvss2->exploitability_score, 1);
    assert_float_equal(6.4, parsed_vulnerabilities->nvd_vulnerabilities->cvss2->impact_score, 1);
    assert_string_equal("CWE-120", parsed_vulnerabilities->nvd_vulnerabilities->cwe);
    assert_string_equal("OR", parsed_vulnerabilities->nvd_vulnerabilities->configuration->operator);
    assert_int_equal(1, parsed_vulnerabilities->nvd_vulnerabilities->configuration->cpe_matches->vulnerable);
    assert_string_equal("cpe:2.3:a:intel:graphics_driver:*:*:*:*:*:*:*:*", parsed_vulnerabilities->nvd_vulnerabilities->configuration->cpe_matches->cpe_uri23);
    assert_string_equal("15.40", parsed_vulnerabilities->nvd_vulnerabilities->configuration->cpe_matches->version_start_including);
    assert_null(parsed_vulnerabilities->nvd_vulnerabilities->configuration->cpe_matches->version_end_including);
    assert_null(parsed_vulnerabilities->nvd_vulnerabilities->configuration->cpe_matches->version_start_excluding);
    assert_string_equal("15.40.44.5107", parsed_vulnerabilities->nvd_vulnerabilities->configuration->cpe_matches->version_end_excluding);
    assert_int_equal(1, parsed_vulnerabilities->nvd_vulnerabilities->configuration->cpe_matches->next->vulnerable);
    assert_string_equal("cpe:2.3:a:intel:graphics_driver:*:*:*:*:*:*:*:*", parsed_vulnerabilities->nvd_vulnerabilities->configuration->cpe_matches->next->cpe_uri23);
    assert_string_equal("15.45", parsed_vulnerabilities->nvd_vulnerabilities->configuration->cpe_matches->next->version_start_including);
    assert_null(parsed_vulnerabilities->nvd_vulnerabilities->configuration->cpe_matches->next->version_end_including);
    assert_null(parsed_vulnerabilities->nvd_vulnerabilities->configuration->cpe_matches->next->version_start_excluding);
    assert_string_equal("https://security.netapp.com/advisory/ntap-20200320-0003/", parsed_vulnerabilities->nvd_vulnerabilities->references->url);
    assert_string_equal("secure@intel.com", parsed_vulnerabilities->nvd_vulnerabilities->references->refsource);
    assert_string_equal("https://www.intel.com/content/www/us/en/security-center/advisory/intel-sa-00315.html", parsed_vulnerabilities->nvd_vulnerabilities->references->next->url);
    assert_string_equal("secure@intel.com", parsed_vulnerabilities->nvd_vulnerabilities->references->next->refsource);

    assert_int_equal(OS_SUCCESS, ret);
    __real_cJSON_Delete(cve_list);
 
}



/* Tests */

int main(void)
{
    const struct CMUnitTest tests[] = {
        // Tests wm_vuldet_clean_version
        cmocka_unit_test_setup_teardown(test_wm_vuldet_clean_version_complete, setup_version, teardown_version),
        cmocka_unit_test_setup_teardown(test_wm_vuldet_clean_version_no_epoch, setup_version, teardown_version),
        cmocka_unit_test_setup_teardown(test_wm_vuldet_clean_version_no_release, setup_version, teardown_version),
        cmocka_unit_test_setup_teardown(test_wm_vuldet_clean_version_no_epoch_release, setup_version, teardown_version),
        // Tests wm_vuldet_get_children
        cmocka_unit_test_setup_teardown(test_wm_vuldet_get_children_prepare_error, setup_children, teardown_children),
        cmocka_unit_test_setup_teardown(test_wm_vuldet_get_children_done, setup_children, teardown_children),
        cmocka_unit_test_setup_teardown(test_wm_vuldet_get_children_OK, setup_children, teardown_children),
        // Tests wm_vuldet_get_siblings
        cmocka_unit_test_setup_teardown(test_wm_vuldet_get_siblings_prepare_error, setup_siblings, teardown_siblings),
        cmocka_unit_test_setup_teardown(test_wm_vuldet_get_siblings_done, setup_siblings, teardown_siblings),
        cmocka_unit_test_setup_teardown(test_wm_vuldet_get_siblings_OK, setup_siblings, teardown_siblings),
        // Tests vuln-detector-unit-test
        cmocka_unit_test(test_vw_vuldet_filter_vulnerabilities_vendor_wrong_Ubuntu),
        cmocka_unit_test(test_vw_vuldet_filter_vulnerabilities_vendor_wrong_Arch),
        cmocka_unit_test(test_vw_vuldet_filter_vulnerabilities_vendor_wrong_Arch_vendor),
        cmocka_unit_test(test_vw_vuldet_filter_vulnerabilities_vendor_wrong_Debian),
        cmocka_unit_test(test_vw_vuldet_filter_vulnerabilities_vendor_wrong_Redhat),
        cmocka_unit_test(test_vw_vuldet_filter_vulnerabilities_vendor_wrong_Amazon),
        cmocka_unit_test(test_vw_vuldet_filter_vulnerabilities_vendor_wrong_SUSE),
        cmocka_unit_test(test_vw_vuldet_filter_vulnerabilities_target_sw_not_whitelist),
        cmocka_unit_test(test_vw_vuldet_filter_vulnerabilities_vendor_target_sw_OK),
        cmocka_unit_test(test_vw_vuldet_filter_vulnerabilities_vendor_target_sw_mac_OK),
        cmocka_unit_test(test_vw_vuldet_filter_vulnerabilities_vendor_target_sw_mac_Err),
        // Tests wm_vuldet_check_generic_package
        cmocka_unit_test(test_wm_vuldet_check_generic_package_prepare_error),
        cmocka_unit_test(test_wm_vuldet_check_generic_package_done),
        cmocka_unit_test(test_wm_vuldet_check_generic_package_skip),
        cmocka_unit_test(test_wm_vuldet_check_generic_package_start_including_no_vulnerable),
        cmocka_unit_test(test_wm_vuldet_check_generic_package_start_including_vulnerable),
        cmocka_unit_test(test_wm_vuldet_check_generic_package_start_excluding_no_vulnerable),
        cmocka_unit_test(test_wm_vuldet_check_generic_package_start_excluding_vulnerable),
        cmocka_unit_test(test_wm_vuldet_check_generic_package_no_starts_no_vulnerable),
        cmocka_unit_test(test_wm_vuldet_check_generic_package_no_starts_vulnerable),
        cmocka_unit_test(test_wm_vuldet_check_generic_package_end_including_no_vulnerable),
        cmocka_unit_test(test_wm_vuldet_check_generic_package_end_including_vulnerable),
        cmocka_unit_test(test_wm_vuldet_check_generic_package_end_excluding_no_vulnerable),
        cmocka_unit_test(test_wm_vuldet_check_generic_package_end_excluding_vulnerable),
        cmocka_unit_test(test_wm_vuldet_check_generic_package_no_ends_no_vulnerable),
        cmocka_unit_test(test_wm_vuldet_check_generic_package_no_ends_vulnerable),
        cmocka_unit_test(test_wm_vuldet_check_generic_package_no_starts_no_ends_children),
        cmocka_unit_test(test_wm_vuldet_check_generic_package_no_children_no_siblings),
        cmocka_unit_test(test_wm_vuldet_check_generic_package_children_invalid),
        cmocka_unit_test(test_wm_vuldet_check_generic_package_children_OK),
        cmocka_unit_test(test_wm_vuldet_check_generic_package_siblings_invalid),
        cmocka_unit_test(test_wm_vuldet_check_generic_package_siblings_OK),
        cmocka_unit_test(test_wm_vuldet_check_generic_package_ignore_package),
        cmocka_unit_test(test_wm_vuldet_check_generic_package_insert_package_error),
        cmocka_unit_test(test_wm_vuldet_check_generic_package_duplicated_package),
        cmocka_unit_test(test_wm_vuldet_check_generic_package_insert_package_OK),
        cmocka_unit_test(test_wm_vuldet_check_generic_package_insert_package_Ubuntu),
        cmocka_unit_test(test_wm_vuldet_check_generic_package_insert_package_Debian),
        cmocka_unit_test(test_wm_vuldet_check_generic_package_insert_package_RedHat),
        cmocka_unit_test(test_wm_vuldet_check_generic_package_insert_package_Arch),
        cmocka_unit_test(test_wm_vuldet_check_generic_package_insert_package_linux_kernel),
        cmocka_unit_test(test_wm_vuldet_check_generic_package_insert_package_mac_os_x),
        cmocka_unit_test(test_wm_vuldet_check_generic_package_insert_package_mac_os_x_server),
        cmocka_unit_test(test_wm_vuldet_check_generic_package_insert_package_mac_os),
        cmocka_unit_test(test_wm_vuldet_check_generic_package_insert_package_macos),
        cmocka_unit_test(test_wm_vuldet_check_generic_package_insert_package_mac_no_vendor),
        cmocka_unit_test(test_wm_vuldet_check_generic_package_insert_package_mac_with_vendor),
        // Tests wm_vuldet_check_specific_package
        cmocka_unit_test(test_wm_vuldet_check_specific_package_pkg_version_NULL),
        cmocka_unit_test(test_wm_vuldet_check_specific_package_pkg_version_empty),
        cmocka_unit_test(test_wm_vuldet_check_specific_package_prepare_error),
        cmocka_unit_test(test_wm_vuldet_check_specific_package_done),
        cmocka_unit_test(test_wm_vuldet_check_specific_package_skip),
        cmocka_unit_test(test_wm_vuldet_check_specific_package_no_vulnerable),
        cmocka_unit_test(test_wm_vuldet_check_specific_package_vulnerable),
        cmocka_unit_test(test_wm_vuldet_check_specific_package_no_children_no_siblings),
        cmocka_unit_test(test_wm_vuldet_check_specific_package_children_invalid),
        cmocka_unit_test(test_wm_vuldet_check_specific_package_children_OK),
        cmocka_unit_test(test_wm_vuldet_check_specific_package_siblings_invalid),
        cmocka_unit_test(test_wm_vuldet_check_specific_package_siblings_OK),
        cmocka_unit_test(test_wm_vuldet_check_specific_package_os_package_valid),
        cmocka_unit_test(test_wm_vuldet_check_specific_package_ignore_package),
        cmocka_unit_test(test_wm_vuldet_check_specific_package_insert_package_error),
        cmocka_unit_test(test_wm_vuldet_check_specific_package_duplicated_package),
        cmocka_unit_test(test_wm_vuldet_check_specific_package_insert_package_OK),
        cmocka_unit_test(test_wm_vuldet_check_specific_package_insert_package_Ubuntu),
        cmocka_unit_test(test_wm_vuldet_check_specific_package_insert_package_Debian),
        cmocka_unit_test(test_wm_vuldet_check_specific_package_insert_package_Arch),
        cmocka_unit_test(test_wm_vuldet_check_specific_package_insert_package_RedHat),
        cmocka_unit_test(test_wm_vuldet_check_specific_package_insert_package_linux_kernel),
        cmocka_unit_test(test_wm_vuldet_check_specific_package_insert_package_mac_os_x),
        cmocka_unit_test(test_wm_vuldet_check_specific_package_insert_package_mac_os_x_server),
        cmocka_unit_test(test_wm_vuldet_check_specific_package_insert_package_mac_os),
        cmocka_unit_test(test_wm_vuldet_check_specific_package_insert_package_macos),
        cmocka_unit_test(test_wm_vuldet_check_specific_package_insert_package_mac_no_vendor),
        cmocka_unit_test(test_wm_vuldet_check_specific_package_insert_package_mac_with_vendor),
        // Tests wm_vuldet_linux_nvd_vulnerabilities
        cmocka_unit_test_setup_teardown(test_wm_vuldet_linux_nvd_vulnerabilities_prepare_error, setup_scan_agent, teardown_scan_agent),
        cmocka_unit_test_setup_teardown(test_wm_vuldet_linux_nvd_vulnerabilities_done, setup_scan_agent, teardown_scan_agent),
        cmocka_unit_test_setup_teardown(test_wm_vuldet_linux_nvd_vulnerabilities_no_source_no_name, setup_scan_agent, teardown_scan_agent),
        cmocka_unit_test_setup_teardown(test_wm_vuldet_linux_nvd_vulnerabilities_source_generic_invalid, setup_scan_agent, teardown_scan_agent),
        cmocka_unit_test_setup_teardown(test_wm_vuldet_linux_nvd_vulnerabilities_source_specific_invalid, setup_scan_agent, teardown_scan_agent),
        cmocka_unit_test_setup_teardown(test_wm_vuldet_linux_nvd_vulnerabilities_source_no_vulnerable, setup_scan_agent, teardown_scan_agent),
        cmocka_unit_test_setup_teardown(test_wm_vuldet_linux_nvd_vulnerabilities_source_vulnerable, setup_scan_agent, teardown_scan_agent),
        cmocka_unit_test_setup_teardown(test_wm_vuldet_linux_nvd_vulnerabilities_name_generic_invalid, setup_scan_agent, teardown_scan_agent),
        cmocka_unit_test_setup_teardown(test_wm_vuldet_linux_nvd_vulnerabilities_name_specific_invalid, setup_scan_agent, teardown_scan_agent),
        cmocka_unit_test_setup_teardown(test_wm_vuldet_linux_nvd_vulnerabilities_name_no_vulnerable, setup_scan_agent, teardown_scan_agent),
        cmocka_unit_test_setup_teardown(test_wm_vuldet_linux_nvd_vulnerabilities_name_vulnerable, setup_scan_agent, teardown_scan_agent),
        cmocka_unit_test_setup_teardown(test_wm_vuldet_linux_nvd_vulnerabilities_pkg_source_version_NULL, setup_scan_agent, teardown_scan_agent),
        cmocka_unit_test_setup_teardown(test_wm_vuldet_linux_nvd_vulnerabilities_os_pkg_mac, setup_scan_agent, teardown_scan_agent),
        cmocka_unit_test_setup_teardown(test_wm_vuldet_linux_nvd_vulnerabilities_app_pkg_mac_no_vendor, setup_scan_agent, teardown_scan_agent),
        cmocka_unit_test_setup_teardown(test_wm_vuldet_linux_nvd_vulnerabilities_app_pkg_mac_with_vendor, setup_scan_agent, teardown_scan_agent),
        // Tests wm_vuldet_json_nvd_parser
        cmocka_unit_test_setup_teardown(test_wm_vuldet_json_nvd_parser_feed_NULL, setup_json_parser_decode, teardown_json_parser_decode),
        cmocka_unit_test_setup_teardown(test_wm_vuldet_json_nvd_parser_vuln_invalid, setup_json_parser_decode, teardown_json_parser_decode),
        cmocka_unit_test_setup_teardown(test_wm_vuldet_json_nvd_parser_vuln_cve_invalid, setup_json_parser_decode, teardown_json_parser_decode),
        cmocka_unit_test_setup_teardown(test_wm_vuldet_json_nvd_parser_node_descriptions, setup_json_parser_nvd, teardown_json_parser_nvd),
        cmocka_unit_test_setup_teardown(test_wm_vuldet_json_nvd_parser_node_metrics, setup_json_parser_nvd, teardown_json_parser_nvd),
        cmocka_unit_test_setup_teardown(test_wm_vuldet_json_nvd_parser_node_weaknesses, setup_json_parser_nvd, teardown_json_parser_nvd),
        cmocka_unit_test_setup_teardown(test_wm_vuldet_json_nvd_parser_node_configurations, setup_json_parser_nvd, teardown_json_parser_nvd),
        cmocka_unit_test_setup_teardown(test_wm_vuldet_json_nvd_parser_node_references, setup_json_parser_nvd, teardown_json_parser_nvd),
        cmocka_unit_test_setup_teardown(test_wm_vuldet_json_nvd_parser_complete, setup_json_parser_decode, teardown_json_parser_decode),
        cmocka_unit_test_setup_teardown(test_wm_vuldet_json_nvd_parser_warnings, setup_json_parser_decode, teardown_json_parser_decode),
        cmocka_unit_test_setup_teardown(test_wm_vuldet_json_nvd_parser_vuln_cve_iterations, setup_json_parser_decode, teardown_json_parser_decode),

    };
    return cmocka_run_group_tests(tests, NULL, NULL);
}

